@@ -1,4 +1,4 @@
-/*  $Id: blast_options.hpp 363884 2012-05-21 15:54:30Z morgulis $
+/*  $Id: blast_options.hpp 438020 2014-06-12 12:13:19Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -127,6 +127,8 @@ public:
     /// Returns the CORE BLAST notion of program type
     EBlastProgramType GetProgramType() const;
 
+    bool IsIterativeSearch() const;
+
     /******************* Lookup table options ***********************/
     /// Returns WordThreshold
     double GetWordThreshold() const;
@@ -302,6 +304,9 @@ public:
     double GetPercentIdentity() const;
     void SetPercentIdentity(double p);
 
+    double GetQueryCovHspPerc() const;
+    void SetQueryCovHspPerc(double p);
+
     int GetMinDiagSeparation() const;
     void SetMinDiagSeparation(int d);
 
