@@ -1,4 +1,4 @@
-/*  $Id: blast_args.hpp 402743 2013-06-10 17:41:20Z boratyng $
+/*  $Id: blast_args.hpp 439436 2014-06-30 13:51:29Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -853,6 +853,12 @@ public:
         eCommaSeparatedValues,
         /// BLAST archive format
         eArchiveFormat,
+        /// JSON seq-align
+        eJsonSeqalign,
+        /// JSON output
+        eJson,
+        /// XML2 output
+        eXml2,
         /// Sentinel value for error checking
         eEndValue
     };
@@ -863,7 +869,8 @@ public:
         m_NumDescriptions(0), m_NumAlignments(0),
         m_DfltNumDescriptions(0), m_DfltNumAlignments(0),
         m_Html(false),
-        m_IsIgBlast(isIgblast)
+        m_IsIgBlast(isIgblast),
+        m_LineLength(align_format::kDfltLineLength)
     {
         if (m_IsIgBlast) {
             m_DfltNumAlignments = m_DfltNumDescriptions = 10;
@@ -903,7 +910,10 @@ public:
     bool HasStructuredOutputFormat() const {
         return m_OutputFormat == eXml || 
             m_OutputFormat == eAsnText ||
-            m_OutputFormat == eAsnBinary;
+            m_OutputFormat == eAsnBinary ||
+            m_OutputFormat == eXml2 ||
+            m_OutputFormat == eJson ||
+            m_OutputFormat == eJsonSeqalign;
     }
 
     /// Display the NCBI GIs in formatted output?
@@ -931,6 +941,9 @@ public:
 
     bool ArchiveFormatRequested(const CArgs& args) const;
 
+    size_t GetLineLength() const {
+    	return m_LineLength;
+    }
 private:
     EOutputFormat m_OutputFormat;   ///< Choice of formatting output
     bool m_ShowGis;                 ///< Display NCBI GIs?
@@ -943,6 +956,7 @@ private:
     /// The format specification for custom output, e.g.: tabular or
     /// comma-separated value (populated if applicable)
     string m_CustomOutputFormatSpec;
+    size_t m_LineLength;
 
 };
 
@@ -1205,6 +1219,14 @@ protected:
     x_CreateOptionsHandle(CBlastOptions::EAPILocality locality,
                           const CArgs& args) = 0;
 
+    /** Creates the BLAST options handle based on the task argument 
+     * @param locality whether the search will be executed locally or remotely [in]
+     * @param task program-specific BLAST named parameter set [in]
+     */
+    CRef<CBlastOptionsHandle>
+    x_CreateOptionsHandleWithTask(CBlastOptions::EAPILocality locality,
+                                  const string& task);
+
     /// Issue warnings when recovering from a search strategy (command line
     /// applications only)
     void x_IssueWarningsForIgnoredOptions(const CArgs& args);
