@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.hpp 418601 2013-11-18 20:02:12Z rafanovi $
+/*  $Id: cmdline_flags.hpp 439436 2014-06-30 13:51:29Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -247,6 +247,7 @@ NCBI_BLASTINPUT_EXPORT extern const string kDfltArgLookupTableMaskingOnlyNucl;
 
 /// Argument to select the number of iterations to perform in PSI-BLAST
 NCBI_BLASTINPUT_EXPORT extern const string kArgPSINumIterations;
+NCBI_BLASTINPUT_EXPORT extern const unsigned int kDfltArgPSINumIterations;
 
 /// Argument to specify a 'checkpoint' file to recover the PSSM from
 NCBI_BLASTINPUT_EXPORT extern const string kArgPSIInputChkPntFile;
@@ -346,6 +347,11 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgRid;
 /// Argument to blast_formatter to request BLAST archive file name
 NCBI_BLASTINPUT_EXPORT extern const string kArgArchive;
 
+/// Argument to specify min query coverage percentage for each hsp
+NCBI_BLASTINPUT_EXPORT extern const string kArgQueryCovHspPerc;
+
+/// Argument to specify line length for displaying alignments
+NCBI_BLASTINPUT_EXPORT extern const string kArgLineLength;
 
 END_SCOPE(blast)
 END_NCBI_SCOPE
