@@ -1,7 +1,7 @@
 #ifndef COMMON___NCBI_EXPORT__H
 #define COMMON___NCBI_EXPORT__H
 
-/*  $Id: ncbi_export.h 411396 2013-08-28 14:45:53Z rafanovi $
+/*  $Id: ncbi_export.h 437786 2014-06-10 14:27:42Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -111,6 +111,7 @@
 #ifdef NCBI_SEQ_EXPORTS
 #  define NCBI_BLASTDB_EXPORTS
 #  define NCBI_BLASTXML_EXPORTS
+#  define NCBI_BLASTXML2_EXPORTS
 #  define NCBI_BLAST_EXPORTS
 #  define NCBI_GENOME_COLLECTION_EXPORTS
 #  define NCBI_SCOREMAT_EXPORTS
@@ -151,6 +152,7 @@
 #  define NCBI_UNIT_TEST_UTIL_EXPORTS
 #  define NCBI_UUDUTIL_EXPORTS
 #  define NCBI_VALERR_EXPORTS
+#  define NCBI_VALID_EXPORTS
 #  define NCBI_BLASTDB_FORMAT_EXPORTS
 #endif
 
@@ -332,6 +334,14 @@
 #  define NCBI_BLASTXML_EXPORT NCBI_DLL_IMPORT
 #endif
 
+/* Export specifier for library blastxml2
+ */
+#ifdef NCBI_BLASTXML2_EXPORTS
+#  define NCBI_BLASTXML2_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_BLASTXML2_EXPORT NCBI_DLL_IMPORT
+#endif
+
 /* Export specifier for library blast
  */
 #ifdef NCBI_BLAST_EXPORTS
@@ -901,6 +911,22 @@
 #  define NCBI_VALERR_EXPORT NCBI_DLL_IMPORT
 #endif
 
+/* Export specifier for library valid
+ */
+#ifdef NCBI_VALID_EXPORTS
+#  define NCBI_VALID_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_VALID_EXPORT NCBI_DLL_IMPORT
+#endif
+
+/* Export specifier for library discrepancy_report
+ */
+#ifdef NCBI_DISCREPANCY_REPORT_EXPORTS
+#  define NCBI_DISCREPANCY_REPORT_EXPORT NCBI_DLL_EXPORT
+#else
+#  define NCBI_DISCREPANCY_REPORT_EXPORT NCBI_DLL_IMPORT
+#endif
+
 /* Export specifier for library cleanup
  */
 #ifdef NCBI_CLEANUP_EXPORTS
