@@ -1,7 +1,7 @@
 #ifndef CONNECT_EXT___NCBI_IFCONF__H
 #define CONNECT_EXT___NCBI_IFCONF__H
 
-/* $Id: ncbi_ifconf.h 405498 2013-07-03 18:00:22Z lavr $
+/* $Id: ncbi_ifconf.h 430177 2014-03-24 17:14:08Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -60,7 +60,7 @@ typedef struct {
     unsigned int address;       /* Primary address, network byte order(b.o.) */
     unsigned int netmask;       /* Primary netmask, network byte order       */
     unsigned int broadcast;     /* Primary broadcast address, network b.o.   */
-    int          nifs;          /* Number of network interfaces detected     */
+    int          nifs;          /* Number of network interfaces seen         */
     int          sifs;          /* Number of network interfaces skipped      */
     size_t       mtu;           /* MTU if known for the returned address     */
 } SNcbiIfConf;
