@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONN_STREAM__HPP
 #define CONNECT___NCBI_CONN_STREAM__HPP
 
-/* $Id: ncbi_conn_stream.hpp 376836 2012-10-04 15:03:27Z lavr $
+/* $Id: ncbi_conn_stream.hpp 424667 2014-01-17 22:34:51Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,7 +43,7 @@
  *
  *   CConn_HttpStream
  *      I/O stream based on HTTP connector (that is, the stream, which
- *      connects to HTTP server and exchanges information using HTTP
+ *      connects to HTTP server and exchanges information using HTTP(S)
  *      protocol).
  *
  *   CConn_ServiceStream
@@ -75,6 +75,7 @@
 #include <connect/ncbi_service_connector.h>
 #include <connect/ncbi_socket_connector.h>
 #include <util/icanceled.hpp>
+#include <utility>
 
 
 /** @addtogroup ConnStreams
@@ -123,7 +124,6 @@ public:
     } EConn_Flag;
     typedef unsigned int TConn_Flags;  ///< bitwise OR of EConn_Flag
 
-public:
     /// Create a stream based on a CONN, which is to be closed upon
     /// stream dtor only if "close" parameter is passed as "true".
     ///
@@ -137,8 +137,8 @@ public:
     /// @param buf_size
     ///  Default size of underlying stream buffer's I/O arena
     /// @param flags
-    ///  Specifies whether to tie input and output -- a tied stream flushes
-    ///  all pending output prior to doing any input.
+    ///  Specifies whether to tie input and output (a tied stream flushes all
+    ///  pending output prior to doing any input) and what to buffer.
     /// @sa
     ///  CONN, ncbi_connection.h
     ///
@@ -151,30 +151,32 @@ public:
      CT_CHAR_TYPE*   ptr      = 0,
      size_t          size     = 0);
 
+    typedef pair<CONNECTOR, EIO_Status> TConn_Pair;
 protected:
     /// Create a stream based on a CONNECTOR --
     /// only for internal use in derived classes.
     ///
-    /// @param connector
+    /// @param TConn_Pair::connector
     ///  A C object of type CONNECTOR (ncbi_connector.h) on top of which
-    ///  the stream is being constructed.  Used internally by individual
-    ///  ctors of specialized streams in this header.  May not be NULL.
+    ///  the stream is being constructed
+    /// @param TConn_Pair::status
+    ///  I/O status to use in underlying streambuf when CONNECTOR is NULL
     /// @param timeout
     ///  Default I/O timeout
     /// @param buf_size
     ///  Default size of underlying stream buffer's I/O arena
     /// @param flags
-    ///  Specifies whether to tie input and output -- a tied stream flushes
-    ///  all pending output prior to doing any input.
+    ///  Specifies whether to tie input and output (a tied stream flushes all
+    ///  pending output prior to doing any input) and what to buffer.
     /// @sa
     ///  CONNECTOR, ncbi_connector.h
     CConn_IOStream
-    (CONNECTOR       connector,
-     const STimeout* timeout  = kDefaultTimeout,
-     size_t          buf_size = kConn_DefaultBufSize,
-     TConn_Flags     flags    = fConn_ReadBuffered | fConn_WriteBuffered,
-     CT_CHAR_TYPE*   ptr      = 0,
-     size_t          size     = 0);
+    (const TConn_Pair& connpair,
+     const STimeout*   timeout  = kDefaultTimeout,
+     size_t            buf_size = kConn_DefaultBufSize,
+     TConn_Flags       flags    = fConn_ReadBuffered | fConn_WriteBuffered,
+     CT_CHAR_TYPE*     ptr      = 0,
+     size_t            size     = 0);
 
 public:
     virtual ~CConn_IOStream();
@@ -207,8 +209,8 @@ public:
     const STimeout* GetTimeout(EIO_Event direction) const;
 
     /// @return
-    ///   Status of the last I/O performed by the underlying CONN in
-    ///   the specified "direction" (either eIO_Open, IO_Read or eIO_Write);
+    ///   Status of the last I/O performed by the underlying CONN in the
+    ///   specified "direction" (either eIO_Open, IO_Read, or eIO_Write);
     ///   if "direction" is not specified (eIO_Close), return status
     ///   of the last CONN I/O performed by the stream.
     /// @sa
@@ -232,7 +234,7 @@ public:
 
     /// @return
     ///   Internal CONNection handle, which is still owned and used by
-    ///   the stream (or NULL if no such connection exists)
+    ///   the stream (or NULL if no such handle exists)
     /// @note
     ///   Connection can have additional flags set for I/O processing.
     /// @sa
@@ -248,9 +250,9 @@ private:
     // Cancellation
     SCONN_Callback        m_CB[3];
     CConstIRef<ICanceled> m_Canceled;
-    static EIO_Status x_IsCanceled(CONN conn, ECONN_Callback type, void* data);
+    static EIO_Status x_IsCanceled(CONN conn, TCONN_Callback type, void* data);
 
-    // Disable copy constructor and assignment.
+    // Disable copy constructor and assignment
     CConn_IOStream(const CConn_IOStream&);
     CConn_IOStream& operator= (const CConn_IOStream&);
 };
@@ -293,9 +295,8 @@ inline CConn_IOStreamSetReadTimeout SetReadTimeout(const STimeout* timeout)
 inline CConn_IOStream& operator>> (CConn_IOStream& is,
                                    const CConn_IOStreamSetReadTimeout& s)
 {
-    if (is.good() && is.SetTimeout(eIO_Read, s.GetTimeout()) != eIO_Success) {
+    if (is.good() && is.SetTimeout(eIO_Read, s.GetTimeout()) != eIO_Success)
         is.clear(IOS_BASE::badbit);
-    }
     return is;
 }
 
@@ -323,9 +324,8 @@ inline CConn_IOStreamSetWriteTimeout SetWriteTimeout(const STimeout* timeout)
 inline CConn_IOStream& operator<< (CConn_IOStream& os,
                                    const CConn_IOStreamSetWriteTimeout& s)
 {
-    if (os.good() && os.SetTimeout(eIO_Write, s.GetTimeout()) != eIO_Success) {
+    if (os.good() && os.SetTimeout(eIO_Write, s.GetTimeout()) != eIO_Success)
         os.clear(IOS_BASE::badbit);
-    }
     return os;
 }
 
@@ -403,7 +403,7 @@ public:
     ///
     /// scheme                          -- must be http or unspecified, checked
     /// host:port                       -- target server
-    /// http_proxy_host:http_proxy_port -- HTTP proxy server to tunnel via
+    /// http_proxy_host:http_proxy_port -- HTTP proxy server to tunnel thru
     /// http_proxy_user:http_proxy_pass -- credentials for the proxy, if needed
     /// http_proxy_leak                 -- ignore bad proxy and connect direct
     /// timeout                         -- timeout to connect to HTTP proxy
@@ -454,7 +454,7 @@ public:
      size_t          buf_size = kConn_DefaultBufSize);
 
     /// This variant uses an existing CSocket to build a stream up on it.
-    /// NOTE:  it revokes all ownership of the "socket"'s internals
+    /// NOTE:  this always revokes all ownership of the "socket"'s internals
     /// (effectively leaving the CSocket empty);  CIO_Exception(eInvalidArg)
     /// is thrown if the internal SOCK is not owned by the passed CSocket.
     /// More details:  <ncbi_socket_connector.h>::SOCK_CreateConnectorOnTop().
@@ -467,11 +467,6 @@ public:
     (CSocket&        socket,       ///< socket, underlying SOCK always grabbed
      const STimeout* timeout  = kDefaultTimeout,
      size_t          buf_size = kConn_DefaultBufSize);
-
-private:
-    // Disable copy constructor and assignment.
-    CConn_SocketStream(const CConn_SocketStream&);
-    CConn_SocketStream& operator= (const CConn_SocketStream&);
 };
 
 
@@ -479,30 +474,29 @@ private:
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// This stream exchanges data with an HTTP server located at the URL:
-/// http://host[:port]/path[?args]
+/// http[s]://host[:port]/path[?args]
 ///
 /// Note that "path" must include a leading slash, "args" can be empty,
 /// in which case the '?' does not get appended to the path.
 ///
 /// "User_header" (if not empty) should be a sequence of lines in the form
-/// 'HTTP-tag: Tag value', with each line separated by a CR LF sequence,
-/// and the last line terminated by a CR LF sequence.  For example:
-/// Content-Encoding: gzip\r\nContent-Length: 123\r\n
+/// 'HTTP-tag: Tag value', with each line separated by a CR LF sequence, and
+/// the last line optionally terminated with a CR LF sequence.  For example:
+/// "Content-Encoding: gzip\r\nContent-Type: application/octet-stream"
 /// It is included in the HTTP-header of each transaction.
 ///
-/// More elaborate specification of the server can be done via
-/// SConnNetInfo structure, which otherwise will be created with the
-/// use of a standard registry section to obtain default values from
-/// (details: <connect/ncbi_connutil.h>).  No user header is added if
-/// the argument is passed as default (or empty string).  To make
-/// sure the user header is passed empty, delete it from net_info
-/// by ConnNetInfo_DeleteUserHeader(net_info).
+/// More elaborate specification(s) of the server can be made via the
+/// SConnNetInfo structure, which otherwise will be created with the use of a
+/// standard registry section to obtain default values from
+/// (details: <connect/ncbi_connutil.h>).  To make sure the actual user header
+/// is empty, remember to delete it from "net_info" with
+/// ConnNetInfo_DeleteUserHeader(net_info).
 ///
 /// THTTP_Flags and other details: <connect/ncbi_http_connector.h>.
 ///
-/// Provided "timeout" is set at connection level, and if different from
-/// kDefaultTimeout, it overrides a value supplied by HTTP connector
-/// (the latter value is kept in SConnNetInfo::timeout).
+/// Provided "timeout" is set at the connection level, and if different from
+/// kDefaultTimeout, it overrides a value supplied by the HTTP connector
+/// (the latter value is kept at SConnNetInfo::timeout).
 ///
 
 class NCBI_XCONNECT_EXPORT CConn_HttpStream : public CConn_IOStream
@@ -577,11 +571,6 @@ private:
                                            void*         data,
                                            unsigned int  count);
     static void              x_Cleanup    (void*         data);
-
-private:
-    // Disable copy constructor and assignment.
-    CConn_HttpStream(const CConn_HttpStream&);
-    CConn_HttpStream& operator= (const CConn_HttpStream&);
 };
 
 
@@ -592,12 +581,12 @@ private:
 /// service is implemented as one of the specified server "types"
 /// (details: <connect/ncbi_server_info.h>).
 ///
-/// Additional specifications can be passed in the SConnNetInfo structure,
+/// Additional specifications can be passed in an SConnNetInfo structure,
 /// otherwise created by using the service name as a registry section
 /// to obtain the information from (details: <connect/ncbi_connutil.h>).
 ///
-/// Provided "timeout" is set at connection level, and if different from
-/// kDefaultTimeout, it overrides the value supplied by underlying connector
+/// Provided "timeout" is set at the connection level, and if different from
+/// kDefaultTimeout, it overrides the value supplied by an underlying connector
 /// (the latter value is kept in SConnNetInfo::timeout).
 ///
 
@@ -606,24 +595,19 @@ class NCBI_XCONNECT_EXPORT CConn_ServiceStream : public CConn_IOStream
 public:
     CConn_ServiceStream
     (const string&         service,
-     TSERV_Type            types       = fSERV_Any,
-     const SConnNetInfo*   net_info    = 0,
-     const SSERVICE_Extra* params      = 0,
-     const STimeout*       timeout     = kDefaultTimeout,
-     size_t                buf_size    = kConn_DefaultBufSize);
+     TSERV_Type            types    = fSERV_Any,
+     const SConnNetInfo*   net_info = 0,
+     const SSERVICE_Extra* params   = 0,
+     const STimeout*       timeout  = kDefaultTimeout,
+     size_t                buf_size = kConn_DefaultBufSize);
 
     CConn_ServiceStream
     (const string&         service,
      const string&         user_header,
-     TSERV_Type            types       = fSERV_Any,
-     const SSERVICE_Extra* params      = 0,
-     const STimeout*       timeout     = kDefaultTimeout,
-     size_t                buf_size    = kConn_DefaultBufSize);
-
-private:
-    // Disable copy constructor and assignment.
-    CConn_ServiceStream(const CConn_ServiceStream&);
-    CConn_ServiceStream& operator= (const CConn_ServiceStream&);
+     TSERV_Type            types    = fSERV_Any,
+     const SSERVICE_Extra* params   = 0,
+     const STimeout*       timeout  = kDefaultTimeout,
+     size_t                buf_size = kConn_DefaultBufSize);
 };
 
 
@@ -648,16 +632,16 @@ public:
     /// Build a stream on top of an existing data area of a specified size.
     /// The contents of the area is what will be read first from the stream.
     /// Writing to the stream will _not_ modify the contents of the area.
-    /// The written data will appear following the initial data block when
-    /// read from the stream.
+    /// When read from the stream, the written data will appear following the
+    /// initial data block.
     /// Ownership of the area pointed to by "ptr" is controlled by the "owner"
     /// parameter, and if the ownership is passed to the stream the area will
-    /// be deleted by "delete[] (char*)" at the stream dtor.  That is,
+    /// be deleted by "delete[] (char*)" from the stream destructor.  That is,
     /// if there are any requirements to be considered for deleting the area
-    /// (like deleting an object or an array of objects), then the
-    /// ownership must not be passed to the stream.
-    /// Note that the area pointed to by "ptr" should not be changed
-    /// while it is still holding the data yet to be read from the stream.
+    /// (like deleting an object or an array of objects), then the ownership
+    /// must not be passed to the stream.
+    /// Note that the area pointed to by "ptr" should not be changed while it
+    /// is still holding the data yet to be read from the stream.
     CConn_MemoryStream(const void* ptr,
                        size_t      size,
                        EOwnership  owner/**no default for safety*/,
@@ -666,20 +650,15 @@ public:
     virtual ~CConn_MemoryStream();
 
     /// The CConnMemoryStream::To* methods allow to obtain unread portion of
-    /// the stream in a single container (as a string or a vector) so that all
-    /// data is kept in sequential memory locations.
-    /// Note that the operation is considered an extraction, so it empties
-    /// the stream.
+    /// the stream into a single container (as a string or a vector) so that
+    /// all data is kept in sequential memory locations.
+    /// Note that the operation is considered an extraction, so it effectively
+    /// empties the stream.
     void    ToString(string*);      ///< fill in the data, NULL is not accepted
     void    ToVector(vector<char>*);///< fill in the data, NULL is not accepted
 
 protected:
     const void* m_Ptr;         ///< pointer to read memory area (if owned)
-
-private:
-    // Disable copy constructor and assignment.
-    CConn_MemoryStream(const CConn_MemoryStream&);
-    CConn_MemoryStream& operator= (const CConn_MemoryStream&);
 };
 
 
@@ -698,9 +677,9 @@ public:
     CConn_PipeStream
     (const string&         cmd,
      const vector<string>& args,
-     CPipe::TCreateFlags   create_flags = 0,
-     const STimeout*       timeout      = kDefaultTimeout,
-     size_t                buf_size     = kConn_DefaultBufSize
+     CPipe::TCreateFlags   flags    = 0,
+     const STimeout*       timeout  = kDefaultTimeout,
+     size_t                buf_size = kConn_DefaultBufSize
      );
     virtual ~CConn_PipeStream();
 
@@ -708,11 +687,6 @@ public:
 
 protected:
     CPipe* m_Pipe; ///< Underlying pipe.
-
-private:
-    // Disable copy constructor and assignment.
-    CConn_PipeStream(const CConn_PipeStream&);
-    CConn_PipeStream& operator= (const CConn_PipeStream&);
 };
 
 
@@ -734,19 +708,14 @@ public:
      const STimeout* timeout     = kDefaultTimeout,
      size_t          buf_size    = kConn_DefaultBufSize
      );
-
-private:
-    // Disable copy constructor and assignment.
-    CConn_NamedPipeStream(const CConn_NamedPipeStream&);
-    CConn_NamedPipeStream& operator= (const CConn_NamedPipeStream&);
 };
 
 
 
 /////////////////////////////////////////////////////////////////////////////
 ///
-/// CConn_FtpStream is an elaborate FTP client, can be used for both
-/// data downloading and/or uploading to and from an FTP server.
+/// CConn_FtpStream is an elaborate FTP client, can be used for data
+/// downloading and/or uploading to and from an FTP server.
 /// See <connect/ncbi_ftp_connector.h> for detailed explanations
 /// of supported features.
 ///
@@ -766,15 +735,18 @@ public:
      size_t               buf_size = kConn_DefaultBufSize
      );
 
+    CConn_FtpStream
+    (const SConnNetInfo&  net_info,
+     TFTP_Flags           flag     = 0,
+     const SFTP_Callback* cmcb     = 0,
+     const STimeout*      timeout  = kDefaultTimeout,
+     size_t               buf_size = kConn_DefaultBufSize
+     );
+
     /// Abort any command in progress, read and discard all input data,
     /// clear stream error state when successful (eIO_Success returns).
     /// @note The call empties both the stream and the underlying CONN.
     virtual EIO_Status Drain(const STimeout* timeout = kDefaultTimeout);
-
-private:
-    // Disable copy constructor and assignment.
-    CConn_FtpStream(const CConn_FtpStream&);
-    CConn_FtpStream& operator= (const CConn_FtpStream&);
 };
 
 
@@ -800,10 +772,17 @@ public:
      size_t               buf_size = kConn_DefaultBufSize
      );
 
-private:
-    // Disable copy constructor and assignment.
-    CConn_FTPDownloadStream(const CConn_FTPDownloadStream&);
-    CConn_FTPDownloadStream& operator= (const CConn_FTPDownloadStream&);
+    CConn_FTPDownloadStream
+    (const SConnNetInfo&  net_info,
+     TFTP_Flags           flag     = 0,
+     const SFTP_Callback* cmcb     = 0,
+     Uint8                offset   = 0, ///< file offset to begin download from
+     const STimeout*      timeout  = kDefaultTimeout,
+     size_t               buf_size = kConn_DefaultBufSize
+     );
+
+protected:
+    void x_InitDownload(const string& file, Uint8 offset);
 };
 
 
@@ -813,24 +792,30 @@ class NCBI_XCONNECT_EXPORT CConn_FTPUploadStream : public CConn_FtpStream
 {
 public:
     CConn_FTPUploadStream
-    (const string&   host,
-     const string&   user,
-     const string&   pass,
-     const string&   file    = kEmptyStr,
-     const string&   path    = kEmptyStr,
-     unsigned short  port    = 0, ///< 0 means default (21 for FTP)
-     TFTP_Flags      flag    = 0,
-     Uint8           offset  = 0, ///< file offset to start upload at
-     const STimeout* timeout = kDefaultTimeout
+    (const string&       host,
+     const string&       user,
+     const string&       pass,
+     const string&       file    = kEmptyStr,
+     const string&       path    = kEmptyStr,
+     unsigned short      port    = 0, ///< 0 means default (21 for FTP)
+     TFTP_Flags          flag    = 0,
+     Uint8               offset  = 0, ///< file offset to start upload at
+     const STimeout*     timeout = kDefaultTimeout
      );
 
-private:
-    // Disable copy constructor and assignment.
-    CConn_FTPUploadStream(const CConn_FTPUploadStream&);
-    CConn_FTPUploadStream& operator= (const CConn_FTPUploadStream&);
+    CConn_FTPUploadStream
+    (const SConnNetInfo& net_info,
+     TFTP_Flags          flag    = 0,
+     Uint8               offset  = 0, ///< file offset to start upload at
+     const STimeout*     timeout = kDefaultTimeout
+     );
+
+protected:
+    void x_InitUpload(const string& file, Uint8 offset);
 };
 
 
+
 #ifdef NCBI_CONN_STREAM_EXPERIMENTAL_API
 
 /////////////////////////////////////////////////////////////////////////////
@@ -846,6 +831,7 @@ CConn_IOStream* NcbiOpenURL(const string& url,
 #endif //NCBI_CONN_STREAM_EXPERIMENTAL_API
 
 
+
 END_NCBI_SCOPE
 
 
