@@ -1,7 +1,7 @@
 #ifndef CONNECT___HTTP_CONNECTOR__H
 #define CONNECT___HTTP_CONNECTOR__H
 
-/* $Id: ncbi_http_connector.h 405502 2013-07-03 18:01:44Z lavr $
+/* $Id: ncbi_http_connector.h 435430 2014-05-15 20:08:41Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -65,13 +65,13 @@ extern "C" {
  *       actually sent to the server side, so the following write will form a
  *       new request, and not get added to the previous one;  also this flag
  *       assures that the connector sends at least an HTTP header on "CLOSE"
- *       and re-"CONNECT", even if no data for HTTP body have been written
+ *       and re-"CONNECT", even if no data for HTTP body have been written.
  *
  * @var fHTTP_KeepHeader
  *       Do not strip HTTP header (i.e. everything up to the first "\r\n\r\n",
  *       including the "\r\n\r\n") from the CGI script's response (including
  *       any server error, which then is made available for reading as well)
- *       *NOTE* this flag disables automatic authorization, redirection, etc.
+ *       *NOTE* this flag disables automatic authorization and redirection.
  *
  * @var fHTTP_UrlDecodeInput
  *       Assume the response body as single-part, URL-encoded;  perform the
@@ -81,7 +81,7 @@ extern "C" {
  *       Do *not* do internal reading into temporary buffer while sending data
  *       to HTTP server;  by default any send operation tries to extract data
  *       as they are coming back from the server in order to prevent stalling
- *       due to data clogging in the connection.
+ *       due to data clogging the connection.
  *
  * @var fHTTP_DropUnread
  *       Do not collect incoming data in "Read" mode before switching into
@@ -94,9 +94,9 @@ extern "C" {
  *       (this flag effectively means having SConnNetInfo::max_try set to 1).
 
  * @var fHTTP_InsecureRedirect
- *       For security reasons the following redirects comprise security risk
- *       and, thus, are prohibited:  switching from https to http, and
- *       re-posting data (regardless of the transport, either http or https);
+ *       For security reasons the following redirects comprise security risk,
+ *       and thus, are prohibited:  switching from https to http, and/or
+ *       re-POSTing data (regardless of the transport, either http or https);
  *       this flag allows such redirects (when encountered) to be honored.
  *
  * @note
@@ -110,7 +110,7 @@ enum EHTTP_Flag {
     fHTTP_AutoReconnect   = 0x1,  /**< See HTTP_CreateConnectorEx()          */
     fHTTP_Flushable       = 0x2,  /**< Connector will really flush on Flush()*/
     fHTTP_KeepHeader      = 0x4,  /**< Keep HTTP header (see limitations)    */
-    fHTTP_UrlEncodeArgs   = 0x8,  /**< URL-encode "info->args" (w/o fragment)*/
+    /*fHTTP_UrlEncodeArgs = 0x8,       URL-encode "info->args" (w/o fragment)*/
     fHTTP_UrlDecodeInput  = 0x10, /**< URL-decode response body              */
     fHTTP_UrlEncodeOutput = 0x20, /**< URL-encode all output data            */
     fHTTP_UrlCodec        = 0x30, /**< fHTTP_UrlDecodeInput | ...EncodeOutput*/
@@ -119,7 +119,7 @@ enum EHTTP_Flag {
     fHTTP_NoAutoRetry     = 0x100,/**< No auto-retries allowed               */
     fHTTP_NoAutomagicSID  = 0x200,/**< Do not add NCBI SID automagically     */
     fHTTP_InsecureRedirect= 0x400,/**< Any redirect will be honored          */
-    /*fHTTP_ReservedFlag  = 0x800,     Do not use                            */
+    fHTTP_AdjustOnRedirect= 0x800 /**< Call adjust routine for redirects, too*/
 };
 typedef unsigned int THTTP_Flags; /**< Bitwise OR of EHTTP_Flag              */
 NCBI_HTTP_CONNECTOR_DEPRECATED
@@ -131,7 +131,7 @@ typedef enum {
     fHCC_UrlDecodeInput   = fHTTP_UrlDecodeInput,
     fHCC_UrlEncodeOutput  = fHTTP_UrlEncodeOutput,
     fHCC_UrlCodec         = fHTTP_UrlCodec,
-    fHCC_UrlEncodeArgs    = fHTTP_UrlEncodeArgs,
+    fHCC_UrlEncodeArgs    = 0x8,  /**< NB: Error-prone semantics, do not use!*/
     fHCC_DropUnread       = fHTTP_DropUnread,
     fHCC_NoUpread         = fHTTP_NoUpread,
     fHCC_Flushable        = fHTTP_Flushable,
@@ -181,7 +181,9 @@ extern NCBI_XCONNECT_EXPORT CONNECTOR HTTP_CreateConnector
  *   micro-session" making a hit when a previous hit has failed;  it is passed
  *   "net_info" as stored within the connector, and the number of previously
  *   unsuccessful consecutive attempts (in the least significant word) since
- *   the connection was opened;  a zero (false) return value ends the retries.
+ *   the connection was opened, pass 0 in that parameter if calling for
+ *   redirects (when fHTTP_AdjustOnRedirect was set);  a zero (false) return
+ *   value ends the retries;
  * - FHTTP_Cleanup() gets called when the connector is about to be destroyed;
  *   "user_data" is guaranteed not to be referenced anymore (so this is a good
  *   place to clean up "user_data" if necessary).
@@ -219,8 +221,8 @@ typedef void        (*FHTTP_Cleanup)
  * If "net_info" does not explicitly specify an HTTP request method (i.e. it
  * has it as "eReqMethod_Any"), then the actual method sent to the HTTP server
  * depends on whether any data have been written to the connection with
- * CONN_Write():  the presense of pending data will cause POST request (with a
- * "Content-Length:" tag provided automatically and reflecting the total
+ * CONN_Write():  the presense of pending data will cause a POST request (with
+ * a "Content-Length:" tag supplied automatically and reflecting the total
  * pending data size), and GET request method will result in the absence of any
  * data.  An explicit value for the request method will cause the specified
  * request to be used regardless of pending data, and will flag an error if any
@@ -242,14 +244,15 @@ typedef void        (*FHTTP_Cleanup)
  *     @note
  *       If <user_header> is neither a NULL pointer nor an empty string, then:
  *       - it must NOT contain any "empty lines":  "\r\n\r\n";
- *       - it should be terminated by a single "\r\n";
+ *       - multiple tags must be separated by "\r\n" (*not* just "\n");
+ *       - it should be terminated by a single "\r\n" (will be added, if not);
  *       - it gets inserted to the HTTP header "as is", without any automatic
- *         checking and / or encoding;
+ *         checking and / or encoding (except for the trailing "\r\n");
  *       - the "user_header" specified in the arguments overrides any user
  *         header that can be provided via the "net_info" argument, see
  *         ConnNetInfo_OverrideUserHeader() from <connect/ncbi_connutil.h>.
  *     @note
- *       Data may depart to the server side earlier if CONN_Flush()'ed in
+ *       Data may depart to the server side earlier if CONN_Flush()'ed in a
  *       fHTTP_Flushable connector, see "flags".
  *  3. After the request has been sent, then the response data from the peer
  *     (usually, a CGI program) can be actually read out.
