@@ -2,7 +2,7 @@
 #define CONNECT_SERVICES__GRID_WORKER_HPP
 
 
-/*  $Id: grid_worker.hpp 417632 2013-11-07 22:59:58Z kazimird $
+/*  $Id: grid_worker.hpp 448239 2014-10-03 14:17:45Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,7 +42,6 @@
 #include "netcache_api.hpp"
 #include "error_codes.hpp"
 #include "grid_app_version_info.hpp"
-#include "timeline.hpp"
 
 #include <connect/connect_export.h>
 
@@ -177,6 +176,8 @@ public:
 class CGridWorkerNode;
 class CWorkerNodeRequest;
 
+struct SWorkerNodeJobContextImpl;
+
 /// Worker Node job context
 ///
 /// Context in which a job is running, gives access to input and output
@@ -184,16 +185,16 @@ class CWorkerNodeRequest;
 ///
 /// @sa IWorkerNodeJob
 ///
-class NCBI_XCONNECT_EXPORT CWorkerNodeJobContext :
-        public CWorkerNodeTimelineEntry
+class NCBI_XCONNECT_EXPORT CWorkerNodeJobContext
 {
-public:
+    NCBI_NET_COMPONENT(WorkerNodeJobContext);
+
     /// Get the associated job structure to access all of its fields.
-    const CNetScheduleJob& GetJob() const {return m_Job;}
-    CNetScheduleJob& GetJob() {return m_Job;}
+    const CNetScheduleJob& GetJob() const;
+    CNetScheduleJob& GetJob();
 
     /// Get a job key
-    const string& GetJobKey() const        { return m_Job.job_id; }
+    const string& GetJobKey() const;
 
     /// Get a job input string.
     ///
@@ -204,7 +205,7 @@ public:
     ///    (NetCache)  In this case use GetIStream method to get a stream with
     ///    an input data for the job
     ///
-    const string& GetJobInput() const      { return m_Job.input; }
+    const string& GetJobInput() const;
 
     /// Set a job's output. This string will be sent to
     /// the queue when job is done.
@@ -213,11 +214,11 @@ public:
     /// To send a large result use GetOStream method. Don't call this
     /// method after GetOStream method is called.
     ///
-    void          SetJobOutput(const string& output) { m_Job.output = output; }
+    void SetJobOutput(const string& output);
 
     /// Set the return code of the job.
     ///
-    void          SetJobRetCode(int ret_code) { m_Job.ret_code = ret_code; }
+    void SetJobRetCode(int ret_code);
 
     /// Get a stream with input data for a job. Stream is based on network
     /// data storage (NetCache). Size of the input data can be determined
@@ -227,12 +228,12 @@ public:
 
     /// Get the size of an input stream
     ///
-    size_t        GetInputBlobSize() const { return m_InputBlobSize; }
+    size_t GetInputBlobSize() const;
 
     /// Put progress message
     ///
-    void          PutProgressMessage(const string& msg,
-                                     bool send_immediately = false);
+    virtual void PutProgressMessage(const string& msg,
+            bool send_immediately = false);
 
     /// Get a stream where a job can write its result
     ///
@@ -254,7 +255,7 @@ public:
     /// This method should be called at the end of the
     /// IWorkerNodeJob::Do() method.
     ///
-    void CommitJobWithFailure(const string& err_msg);
+    void CommitJobWithFailure(const string& err_msg, bool no_retries = false);
 
     /// Schedule the job for return.
     ///
@@ -270,6 +271,11 @@ public:
     ///
     void ReturnJob();
 
+    /// Put the job back in the queue with the specified affinity and group.
+    ///
+    /// This method should be called before exiting IWorkerNodeJob::Do().
+    void RescheduleJob(const string& affinity, const string& group = kEmptyStr);
+
     /// Check if job processing must be aborted.
     ///
     /// This method must be called periodically from within the Do()
@@ -295,7 +301,7 @@ public:
     ///    successfully finished.
     /// 3. The job has expired.
     ///
-    CNetScheduleAdmin::EShutdownLevel GetShutdownLevel();
+    virtual CNetScheduleAdmin::EShutdownLevel GetShutdownLevel();
 
     /// Get a name of a queue where this node is connected to.
     ///
@@ -316,7 +322,7 @@ public:
     /// @param runtime_inc
     ///    Estimated time in seconds(from the current moment) to
     ///    finish the job.
-    void JobDelayExpiration(unsigned runtime_inc);
+    virtual void JobDelayExpiration(unsigned runtime_inc);
 
     /// Check if logging was requested in config file
     ///
@@ -328,79 +334,30 @@ public:
     /// requested by another job this job will be returned back to the queue.
     void RequestExclusiveMode();
 
-    const string& GetJobOutput() const { return m_Job.output; }
+    const string& GetJobOutput() const;
 
-    CNetScheduleAPI::TJobMask GetJobMask() const { return m_Job.mask; }
+    CNetScheduleAPI::TJobMask GetJobMask() const;
 
-    size_t GetMaxServerOutputSize();
-
-    unsigned int GetJobNumber() const  { return m_JobNumber; }
+    unsigned int GetJobNumber() const;
 
     enum ECommitStatus {
         eDone,
         eFailure,
         eReturn,
+        eRescheduled,
         eNotCommitted,
         eCanceled
     };
 
-    bool IsJobCommitted() const    { return m_JobCommitted != eNotCommitted; }
-    ECommitStatus GetCommitStatus() const    { return m_JobCommitted; }
+    bool IsJobCommitted() const;
+    ECommitStatus GetCommitStatus() const;
     static const char* GetCommitStatusDescription(ECommitStatus commit_status);
 
-    bool IsCanceled() const { return m_JobCommitted == eCanceled; }
+    bool IsCanceled() const;
 
     IWorkerNodeCleanupEventSource* GetCleanupEventSource();
 
-    CGridWorkerNode& GetWorkerNode() const {return m_WorkerNode;}
-
-    CWorkerNodeJobContext(CGridWorkerNode& worker_node);
-
-private:
-    string& SetJobOutput()             { return m_Job.output; }
-    size_t& SetJobInputBlobSize()      { return m_InputBlobSize; }
-    bool IsJobExclusive() const        { return m_ExclusiveJob; }
-    const string& GetErrMsg() const    { return m_Job.error_msg; }
-
-    friend class CJobCommitterThread;
-    friend class CWorkerNodeRequest;
-    /// Only a CGridWorkerNode can create an instance of this class
-    friend class CGridWorkerNode;
-
-    void x_SetCanceled() { m_JobCommitted = eCanceled; }
-    void CheckIfCanceled();
-
-    void x_PrintRequestStop();
-    void x_RunJob();
-    void x_SendJobResults();
-
-    void Reset();
-
-    CGridWorkerNode&     m_WorkerNode;
-    CNetScheduleJob      m_Job;
-    ECommitStatus        m_JobCommitted;
-    size_t               m_InputBlobSize;
-    unsigned int         m_JobNumber;
-    bool                 m_ExclusiveJob;
-
-    CRef<IWorkerNodeCleanupEventSource> m_CleanupEventSource;
-
-    CRef<CRequestContext> m_RequestContext;
-    CRequestRateControl m_StatusThrottler;
-    CRequestRateControl m_ProgressMsgThrottler;
-    CNetScheduleExecutor m_NetScheduleExecutor;
-    CNetCacheAPI m_NetCacheAPI;
-    auto_ptr<CNcbiIstream> m_RStream;
-    auto_ptr<IEmbeddedStreamWriter> m_Writer;
-    auto_ptr<CNcbiOstream> m_WStream;
-
-    CDeadline m_CommitExpiration;
-    bool      m_FirstCommitAttempt;
-
-    /// The copy constructor and the assignment operator
-    /// are prohibited
-    CWorkerNodeJobContext(const CWorkerNodeJobContext&);
-    CWorkerNodeJobContext& operator=(const CWorkerNodeJobContext&);
+    CGridWorkerNode GetWorkerNode() const;
 };
 
 class CWorkerNodeIdleThread;
@@ -588,6 +545,7 @@ public:
         eJobFailed,
         eJobSucceeded,
         eJobReturned,
+        eJobRescheduled,
         eJobCanceled,
         eJobLost
     };
@@ -597,10 +555,9 @@ public:
     virtual void Notify(const CWorkerNodeJobContext& job, EEvent event) = 0;
 };
 
-class CWorkerNodeIdleThread;
-class CJobCommitterThread;
 class IGridWorkerNodeApp_Listener;
-class CWNJobWatcher;
+
+struct SGridWorkerNodeImpl;
 /// Grid Worker Node
 ///
 /// It gets jobs from a NetSchedule server and runs them simultaneously
@@ -612,13 +569,11 @@ class CWNJobWatcher;
 ///
 class NCBI_XCONNECT_EXPORT CGridWorkerNode
 {
-public:
+    NCBI_NET_COMPONENT(GridWorkerNode);
+
     /// Construct a worker node using class factories
     ///
-    CGridWorkerNode(CNcbiApplication& app,
-        IWorkerNodeJobFactory* job_factory);
-
-    virtual ~CGridWorkerNode();
+    CGridWorkerNode(CNcbiApplication& app, IWorkerNodeJobFactory* job_factory);
 
     void Init();
 
@@ -632,65 +587,45 @@ public:
 
     void RequestShutdown();
 
-    void ForceSingleThread() { m_SingleThreadForced = true; }
-
-    void AddJobWatcher(IWorkerNodeJobWatcher& job_watcher,
-                          EOwnership owner = eNoOwnership);
+    void ForceSingleThread();
 
     void SetListener(IGridWorkerNodeApp_Listener* listener);
 
-    IWorkerNodeJobFactory& GetJobFactory() { return *m_JobProcessorFactory; }
+    IWorkerNodeJobFactory& GetJobFactory();
 
     /// Get the maximum threads running simultaneously
     ///
-    unsigned int GetMaxThreads() const { return m_MaxThreads; }
+    unsigned int GetMaxThreads() const;
 
     /// Get total memory limit (automatic restart if node grows more than that)
     ///
-    Uint8 GetTotalMemoryLimit() const { return m_TotalMemoryLimit; }
+    Uint8 GetTotalMemoryLimit() const;
 
     /// Get total time limit (automatic restart after that)
     ///
-    int GetTotalTimeLimit() const { return m_TotalTimeLimit; }
-    time_t GetStartupTime() const { return m_StartupTime; }
-    unsigned GetQueueTimeout() const {return m_QueueTimeout;}
+    int GetTotalTimeLimit() const;
+    time_t GetStartupTime() const;
+    unsigned GetQueueTimeout() const;
 
     bool IsHostInAdminHostsList(const string& host) const;
 
-    unsigned GetCommitJobInterval() const {return m_CommitJobInterval;}
-    unsigned GetCheckStatusPeriod() const {return m_CheckStatusPeriod;}
+    unsigned GetCommitJobInterval() const;
+    unsigned GetCheckStatusPeriod() const;
     size_t GetServerOutputSize();
 
     const string& GetQueueName() const;
 
     const string& GetClientName() const;
 
-    string GetAppName() const
-    {
-        CFastMutexGuard guard(m_JobProcessorMutex);
-        return m_JobProcessorFactory->GetAppName();
-    }
-    string GetAppVersion() const
-    {
-        CFastMutexGuard guard(m_JobProcessorMutex);
-        return m_JobProcessorFactory->GetAppVersion();
-    }
-    string GetBuildDate() const
-    {
-        CFastMutexGuard guard(m_JobProcessorMutex);
-        return m_JobProcessorFactory->GetAppBuildDate();
-    }
+    string GetAppName() const;
+    string GetAppVersion() const;
+    string GetBuildDate() const;
 
     const string& GetServiceName() const;
 
-    CNetCacheAPI GetNetCacheAPI() const { return m_NetCacheAPI; }
-    CNetScheduleAPI GetNetScheduleAPI() const { return m_NetScheduleAPI; }
-    CNetScheduleExecutor GetNSExecutor() const { return m_NSExecutor; }
-
-    bool EnterExclusiveMode();
-    void LeaveExclusiveMode();
-    bool IsExclusiveMode() const;
-    bool WaitForExclusiveJobToFinish();
+    CNetCacheAPI GetNetCacheAPI() const;
+    CNetScheduleAPI GetNetScheduleAPI() const;
+    CNetScheduleExecutor GetNSExecutor() const;
 
     /// Disable the automatic logging of request-start and
     /// request-stop events by the framework itself.
@@ -704,132 +639,11 @@ public:
 
     IWorkerNodeCleanupEventSource* GetCleanupEventSource();
 
-private:
-    auto_ptr<IWorkerNodeJobFactory>      m_JobProcessorFactory;
-
-    CNetCacheAPI m_NetCacheAPI;
-    CNetScheduleAPI m_NetScheduleAPI;
-    CNetScheduleExecutor m_NSExecutor;
-
-    unsigned int                 m_MaxThreads;
-    unsigned int                 m_NSTimeout;
-    mutable CFastMutex           m_JobProcessorMutex;
-    unsigned                     m_CommitJobInterval;
-    unsigned                     m_CheckStatusPeriod;
-    CSemaphore                   m_ExclusiveJobSemaphore;
-    bool                         m_IsProcessingExclusiveJob;
-    Uint8                        m_TotalMemoryLimit;
-    unsigned                     m_TotalTimeLimit;
-    time_t                       m_StartupTime;
-    unsigned                     m_QueueTimeout;
-
-    typedef map<IWorkerNodeJobWatcher*,
-            AutoPtr<IWorkerNodeJobWatcher> > TJobWatchers;
-    CFastMutex m_JobWatcherMutex;
-    TJobWatchers m_Watchers;
-
-    CRef<IWorkerNodeCleanupEventSource> m_CleanupEventSource;
-
-    IWorkerNodeJob* GetJobProcessor();
-
-    friend class CWorkerNodeJobContext;
-
-    void x_NotifyJobWatchers(const CWorkerNodeJobContext& job,
-                            IWorkerNodeJobWatcher::EEvent event);
-
-    set<SServerAddress> m_Masters;
-    set<unsigned int> m_AdminHosts;
-
-private:
-    struct SNotificationTimelineEntry : public CWorkerNodeTimelineEntry
-    {
-        SServerAddress m_ServerAddress;
-
-        unsigned m_DiscoveryIteration;
-
-        SNotificationTimelineEntry(const SServerAddress& server_address,
-                unsigned discovery_iteration) :
-            m_ServerAddress(server_address),
-            m_DiscoveryIteration(discovery_iteration)
-        {
-        }
-
-        // Special constructor for m_TimelineSearchPattern (see below).
-        SNotificationTimelineEntry() :
-            m_ServerAddress(0, 0),
-            m_DiscoveryIteration(0)
-        {
-        }
-
-        bool IsDiscoveryAction() const {return m_DiscoveryIteration == 0;}
-
-        struct SLess {
-            bool operator ()(const SNotificationTimelineEntry* left,
-                    const SNotificationTimelineEntry* right) const
-            {
-                return left->m_ServerAddress < right->m_ServerAddress;
-            }
-        };
-    };
-
-    unsigned m_DiscoveryIteration;
-
-    CWorkerNodeTimeline<SNotificationTimelineEntry>
-            m_ImmediateActions, m_Timeline;
-
-    typedef set<SNotificationTimelineEntry*,
-            SNotificationTimelineEntry::SLess> TTimelineEntries;
-
-    TTimelineEntries m_TimelineEntryByAddress;
-
-    SNotificationTimelineEntry m_TimelineSearchPattern;
-
-    bool x_PerformTimelineAction(SNotificationTimelineEntry* action,
-            CNetScheduleJob& job);
-    void x_ProcessRequestJobNotification();
-    bool x_WaitForNewJob(CNetScheduleJob& job);
-    bool x_GetNextJob(CNetScheduleJob& job);
-
-    friend class CWNJobWatcher;
-    friend class CWorkerNodeRequest;
-    void x_ReturnJob(const CNetScheduleJob& job);
-    bool x_AreMastersBusy() const;
-
-    auto_ptr<IWorkerNodeInitContext> m_WorkerNodeInitContext;
-
-    CRef<CJobCommitterThread> m_JobCommitterThread;
-    CRef<CWorkerNodeIdleThread>  m_IdleThread;
-
-    auto_ptr<IGridWorkerNodeApp_Listener> m_Listener;
-
-    CNcbiApplication& m_App;
-    bool m_SingleThreadForced;
-    bool m_LogRequested;
-    bool m_ProgressLogRequested;
-    size_t m_QueueEmbeddedOutputSize;
+    void Suspend(bool pullback, unsigned timeout);
+    void Resume();
+    bool IsSuspended() const;
 };
 
-inline const string& CGridWorkerNode::GetQueueName() const
-{
-    return GetNetScheduleAPI().GetQueueName();
-}
-
-inline const string& CGridWorkerNode::GetClientName() const
-{
-    return GetNetScheduleAPI().GetService().GetServerPool().GetClientName();
-}
-
-inline const string& CGridWorkerNode::GetServiceName() const
-{
-    return GetNetScheduleAPI().GetService().GetServiceName();
-}
-
-inline bool CGridWorkerNode::IsExclusiveMode() const
-{
-    return m_IsProcessingExclusiveJob;
-}
-
-
 class NCBI_XCONNECT_EXPORT CGridWorkerNodeException : public CException
 {
 public:
