@@ -1,7 +1,7 @@
 #ifndef CONN___NETCACHE_API__HPP
 #define CONN___NETCACHE_API__HPP
 
-/*  $Id: netcache_api.hpp 420507 2013-12-03 19:01:36Z kazimird $
+/*  $Id: netcache_api.hpp 447467 2014-09-25 20:55:03Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,8 +37,8 @@
 /// NetCache client specs.
 ///
 
+#include "netschedule_api.hpp"
 #include "netcache_admin.hpp"
-#include "netservice_api.hpp"
 #include "netcache_api_expt.hpp"
 #include "netcache_key.hpp"
 #include "named_parameters.hpp"
@@ -162,7 +162,8 @@ class NCBI_XCONNECT_EXPORT CNetCacheAPI
     ///   parameters in.  If empty string is passed, then the section
     ///   name "netcache_api" will be used.
     explicit CNetCacheAPI(EAppRegistry use_app_reg,
-        const string& conf_section = kEmptyStr);
+            const string& conf_section = kEmptyStr,
+            CNetScheduleAPI::TInstance ns_api = NULL);
 
     /// Constructs a CNetCacheAPI object and initializes it with
     /// parameters read from the specified registry object.
@@ -173,7 +174,8 @@ class NCBI_XCONNECT_EXPORT CNetCacheAPI
     ///   parameters in.  If empty string is passed, then the section
     ///   name "netcache_api" will be used.
     explicit CNetCacheAPI(const IRegistry& reg,
-        const string& conf_section = kEmptyStr);
+            const string& conf_section = kEmptyStr,
+            CNetScheduleAPI::TInstance ns_api = NULL);
 
     /// Constructs a CNetCacheAPI object and initializes it with
     /// parameters read from the specified configuration object.
@@ -184,12 +186,15 @@ class NCBI_XCONNECT_EXPORT CNetCacheAPI
     ///   parameters.  If empty string is passed, then the section
     ///   name "netcache_api" will be used.
     explicit CNetCacheAPI(CConfig* conf,
-        const string& conf_section = kEmptyStr);
+            const string& conf_section = kEmptyStr,
+            CNetScheduleAPI::TInstance ns_api = NULL);
 
-    explicit CNetCacheAPI(const string& client_name);
+    explicit CNetCacheAPI(const string& client_name,
+            CNetScheduleAPI::TInstance ns_api = NULL);
 
     /// Construct client, working with the specified service
-    CNetCacheAPI(const string& service_name, const string& client_name);
+    CNetCacheAPI(const string& service_name, const string& client_name,
+            CNetScheduleAPI::TInstance ns_api = NULL);
 
     /// Put BLOB to server.  This method is blocking and waits
     /// for a confirmation from NetCache after all data is
@@ -418,10 +423,6 @@ class NCBI_XCONNECT_EXPORT CNetCacheAPI
     CNcbiIstream* GetIStream(const string& key, size_t* blob_size = NULL,
             const CNamedParameterList* optional = NULL);
 
-    /// Retrieve BLOB's owner information as registered by the server
-    string GetOwner(const string& blob_id,
-            const CNamedParameterList* optional = NULL);
-
     /// Remove BLOB by key
     void Remove(const string& blob_id,
             const CNamedParameterList* optional = NULL);
@@ -472,18 +473,6 @@ class NCBI_XCONNECT_EXPORT CNetCacheAPI
     CCompoundIDPool GetCompoundIDPool();
 };
 
-NCBI_DEPRECATED_CLASS NCBI_XCONNECT_EXPORT CNetCachePasswordGuard
-{
-public:
-    CNetCachePasswordGuard(CNetCacheAPI::TInstance nc_api,
-        const string& password);
-
-    CNetCacheAPI* operator ->() {return &m_NetCacheAPI;}
-
-private:
-    CNetCacheAPI m_NetCacheAPI;
-};
-
 NCBI_DECLARE_INTERFACE_VERSION(SNetCacheAPIImpl, "xnetcacheapi", 1, 1, 0);
 
 extern NCBI_XCONNECT_EXPORT const char* const kNetCacheAPIDriverName;
@@ -566,6 +555,11 @@ private:
     CBlobStorage_NetCache& operator=(CBlobStorage_NetCache&);
 };
 
+/// @internal
+NCBI_XCONNECT_EXPORT
+ERW_Result g_ReadFromNetCache(IReader* reader,
+        char* buf, size_t count, size_t* bytes_read);
+
 /* @} */
 
 END_NCBI_SCOPE
