@@ -1,7 +1,7 @@
 #ifndef CONN___NETSCHEDULE_API__HPP
 #define CONN___NETSCHEDULE_API__HPP
 
-/*  $Id: netschedule_api.hpp 418125 2013-11-14 19:16:45Z kazimird $
+/*  $Id: netschedule_api.hpp 448239 2014-10-03 14:17:45Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,10 +38,10 @@
 /// NetSchedule client specs.
 ///
 
-#include <connect/services/netschedule_key.hpp>
-#include <connect/services/netschedule_api_expt.hpp>
-#include <connect/services/netservice_api.hpp>
-#include <connect/services/compound_id.hpp>
+#include "netschedule_key.hpp"
+#include "netschedule_api_expt.hpp"
+#include "netservice_api.hpp"
+#include "compound_id.hpp"
 
 #include <corelib/plugin_manager.hpp>
 
@@ -55,6 +55,14 @@ const unsigned int kNetScheduleMaxDBDataSize = 2048;
 const unsigned int kNetScheduleMaxDBErrSize = 4096;
 
 
+/// Defines whether the job queue is paused, and if so, defines
+/// the pause mode set by the administrator.
+enum ENetScheduleQueuePauseMode {
+    eNSQ_NoPause,
+    eNSQ_WithPullback,
+    eNSQ_WithoutPullback
+};
+
 /** @addtogroup NetScheduleClient
  *
  * @{
@@ -62,6 +70,7 @@ const unsigned int kNetScheduleMaxDBErrSize = 4096;
 
 class CNetScheduleSubmitter;
 class CNetScheduleExecutor;
+class CNetScheduleJobReader;
 class CNetScheduleAdmin;
 
 struct CNetScheduleJob;
@@ -215,6 +224,9 @@ class NCBI_XCONNECT_EXPORT CNetScheduleAPI
     /// Create an instance of CNetScheduleExecutor.
     CNetScheduleExecutor GetExecutor();
 
+    /// Create an instance of CNetScheduleJobReader.
+    CNetScheduleJobReader GetJobReader();
+
     CNetScheduleAdmin GetAdmin();
 
     CNetService GetService();
@@ -226,8 +238,15 @@ class NCBI_XCONNECT_EXPORT CNetScheduleAPI
 
     const SServerParams& GetServerParams();
 
+    typedef map<string, string> TQueueParams;
+    void GetQueueParams(const string& queue_name, TQueueParams& queue_params);
+    void GetQueueParams(TQueueParams& queue_params);
+
     /// Get job details
-    EJobStatus GetJobDetails(CNetScheduleJob& job, time_t* job_exptime = NULL);
+    EJobStatus GetJobDetails(
+            CNetScheduleJob& job,
+            time_t* job_exptime = NULL,
+            ENetScheduleQueuePauseMode* pause_mode = NULL);
 
     /// Update the progress_message field of the job structure.
     /// Prior to calling this method, the caller must set the
@@ -241,6 +260,30 @@ class NCBI_XCONNECT_EXPORT CNetScheduleAPI
 
     void SetClientSession(const string& client_session);
 
+    enum ENetScheduleWarningType {
+        eWarnUnknown,
+        eWarnAffinityNotFound,
+        eWarnAffinityNotPreferred,
+        eWarnAffinityAlreadyPreferred,
+        eWarnGroupNotFound,
+        eWarnJobNotFound,
+        eWarnJobAlreadyCanceled,
+        eWarnJobAlreadyDone,
+        eWarnJobAlreadyFailed,
+        eWarnJobPassportOnlyMatch,
+        eWarnNoParametersChanged,
+        eWarnConfigFileNotChanged,
+        eWarnAlertNotFound,
+        eWarnAlertAlreadyAcknowledged,
+        eWarnSubmitsDisabledForServer,
+        eWarnQueueAlreadyPaused,
+        eWarnQueueNotPaused,
+        eWarnCommandObsolete,
+    };
+    static ENetScheduleWarningType ExtractWarningType(string& warn_msg);
+    static const char* WarningTypeToString(
+            ENetScheduleWarningType warning_type);
+
     /// @internal
     void UpdateAuthString();
 
@@ -271,7 +314,7 @@ class NCBI_XCONNECT_EXPORT CNetScheduleAPI
 };
 
 
-/////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
 ////
 /// Job description
 ///
@@ -291,6 +334,9 @@ struct CNetScheduleJob
     {
         input.erase();
         affinity.erase();
+        client_ip.erase();
+        session_id.erase();
+        page_hit_id.erase();
         mask = CNetScheduleAPI::eEmptyMask;
         job_id.erase();
         ret_code = 0;
@@ -311,6 +357,7 @@ struct CNetScheduleJob
 
     string client_ip;
     string session_id;
+    string page_hit_id;
 
     CNetScheduleAPI::TJobMask  mask;
 
@@ -470,12 +517,16 @@ class NCBI_XCONNECT_EXPORT CNetScheduleSubmitter
     /// @param job_exptime
     ///    Number of seconds since EPOCH when the job will expire
     ///    on the server.
+    /// @param pause_mode
+    ///    Queue pause mode set by the administrator.
     ///
     /// @return The current job status. eJobNotFound is returned if the job
     ///         cannot be found (job record has expired).
     ///
     CNetScheduleAPI::EJobStatus GetJobStatus(
-            const string& job_key, time_t* job_exptime = NULL);
+            const string& job_key,
+            time_t* job_exptime = NULL,
+            ENetScheduleQueuePauseMode* pause_mode = NULL);
 
     /// Get full information about the specified job.
     ///
@@ -491,10 +542,12 @@ class NCBI_XCONNECT_EXPORT CNetScheduleSubmitter
     /// @return The current job status.
     ///
     CNetScheduleAPI::EJobStatus GetJobDetails(
-            CNetScheduleJob& job, time_t* job_exptime = NULL);
+            CNetScheduleJob& job,
+            time_t* job_exptime = NULL,
+            ENetScheduleQueuePauseMode* pause_mode = NULL);
 };
 
-/////////////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
 ////
 struct SNetScheduleExecutorImpl;
 
@@ -624,7 +677,19 @@ class NCBI_XCONNECT_EXPORT CNetScheduleExecutor
     ///     NetSchedule job description structure. its error_msg
     ///     and optionally ret_code and output fields should be set
     ///
-    void PutFailure(const CNetScheduleJob& job);
+    /// @param no_retries
+    ///     Instruct NetSchedule to fail the job permanently.
+    ///     The job will not be rerun regardless of whether retries
+    ///     are enabled for the queue or not.
+    ///
+    void PutFailure(const CNetScheduleJob& job, bool no_retries = false);
+
+    /// Reschedule a job with new affinity and/or group information.
+    ///
+    /// This method requires that the following fields of the specified
+    /// CNetScheduleJob structure are filled: 'job_id', 'auth_token',
+    /// 'affinity' and/or 'group'.
+    void Reschedule(const CNetScheduleJob& job);
 
     /// Get the current status of the specified job. Unlike the similar
     /// method from CNetScheduleSubmitter, this method does not prolong
@@ -635,12 +700,16 @@ class NCBI_XCONNECT_EXPORT CNetScheduleExecutor
     /// @param job_exptime
     ///    Number of seconds since EPOCH when the job will expire
     ///    on the server.
+    /// @param pause_mode
+    ///    Queue pause mode set by the administrator.
     ///
     /// @return The current job status. eJobNotFound is returned if the job
     ///         cannot be found (job record has expired).
     ///
-    CNetScheduleAPI::EJobStatus GetJobStatus(const string& job_key,
-            time_t* job_exptime = NULL);
+    CNetScheduleAPI::EJobStatus GetJobStatus(
+            const string& job_key,
+            time_t* job_exptime = NULL,
+            ENetScheduleQueuePauseMode* pause_mode = NULL);
 
     /// Switch the job back to the "Pending" status. It will be
     /// run again on a different worker node.
@@ -688,7 +757,67 @@ class NCBI_XCONNECT_EXPORT CNetScheduleExecutor
 // Definition for backward compatibility.
 typedef CNetScheduleExecutor CNetScheduleExecuter;
 
-/////////////////////////////////////////////////////////////////////////////////////
+
+struct SNetScheduleJobReaderImpl;
+
+/// Smart pointer to a part of the NetSchedule API that allows to
+/// retrieve completed jobs.
+/// Objects of this class are returned by CNetScheduleAPI::GetJobReader().
+///
+/// @sa CNetScheduleAPI
+///
+class NCBI_XCONNECT_EXPORT CNetScheduleJobReader
+{
+    NCBI_NET_COMPONENT(NetScheduleJobReader);
+
+    /// Restrict job retrieval to the specified job group name.
+    ///
+    void SetJobGroup(const string& group_name);
+
+    /// Restrict job retrieval to the specified affinity.
+    ///
+    void SetAffinity(const string& affinity);
+
+    /// Possible outcomes of ReadNextJob() calls.
+    enum EReadNextJobResult {
+        eRNJ_JobReady,      ///< A job is returned.
+        eRNJ_Timeout,       ///< Operation has timed out.
+        eRNJ_NoMoreJobs,    ///< There are no more jobs matching
+                            ///< the requested criteria on the server.
+        eRNJ_Interrupt,     ///< ReadNextJob() has been interrupted.
+    };
+
+    /// Wait and return the next completed job.
+    ///
+    /// @param job     Structure to fill with information about the job.
+    ///                The following fields of the structure will be filled:
+    ///                - job_id
+    ///                - auth_token
+    ///                - affinity
+    ///                - client_ip
+    ///                - session_id
+    ///                - page_hit_id
+    ///
+    /// @param job_status Pointer to a variable where to store the final
+    ///                   status of the job.
+    ///
+    /// @param timeout Timeout to wait for job completion.
+    ///                If NULL, the method returns immediately.
+    ///
+    /// @return True if a completed job is returned. False if there
+    ///         are no more jobs matching the specified criteria.
+    ///
+    EReadNextJobResult ReadNextJob(CNetScheduleJob* job,
+        CNetScheduleAPI::EJobStatus* job_status,
+        CTimeout* timeout = NULL);
+
+    /// Abort waiting for a completed job in the above method.
+    /// Can be called from a signal handler or a parallel thread.
+    ///
+    void InterruptReading();
+};
+
+////////////////////////////////////////////////////////////////////////////////
 ////
 
 struct SNetScheduleAdminImpl;
@@ -711,24 +840,6 @@ class NCBI_XCONNECT_EXPORT CNetScheduleAdmin
             const string& affinity_token = kEmptyStr,
             const string& job_group = kEmptyStr);
 
-    /// Affinity information returned by AffinitySnapshot().
-    struct SAffinityInfo {
-        unsigned pending_job_count;
-        unsigned running_job_count;
-        unsigned dedicated_workers;
-        unsigned tentative_workers;
-    };
-    /// Affinity map, shows the number of jobs and worker nodes per affinity.
-    typedef map<string, SAffinityInfo> TAffinityMap;
-
-    /// For each affinity, returns the number of jobs associated with it
-    /// as well as the number of worker nodes that are handling or have
-    /// requested this affinity.
-    ///
-    /// @param affinity_map
-    ///    Map of affinity tokens to job and worker node counters.
-    void AffinitySnapshot(TAffinityMap& affinity_map);
-
     /// Create an instance of the given queue class.
     /// @param qname
     ///    Name of the queue to create
@@ -801,6 +912,8 @@ class NCBI_XCONNECT_EXPORT CNetScheduleAdmin
     void PrintServerStatistics(CNcbiOstream& output_stream,
         EStatisticsOptions opt = eStatisticsBrief);
 
+    void PrintHealth(CNcbiOstream& output_stream);
+
     void DumpQueue(CNcbiOstream& output_stream,
         const string& start_after_job = kEmptyStr,
         size_t job_count = 0,
