@@ -1,7 +1,7 @@
 #ifndef NCBIEXPT__HPP
 #define NCBIEXPT__HPP
 
-/*  $Id: ncbiexpt.hpp 354590 2012-02-28 16:30:13Z ucko $
+/*  $Id: ncbiexpt.hpp 448239 2014-10-03 14:17:45Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -540,8 +540,15 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 
 /// Generic macro to throw an exception, given the exception class,
 /// error code and message string.
-#define NCBI_THROW(exception_class, err_code, message) \
-    NCBI_EXCEPTION_THROW(NCBI_EXCEPTION(exception_class, err_code, message))
+/// The err_code argument may include manipulators to set additional
+/// options, e.g. eMyErrCode | Console | Severity(eDiag_Info). In any
+/// case the error code value must be the first in the combination.
+#define NCBI_THROW(exception_class, err_code, message)                  \
+    do {                                                                \
+        USING_SCOPE(NCBI_NS_NCBI::ncbi_ex_manip);                       \
+        NCBI_EXCEPTION_THROW(NCBI_EXCEPTION(exception_class, err_code,  \
+                                            message));                  \
+    } while (0)
 
 /// Throw a quick-and-dirty runtime exception of type 'CException' with
 /// the given error message and error code 'eUnknown'.
@@ -560,22 +567,30 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
     NCBI_THROW_FMT(NCBI_NS_NCBI::CException, eUnknown, message)
 
 /// Generic macro to make an exception, given the exception class,
-/// previous exception , error code and message string.
-#define NCBI_EXCEPTION_EX(prev_exception, exception_class, err_code, message)\
-    NCBI_EXCEPTION_VAR_EX(NCBI_EXCEPTION_EMPTY_NAME, &(prev_exception),      \
+/// previous exception, error code and message string.
+#define NCBI_EXCEPTION_EX(prev_exception, exception_class, err_code, message) \
+    NCBI_EXCEPTION_VAR_EX(NCBI_EXCEPTION_EMPTY_NAME, &(prev_exception), \
                           exception_class, err_code, message)
 
 /// Generic macro to re-throw an exception.
+/// The err_code argument may include manipulators to set additional
+/// options, e.g. eMyErrCode | Console | Severity(eDiag_Info). In any
+/// case the error code value must be the first in the combination.
 #define NCBI_RETHROW(prev_exception, exception_class, err_code, message) \
-    throw NCBI_EXCEPTION_EX(prev_exception, exception_class, err_code, message)
+    do {                                                                \
+        USING_SCOPE(NCBI_NS_NCBI::ncbi_ex_manip);                       \
+        throw NCBI_EXCEPTION_EX(prev_exception, exception_class,        \
+                                err_code, message);                     \
+    } while (0)
 
 /// The same as NCBI_RETHROW but with message processed as output to ostream.
 #define NCBI_RETHROW_FMT(prev_exception, exception_class, err_code, message) \
     NCBI_RETHROW(prev_exception, exception_class, err_code, FORMAT(message))
 
 /// Generic macro to re-throw the same exception.
-#define NCBI_RETHROW_SAME(prev_exception, message)              \
-    do { prev_exception.AddBacklog(DIAG_COMPILE_INFO, message, prev_exception.GetSeverity()); \
+#define NCBI_RETHROW_SAME(prev_exception, message)                \
+    do { prev_exception.AddBacklog(DIAG_COMPILE_INFO, message,    \
+                                   prev_exception.GetSeverity()); \
     throw; }  while (0)
 
 /// Generate a report on the exception.
@@ -592,9 +607,9 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 /// given subcode.
 #define NCBI_REPORT_EXCEPTION_XX(err_name, err_subcode, title, ex)   \
     NCBI_CHECK_ERR_SUBCODE_X_NAME(err_name, err_subcode);            \
-    NCBI_NS_NCBI::CExceptionReporter::ReportDefaultEx(               \
-                NCBI_ERRCODE_X_NAME(err_name), err_subcode,          \
-                DIAG_COMPILE_INFO, title, ex, NCBI_NS_NCBI::eDPF_Default)
+    NCBI_NS_NCBI::CExceptionReporter::ReportDefaultEx                \
+    (NCBI_ERRCODE_X_NAME(err_name), err_subcode,                     \
+     DIAG_COMPILE_INFO, title, ex, NCBI_NS_NCBI::eDPF_Default)
 
 
 
@@ -605,6 +620,82 @@ const T& DbgPrintNP(const CDiagCompileInfo& info,
 class CExceptionReporter;
 
 
+// Base helper class for passing options to exceptions.
+class CExceptionArgs_Base
+{
+public:
+    typedef int      TErrCodeVal;
+    typedef int      TFlags;
+    typedef EDiagSev TSeverity;
+
+    CExceptionArgs_Base(void) : m_ErrCode(0), m_Flags(0), m_Severity(eDiag_Error) {}
+
+    void SetErrCodeVal(TErrCodeVal err_code) { m_ErrCode = err_code; }
+    TErrCodeVal GetErrCodeVal(void) const { return m_ErrCode; }
+
+    void SetFlags(TFlags flags) { m_Flags = flags; }
+    TFlags GetFlags(void) const { return m_Flags; }
+
+    void SetSeverity(TSeverity severity) { m_Severity = severity; }
+    TSeverity GetSeverity(void) const { return m_Severity; }
+
+    void SetModule(const string& module) { m_Module = module; }
+    const string& GetModule(void) const { return m_Module; }
+    bool IsSetModule(void) const { return !m_Module.empty(); }
+
+private:
+    TErrCodeVal m_ErrCode;
+    TFlags      m_Flags;
+    TSeverity   m_Severity;
+    string      m_Module;
+};
+
+
+class CExceptionArgsManip
+{
+public:
+    virtual ~CExceptionArgsManip(void) {}
+
+    virtual void operator()(CExceptionArgs_Base& args) const = 0;
+};
+
+typedef void (*FExceptionArgsManip)(CExceptionArgs_Base&);
+
+/// Manipulator function wrapper.
+class CExceptionArgsManip_Wrapper : public CExceptionArgsManip
+{
+public:
+    CExceptionArgsManip_Wrapper(FExceptionArgsManip f) : m_Func(f) { }
+
+    void operator()(CExceptionArgs_Base& args) const { (*m_Func)(args); }
+
+private:
+    FExceptionArgsManip m_Func;
+};
+
+
+template<class TErrCode>
+class CExceptionArgs : public CExceptionArgs_Base
+{
+public:
+    CExceptionArgs(TErrCode err_code) { SetErrCodeVal(err_code); }
+
+    TErrCode GetErrCode(void) const { return TErrCode(GetErrCodeVal()); }
+
+    CExceptionArgs<TErrCode>& operator|(const CExceptionArgsManip& manip)
+    {
+        manip(*this);
+        return *this;
+    }
+
+    CExceptionArgs<TErrCode>& operator|(const CExceptionArgsManip_Wrapper& manip)
+    {
+        manip(*this);
+        return *this;
+    }
+};
+
+
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// CException --
@@ -630,6 +721,19 @@ public:
     };
     typedef int TErrCode;
 
+    /// Miscellaneous generic hints, flags and attributes
+    enum EFlags {
+        /// Mark the exception with this flag if the exception is supposed
+        /// to be extra-visible, such as to go to a console or a dialog
+        /// that's looked at by the end user. Naturally, the message should be
+        /// clear and informative enough to be actually helpful for the user.
+        /// If such "console" exception is passed to the diagnostic (e.g.
+        /// "ERR_POST(ex);", then the posting will be automatically marked as
+        /// "console" too (so it'll work as "ERR_POST(Console << ex);").
+        fConsole = (1 << 0)
+    };
+    typedef int TFlags;  //< Bit-wise OR of "EFlags"
+
     /// Constructor.
     ///
     /// When throwing an exception initially, "prev_exception" must be 0.
@@ -637,7 +741,13 @@ public:
                const CException* prev_exception,
                EErrCode err_code,
                const string& message,
-               EDiagSev severity = eDiag_Error );
+               EDiagSev severity = eDiag_Error,
+               TFlags flags = 0);
+
+    CException(const CDiagCompileInfo& info,
+               const CException* prev_exception,
+               const CExceptionArgs<EErrCode>& args,
+               const string& message);
 
     /// Copy constructor.
     CException(const CException& other);
@@ -722,7 +832,7 @@ public:
     EDiagSev GetSeverity(void) const { return m_Severity; }
 
     /// Set exception severity.
-    void SetSeverity(EDiagSev severity);
+    CException& SetSeverity(EDiagSev severity);
 
     /// Get class name as a string.
     virtual const char* GetType(void) const;
@@ -758,7 +868,7 @@ public:
     TErrCode GetErrCode(void) const;
 
     /// Get message string.
-    const string& GetMsg(void) const { return m_Msg; }
+    const string& GetMsg(void) const;
 
     /// Get "previous" exception from the backlog.
     const CException* GetPredecessor(void) const { return m_Predecessor; }
@@ -769,6 +879,15 @@ public:
     /// Destructor.
     virtual ~CException(void) throw();
 
+    /// Check if the flag is set
+    bool IsSetFlag(EFlags flag) const { return (m_Flags & flag) != 0; }
+
+    /// Set flag (add to other flags)
+    CException& SetFlag(EFlags flag) { m_Flags |= flag; return *this; }
+
+    /// Unset flag (other flags are left as is)
+    CException& UnsetFlag(EFlags flag) { m_Flags &= ~flag; return *this; }
+
 protected:
     /// Constructor with no arguments.
     ///
@@ -787,6 +906,10 @@ protected:
                         const CException*       prev_exception,
                         EDiagSev                severity);
 
+    /// Process additional arguments. Derived classes may need
+    /// to cast args to CExceptionArgs<CDerivedException::EErrCode>.
+    virtual void x_InitArgs(const CExceptionArgs_Base& args);
+
     /// Helper method for copying exception data.
     virtual void x_Assign(const CException& src);
 
@@ -826,11 +949,102 @@ private:
 
     auto_ptr<CStackTrace> m_StackTrace; ///< Saved stack trace
 
+    TFlags       m_Flags;            ///< Flags, hints, attributes
+
     /// Private assignment operator to prohibit assignment.
     CException& operator= (const CException&);
 };
 
 
+/// Exception manipulators. Can be combined with error code using operator|().
+
+BEGIN_SCOPE(ncbi_ex_manip)
+
+template<class TErrCode>
+CExceptionArgs<TErrCode> operator|(TErrCode                   err_code,
+                                   const CExceptionArgsManip& manip)
+{
+    CExceptionArgs<TErrCode> args(err_code);
+    manip(args);
+    return args;
+}
+
+
+template<class TErrCode>
+CExceptionArgs<TErrCode> operator|(TErrCode                           err_code,
+                                   const CExceptionArgsManip_Wrapper& manip)
+{
+    CExceptionArgs<TErrCode> args(err_code);
+    manip(args);
+    return args;
+}
+
+
+inline
+void Console(CExceptionArgs_Base& args)
+{
+    args.SetFlags(args.GetFlags() | CException::fConsole);
+}
+
+inline
+void Trace(CExceptionArgs_Base& args)
+{
+    args.SetSeverity(eDiag_Trace);
+}
+
+inline
+void Info(CExceptionArgs_Base& args)
+{
+    args.SetSeverity(eDiag_Info);
+}
+
+inline
+void Warning(CExceptionArgs_Base& args)
+{
+    args.SetSeverity(eDiag_Warning);
+}
+
+inline
+void Error(CExceptionArgs_Base& args)
+{
+    args.SetSeverity(eDiag_Error);
+}
+
+inline
+void Critical(CExceptionArgs_Base& args)
+{
+    args.SetSeverity(eDiag_Critical);
+}
+
+inline
+void Fatal(CExceptionArgs_Base& args)
+{
+    args.SetSeverity(eDiag_Fatal);
+}
+
+
+/// Module manipulator
+
+class Module : public CExceptionArgsManip
+{
+public:
+    Module(const string& module) : m_Module(module) {}
+    Module(const char* module) : m_Module(module) {}
+    virtual ~Module(void) {}
+
+    virtual void operator()(CExceptionArgs_Base& args) const
+    {
+        args.SetModule(m_Module);
+    }
+
+private:
+    string m_Module;
+};
+
+
+END_SCOPE(ncbi_ex_manip)
+
+
 /// Return valid pointer to uppermost derived class only if "from" is _really_
 /// the object of the desired type.
 ///
@@ -891,15 +1105,33 @@ private: \
 ///
 /// This can be used ONLY if the derived class does not have any additional
 /// (non-standard) data members.
-#define NCBI_EXCEPTION_DEFAULT(exception_class, base_class)         \
-public:                                                             \
-    exception_class(const CDiagCompileInfo& info,                   \
-        const CException* prev_exception,                           \
-                    EErrCode err_code,const string& message,        \
-                    EDiagSev severity = eDiag_Error)                \
-        : base_class(info, prev_exception,                          \
-            (base_class::EErrCode) CException::eInvalid, (message)) \
-    NCBI_EXCEPTION_DEFAULT_IMPLEMENTATION(exception_class, base_class)
+#define NCBI_EXCEPTION_DEFAULT(exception_class, base_class)             \
+public:                                                                 \
+    exception_class(const CDiagCompileInfo& info,                       \
+        const CException* prev_exception, EErrCode err_code,            \
+        const string& message, EDiagSev severity = eDiag_Error)         \
+        : base_class(info, prev_exception,                              \
+            (base_class::EErrCode) CException::eInvalid, (message))     \
+    {                                                                   \
+        x_Init(info, message, prev_exception, severity);                \
+        x_InitErrCode((CException::EErrCode) err_code);                 \
+    }                                                                   \
+    exception_class(const CDiagCompileInfo& info,                       \
+        const CException* prev_exception,                               \
+        const CExceptionArgs<EErrCode>& args,                           \
+        const string& message)                                          \
+        : base_class(info, prev_exception,                              \
+            (base_class::EErrCode) CException::eInvalid, (message))     \
+    {                                                                   \
+        x_Init(info, message, prev_exception, args.GetSeverity());      \
+        x_InitArgs(args);                                               \
+        x_InitErrCode((CException::EErrCode) args.GetErrCode());        \
+    }                                                                   \
+    NCBI_EXCEPTION_DEFAULT_IMPLEMENTATION_COMMON(exception_class,       \
+        base_class)                                                     \
+private:                                                                \
+    /* for the sake of semicolon at the end of macro...*/               \
+    static void xx_unused_##exception_class(void)
 
 
 /// Helper macro added to support templatized exceptions.
@@ -1054,6 +1286,27 @@ public:
     NCBI_EXCEPTION_DEFAULT(CCoreException, CException);
 };
 
+/////////////////////////////////////////////////////////////////////////////
+///
+/// CInvalidParamException --
+///
+/// Exception to report configuration or command line parameter errors
+/// (external error sources as opposed to CCoreException::eInvalidArg).
+
+class NCBI_XNCBI_EXPORT CInvalidParamException : public CException
+{
+public:
+    enum EErrCode {
+        eUndefined = 1,         ///< Parameter is missing/undefined
+        eInvalidCharacter       ///< Parameter value contains invalid character
+    };
+
+    /// Translate from an error code value to its string representation.
+    virtual const char* GetErrCodeString(void) const;
+
+    // Standard exception boilerplate code.
+    NCBI_EXCEPTION_DEFAULT(CInvalidParamException, CException);
+};
 
 
 // Some implementations return char*, so strict compilers may refuse
