@@ -1,7 +1,7 @@
 #ifndef CORELIB__NCBITIME__HPP
 #define CORELIB__NCBITIME__HPP
 
-/*  $Id: ncbitime.hpp 410651 2013-08-20 17:43:28Z rafanovi $
+/*  $Id: ncbitime.hpp 441072 2014-07-19 17:42:37Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,10 +35,10 @@
 
 /// @file ncbitime.hpp
 /// Defines:
-///   CTimeFormat    - storage class for fime format.
+///   CTimeFormat    - storage class for time format.
 ///   CTime          - standard Date/Time class to represent an absolute time.
 ///   CTimeSpan      - class to represents a relative time span.
-///   CStopWatch     - stop watch class to measure elasped time.
+///   CStopWatch     - stop watch class to measure elapsed time.
 ///   CTimeout       - timeout interval for various I/O etc activity.
 ///   CNanoTimeout   - timeout interval with nanoseconds precision.
 ///   CDeadline      - timeout that use absolute time mark (deadline time).
@@ -123,28 +123,36 @@ class NCBI_XNCBI_EXPORT CTimeFormat
 public:
     /// Flags.
     ///
+    /// @note
+    ///   It not specified otherwise, format have fDefault value "by default",
+    ///   that mean simple format string and strict matching.
     /// @sa SetFormat, AsString
     enum EFlags {
         /// Use single characters as format symbols.
         fFormat_Simple     = (1 << 0),
-        /// Specify that each format symbol have a preceding symbol '$'.
-        /// This can be useful if you want to include to format string
-        /// a characters that are a format symbols. 
+        /// Specify each format symbol with a preceding symbol '$'.
+        /// This can be useful if your format string includes output characters
+        /// that otherwise can be treated as format symbols.
         /// To include symbol '$' use '$$'.
         fFormat_Ncbi       = (1 << 1),
+
         /// A time string should strictly match the format string.
         fMatch_Strict      = (1 << 5),       ///< eg "Y" and "1997"
-        /// A format string can have extra leading format symbols,
+        /// A time/format string can have extra trailing format symbols,
         /// that do not have matching symbols in the time string.
         /// Corresponding time components will be initialized by default
         /// in the time object.
         fMatch_ShortTime   = (1 << 6),       ///< eg "Y/M/D h:m:s" and "1997"
         fMatch_ShortFormat = (1 << 7),       ///< eg "Y" and "1997/07/16"
-        fMatch_Weak        = fMatch_ShortFormat | fMatch_ShortTime,
+        /// Combination of both modes above.
+        /// Note that it matches until time or format string have symbols.
+        /// It not allow unprocessed symbols left in both time and format
+        /// strings at the same time.
+        fMatch_Weak        = fMatch_ShortTime | fMatch_ShortFormat,
         /// Default flags
         fDefault           = fFormat_Simple | fMatch_Strict,
 
-        /// "Enums", used for backward compatibility. Please use flags instead.
+        /// "Enum"s, used for backward compatibility. Please use flags instead.
         eNcbiSimple        = fFormat_Simple,
         eNcbi              = fFormat_Ncbi,
         eDefault           = fDefault
@@ -168,7 +176,7 @@ public:
     CTimeFormat(void);
 
     /// Copy constructor.
-    CTimeFormat(const CTimeFormat& format);
+    CTimeFormat(const CTimeFormat& fmt);
 
     /// Constructor.
     ///
@@ -181,22 +189,22 @@ public:
     CTimeFormat(const string& fmt, TFlags flags = fDefault);
 
     /// Assignment operator.
-    CTimeFormat& operator= (const CTimeFormat& format);
+    CTimeFormat& operator= (const CTimeFormat& fmt);
 
     /// Set the current time format.
     ///
     /// @param fmt
-    ///   String of letters describing the time format.
+    ///   String of symbols describing the time format.
     /// @param flags
     ///   Flags specifying how to match a time string against format string.
     /// @sa
-    ///   GetFormat, EFormat
+    ///   GetFormat, EFormat, EFlags
     void SetFormat(const char* fmt, TFlags flags = fDefault);
 
     /// Set the current time format.
     ///
     /// @param fmt
-    ///   String of letters describing the time format.
+    ///   String of symbols describing the time format.
     /// @param flags
     ///   Flags specifying how to match a time string against format string.
     /// @sa
@@ -206,7 +214,7 @@ public:
     /// Get format string.
     ///
     /// @return
-    ///   A string of letters describing the time format.
+    ///   A string of symbols describing the time format.
     /// @sa SetFormat, GetFlags
     const string& GetString(void) const;
 
@@ -222,7 +230,7 @@ public:
 
     /// Get predefined format.
     /// @param fmt
-    ///   String of letters describing the time format.
+    ///   String of symbols describing the time format.
     /// @param fmt_type
     ///   Specify type of the format string.
     /// @return
@@ -268,7 +276,7 @@ public:
 
     /// Which initial value to use for timezone.
     enum ETimeZone {
-        eLocal,       ///< Use local time
+        eLocal = 1,   ///< Use local time
         eGmt          ///< Use GMT (Greenwich Mean Time)
     };
 
@@ -426,9 +434,11 @@ public:
     ///   implies the format, that was previously setup using SetFormat()
     ///   method, or default "M/D/Y h:m:s".
     /// @param tz
-    ///   Whether to use local time (default) or GMT.
-    /// @param tzp
-    ///   What time zone precision to use.
+    ///   If current format contains 'Z', then objects timezone will be set to:
+    ///     - eGMT if "str" has word "GMT" in the appropriate position;
+    ///     - eLocal otherwise.
+    ///   If current format does not contain 'Z', objects timezone
+    ///   will be set to 'tz' value.
     /// @sa AsString, operator=
     explicit CTime(const string& str, const CTimeFormat& fmt = kEmptyStr,
                    ETimeZone tz = eLocal,
@@ -440,16 +450,18 @@ public:
     /// Assignment operator.
     CTime& operator= (const CTime& t);
 
-    /// Assignment operator.
+    /// Assignment operator from string.
     ///
     /// If current format contains 'Z', then objects timezone will be set to:
     ///   - eGMT if "str" has word "GMT" in the appropriate position;
     ///   - eLocal otherwise.
     /// If current format does not contain 'Z', objects timezone
     /// will not be changed.
-    /// NOTE: This operator expect a string in the format, 
-    ///       that was previously setup using SetFormat() method.
-    /// @sa CTime constructor from string, AsString
+    /// @note
+    ///   This operator expect a string in the format, that was previously
+    ///   set using SetFormat() method.
+    /// @sa
+    ///   CTime constructor from string, AsString
     CTime& operator= (const string& str);
 
     /// Set time using time_t time value.
@@ -542,15 +554,15 @@ public:
     /// Set the current time format.
     ///
     /// The default format is: "M/D/Y h:m:s".
-    /// @param format
-    ///   An object contains string of letters describing the time
-    ///   format and its type. The format letters have
-    ///   the following meanings:
+    /// @param fmt
+    ///   An object contains string of symbols describing the time
+    ///   format and its type. The following format letters have
+    ///   the special meaning:
     ///   - Y = year with century
     ///   - y = year without century           (00-99)
     ///   - M = month as decimal number        (01-12)
     ///   - B = full month name                (January-December)
-    ///   - b = abbeviated month name          (Jan-Dec)
+    ///   - b = abbreviated month name         (Jan-Dec)
     ///   - D = day as decimal number          (01-31)
     ///   - d = day as decimal number (w/o 0)  (1-31)
     ///   - H = hour in 12-hour format         (00-12)
@@ -560,6 +572,9 @@ public:
     ///   - l = milliseconds as decimal number (000-999)
     ///   - r = microseconds as decimal number (000000-999999)
     ///   - S = nanosecond as decimal number   (000000000-999999999)
+    ///   - G = seconds and fraction part of second as double value
+    ///         ("s.nnn") with floating number (1-9) of digits after dot.
+    ///   - g = same as "G" but w/o leading "0" if number of seconds < 10.
     ///   - P = am/pm                          (AM/PM)
     ///   - p = am/pm                          (am/pm)
     ///   - Z = timezone format                (GMT or none)
@@ -569,16 +584,16 @@ public:
     ///   - w = abbreviated day of week name   (Sun-Sat)
     ///
     ///   Format string can represent date/time partially, in this case
-    ///   current time, or defaut values, will be used to amplify time
+    ///   current time, or default values, will be used to amplify time
     ///   object, if possible. Current date/time cannot be used
     ///   if format string contains "z" (time shift) format symbol.
     ///   Also, it cannot be used if time format is ambiguous, like "Y/D".
     ///   Note, that you still can use "Y/M", or even "Y", where month and
-    ///   day will be defined to 1; or "M/D", where year will be set as
+    ///   day will be defined as 1; or "M/D", where year will be set as the
     ///   current year.
     /// @sa
     ///   CTimeFormat, GetFormat, AsString
-    static void SetFormat(const CTimeFormat& format);
+    static void SetFormat(const CTimeFormat& fmt);
 
     /// Get the current time format.
     ///
@@ -642,10 +657,10 @@ public:
     ///   Output timezone. This is a difference in seconds between GMT time
     ///   and local time for some place (for example, for EST5 timezone
     ///   its value is 18000). This parameter works only with local time.
-    ///   If the time object have GMT time that it is ignored.
+    ///   If the time object contains GMT time then it is ignored.
     ///   Before transformation to string the time will be converted to output
-    ///   timezone. Timezone can be printed as string 'GMT[+|-]HHMM' using
-    ///   format symbol 'z'. By default current timezone is used.
+    ///   timezone. Timezone can be printed as a string 'GMT[+|-]HHMM' using
+    ///   the format symbol 'z'. By default the current timezone is used.
     /// @sa
     ///   GetFormat, SetFormat
     string AsString(const CTimeFormat& format = kEmptyStr,
@@ -842,7 +857,7 @@ public:
 
     /// Get number of days in the month.
     ///
-    /// Number of days = 1..31
+    /// Number of days = 28..31
     int DaysInMonth(void) const;
 
     /// Add specified years and adjust for daylight saving time.
@@ -939,7 +954,7 @@ public:
     ///
     /// Round stored time to specified precision. All time components with
     /// precision less that specified will be zero-filled, all other
-    /// components will be adjusted accondingly to rules for rounding
+    /// components will be adjusted accordingly to rules for rounding
     /// numbers.
     /// @param precision
     ///   Rounding precision. 
@@ -1104,8 +1119,8 @@ public:
     string TimeZoneName(void);
 
 private:
-    /// Helper method to set time value from string "str" using "format".
-    void x_Init(const string& str, const CTimeFormat& format);
+    /// Helper method to set time value from string "str" using format "fmt".
+    void x_Init(const string& str, const CTimeFormat& fmt);
 
     /// Helper method to set time from 'time_t' -- If "t" not specified,
     /// then set to current time.
@@ -1163,7 +1178,7 @@ private:
         // as stored during the last call to x_AdjustTime***().
         Int4          adjTimeDiff NCBI_TIME_BITFIELD(18);
         // Timezone and precision
-        ETimeZone     tz          NCBI_TIME_BITFIELD(2);  // local/GMT
+        ETimeZone     tz          NCBI_TIME_BITFIELD(3);  // local/GMT
         ETimeZonePrecision tzprec NCBI_TIME_BITFIELD(4);  // Time zone precision
         NCBI_TIME_EMPTY_BITFIELD  // Force alignment
         Int4          nanosec;
@@ -1271,6 +1286,10 @@ public:
     ///   - s = seconds, "S" modulo 60 (-59 - 59)
     ///   - N = total whole number of nanoseconds stored in the time span
     ///   - n = nanoseconds (-999999999 - 999999999)
+    ///   - G = total whole number of seconds and part of second as double value
+    ///         ("S.nnn") with floating number (1-9) of digits after dot.
+    ///   - g = seconds, "S" modulo 60 and part of second as double value
+    ///         ("s.nnn") with floating number (1-9) of digits after dot.
     /// @sa
     ///   CTimeFormat, GetFormat, AsString
     static void SetFormat(const CTimeFormat& format);
@@ -1286,14 +1305,14 @@ public:
 
     /// Transform time span to string.
     ///
-    /// @param format
+    /// @param fmt
     ///   Format specifier used to convert time span to string.
-    ///   If "format" is not defined, then GetFormat() will be used.
+    ///   If format is not defined, then GetFormat() will be used.
     /// @return
     ///   A string representation of time span in specified format.
     /// @sa
     ///   CTimeFormat, GetFormat, SetFormat
-    string AsString(const CTimeFormat& format = kEmptyStr) const;
+    string AsString(const CTimeFormat& fmt = kEmptyStr) const;
 
     /// Return span time as string using the format returned by GetFormat().
     operator string(void) const;
@@ -1347,14 +1366,14 @@ public:
     ///   Rounding mode. By default time span will be truncated at last value
     ///   specified by precision. If mode is eRound, that time span will be
     ///   arithmetically rounded by precision level. 
-    ///   This parameters is ignored for eSSP_Smart precision level and
-    ///   always works as eTrunc.
     /// @param zero_mode
     ///   Mode to print or skip zero parts of time span which should be
     ///   printed but have 0 value. Apply to middle and trailing zero parts
     ///   of time span only. Leading zeros will be ignored in common case
     ///   in any mode, they can be printed only if it is impossible
     ///   to represent time span otherwise.
+    ///   This parameters is ignored for eSSP_Smart precision level and
+    ///   always works as eSSZ_SkipZero.
     /// @return
     ///   A string representation of time span.
     /// @sa
@@ -1461,12 +1480,18 @@ private:
     /// Seconds after the minute = -59..59
     int x_Second(void) const;
 
-    /// Helper method to set time value from string "str" using "format".
-    void x_Init(const string& str, const CTimeFormat& format);
+    /// Helper method to set time value from string "str" using format "fmt".
+    void x_Init(const string& str, const CTimeFormat& fmt);
 
     /// Helper method to normalize stored time value.
     void x_Normalize(void);
 
+    /// Helpers for AsSmartString()
+    string x_AsSmartString_Smart_Big  (ERound rounding);
+    string x_AsSmartString_Smart_Small(ERound rounding);
+    string x_AsSmartString_Precision  (ESmartStringPrecision precision,
+                                       ERound rounding, ESmartStringZeroMode zero_mode);
+
 private:
     long  m_Sec;      ///< Seconds part of the time span
     long  m_NanoSec;  ///< Nanoseconds after the second
@@ -1621,33 +1646,33 @@ public:
 class NCBI_XNCBI_EXPORT CDeadline
 {
 public:
-    /// Initialize absolute timeout using seconds and nanoseconds
+    /// Initialize deadline using seconds and nanoseconds
     /// (adding to the current time)
     /// @param seconds
     ///   Number of seconds to add to the current time
     /// @param nanoseconds
     ///   Number of nanoseconds to add to the current time
-    CDeadline(unsigned int seconds, unsigned int nanoseconds = 0);
+    CDeadline(unsigned int rel_seconds, unsigned int rel_nanoseconds = 0);
 
-    /// Initialize absolute timeout by adding relative one to the current time
+    /// Initialize deadline by adding relative timeout to the current time.
     CDeadline(const CTimeout& timeout);
 
-    /// Check if the timeout is infinite
+    /// Check if the deadline is infinite.
     bool IsInfinite(void) const { return m_Infinite; }
 
-    /// Check if the timeout is expired
+    /// Check if the deadline is expired.
     bool IsExpired(void) const
     { return !IsInfinite()  &&  GetRemainingTime().IsZero(); }
 
     /// Get the number of seconds and nanoseconds (since 1/1/1970).
-    /// Throw an exception if the timeout is infinite.
+    /// Throw an exception if the deadline is infinite.
     void GetExpirationTime(time_t* sec, unsigned int* nanosec) const;
 
-    /// Get time left to the expiration time
+    /// Get time left to the expiration.
     CNanoTimeout GetRemainingTime(void) const;
 
     /// Compare two CDeadline values.
-    bool operator <(const CDeadline& right_hand_operand) const;
+    bool operator < (const CDeadline& right_hand_operand) const;
 
 private:
     void x_Now(void);
@@ -1714,7 +1739,7 @@ private:
 ///
 /// CStopWatch --
 ///
-/// Define a stop watch class to measure elasped time.
+/// Define a stop watch class to measure elapsed time.
 
 class NCBI_XNCBI_EXPORT CStopWatch
 {
@@ -1731,7 +1756,7 @@ public:
 
     /// Constructor.
     /// Start timer if argument is true.
-    /// @deprecated Use CStopWatch(EStat) constuctor instead.
+    /// @deprecated Use CStopWatch(EStat) constructor instead.
     NCBI_DEPRECATED_CTOR(CStopWatch(bool start));
 
     /// Start the timer.
@@ -1764,18 +1789,18 @@ public:
 
     /// Set the current stopwatch time format.
     ///
-    /// The default format is: "-S.n".
-    /// @param format
+    /// The default format is: "S.n".
+    /// @param fmt
     ///   Format specifier used to convert time span to string.
-    ///   If "format" is not defined, then GetFormat() will be used.
+    ///   If format is not defined, then GetFormat() will be used.
     ///   Uses the same time format as CTimeSpan class.
     /// @sa
     ///   CTimeFormat, CTimeSpan::SetFormat, AsString
-    static void SetFormat(const CTimeFormat& format);
+    static void SetFormat(const CTimeFormat& fmt);
 
     /// Get the current stopwatch time format.
     ///
-    /// The default format is: "-S.n".
+    /// The default format is: "S.n".
     /// @return
     ///   An object describing the time format.
     ///   The letters having the same means that for CTimeSpan.
@@ -1788,13 +1813,13 @@ public:
     /// According to used OS, the double representation can provide much
     /// finer grained time control. The string representation is limited
     /// by nanoseconds.
-    /// @param format
-    ///   If "format" is not defined, then GetFormat() will be used.
+    /// @param fmt
+    ///   If format is not defined, then GetFormat() will be used.
     ///   Format specifier used to convert value returned by Elapsed()
     ///   to string.
     /// @sa
     ///   CTimeSpan::AsString, CTimeFormat, Elapsed, GetFormat, SetFormat
-    string AsString(const CTimeFormat& format = kEmptyStr) const;
+    string AsString(const CTimeFormat& fmt = kEmptyStr) const;
 
     /// Return stopwatch time as string using the format returned
     /// by GetFormat().
@@ -1996,18 +2021,10 @@ ostream& operator<< (ostream& os, const CTime& t)
 //  CTimeFormat
 //
 
-inline
-void CTimeFormat::SetFormat(const string& fmt, TFlags flags)
-{
-    m_Str   = fmt;
-    m_Flags = flags;
-}
-
 inline
 void CTimeFormat::SetFormat(const char* fmt, TFlags flags)
 {
-    m_Str   = fmt;
-    m_Flags = flags;
+    SetFormat(string(fmt), flags);
 }
 
 inline
