@@ -1,7 +1,7 @@
 #ifndef SEQ_LOC_MAPPER_BASE__HPP
 #define SEQ_LOC_MAPPER_BASE__HPP
 
-/*  $Id: seq_loc_mapper_base.hpp 371868 2012-08-13 15:10:25Z rafanovi $
+/*  $Id: seq_loc_mapper_base.hpp 444821 2014-08-28 12:07:07Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -281,7 +281,6 @@ private:
 ///  Mapping locations and alignments between bioseqs through seq-locs,
 ///  features, alignments or between parts of segmented bioseqs.
 
-
 class NCBI_SEQ_EXPORT CSeq_loc_Mapper_Base : public CObject
 {
 public:
@@ -296,10 +295,19 @@ public:
         /// Ignore internal dense-seg structure - map each
         /// dense-seg according to the total ranges involved
         fAlign_Dense_seg_TotalRange = 0x01,
+
         /// Flags used to indicate mapping direction when mapping
         /// through a sparse-seg.
         fAlign_Sparse_ToFirst       = 0x00, ///< Map to first-id
-        fAlign_Sparse_ToSecond      = 0x02  ///< Map to second-id
+        fAlign_Sparse_ToSecond      = 0x02, ///< Map to second-id
+        
+        /// Flag used when mapping through a seq-map (this includes
+        /// mapping through a bioseq or a GC-assembly). If set, each
+        /// call to Map() goes only one level up or down, unlike normal
+        /// mode which maps from any level as far up/down as possible.
+        /// The result of mapping can be mapped further by making another
+        /// call to Map().
+        fMapSingleLevel             = 0x04
     };
     typedef int TMapOptions;
 
@@ -435,22 +443,49 @@ public:
     /// The data may be truncated to match the new location.
     CRef<CSeq_graph> Map(const CSeq_graph& src_graph);
 
+    /// Flags defining seq-annot mapping options.
+    enum FAnnotMapFlag {
+        fAnnotMap_Location = 1 << 0, ///< Map seq-feat locations
+        fAnnotMap_Product  = 1 << 1, ///< Map seq-feat products
+        fAnnotMap_Both     = fAnnotMap_Location | fAnnotMap_Product,
+
+        /// Remove annotations which can not be mapped with this mapper.
+        /// If the flag is not set, the original annotation is stored
+        /// in the seq-annot.
+        fAnnotMap_RemoveNonMapping = 1 << 2,
+
+        /// Throw exception if an annotation can not be mapped.
+        fAnnotMap_ThrowOnFailure = 1 << 3,
+
+        fAnnotMap_Default = fAnnotMap_Both
+    };
+    typedef int TAnnotMapFlags;
+
+    /// Result of seq-annot mapping
+    enum EMapResult {
+        /// No annotation was mapped, the input seq-annot is unchanged.
+        eMapped_None = 0,
+        /// Some (not all) annotations were mapped.
+        eMapped_Some,
+        /// All annotations were mapped, none was removed.
+        eMapped_All
+    };
+
     /// Map each object from the Seq-annot and replace the original
     /// with the mapped one.
-    void Map(CSeq_annot& annot);
+    EMapResult Map(CSeq_annot& annot, TAnnotMapFlags flags = fAnnotMap_Default);
 
     /// Check if the last mapping resulted in partial location
     /// (not all ranges from the original location could be mapped
     /// to the target).
-    bool             LastIsPartial(void);
+    bool LastIsPartial(void);
 
-    typedef set<CSeq_id_Handle>        TSynonyms;
+    typedef set<CSeq_id_Handle> TSynonyms;
 
-    // Collect synonyms for the given seq-id and put them in the container.
-    // The default implementation just adds the id to the list of synonyms.
-    // Any overriden method should do at least the same.
-    void CollectSynonyms(const CSeq_id_Handle& id,
-                         TSynonyms&            synonyms) const;
+    // Collect synonyms for the id, store mapping of each synonym
+    // to the primary id. Returns primary id for the argument or the
+    // argument itself.
+    const CSeq_id_Handle& CollectSynonyms(const CSeq_id_Handle& id) const;
 
     // Sequence type - to recalculate coordinates.
     enum ESeqType {
@@ -493,12 +528,10 @@ protected:
     // row containing the id and sets it as mapping target. All other
     // rows become mapping source.
     void x_InitializeAlign(const CSeq_align& map_align,
-                           const CSeq_id&    to_id,
-                           TMapOptions       opts);
+                           const CSeq_id&    to_id);
     // Initialize the mapper from an alignment, map to the specified row.
     void x_InitializeAlign(const CSeq_align& map_align,
-                           size_t            to_row,
-                           TMapOptions       opts);
+                           size_t            to_row);
 
     // Create dummy mapping from the whole destination location to itself.
     // This will prevent truncation of ranges already on the target.
@@ -640,15 +673,13 @@ private:
 
     // Initialize the mapper from different alignment types.
     void x_InitAlign(const CDense_diag& diag, size_t to_row);
-    void x_InitAlign(const CDense_seg& denseg, size_t to_row,
-                     TMapOptions opts);
+    void x_InitAlign(const CDense_seg& denseg, size_t to_row);
     void x_InitAlign(const CStd_seg& sseg, size_t to_row);
     void x_InitAlign(const CPacked_seg& pseg, size_t to_row);
     void x_InitSpliced(const CSpliced_seg& spliced,
                        const CSeq_id&      to_id);
     void x_InitSpliced(const CSpliced_seg& spliced, ESplicedRow to_row);
-    void x_InitSparse(const CSparse_seg& sparse, int to_row,
-                      TMapOptions opts);
+    void x_InitSparse(const CSparse_seg& sparse, int to_row);
 
     void x_IterateExonParts(const CSpliced_exon::TParts& parts,
                             ESplicedRow                  to_row,
@@ -762,6 +793,13 @@ private:
     void x_Map_PackedInt_Element(const CSeq_interval& si);
     void x_Map_PackedPnt_Element(const CPacked_seqpnt& pp, TSeqPos p);
 
+    // Get main seq-id for a synonym. If no mapping exists, returns the
+    // original id.
+    const CSeq_id_Handle& x_GetPrimaryId(const CSeq_id_Handle& synonym) const;
+
+    typedef map<CSeq_id_Handle, CSeq_id_Handle> TSynonymMap;
+    typedef map<CSeq_id_Handle, TSeqPos> TLengthMap;
+
     // How to merge mapped locations.
     EMergeFlags          m_MergeFlag;
     // How to treat gaps (Null sub-locations) if any.
@@ -782,6 +820,12 @@ private:
     // Collected ranges for mapped graph. Used to adjust mapped graph data.
     CRef<CGraphRanges>   m_GraphRanges;
 
+    // Map each synonym to a primary seq-id.
+    mutable TSynonymMap  m_SynonymMap;
+
+    // Map each primary seq-id to sequence length.
+    mutable TLengthMap   m_LengthMap;
+
 protected:
     // Storage for sequence types.
     mutable TSeqTypeById m_SeqTypes;
@@ -801,6 +845,8 @@ protected:
     int                  m_CurrentGroup;
     // Control how fuzz is generated and propagated
     TFuzzOption          m_FuzzOption;
+    // Misc mapping options
+    TMapOptions          m_MapOptions;
 
     // Sequence info provider
     mutable CRef<IMapper_Sequence_Info> m_SeqInfo;
@@ -829,6 +875,13 @@ public:
 
     /// Get mapping ranges.
     const CMappingRanges& GetMappingRanges(void) const { return *m_Mappings; }
+
+    /// NOTE: In most cases CollectSynonyms(const CSeq_id_Handle& id) should
+    /// be used instead, since it takes care of synonym storage and mapping.
+    /// This method does nothing but storing synonyms in the container.
+    void CollectSynonyms(const CSeq_id_Handle& id,
+                         TSynonyms&            synonyms) const;
+
 };
 
 
@@ -1034,11 +1087,12 @@ inline
 CSeq_loc_Mapper_Base::ESeqType
 CSeq_loc_Mapper_Base::GetSeqTypeById(const CSeq_id_Handle& idh) const
 {
-    TSeqTypeById::const_iterator it = m_SeqTypes.find(idh);
+    CSeq_id_Handle primary_id = CollectSynonyms(idh);
+    TSeqTypeById::const_iterator it = m_SeqTypes.find(primary_id);
     if (it != m_SeqTypes.end()) {
         return it->second;
     }
-    return GetSeqType(idh);
+    return GetSeqType(primary_id);
 }
 
 
