@@ -1,7 +1,7 @@
 #ifndef SEQ_FEAT_HANDLE__HPP
 #define SEQ_FEAT_HANDLE__HPP
 
-/*  $Id: seq_feat_handle.hpp 382535 2012-12-06 19:21:37Z vasilche $
+/*  $Id: seq_feat_handle.hpp 397491 2013-04-26 17:42:27Z kornbluh $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -637,10 +637,21 @@ public:
     /// Add a qualifier to this feature
     void AddQualifier(const string& qual_name, const string& qual_val);
 
+    /// Remove all qualifiers with the given name on this feature.
+    void RemoveQualifier(const string& qual_name);
+
     /// add a DB xref to this feature
     void AddDbxref(const string& db_name, const string& db_key);
     void AddDbxref(const string& db_name, int db_key);
 
+    /// Add the given exception_text and set the except flag to true.
+    /// If the exception_text is already there, it just sets the except flag.
+    void AddExceptText(const string & exception_text);
+
+    /// Remove all instances of the given exception text in this feature,
+    /// and reset the except flag if there are no exception texts left.
+    void RemoveExceptText(const string & exception_text);
+
     /// Add feature id
     void AddFeatId(int id);
     void AddFeatId(const string& id);
