@@ -1,7 +1,7 @@
 #ifndef SEQ_LOC_MAPPER__HPP
 #define SEQ_LOC_MAPPER__HPP
 
-/*  $Id: seq_loc_mapper.hpp 407973 2013-07-25 13:25:29Z grichenk $
+/*  $Id: seq_loc_mapper.hpp 425911 2014-02-04 17:57:38Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -128,7 +128,21 @@ public:
     /// @param direction
     ///  Direction of mapping: up (from segments to master) or down.
     CSeq_loc_Mapper(CBioseq_Handle   target_seq,
-                    ESeqMapDirection direction);
+                    ESeqMapDirection direction,
+                    TMapOptions      opts = 0);
+
+    /// Mapping between segments and the top level sequence.
+    /// @param target_seq
+    ///  Top level bioseq
+    /// @param direction
+    ///  Direction of mapping: up (from segments to master) or down.
+    /// @param selector
+    ///  Seq-map selector with additional restrictions (range, strand etc.).
+    ///  Some properties of the selector are always adjusted by the mapper.
+    CSeq_loc_Mapper(CBioseq_Handle   target_seq,
+                    ESeqMapDirection direction,
+                    SSeqMapSelector  selector,
+                    TMapOptions      opts = 0);
 
     /// Mapping through a seq-map.
     /// @param seq_map
@@ -141,46 +155,8 @@ public:
     CSeq_loc_Mapper(const CSeqMap&   seq_map,
                     ESeqMapDirection direction,
                     const CSeq_id*   top_level_id = 0,
-                    CScope*          scope = 0);
-
-    /// Mapping between segments and the top level sequence limited by depth.
-    /// @param depth
-    ///  Mapping depth. Depth of 0 converts synonyms.
-    /// @param top_level_seq
-    ///  Top level bioseq
-    /// @param direction
-    ///  Direction of mapping: up (from segments to master) or down.
-    CSeq_loc_Mapper(size_t                depth,
-                    const CBioseq_Handle& top_level_seq,
-                    ESeqMapDirection      direction);
-
-    /// Depth-limited mapping through a seq-map.
-    /// @param depth
-    ///  Mapping depth. Depth of 0 converts synonyms.
-    /// @param seq_map
-    ///  Sequence map defining the mapping
-    /// @param direction
-    ///  Direction of mapping: up (from segments to master) or down.
-    /// @param top_level_id
-    ///  Explicit destination id when mapping up, may be used with
-    ///  seq-maps constructed from a seq-loc with multiple ids.
-    CSeq_loc_Mapper(size_t           depth,
-                    const CSeqMap&   top_level_seq,
-                    ESeqMapDirection direction,
-                    const CSeq_id*   top_level_id = 0,
-                    CScope*          scope = 0);
-
-    /// Mapping between segments and the top level sequence.
-    /// @param target_seq
-    ///  Top level bioseq
-    /// @param direction
-    ///  Direction of mapping: up (from segments to master) or down.
-    /// @param selector
-    ///  Seq-map selector with additional restrictions (range, strand etc.).
-    ///  Some properties of the selector are always adjusted by the mapper.
-    CSeq_loc_Mapper(CBioseq_Handle   target_seq,
-                    ESeqMapDirection direction,
-                    SSeqMapSelector  selector);
+                    CScope*          scope = 0,
+                    TMapOptions      opts = 0);
 
     /// Mapping through a seq-map.
     /// @param seq_map
@@ -197,14 +173,46 @@ public:
                     ESeqMapDirection direction,
                     SSeqMapSelector  selector,
                     const CSeq_id*   top_level_id = 0,
-                    CScope*          scope = 0);
+                    CScope*          scope = 0,
+                    TMapOptions      opts = 0);
+
+    /// Mapping between segments and the top level sequence limited by depth.
+    /// @param depth
+    ///  Mapping depth. Depth of 0 converts synonyms.
+    /// @param top_level_seq
+    ///  Top level bioseq
+    /// @param direction
+    ///  Direction of mapping: up (from segments to master) or down.
+    CSeq_loc_Mapper(size_t                depth,
+                    const CBioseq_Handle& top_level_seq,
+                    ESeqMapDirection      direction,
+                    TMapOptions           opts = 0);
+
+    /// Depth-limited mapping through a seq-map.
+    /// @param depth
+    ///  Mapping depth. Depth of 0 converts synonyms.
+    /// @param seq_map
+    ///  Sequence map defining the mapping
+    /// @param direction
+    ///  Direction of mapping: up (from segments to master) or down.
+    /// @param top_level_id
+    ///  Explicit destination id when mapping up, may be used with
+    ///  seq-maps constructed from a seq-loc with multiple ids.
+    CSeq_loc_Mapper(size_t           depth,
+                    const CSeqMap&   top_level_seq,
+                    ESeqMapDirection direction,
+                    const CSeq_id*   top_level_id = 0,
+                    CScope*          scope = 0,
+                    TMapOptions      opts = 0);
 
     /// Destination of seq-id mapping through a GC-Assembly.
     enum EGCAssemblyAlias {
-        eGCA_Genbank, ///< Map to GenBank alias
-        eGCA_Refseq,  ///< Map to RefSeq alias
-        eGCA_UCSC,    ///< Map to UCSC alias
-        eGCA_Other    ///< Map to 'private' seq-id
+        eGCA_Genbank,    ///< Map to GenBank alias, prefer GI
+        eGCA_GenbankAcc, ///< Map to GenBank alias, prefer acc.ver
+        eGCA_Refseq,     ///< Map to RefSeq alias, prefer GI
+        eGCA_RefseqAcc,  ///< Map to RefSeq alias, prefer acc.ver
+        eGCA_UCSC,       ///< Map to UCSC alias
+        eGCA_Other       ///< Map to 'private' seq-id
     };
 
     /// Using CScope for virtual bioseqs created from GC-Assemblies.
@@ -227,7 +235,8 @@ public:
                     ESeqMapDirection    direction,
                     SSeqMapSelector     selector,
                     CScope*             scope = 0,
-                    EScopeFlag          scope_flag = eCopyScope);
+                    EScopeFlag          scope_flag = eCopyScope,
+                    TMapOptions         opts = 0);
 
     ~CSeq_loc_Mapper(void);
 
@@ -247,33 +256,29 @@ private:
                             size_t           depth,
                             const CSeq_id*   top_id,
                             ESeqMapDirection direction);
+    void x_InitializeSeqMap(const CSeqMap&   seq_map,
+                            SSeqMapSelector  selector,
+                            const CSeq_id*   top_id,
+                            ESeqMapDirection direction);
     void x_InitializeSeqMap(CSeqMap_CI       seg_it,
                             const CSeq_id*   top_id,
                             ESeqMapDirection direction);
-    void x_InitializeBioseq(const CBioseq_Handle& bioseq,
-                            const CSeq_id*        top_id,
-                            ESeqMapDirection      direction);
-    void x_InitializeBioseq(const CBioseq_Handle& bioseq,
-                            size_t                depth,
-                            const CSeq_id*        top_id,
-                            ESeqMapDirection      direction);
+    void x_InitializeSeqMapUp(CSeqMap_CI       seg_it,
+                              const CSeq_id*   top_id);
+    void x_InitializeSeqMapDown(CSeqMap_CI       seg_it,
+                                const CSeq_id*   top_id);
+    void x_InitializeSeqMapSingleLevel(CSeqMap_CI       seg_it,
+                                       const CSeq_id*   top_id,
+                                       ESeqMapDirection direction);
 
     // Parse GC-Assembly, collect mappings for each seq-id to the
     // selected alias type.
     void x_InitGCAssembly(const CGC_Assembly& gc_assembly,
                           EGCAssemblyAlias    to_alias);
-    void x_InitGCAssembly(const CGC_Assembly& gc_assembly,
-                          ESeqMapDirection    direction,
-                          SSeqMapSelector     selector);
     // Parse GC-Sequence, collect mappings for each seq-id to the
     // selected alias type.
     void x_InitGCSequence(const CGC_Sequence& gc_seq,
                           EGCAssemblyAlias    to_alias);
-    void x_InitGCSequence(const CGC_Sequence& gc_seq,
-                          ESeqMapDirection    direction,
-                          SSeqMapSelector     selector,
-                          const CGC_Sequence* parent_seq,
-                          CRef<CSeq_id>       override_id);
 
     // Creates and adds to the scope a virtual bioseq. If delta is not null,
     // the bioseq uses it for the instance. Otherwise a virtual bioseq without
