@@ -161,6 +161,10 @@ private:
         const CBioseq_Handle& bsh
     );
 
+    void x_AdjustProteinTitleSuffix (
+        const CBioseq_Handle& bsh
+    );
+
 private:
     /// internal feature tree for parent mapping
     CRef<feature::CFeatTree> m_Feat_Tree;
@@ -177,6 +181,7 @@ private:
     /// seq-inst fields
     bool m_IsNA;
     bool m_IsAA;
+    CSeq_inst::TTopology m_Topology;
 
     bool m_IsSeg;
     bool m_IsDelta;
@@ -189,6 +194,7 @@ private:
     bool m_IsNR;
     bool m_IsPatent;
     bool m_IsPDB;
+    bool m_IsWP;
     bool m_ThirdParty;
     bool m_WGSMaster;
     bool m_TSAMaster;
@@ -229,8 +235,13 @@ private:
     /// biosource fields
     CConstRef<CBioSource> m_Source;
     CTempString m_Taxname;
+    bool m_Multispecies;
     CBioSource::TGenome m_Genome;
 
+    string m_FirstSuperKingdom;
+    string m_SecondSuperKingdom;
+    bool m_IsCrossKingdom;
+
     /// subsource fields
     CTempString m_Chromosome;
     CTempString m_Clone;
@@ -252,7 +263,14 @@ private:
     string m_rEnzyme;
 
     /// exception fields
-    CTextFsm<int> m_Low_Quality_Fsa;
+
+    /// (Careful: CTextFsm has no virtual destructor)
+    class CLowQualityTextFsm : public CTextFsm<int> {
+    public:
+        CLowQualityTextFsm(void);
+    };
+
+    static CSafeStatic<CLowQualityTextFsm> ms_p_Low_Quality_Fsa;
 };
 
 
