@@ -1,7 +1,7 @@
 #ifndef FEAT__HPP
 #define FEAT__HPP
 
-/*  $Id: feature.hpp 382535 2012-12-06 19:21:37Z vasilche $
+/*  $Id: feature.hpp 441408 2014-07-23 18:59:17Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -174,7 +174,14 @@ public:
     /// Construct empty tree.
     CFeatTree(void);
     /// Construct a tree with features collected by a CFeat_CI.
+    explicit
     CFeatTree(CFeat_CI it);
+    /// Construct a tree with features from a Seq-annot
+    explicit
+    CFeatTree(const CSeq_annot_Handle& sah);
+    /// Construct a tree with features from a Seq-entry
+    explicit
+    CFeatTree(const CSeq_entry_Handle& seh);
     /// Destructor.
     ~CFeatTree(void);
 
@@ -336,7 +343,7 @@ public:
         size_t m_AddIndex;
         CMappedFeat m_Feat;
         CRange<TSeqPos> m_MasterRange;
-        CConstRef<CGb_qual> m_TranscriptId;
+        bool m_CanMatchByQual;
         bool m_IsSetParent, m_IsSetChildren, m_MultiId;
         enum EIsLinkedToRoot NCBI_PACKED_ENUM_TYPE(Int1) {
             eIsLinkedToRoot_unknown,
