@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___CONTEXT__HPP
 #define OBJTOOLS_FORMAT___CONTEXT__HPP
 
-/*  $Id: context.hpp 412079 2013-09-04 17:32:35Z rafanovi $
+/*  $Id: context.hpp 437942 2014-06-11 16:30:58Z kornbluh $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -96,6 +96,7 @@ public:
     // Get the bioseq's handle (and those of its neighbors)
     CBioseq_Handle& GetPrevHandle(void) { return m_PrevHandle; }
     CBioseq_Handle& GetHandle(void) { return m_Handle; }
+    const CBioseq_Handle& GetHandle(void) const { return m_Handle; }
     CBioseq_Handle& GetNextHandle(void) { return m_NextHandle; }
     CScope& GetScope(void) { return m_Handle.GetScope(); }
     feature::CFeatTree& GetFeatTree(void) { return m_FeatTree; }
@@ -200,6 +201,7 @@ public:
 
     const string &GetFinishingStatus(void) const;
     bool IsGenomeAssembly(void) const;
+    bool IsCrossKingdom(void) const;
 
     NCBI_DEPRECATED bool IsUnverified(void) const;
     enum FUnverified {
@@ -335,6 +337,7 @@ private:
     bool m_HasOperon;
     bool m_HasMultiIntervalGenes;
     bool m_IsGenomeAssembly;
+    bool m_IsCrossKingdom;
     TUnverified m_fUnverified;
     bool m_ShowAnnotCommentAsCOMMENT;
 
@@ -592,6 +595,12 @@ bool CBioseqContext::IsGenomeAssembly(void) const
     return m_IsGenomeAssembly;
 }
 
+inline
+bool CBioseqContext::IsCrossKingdom(void) const
+{
+    return m_IsCrossKingdom;
+}
+
 inline
 bool CBioseqContext::IsUnverified(void) const
 {
