@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.hpp 412266 2013-09-05 17:37:26Z rafanovi $
+/*  $Id: bed_reader.hpp 448862 2014-10-09 16:32:34Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,12 +34,13 @@
 #define OBJTOOLS_READERS___BEDREADER__HPP
 
 #include <corelib/ncbistd.hpp>
-#include <objtools/readers/reader_base.hpp>
-#include <objtools/readers/message_listener.hpp>
 #include <objects/seq/Seq_annot.hpp>
 #include <objects/seqloc/Seq_id.hpp>
 #include <objects/seqloc/Seq_loc.hpp>
 #include <objects/seqloc/Seq_interval.hpp>
+#include <objects/seqset/Seq_entry.hpp>
+#include <objtools/readers/reader_base.hpp>
+#include <objtools/readers/message_listener.hpp>
 
 BEGIN_NCBI_SCOPE
 
@@ -191,36 +192,11 @@ public:
         ILineReader& lr,
         IMessageListener* pErrors=0 );
 
-    /// Read all objects from given insput stream, returning them as a vector of
-    /// Seq-annots, each containing a feature table.
-    /// @param annots
-    ///   (out) vector containing read Seq-annots
-    /// @param istr
-    ///   input stream to read from.
-    /// @param pErrors
-    ///   pointer to optional error container object. 
-    ///  
-    virtual void
-    ReadSeqAnnots(
-        vector< CRef<CSeq_annot> >& annots,
+    virtual CRef< CSeq_annot >
+    ReadSeqAnnot(
         CNcbiIstream& istr,
         IMessageListener* pErrors=0 );
-                        
-    /// Read all objects from given insput stream, returning them as a vector of
-    /// Seq-annots, each containing a feature table.
-    /// @param annots
-    ///   (out) vector containing read Seq-annots
-    /// @param lr
-    ///   line reader to read from.
-    /// @param pErrors
-    ///   pointer to optional error container object. 
-    ///  
-    virtual void
-    ReadSeqAnnots(
-        vector< CRef<CSeq_annot> >& annots,
-        ILineReader& lr,
-        IMessageListener* pErrors=0 );
-                        
+
     virtual bool 
     ReadTrackData(
         ILineReader&,
@@ -230,7 +206,6 @@ public:
 protected:
     virtual bool xParseTrackLine(
         const string&,
-        CRef< CSeq_annot >&,
         IMessageListener*);
         
     bool xParseFeature(
@@ -268,10 +243,6 @@ protected:
         unsigned int,
         IMessageListener*);
 
-    bool xParseComment(
-        const string&,
-        CRef<CSeq_annot>&);
-
     void x_SetFeatureLocation(
         CRef<CSeq_feat>&,
         const vector<string>&);
@@ -324,7 +295,7 @@ protected:
         CRef<CSeq_feat>&,
         const vector<string>&);
 
-    virtual void x_SetTrackData(
+    virtual void xSetTrackData(
         CRef<CSeq_annot>&,
         CRef<CUser_object>&,
         const string&,
@@ -345,15 +316,6 @@ protected:
         CRawBedRecord&,
         IMessageListener*);
 
-    bool 
-    xGetLine(
-        ILineReader&,
-        string&);
-
-    bool 
-    xCommentLine(
-        const string&) const;
-
     static void xCleanColumnValues(
         vector<string>&);
 
