@@ -1,4 +1,4 @@
-/*  $Id: line_error.hpp 413670 2013-09-17 17:08:28Z ivanov $
+/*  $Id: line_error.hpp 423972 2014-01-10 13:51:01Z kornbluh $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,6 +35,7 @@
 
 #include <corelib/ncbistd.hpp>
 #include <corelib/ncbimisc.hpp>
+#include <corelib/ncbiobj.hpp>
 #include <objtools/readers/reader_exception.hpp>
 
 
@@ -97,6 +98,14 @@ public:
         eProblem_GeneralParsingError
     };
 
+    /// This is here because the copy constructor may be protected
+    /// eventually.
+    virtual ILineError *Clone(void) const {
+        /// throw instead of making it pure virtual for backward
+        /// compatibility.
+        NCBI_USER_THROW("not implemented: ILineError::Clone");
+    }
+
     virtual ~ILineError(void) throw() {}
 
     virtual EProblem
@@ -298,6 +307,39 @@ public:
         }
         out << endl;
     };
+
+    // dump the XML on one line since some tools assume that
+    virtual void DumpAsXML(
+        std::ostream& out ) const
+    {
+        out << "<message severity=\"" << NStr::XmlEncode(SeverityStr()) << "\" "
+            << "problem=\"" << NStr::XmlEncode(ProblemStr()) << "\" ";
+        const string & seqid = SeqId();
+        if (!seqid.empty()) {
+            out << "seqid\"" << NStr::XmlEncode(seqid) << "\" ";
+        }
+        out << "line=\"" << Line() << "\" ";
+        const string & feature = FeatureName();
+        if (!feature.empty()) {
+            out << "feature_name=\"" << NStr::XmlEncode(feature) << "\" ";
+        }
+        const string & qualname = QualifierName();
+        if (!qualname.empty()) {
+            out << "qualifier_name=\"" << NStr::XmlEncode(qualname) << "\" ";
+        }
+        const string & qualval = QualifierValue();
+        if (!qualval.empty()) {
+            out << "qualifier_value=\"" << NStr::XmlEncode(qualval) << "\" ";
+        }
+        out << ">";
+
+        // child nodes
+        ITERATE(TVecOfLines, line_it, OtherLines()) {
+            out << "<other_line>" << *line_it << "</other_line>";
+        }
+
+        out << "</message>" << endl;
+    };
 };
     
 //  ============================================================================
@@ -306,7 +348,12 @@ class NCBI_XOBJUTIL_EXPORT CLineError:
     public ILineError
 {
 public:
-    CLineError(
+
+    /// Use this because the constructor is protected.
+    ///
+    /// @returns
+    ///   Caller is responsible for the return value.
+    static CLineError* Create(
         EProblem eProblem,
         EDiagSev eSeverity,
         const std::string& strSeqId,
@@ -314,13 +361,17 @@ public:
         const std::string & strFeatureName = string(""),
         const std::string & strQualifierName = string(""),
         const std::string & strQualifierValue = string(""),
-        const std::string & strErrorMessage = string(""))
-    : m_eProblem(eProblem), m_eSeverity( eSeverity ), m_strSeqId(strSeqId), m_uLine( uLine ), 
-      m_strFeatureName(strFeatureName), m_strQualifierName(strQualifierName), 
-      m_strQualifierValue(strQualifierValue), m_strErrorMessage(strErrorMessage)
-     { }
-    
+        const std::string & strErrorMessage = string(""),
+        const TVecOfLines & vecOfOtherLines = TVecOfLines() );
+
+    /// Use this because copy ctor is protected.
+    virtual ILineError *Clone(void) const;
+
     virtual ~CLineError(void) throw() {}
+
+    /// copy constructor is protected so please use this function to
+    /// throw the object.
+    NCBI_NORETURN void Throw(void) const;
        
     void PatchLineNumber(
         unsigned int uLine) { m_uLine = uLine; };
@@ -368,6 +419,7 @@ public:
     const std::string &ErrorMessage(void) const { return m_strErrorMessage; }
 
 protected:
+
     EProblem m_eProblem;
     EDiagSev m_eSeverity;
     std::string m_strSeqId;
@@ -377,18 +429,41 @@ protected:
     std::string m_strQualifierValue;
     std::string m_strErrorMessage;
     TVecOfLines m_vecOfOtherLines;
+
+    /// protected instead of public.  Please use the Create function instead.
+    CLineError(
+        EProblem eProblem,
+        EDiagSev eSeverity,
+        const std::string& strSeqId,
+        unsigned int uLine,
+        const std::string & strFeatureName,
+        const std::string & strQualifierName,
+        const std::string & strQualifierValue,
+        const std::string & strErrorMessage,
+        const TVecOfLines & m_vecOfOtherLine);
+
+    /// protected instead of public.  Please use the Throw function to throw
+    /// this exception and try to avoid using the copy constructor at all.
+    CLineError(const CLineError & rhs );
 };
 
 //  ============================================================================
 class NCBI_XOBJUTIL_EXPORT CObjReaderLineException
 //  ============================================================================
-    : public CObjReaderParseException, public ILineError
+    // must inherit from ILineError first due to the way CObject detects
+    // whether or not it's on the heap.
+    : public ILineError, public CObjReaderParseException
 {
 public:
 
     using CObjReaderParseException::EErrCode;
 
-    CObjReaderLineException(
+    /// Please use this instead of the constructor because the ctor
+    /// is protected.
+    ///
+    /// @returns
+    ///   Caller is responsible for the return value.
+    static CObjReaderLineException* Create(
         EDiagSev eSeverity,
         unsigned int uLine,
         const std::string &strMessage,
@@ -397,27 +472,16 @@ public:
         const std::string & strFeatureName = string(""),
         const std::string & strQualifierName = string(""),
         const std::string & strQualifierValue = string(""),
-        CObjReaderLineException::EErrCode eErrCode = eFormat
-        )
-        : CObjReaderParseException( DIAG_COMPILE_INFO, 0, static_cast<CObjReaderParseException::EErrCode>(CException::eInvalid), strMessage, uLine,
-        eDiag_Info ), 
-        m_eProblem(eProblem), m_strSeqId(strSeqId), m_uLineNumber(uLine),
-        m_strFeatureName(strFeatureName), m_strQualifierName(strQualifierName), 
-        m_strQualifierValue(strQualifierValue), m_strErrorMessage(strMessage)
-    {
-        SetSeverity( eSeverity );
-        x_InitErrCode(static_cast<CException::EErrCode>(eErrCode));
-    }
+        CObjReaderLineException::EErrCode eErrCode = eFormat,
+        const TVecOfLines & vecOfOtherLines = TVecOfLines()
+        );
 
-    CObjReaderLineException(const CObjReaderLineException & rhs ) :
-        CObjReaderParseException( rhs ), 
-            m_eProblem(rhs.Problem()), m_strSeqId(rhs.SeqId()), m_uLineNumber(rhs.Line()), 
-            m_strFeatureName(rhs.FeatureName()), m_strQualifierName(rhs.QualifierName()),
-            m_strQualifierValue(rhs.QualifierValue()), m_strErrorMessage(rhs.ErrorMessage())
-    {
-        SetSeverity( rhs.Severity() );
-        x_InitErrCode(static_cast<CException::EErrCode>(rhs.x_GetErrCode()));
-    }
+    /// Use instead of copy constructor, which is protected.
+    virtual ILineError *Clone(void) const;
+
+    // Copy constructor is protected, so please use 
+    /// this function to throw this object.
+    NCBI_NORETURN void Throw(void) const;
 
     ~CObjReaderLineException(void) throw() { }
 
@@ -439,7 +503,7 @@ public:
 
     std::string ProblemStr() const;
 
-    std::string Message() const { return GetMsg(); }
+    std::string Message() const { return ( GetMsg().empty() ? ILineError::Message() : GetMsg()); }
     
     //
     //  Cludge alert: The line number may not be known at the time the exception
@@ -457,6 +521,7 @@ public:
     }
 
 protected:
+
     EProblem m_eProblem;
     std::string m_strSeqId;
     unsigned int m_uLineNumber;
@@ -465,6 +530,23 @@ protected:
     std::string m_strQualifierValue;
     std::string m_strErrorMessage;
     TVecOfLines m_vecOfOtherLines;
+
+    /// protected instead of public.  Please use the Create function instead.
+    CObjReaderLineException(
+        EDiagSev eSeverity,
+        unsigned int uLine,
+        const std::string &strMessage,
+        EProblem eProblem = eProblem_GeneralParsingError,
+        const std::string& strSeqId = string(""),
+        const std::string & strFeatureName = string(""),
+        const std::string & strQualifierName = string(""),
+        const std::string & strQualifierValue = string(""),
+        CObjReaderLineException::EErrCode eErrCode = eFormat,
+        const TVecOfLines & vecOfOtherLines = TVecOfLines()
+        );
+
+    /// Protected, so use Clone or Throw instead.
+    CObjReaderLineException(const CObjReaderLineException & rhs );
 };
 
     
