@@ -1,4 +1,4 @@
-/*  $Id: reader_base.hpp 407582 2013-07-23 11:15:22Z kornbluh $
+/*  $Id: reader_base.hpp 448871 2014-10-09 16:35:08Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,12 +37,14 @@
 #include <objects/seq/Seq_annot.hpp>
 #include <util/format_guess.hpp>
 #include <util/line_reader.hpp>
+#include <util/icanceled.hpp>
 #include <objtools/readers/line_error.hpp>
 
 BEGIN_NCBI_SCOPE
 
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
+class CSeq_entry;
 class IMessageListener;
 class CObjReaderLineException;
 class CTrackData;
@@ -147,6 +149,36 @@ public:
         ILineReader& lr,
         IMessageListener* pErrors=0 );
 
+    /// Read all objects from given insput stream, returning them as a vector of
+    /// Seq-annots.
+    /// @param annots
+    ///   (out) vector containing read Seq-annots
+    /// @param istr
+    ///   input stream to read from.
+    /// @param pErrors
+    ///   pointer to optional error container object. 
+    ///  
+    virtual void
+    ReadSeqAnnots(
+        vector< CRef<CSeq_annot> >& annots,
+        CNcbiIstream& istr,
+        IMessageListener* pErrors=0 );
+                        
+    /// Read all objects from given insput stream, returning them as a vector of
+    /// Seq-annots.
+    /// @param annots
+    ///   (out) vector containing read Seq-annots
+    /// @param lr
+    ///   line reader to read from.
+    /// @param pErrors
+    ///   pointer to optional error container object. 
+    ///  
+    virtual void
+    ReadSeqAnnots(
+        vector< CRef<CSeq_annot> >& annots,
+        ILineReader& lr,
+        IMessageListener* pErrors=0 );
+                        
     /// Read an object from a given input stream, render it as a single
     /// Seq-entry, if possible. Return empty Seq-entry otherwise.
     /// @param istr
@@ -171,41 +203,88 @@ public:
         ILineReader& lr,
         IMessageListener* pErrors=0 );
        
+    void
+    SetProgressReportInterval(
+        unsigned int intv );
+
+    void
+    SetCanceler(
+        ICanceled* =0);
+
+    bool
+    IsCanceled() const { return m_pCanceler && m_pCanceler->IsCanceled(); };
+
 protected:
-    virtual void x_AssignTrackData(
+    virtual bool xGetLine(
+        ILineReader&,
+        string&);
+
+    virtual bool xUngetLine(
+        ILineReader&);
+
+    virtual bool xIsCommentLine(
+        const CTempString& );
+
+    virtual bool xIsTrackLine(
+        const CTempString& );
+
+    virtual bool xIsBrowserLine(
+        const CTempString& );
+
+    virtual void xAssignTrackData(
         CRef<CSeq_annot>& );
                 
-    virtual bool x_ParseBrowserLine(
+    virtual bool xParseBrowserLine(
         const string&,
         CRef<CSeq_annot>&,
         IMessageListener*);
         
-    virtual bool x_ParseTrackLine(
+    virtual bool xParseTrackLine(
         const string&,
-        CRef<CSeq_annot>&,
         IMessageListener*);
         
-    virtual void x_SetBrowserRegion(
+    virtual bool xParseBrowserLine(
+        const string&,
+        IMessageListener*);
+        
+    virtual void xSetBrowserRegion(
         const string&,
         CAnnot_descr&,
         IMessageListener*);
 
-    virtual void x_SetTrackData(
+    virtual void xSetTrackData(
         CRef<CSeq_annot>&,
         CRef<CUser_object>&,
         const string&,
-        const string& );
+        const string&);
                 
-    virtual void x_AddConversionInfo(
+    virtual void xAddConversionInfo(
         CRef< CSeq_annot >&,
         IMessageListener* );
                     
-    virtual void x_AddConversionInfo(
+    virtual void xAddConversionInfo(
         CRef< CSeq_entry >&,
-        IMessageListener* );
+        IMessageListener*);
     
-    virtual CRef<CUser_object> x_MakeAsnConversionInfo(
-        IMessageListener* );
+    virtual CRef<CUser_object> xMakeAsnConversionInfo(
+        IMessageListener*);
+
+    bool xParseComment(
+        const CTempString&,
+        CRef<CSeq_annot>&);
+
+    virtual bool xReadInit();
+
+    virtual bool xProgressInit(
+        CNcbiIstream& istr);
+
+    void xReportProgress(
+        IMessageListener* =0 );
+
+    bool xIsReportingProgress() const;
+
+    
+    bool xIsOperationCanceled() const;
 
     void
     ProcessError(
@@ -231,8 +310,14 @@ protected:
     //  Data:
     //
     unsigned int m_uLineNumber;
+    unsigned int m_uProgressReportInterval;
+    unsigned int m_uNextProgressReport;
+    ios::streampos m_uMaxFilePos;
+
     TReaderFlags       m_iFlags;
     CTrackData*  m_pTrackDefaults;
+    CNcbiIstream* m_pInStream;
+    ICanceled* m_pCanceler;
 };
 
 END_objects_SCOPE
