@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___SOURCE_MOD_PARSER__HPP
 #define OBJTOOLS_READERS___SOURCE_MOD_PARSER__HPP
 
-/*  $Id: source_mod_parser.hpp 399305 2013-05-13 19:13:43Z grichenk $
+/*  $Id: source_mod_parser.hpp 448727 2014-10-08 15:47:13Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -75,6 +75,9 @@ class CBioseq;
 /// for allowing them to contain bracketed modifiers of the form [key=value],
 /// as documented at http://www.ncbi.nlm.nih.gov/Sequin/modifiers.html .
 
+template<class _T>
+class CAutoInitDesc;
+
 class NCBI_XOBJREAD_EXPORT CSourceModParser
 {
 public:
@@ -181,6 +184,18 @@ public:
     /// Allows user to get the list of bad mods found by this
     const TMods & GetBadMods(void) const { return m_BadMods; }
 
+    class CModFilter : public CObject {
+    public:
+        virtual ~CModFilter(void) { }
+        virtual bool operator()( const CTempString & mod_name ) = 0;
+    };
+    /// The filter indicates whether certain mods are okay or not.
+    ///
+    /// @param pModFilter
+    ///   If this is an unset CRef, it turns off filtering.
+    void SetModFilter( CRef<CModFilter> pModFilter ) { 
+        m_pModFilter = pModFilter; }
+
     /// Given a mod name (e.g. "topology"), it returns the set of acceptable
     /// values (e.g. "linear", "circular", etc.).  If the
     /// mod is unknown or free-form, the returned set will be blank.
@@ -198,11 +213,13 @@ private:
     TMods m_Mods;
     TMods m_BadMods;
 
-    void x_ApplyMods(CAutoInitRef<CBioSource>& bsrc, CTempString organism);
-    void x_ApplyMods(CAutoInitRef<CMolInfo>& mi);
+    CRef<CModFilter> m_pModFilter;
+
+    void x_ApplyMods(CAutoInitDesc<CBioSource>& bsrc, CTempString organism);
+    void x_ApplyMods(CAutoInitDesc<CMolInfo>& mi);
     void x_ApplyMods(CAutoInitRef<CGene_ref>& gene);
     void x_ApplyMods(CAutoInitRef<CProt_ref>& prot);
-    void x_ApplyMods(CAutoInitRef<CGB_block>& gbb);
+    void x_ApplyMods(CAutoInitDesc<CGB_block>& gbb);
     void x_ApplyMods(CAutoInitRef<CSeq_hist>& hist);
     // void x_ApplyMods(CAutoInitRef<CSubmit_block>& sb);
     void x_ApplyTPAMods(CAutoInitRef<CUser_object>& tpa);
@@ -219,24 +236,6 @@ private:
 
 //////////////////////////////////////////////////////////////////////
 
-inline
-void CSourceModParser::ApplyMods(CBioSource& bsrc, CTempString organism)
-{
-    CAutoInitRef<CBioSource> ref;
-    ref.Set(&bsrc);
-    x_ApplyMods(ref, organism);
-}
-
-
-inline
-void CSourceModParser::ApplyMods(CMolInfo& mi)
-{
-    CAutoInitRef<CMolInfo> ref;
-    ref.Set(&mi);
-    x_ApplyMods(ref);
-}
-
-
 inline
 void CSourceModParser::ApplyMods(CGene_ref& gene)
 {
@@ -255,15 +254,6 @@ void CSourceModParser::ApplyMods(CProt_ref& prot)
 }
 
 
-inline
-void CSourceModParser::ApplyMods(CGB_block& gbb)
-{
-    CAutoInitRef<CGB_block> ref;
-    ref.Set(&gbb);
-    x_ApplyMods(ref);
-}
-
-
 inline
 void CSourceModParser::ApplyMods(CSeq_hist& hist)
 {
