@@ -1,4 +1,4 @@
-/*  $Id: wiggle_reader.hpp 416067 2013-10-23 18:17:46Z rafanovi $
+/*  $Id: wiggle_reader.hpp 448858 2014-10-09 16:31:43Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -82,6 +82,7 @@ struct SVarStepInfo {
 //  ============================================================================
 struct SValueInfo {
 //  ============================================================================
+    string m_Chrom;
     TSeqPos m_Pos;
     TSeqPos m_Span;
     double m_Value;
@@ -92,6 +93,9 @@ struct SValueInfo {
         return m_Pos + m_Span;
     }
     bool operator<(const SValueInfo& v) const {
+        if (m_Chrom != v.m_Chrom) {
+            return m_Chrom < v.m_Chrom;
+        }
         return m_Pos < v.m_Pos;
     }
 };
@@ -238,9 +242,8 @@ class NCBI_XOBJREAD_EXPORT CWiggleReader
     : public CReaderBase
 {
 public:
-    //
-    //  object management:
-    //
+    typedef vector<SValueInfo> TValues;
+
 public:
     CWiggleReader( 
         int =fDefaults );
@@ -253,11 +256,11 @@ public:
 public:
     enum EWiggleFlags {
         fDefaults = 0,
-        fJoinSame = 1<<0,
-        fAsByte = 1<<1,
-        fAsGraph = 1<<2,
-        fDumpStats = 1<<3,
-        fAsRaw = 1<<4,
+        fJoinSame = 1<<8,
+        fAsByte = 1<<9,
+        fAsGraph = 1<<10,
+        fDumpStats = 1<<11,
+        fAsRaw = 1<<12,
     };
     typedef int TFlags;
 
@@ -266,18 +269,6 @@ public:
         ILineReader&,
         IMessageListener* =0 );
 
-    virtual void
-    ReadSeqAnnots(
-        vector< CRef<CSeq_annot> >&,
-        CNcbiIstream&,
-        IMessageListener* =0 );
-                        
-    virtual void
-    ReadSeqAnnots(
-        vector< CRef<CSeq_annot> >&,
-        ILineReader&,
-        IMessageListener* =0 );
-                
     virtual CRef< CSerialObject >
     ReadObject(
         ILineReader&,
@@ -293,11 +284,23 @@ public:
     //  helpers:
     //
 protected:
-    void 
-    xReadBrowser();
+    CRef< CSeq_annot >
+    xReadSeqAnnotGraph(
+        ILineReader&,
+        IMessageListener* =0 );
 
-    void 
-    xReadTrack(
+    CRef< CSeq_annot >
+    xReadSeqAnnotTable(
+        ILineReader&,
+        IMessageListener* =0 );
+
+    bool 
+    xProcessBrowserLine(
+        IMessageListener*);
+
+    bool 
+    xParseTrackLine(
+        const string&,
         IMessageListener*);
 
     void
@@ -336,31 +339,27 @@ protected:
 
     void 
     xReadBedLine(
-        CTempString chrom,
+        const string& chrom,
         IMessageListener*);
 
     CRef<CSeq_annot>
     xGetAnnot();
 
-    bool 
-    xGetLine(
-        ILineReader&);
-
     bool 
     xCommentLine() const;
 
-    CTempString 
+    string 
     xGetWord(
         IMessageListener*);
 
     bool 
     xSkipWS();
 
-    CTempString 
+    string 
     xGetParamName(
         IMessageListener*);
 
-    CTempString 
+    string 
     xGetParamValue(
         IMessageListener*);
 
@@ -436,24 +435,25 @@ protected:
 
     void 
     xSetChrom(
-        CTempString chrom);
+        const string& chrom);
+
+    CRef<CSeq_loc> 
+    xGetContainingLoc();
+
+    bool
+    xValuesAreFromSingleSequence() const;
 
     //
     //  data:
     //
 protected:
-    CTempStringEx m_CurLine;
+    string m_CurLine;
     string m_ChromId;
-    typedef vector<SValueInfo> TValues;
     TValues m_Values;
     double m_GapValue;
     bool m_SingleAnnot;
     bool m_OmitZeros;
-    string m_TrackName;
-    string m_TrackDescription;
-    typedef map<string, string> TTrackParams;
-    TTrackParams m_TrackParams;
-    string m_TrackTypeValue;
+
     enum ETrackType {
         eTrackType_invalid,
         eTrackType_wiggle_0,
