@@ -1,7 +1,7 @@
 #ifndef CHECKSUM__HPP
 #define CHECKSUM__HPP
 
-/*  $Id: checksum.hpp 365172 2012-06-04 13:42:54Z lavr $
+/*  $Id: checksum.hpp 448175 2014-10-02 18:33:47Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -61,10 +61,12 @@ class NCBI_XUTIL_EXPORT CChecksum
 public:
     /// Method used to compute control sum.
     enum EMethod {
-        eNone,             ///< No checksum in file
-        eCRC32,            ///< 32-bit Cyclic Redundancy Check
-        eMD5,              ///< Message Digest version 5
-        eCRC32ZIP,         ///< Exact zip CRC32, slightly differs from eCRC32
+        eNone,             ///< No checksum in file.
+        eCRC32,            ///< 32-bit Cyclic Redundancy Check.
+        eCRC32ZIP,         ///< Exact zip CRC32, slightly differs from eCRC32.
+        eCRC32INSD,        ///< Inverted CRC32ZIP. Hash function used in
+                           ///< the ID system to verify sequence uniqueness.
+        eMD5,              ///< Message Digest version 5.
         eAdler32,          ///< A bit faster than CRC32ZIP, not recommended
                            ///< for small data sizes.
         eDefault = eCRC32
@@ -165,11 +167,12 @@ Uint4      ComputeFileCRC32(const string& path);
 ///
 /// Stream class to compute checksum for written data.
 ///
-class CChecksumStreamWriter : public IWriter
+class NCBI_XUTIL_EXPORT CChecksumStreamWriter : public IWriter
 {
 public:
     /// Construct object to compute checksum for written data.
     CChecksumStreamWriter(CChecksum::EMethod method);
+    virtual ~CChecksumStreamWriter(void);
 
     /// Virtual methods from IWriter
     virtual ERW_Result Write(const void* buf, size_t count,
@@ -267,29 +270,6 @@ Uint4 ComputeFileCRC32(const string& path)
     return ComputeFileChecksum(path, CChecksum::eCRC32).GetChecksum();
 }
 
-inline
-CChecksumStreamWriter::CChecksumStreamWriter(CChecksum::EMethod method)
-    : m_Checksum(method)
-{
-}
-
-inline
-ERW_Result CChecksumStreamWriter::Write(const void* buf, size_t count,
-                                        size_t* bytes_written)
-{
-    m_Checksum.AddChars((const char*)buf, count);
-    if (bytes_written) {
-        *bytes_written = count;
-    }
-    return eRW_Success;
-}
-
-inline
-ERW_Result CChecksumStreamWriter::Flush(void)
-{
-    return eRW_Success;
-}
-
 inline
 const CChecksum& CChecksumStreamWriter::GetChecksum(void) const
 {
