@@ -1,7 +1,7 @@
 #ifndef UTIL_COMPRESS__COMPRESS__HPP
 #define UTIL_COMPRESS__COMPRESS__HPP
 
-/*  $Id: compress.hpp 367639 2012-06-27 12:34:44Z ivanov $
+/*  $Id: compress.hpp 430126 2014-03-24 12:25:47Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -41,6 +41,7 @@
  * @{
  */
 
+
 BEGIN_NCBI_SCOPE
 
 
@@ -50,6 +51,31 @@ const streamsize kCompressionDefaultBufSize = 16*1024;
 /// Macro to report errors in compression API.
 #define ERR_COMPRESS(subcode, message) ERR_POST_X(subcode, Warning << message)
 
+/// Macro to catch and handle exceptions (from streams in the destructor)
+#define COMPRESS_HANDLE_EXCEPTIONS(subcode, message)                  \
+    catch (CException& e) {                                           \
+        try {                                                         \
+            NCBI_REPORT_EXCEPTION_X(subcode, message, e);             \
+        } catch (...) {                                               \
+        }                                                             \
+    }                                                                 \
+    catch (exception& e) {                                            \
+        try {                                                         \
+            ERR_POST_X(subcode, Error                                 \
+                       << "[" << message                              \
+                       << "] Exception: " << e.what());               \
+        } catch (...) {                                               \
+        }                                                             \
+    }                                                                 \
+    catch (...) {                                                     \
+        try {                                                         \
+            ERR_POST_X(subcode, Error                                 \
+                       << "[" << message << "] Unknown exception");   \
+        } catch (...) {                                               \
+        }                                                             \
+    }                                                                 \
+
+
 // Forward declaration
 class CCompressionFile;
 class CCompressionStreambuf;
@@ -85,8 +111,10 @@ public:
     };
 
     /// Compression flags. The flag selection depends from compression
-    /// algorithm implementation.
-    typedef unsigned int TFlags;    ///< Bitwise OR of EFlags*
+    /// algorithm implementation. For examples see the flags defined
+    /// in the derived classes:  CBZip2Compression::EFlags,
+    /// CLZOCompression::EFlags, CZipCompression::EFlags, etc.
+    typedef unsigned int TFlags;    ///< Bitwise OR of CXxxCompression::EFlags
 
 public:
     /// Destructor
