@@ -1,7 +1,7 @@
 #ifndef UTIL___VALUE_CONVERT__HPP
 #define UTIL___VALUE_CONVERT__HPP
 
-/* $Id: value_convert.hpp 362689 2012-05-10 14:06:40Z ucko $
+/* $Id: value_convert.hpp 423993 2014-01-10 16:41:29Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1676,7 +1676,7 @@ template <typename CP, typename FROM>
 inline
 string operator+(const value_slice::CValueConvert<CP, FROM>& value, const string& s)
 {
-    string str_value = value;
+    string str_value = value.operator string();
 
     str_value += s;
     return str_value;
@@ -1701,7 +1701,7 @@ template <typename CP, typename FROM>
 inline
 string operator+(const value_slice::CValueConvert<CP, FROM>& value, const char* s)
 {
-    string str_value = value;
+    string str_value = value.operator string();
 
     if (s) {
         str_value += s;
