@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: check_make_unix.sh 416464 2013-10-28 19:08:09Z rafanovi $
+# $Id: check_make_unix.sh 429809 2014-03-19 16:54:53Z ivanov $
 # Author:  Vladimir Ivanov, NCBI 
 #
 ###########################################################################
@@ -52,6 +52,11 @@ x_top_srcdir=$4
 x_target_dir=$5
 x_out=$6
 
+# Detect Cygwin
+case `uname -s` in
+   CYGWIN* ) cygwin=true  ;;
+   *)        cygwin=false ;;
+esac
 
 # Check for build dir
 if test ! -z "$x_build_dir"; then
@@ -152,6 +157,7 @@ conf_dir="$x_conf_dir"
 compile_dir="$x_compile_dir"
 bin_dir="$x_bin_dir"
 script="$x_out"
+cygwin=$cygwin
 
 res_journal="\$script.journal"
 res_log="\$script.log"
@@ -306,10 +312,11 @@ fi
 
 # Path to test data, used by some scripts and applications
 if test -z "\$NCBI_TEST_DATA"; then
-    case `uname -s` in
-       CYGWIN* ) NCBI_TEST_DATA=//snowman/win-coremake/Scripts/test_data ;;
-       *)        NCBI_TEST_DATA=/am/ncbiapdata/test_data ;;
-    esac
+    if [ \$cygwin = true ]; then
+       NCBI_TEST_DATA=//snowman/win-coremake/Scripts/test_data
+    else
+       NCBI_TEST_DATA=/am/ncbiapdata/test_data
+    fi
     export NCBI_TEST_DATA
 fi
 
@@ -407,7 +414,9 @@ fi
 
 if test "\$NCBI_CHECK_SETLIMITS" != "0"; then
    ulimit -c 1000000
-   ulimit -v 4000000
+   if [ \$cygwin = false ]; then
+      ulimit -v 4000000
+   fi
 fi
 
 
@@ -545,6 +554,10 @@ RunTest()
                     echo "\$x_name"
                     echo "======================================================================"
                     echo 
+                    if test "\$x_run" != "\$x_name"; then
+                       echo "Command line: \$x_run"
+                       echo 
+                    fi
                 ) > \$x_test_out 2>&1
 
                 # Remove old core file if it exist (for clarity of the test)
