@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# $Id: install.sh 136687 2008-08-06 13:15:54Z ucko $
+# $Id: install.sh 429376 2014-03-13 19:37:01Z ucko $
 # Author:  Denis Vakatov, NCBI 
 #
 ###########################################################################
@@ -16,8 +16,8 @@
 
 echo "[`date`]"
 
-svn_location=`echo '$URL: https://svn.ncbi.nlm.nih.gov/repos/toolkit/release/blast/2.2.29/c++/scripts/common/impl/install.sh $' | sed "s%\\$[U]RL: *\\([^$][^$]*\\) \\$.*%\\1%"`
-svn_revision=`echo '$Revision: 136687 $' | sed "s%\\$[R]evision: *\\([^$][^$]*\\) \\$.*%\\1%"`
+svn_location=`echo '$URL: https://svn.ncbi.nlm.nih.gov/repos/toolkit/release/blast/2.2.30/c++/scripts/common/impl/install.sh $' | sed "s%\\$[U]RL: *\\([^$][^$]*\\) \\$.*%\\1%"`
+svn_revision=`echo '$Revision: 429376 $' | sed "s%\\$[R]evision: *\\([^$][^$]*\\) \\$.*%\\1%"`
 
 script_name=`basename $0`
 script_dir=`dirname $0`
@@ -98,9 +98,14 @@ EOF
 #####  INSTALL
 
 
-# Directories to install
-install_dirs="include compilers scripts build_info"
-if test "$with_src" != "no"; then 
+# Top-level entities (mostly directories) to install
+install_dirs="include compilers scripts"
+if test -f "$top_srcdir/build_info"; then
+    install_dirs="$install_dirs build_info"
+fi
+if test "$with_src" = "no"; then
+    install_dirs="$install_dirs src/build-system"
+else
     install_dirs="$install_dirs src configure"
 fi
 if test "$with_doc" != "no" -a -d "$top_srcdir/doc"; then
