@@ -1,4 +1,4 @@
-/*  $Id: blast_options_builder.cpp 403214 2013-06-13 13:25:50Z fongah2 $
+/*  $Id: blast_options_builder.cpp 431529 2014-04-04 14:15:53Z fongah2 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -47,14 +47,15 @@ BEGIN_SCOPE(blast)
 CBlastOptionsBuilder::
 CBlastOptionsBuilder(const string                & program,
                      const string                & service,
-                     CBlastOptions::EAPILocality   locality)
+                     CBlastOptions::EAPILocality   locality,
+                     bool ignore_unsupported_options)
     : m_Program        (program),
       m_Service        (service),
       m_PerformCulling (false),
       m_HspRangeMax    (0),
       m_QueryRange     (TSeqRange::GetEmpty()),
       m_Locality       (locality),
-      m_IgnoreUnsupportedOptions(false),
+      m_IgnoreUnsupportedOptions(ignore_unsupported_options),
       m_ForceMbIndex   (false)
 {
 }
@@ -365,6 +366,8 @@ x_ProcessOneOption(CBlastOptionsHandle        & opts,
     case 'Q':
         if (CBlast4Field::Get(eBlastOpt_QueryGeneticCode).Match(p)) {
             bo.SetQueryGeneticCode(v.GetInteger());
+        }else if (CBlast4Field::Get(eBlastOpt_QueryCovHspPerc).Match(p)) {
+            opts.SetQueryCovHspPerc(v.GetReal());
         } else {
             found = false;
         }
