@@ -1,6 +1,6 @@
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_options_cxx.cpp 371842 2012-08-13 13:56:38Z fongah2 $";
+    "$Id: blast_options_cxx.cpp 438018 2014-06-12 11:52:14Z fongah2 $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
@@ -560,6 +560,10 @@ void CBlastOptionsRemote::SetValue(EBlastOptIdx opt, const double & v)
         x_SetParam(CBlast4Field::Get(opt), v);
         return;
 
+    case eBlastOpt_QueryCovHspPerc:
+        x_SetParam(CBlast4Field::Get(opt), v);
+        return;
+
     case eBlastOpt_InclusionThreshold:
         x_SetParam(CBlast4Field::Get(opt), v);
         return;
@@ -841,6 +845,18 @@ CBlastOptions::SetProgram(EProgram p)
     }
 }
 
+bool
+CBlastOptions::IsIterativeSearch() const{
+    switch ( GetProgram()) {
+    	case ePSIBlast :
+    	case ePSITblastn :
+    	case eDeltaBlast :
+    		return true;
+    	break;
+    	default :
+    		return false;
+    }
+}
 /******************* Lookup table options ***********************/
 double 
 CBlastOptions::GetWordThreshold() const
@@ -1719,6 +1735,26 @@ CBlastOptions::SetPercentIdentity(double p)
     }
 }
 
+double
+CBlastOptions::GetQueryCovHspPerc() const
+{
+    if (! m_Local) {
+        x_Throwx("Error: GetQueryCovHspPerc() not available.");
+    }
+    return m_Local->GetQueryCovHspPerc();
+}
+void
+CBlastOptions::SetQueryCovHspPerc(double p)
+{
+    if (m_Local) {
+        m_Local->SetQueryCovHspPerc(p);
+    }
+    if (m_Remote) {
+        m_Remote->SetValue(eBlastOpt_QueryCovHspPerc, p);
+    }
+}
+
+
 int 
 CBlastOptions::GetMinDiagSeparation() const
 {
