@@ -1,4 +1,4 @@
-/*  $Id: blast_options_local_priv.hpp 363884 2012-05-21 15:54:30Z morgulis $
+/*  $Id: blast_options_local_priv.hpp 436114 2014-05-23 14:32:59Z madden $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -202,6 +202,9 @@ public:
     double GetPercentIdentity() const;
     void SetPercentIdentity(double p);
 
+    double GetQueryCovHspPerc() const;
+    void SetQueryCovHspPerc(double p);
+
     int GetMinDiagSeparation() const;
     void SetMinDiagSeparation(int d);
 
@@ -560,6 +563,10 @@ inline void
 CBlastOptionsLocal::SetWordSize(int ws)
 {
     m_LutOpts->word_size = ws;
+    if (m_LutOpts->lut_type == eCompressedAaLookupTable && ws < 5)
+	m_LutOpts->lut_type = eAaLookupTable;
+    else if (m_LutOpts->lut_type == eAaLookupTable && ws >= 5)
+	m_LutOpts->lut_type = eCompressedAaLookupTable;
 }
 
 inline unsigned char
@@ -1226,6 +1233,18 @@ CBlastOptionsLocal::SetPercentIdentity(double p)
     m_HitSaveOpts->percent_identity = p;
 }
 
+inline double
+CBlastOptionsLocal::GetQueryCovHspPerc() const
+{
+    return m_HitSaveOpts->query_cov_hsp_perc;
+}
+
+inline void
+CBlastOptionsLocal::SetQueryCovHspPerc(double p)
+{
+    m_HitSaveOpts->query_cov_hsp_perc = p;
+}
+
 inline int
 CBlastOptionsLocal::GetMinDiagSeparation() const
 {
