@@ -1,4 +1,4 @@
-/*  $Id: blast_rps_options.cpp 383627 2012-12-17 15:57:20Z boratyng $
+/*  $Id: blast_rps_options.cpp 440360 2014-07-11 17:28:33Z madden $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -89,7 +89,7 @@ CBlastRPSOptionsHandle::SetGappedExtensionDefaults()
     SetGapTrigger(BLAST_GAP_TRIGGER_PROT);
     m_Opts->SetGapExtnAlgorithm(eDynProgScoreOnly);
     m_Opts->SetGapTracebackAlgorithm(eDynProgTbck);
-    SetCompositionBasedStats(false);
+    SetCompositionBasedStats(true);
 }
 
 
@@ -116,6 +116,8 @@ CBlastRPSOptionsHandle::SetHitSavingOptionsDefaults()
     SetCutoffScore(0); // will be calculated based on evalue threshold,
     // effective lengths and Karlin-Altschul params in BLAST_Cutoffs_simple
     // and passed to the engine in the params structure
+
+    SetQueryCovHspPerc(0);
 }
 
 void
