@@ -1,4 +1,4 @@
-/* $Id: blast_engine.c 419242 2013-11-22 18:43:29Z rafanovi $
+/* $Id: blast_engine.c 442543 2014-08-04 21:22:48Z mcelhany $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -55,7 +55,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_engine.c 419242 2013-11-22 18:43:29Z rafanovi $";
+    "$Id: blast_engine.c 442543 2014-08-04 21:22:48Z mcelhany $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_engine.h>
@@ -84,8 +84,8 @@ static char const rcsid[] =
 
 NCBI_XBLAST_EXPORT const int   kBlastMajorVersion = 2;
 NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 2;
-NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 29;
-NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "December-16-2013";
+NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 30;
+NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "August-04-2014";
 
 /** Structure to be passed to s_BlastSearchEngineCore, containing pointers 
     to various preallocated structures and arrays. */
@@ -1117,26 +1117,101 @@ s_RPSPreliminarySearchEngine(EBlastProgramType program_number,
     return status;
 }
 
-static void 
+static void
+s_AdjustSubjectForTranslatedSraSearch(BlastHSPList* hsp_list, Uint1 offset, Int4 length)
+{
+	int i = 0;
+	BlastHSP ** hsp_array = hsp_list->hsp_array;
+	for(i=0; i < hsp_list->hspcnt; i++) {
+		BlastHSP * hsp = hsp_array[i];
+		Boolean fix_co = FALSE;
+
+		if(hsp->subject.frame > 0) {
+			switch (offset) {
+			case 1:
+				if(hsp->subject.frame == 1) {
+					hsp->subject.frame = 3;
+					fix_co = TRUE;
+				}
+				else if(hsp->subject.frame == 2) {
+					hsp->subject.frame = 1;
+				}
+				else if (hsp->subject.frame == 3) {
+					hsp->subject.frame = 2;
+				}
+			break;
+			case 2:
+				if(hsp->subject.frame == 1) {
+					hsp->subject.frame = 2;
+					fix_co = TRUE;
+				}
+				else if(hsp->subject.frame == 2) {
+					hsp->subject.frame = 3;
+					fix_co = TRUE;
+				}
+				else if (hsp->subject.frame == 3) {
+					hsp->subject.frame = 1;
+				}
+			break;
+			case 3:
+					fix_co = TRUE;
+			break;
+			default:
+			break;
+			}
+
+			if(fix_co) {
+				if(hsp->subject.offset == 0)
+					hsp->query.offset +=1;
+
+				if(hsp->subject.offset > 0)
+					hsp->subject.offset -=1;
+
+				hsp->subject.end -=1;
+				hsp->subject.gapped_start -=1;
+			}
+		}
+		else {
+			Int4 max_end = length - offset;
+			Int4 subj_end = (hsp->subject.end + 1) *CODON_LENGTH -hsp->subject.frame -2;
+			if(subj_end > max_end)
+			{
+				hsp->subject.end -=1;
+				hsp->query.end -=1;
+			}
+		}
+	}
+}
+
+static void
 s_AdjustSubjectForSraSearch(BlastHSPList* hsp_list, Uint1 offset )
 {
 	int i = 0;
 	BlastHSP ** hsp_array = hsp_list->hsp_array;
+
 	for(i=0; i < hsp_list->hspcnt; i++)
 	{
 		BlastHSP * hsp = hsp_array[i];
+
 		if(hsp->subject.offset <= offset)
 		{
+			unsigned int q_offset = offset - hsp->subject.offset;
 			hsp->subject.offset = 0;
-			hsp->query.offset += offset;
+			hsp->query.offset += q_offset;
 
 			if(hsp->subject.gapped_start <= offset)
 			{
 				hsp->subject.gapped_start =  0;
-				hsp->query.gapped_start = hsp->query.offset;
 			}
 			else
+			{
 				hsp->subject.gapped_start -= offset;
+			}
+
+			if(hsp->query.gapped_start < hsp->query.offset)
+			{
+				hsp->query.gapped_start += q_offset;
+			}
 		}
 		else
 		{
@@ -1151,6 +1226,7 @@ s_AdjustSubjectForSraSearch(BlastHSPList* hsp_list, Uint1 offset )
 	}
 }
 
+
 Int4 
 BLAST_PreliminarySearchEngine(EBlastProgramType program_number, 
     BLAST_SequenceBlk* query, BlastQueryInfo* query_info,
@@ -1288,6 +1364,13 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
       if (hsp_list && hsp_list->hspcnt > 0) {
          int query_index=0; /* Used to loop over queries below. */
          if (!gapped_calculation) {
+        	 if(seq_arg.seq->bases_offset > 0)
+        	 {
+        		 if (Blast_SubjectIsTranslated(program_number))
+        		 	 s_AdjustSubjectForTranslatedSraSearch(hsp_list, seq_arg.seq->bases_offset, seq_arg.seq->length);
+        		 else
+        			 s_AdjustSubjectForSraSearch(hsp_list, seq_arg.seq->bases_offset);
+        	 }
             /* The following must be performed for any ungapped 
                search with a nucleotide database. */
                status = 
@@ -1336,15 +1419,19 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
         	   status = s_Blast_HSPListReapByPrelimEvalue(hsp_list, hit_params);
                }
  
+               Blast_HSPListReapByQueryCoverage(hsp_list,hit_params->options, query_info, program_number);
             /* Calculate and fill the bit scores, since there will be no
                traceback stage where this can be done. */
             Blast_HSPListGetBitScores(hsp_list, gapped_calculation, sbp);
          } 
          
          // This should only happen for sra searches
-         if(seq_arg.seq->bases_offset > 0)
+         if((seq_arg.seq->bases_offset > 0) && (gapped_calculation))
          {
-        	s_AdjustSubjectForSraSearch(hsp_list, seq_arg.seq->bases_offset);
+        	 if (Blast_SubjectIsTranslated(program_number))
+        		 s_AdjustSubjectForTranslatedSraSearch(hsp_list, seq_arg.seq->bases_offset, seq_arg.seq->length);
+        	 else
+        		 s_AdjustSubjectForSraSearch(hsp_list, seq_arg.seq->bases_offset);
          }
 
          /* Save the results. */
