@@ -1,4 +1,4 @@
-/* $Id: blast_traceback.c 419604 2013-11-26 22:49:06Z rafanovi $
+/* $Id: blast_traceback.c 433477 2014-04-25 13:43:51Z boratyng $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_traceback.c 419604 2013-11-26 22:49:06Z rafanovi $";
+    "$Id: blast_traceback.c 433477 2014-04-25 13:43:51Z boratyng $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_traceback.h>
@@ -821,6 +821,30 @@ EBlastEncoding Blast_TracebackGetEncoding(EBlastProgramType program_number)
     }
 }
 
+/** Delete hsps below query coverage percentage
+ * @param results All results after traceback [in] [out]
+ * @param hit_options hits options [in]
+ * @param query_info  query info [in]
+ * @param program_number  blast program number [in]
+ */
+static void
+s_BlastResultsReapByQueryCoverage(BlastHSPResults* results, const BlastHitSavingOptions* hit_options,
+        const BlastQueryInfo* query_info, EBlastProgramType program_number)
+{
+   Int4 query_index, subject_index;
+   BlastHitList* hit_list;
+
+   for (query_index = 0; query_index < results->num_queries; ++query_index) {
+      if (!(hit_list = results->hitlist_array[query_index]))
+         continue;
+      for (subject_index = 0;
+           subject_index < hit_list->hsplist_count; ++subject_index) {
+    	  	  BlastHSPList * hsp_list = hit_list->hsplist_array[subject_index];
+    	  	  Blast_HSPListReapByQueryCoverage(hsp_list, hit_options, query_info, program_number);
+      }
+   }
+}
+
 
 /** Delete extra subject sequences hits, if after-traceback hit list size is
  * smaller than preliminary hit list size.
@@ -1076,6 +1100,15 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
          sbp->kbp_gap[index] = Blast_KarlinBlkNew();
          Blast_KarlinBlkCopy(sbp->kbp_gap[index], sbp->kbp_gap[valid_kb_index]);
       }
+      if (sbp->kbp_gap_psi[index] == NULL) {
+         sbp->kbp_gap_psi[index] = Blast_KarlinBlkNew();
+         Blast_KarlinBlkCopy(sbp->kbp_gap_psi[index],
+                             sbp->kbp_gap_psi[valid_kb_index]);
+      }
+      if (sbp->kbp_psi[index] == NULL) {
+         sbp->kbp_psi[index] = Blast_KarlinBlkNew();
+         Blast_KarlinBlkCopy(sbp->kbp_psi[index], sbp->kbp_psi[valid_kb_index]);
+      }
    }
 
    /* for CBS the scaling of PSSM is ignored.  So we scale the gap parameters back */
@@ -1375,7 +1408,6 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
     BlastScoringParameters* score_params = NULL;
     const BlastExtensionParameters* ext_params = NULL;
     const BlastHitSavingParameters* hit_params = NULL;
-    const BlastEffectiveLengthsParameters* eff_len_params = NULL;
     BlastGapAlignStruct *gap_align = NULL;
     const BlastSeqSrc* seq_src = NULL;
     Int4 default_db_genetic_code = db_options->genetic_code;
@@ -1390,7 +1422,6 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
     score_params = thread_data->tld[0]->score_params;
     ext_params = thread_data->tld[0]->ext_params;
     hit_params = thread_data->tld[0]->hit_params;
-    eff_len_params = thread_data->tld[0]->eff_len_params;
 
     s_SThreadLocalDataArraySetGapXDropoffFinal(thread_data);
 
@@ -1670,6 +1701,10 @@ BLAST_ComputeTraceback_MT(EBlastProgramType program_number,
     if (BlastSeqSrcGetTotLen(seq_src) > 0)
         Blast_HSPResultsSortByEvalue(results);
 
+    if(hit_params->options->query_cov_hsp_perc > 0) {
+    	s_BlastResultsReapByQueryCoverage(results, hit_params->options,
+            						  	  query_info, program_number);
+    }
     /* Eliminate extra hits from results, if preliminary hit list size is
        larger than the final hit list size */
     s_BlastPruneExtraHits(results, hit_params->options->hitlist_size);
