@@ -1,4 +1,4 @@
-/*  $Id: redoalignment_unit_test.cpp 416467 2013-10-28 19:09:08Z rafanovi $ 
+/*  $Id: redoalignment_unit_test.cpp 439568 2014-07-01 16:13:36Z madden $ 
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -64,6 +64,9 @@
 #include <algo/blast/blastinput/blast_input.hpp>
 #include <algo/blast/blastinput/blast_fasta_input.hpp>
 
+#include <algo/blast/composition_adjustment/composition_constants.h>
+#include <algo/blast/composition_adjustment/matrix_frequency_data.h>
+
 #include "test_objmgr.hpp"
 #include "blast_test_util.hpp"
 
@@ -950,6 +953,12 @@ BOOST_AUTO_TEST_CASE(testRedoAlignmentUseXdropEvalue) {
 // the PSSM and K parameter, which we cannot do unless we support reading
 // checkpoint files. QA should be added later 
 
+
+BOOST_AUTO_TEST_CASE(CheckLowerCaseMatrix)
+{
+      BOOST_REQUIRE(Blast_FrequencyDataIsAvailable("blosum62") == 1);
+}
+
 BOOST_AUTO_TEST_SUITE_END()
 
 /*
