@@ -1,4 +1,4 @@
-/*  $Id: blast_formatter.cpp 416241 2013-10-24 19:22:03Z rafanovi $
+/*  $Id: blast_formatter.cpp 439436 2014-06-30 13:51:29Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_formatter.cpp 416241 2013-10-24 19:22:03Z rafanovi $";
+    "$Id: blast_formatter.cpp 439436 2014-06-30 13:51:29Z fongah2 $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -315,6 +315,11 @@ int CBlastFormatterApp::PrintFormattedOutput(void)
                            fmt_args.GetCustomOutputFormatSpec(),
                            kTask == "megablast",
                            opts.GetMBIndexLoaded());
+    formatter.SetLineLength(fmt_args.GetLineLength());
+    if((fmt_args.GetFormattedOutputChoice() ==  CFormattingArgs::eXml2 ||
+        fmt_args.GetFormattedOutputChoice() ==  CFormattingArgs::eJson)
+        && args[kArgOutput].AsString() != "-")
+           	formatter.SetBaseFile(args[kArgOutput].AsString());
     CRef<CSearchResultSet> results = m_RmtBlast->GetResultSet();
     formatter.PrintProlog();
     bool isPsiBlast = ("psiblast" == kTask);
