@@ -1,4 +1,4 @@
-/*  $Id: blastx_app.cpp 417381 2013-11-06 16:06:11Z rafanovi $
+/*  $Id: blastx_app.cpp 438209 2014-06-13 14:26:12Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-	"$Id: blastx_app.cpp 417381 2013-11-06 16:06:11Z rafanovi $";
+	"$Id: blastx_app.cpp 438209 2014-06-13 14:26:12Z fongah2 $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -147,6 +147,10 @@ int CBlastxApp::Run(void)
                                fmt_args->GetCustomOutputFormatSpec());
         
         formatter.SetQueryRange(query_opts->GetRange());
+        if((fmt_args->GetFormattedOutputChoice() ==  CFormattingArgs::eXml2 ||
+            fmt_args->GetFormattedOutputChoice() ==  CFormattingArgs::eJson)
+                   && args[kArgOutput].AsString() != "-")
+                	formatter.SetBaseFile(args[kArgOutput].AsString());
         formatter.PrintProlog();
 
         /*** Process the input ***/
