@@ -1,4 +1,4 @@
-/*  $Id: rpsblast_app.cpp 391262 2013-03-06 17:58:48Z rafanovi $
+/*  $Id: rpsblast_app.cpp 439436 2014-06-30 13:51:29Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-	"$Id: rpsblast_app.cpp 391262 2013-03-06 17:58:48Z rafanovi $";
+	"$Id: rpsblast_app.cpp 439436 2014-06-30 13:51:29Z fongah2 $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -127,6 +127,8 @@ int CRPSBlastApp::Run(void)
                                      query_opts->GetParseDeflines(),
                                      query_opts->GetRange());
         iconfig.SetQueryLocalIdMode();
+        if(IsIStreamEmpty(m_CmdLineArgs->GetInputStream()))
+           	NCBI_THROW(CArgException, eNoValue, "Query is Empty!");
         CBlastFastaInputSource fasta(m_CmdLineArgs->GetInputStream(), iconfig);
         CBlastInput input(&fasta, m_CmdLineArgs->GetQueryBatchSize());
 
@@ -149,6 +151,12 @@ int CRPSBlastApp::Run(void)
                                db_adapter->GetFilteringAlgorithm(),
                                fmt_args->GetCustomOutputFormatSpec());
 
+        formatter.SetQueryRange(query_opts->GetRange());
+        formatter.SetLineLength(fmt_args->GetLineLength());
+        if((fmt_args->GetFormattedOutputChoice() ==  CFormattingArgs::eXml2 ||
+            fmt_args->GetFormattedOutputChoice() ==  CFormattingArgs::eJson)
+           && args[kArgOutput].AsString() != "-")
+        	formatter.SetBaseFile(args[kArgOutput].AsString());
         formatter.PrintProlog();
 
         /*** Process the input ***/
