@@ -1,4 +1,4 @@
-/*  $Id: tblastx_app.cpp 417381 2013-11-06 16:06:11Z rafanovi $
+/*  $Id: tblastx_app.cpp 439436 2014-06-30 13:51:29Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,7 +33,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-	"$Id: tblastx_app.cpp 417381 2013-11-06 16:06:11Z rafanovi $";
+	"$Id: tblastx_app.cpp 439436 2014-06-30 13:51:29Z fongah2 $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -147,6 +147,11 @@ int CTblastxApp::Run(void)
                                fmt_args->GetCustomOutputFormatSpec());
         
         formatter.SetQueryRange(query_opts->GetRange());
+        formatter.SetLineLength(fmt_args->GetLineLength());
+        if((fmt_args->GetFormattedOutputChoice() ==  CFormattingArgs::eXml2 ||
+           fmt_args->GetFormattedOutputChoice() ==  CFormattingArgs::eJson)
+           && args[kArgOutput].AsString() != "-")
+        	formatter.SetBaseFile(args[kArgOutput].AsString());
         formatter.PrintProlog();
 
         /*** Process the input ***/
