@@ -1,4 +1,4 @@
-/*  $Id: blastdbcheck.cpp 389294 2013-02-14 18:43:48Z rafanovi $
+/*  $Id: blastdbcheck.cpp 448483 2014-10-07 12:11:08Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -42,6 +42,7 @@
 #include <corelib/ncbi_mask.hpp>
 
 #include <algo/blast/blastinput/blast_input.hpp>
+#include <algo/blast/api/rps_aux.hpp>
 #include <objtools/align_format/align_format_util.hpp>
 #include "../blast/blast_app_util.hpp"
 
@@ -230,6 +231,9 @@ void CBlastDbCheckApplication::Init(void)
         ("must_have_taxids", 
          "Require that all sequences in the database have taxid set.");
 
+    arg_desc->AddFlag
+        ("cdd_delta", "Do aditional tests for a CDD database for DELTA-BLAST");
+
     // Setup arg.descriptions for this application
     SetupArgDescriptions(arg_desc.release());
 }
@@ -496,7 +500,6 @@ bool CTestAction::TestOID(CSeqDB & db, TSeen & seen, int oid)
         if (bs.Empty()) {
             throw runtime_error("no bioseq");
         }
-        cerr << MSerial_AsnText << *bs << endl;
 
         // If requested, make sure there's a taxID set for this Bioseq
         if (((m_Flags & e_TaxIDSet) != 0) && (bs->GetTaxId() == 0)) {
@@ -815,7 +818,7 @@ private:
             CFile f(CDirEntry::MakePath(dir, tokens[i]));
 
             // File naming check 
-            if (tokens[0] == "OIDLIST" && f.GetExt() != "msk") {
+            if (tokens[0] == "OIDLIST" && f.GetExt() != ".msk") {
                 Log(name, e_Details) << "  [WARNING] oidlist file " << tokens[i]
                                << " does not have .msk extension" << endl;
             }
@@ -1157,6 +1160,196 @@ bool CDirTest::Test(CTestActionList & action)
     return (tot_faults == 0);
 }
 
+// Test comparing CDD header files to check that the numbers of records match.
+class CCddHeadersTest : public CTestAction
+{
+public:
+
+    CCddHeadersTest(CBlastDbCheckLog& log, int flags)
+        : CTestAction(log, "CDD_Headers", flags), m_NumFailures(0) {}
+
+    virtual int DoTest(CSeqDB& db, TSeen& /* seen */);
+
+protected:
+    bool x_GetHeader(CSeqDB& db);
+    bool x_TestFreqRatios(CSeqDB& db);
+
+protected:
+    vector<Int4> m_Offsets;
+    Int4 m_NumProfiles;
+    Int4 m_MagicNumber;
+    int m_NumFailures;
+};
+
+int CCddHeadersTest::DoTest(CSeqDB& db, TSeen& seen)
+{
+    if (x_GetHeader(db)) {
+        x_TestFreqRatios(db);
+    }
+
+    return m_NumFailures;
+}
+
+bool CCddHeadersTest::x_GetHeader(CSeqDB& db)
+{
+    try {
+        const string& db_name = db.GetDBNameList();
+        CBlastRPSInfo rps_info(db_name, CBlastRPSInfo::fRpsBlast);
+
+        // copy header information
+        BlastRPSProfileHeader* header = rps_info()->profile_header;
+        m_NumProfiles = header->num_profiles;
+        m_MagicNumber = header->magic_number;
+        m_Offsets.reserve(m_NumProfiles);
+        for (int i=0;i < m_NumProfiles;i++) {
+            m_Offsets.push_back(header->start_offsets[i]);
+        }
+    }
+    catch (exception& e) {
+        Log(db, e_Brief) << "  [ERROR] caught exception while reading rps header "
+                           << endl;
+        Log(db, e_Details) << e.what() << endl;
+        m_NumFailures++;
+    }
+
+    return (m_NumFailures == 0);
+}
+
+bool CCddHeadersTest::x_TestFreqRatios(CSeqDB& db)
+{
+    try {
+        const string& db_name = db.GetDBNameList();
+        CBlastRPSInfo rps_info(db_name, CBlastRPSInfo::fFreqRatiosFile);
+
+        BlastRPSProfileHeader* header = (BlastRPSProfileHeader*)rps_info()->freq_ratios_header;
+        if (header->magic_number != m_MagicNumber) {
+            Log(db, e_Brief) << "  [ERROR] Bad magic number of the "
+                " .freq file" << endl;
+            m_NumFailures++;
+        }
+
+        if (header->num_profiles != m_NumProfiles) {
+            Log(db, e_Brief) << "  [ERROR] Wrong number of profiles in the "
+                " .freq file: " << header->num_profiles << endl;
+            m_NumFailures++;
+        }
+
+        for (int i=0;i < m_NumProfiles;i++) {
+            if (header->start_offsets[i] != m_Offsets[i]) {
+                Log(db, e_Brief) << "  [ERROR] Header offsets differ between .rps "
+                    "and .freq files" << endl;
+                m_NumFailures++;
+            }
+        }
+    }
+    catch (exception& e) {
+        Log(db, e_Brief) << "  [ERROR] caught exception while reading "
+            ".freq freqs header " << endl;
+        Log(db, e_Details) << e.what() << endl;
+        m_NumFailures++;
+    }
+
+    return (m_NumFailures == 0);
+}
+
+
+class CCddDeltaHeadersTest : public CCddHeadersTest
+{
+public:
+    CCddDeltaHeadersTest(CBlastDbCheckLog& log, int flags)
+        : CCddHeadersTest(log, flags) {}
+
+    virtual int DoTest(CSeqDB& db, TSeen& /* seen */);
+
+private:
+    bool x_TestWeightedCounts(CSeqDB& db);
+    bool x_TestObservations(CSeqDB& db);
+};
+
+
+int CCddDeltaHeadersTest::DoTest(CSeqDB& db, TSeen& seen)
+{
+    if (x_GetHeader(db)) {
+        x_TestFreqRatios(db);
+        x_TestWeightedCounts(db);
+        x_TestObservations(db);
+    }
+
+    return m_NumFailures;
+}
+
+
+bool CCddDeltaHeadersTest::x_TestWeightedCounts(CSeqDB& db)
+{
+    try {
+        const string& db_name = db.GetDBNameList();
+        CBlastRPSInfo rps_info(db_name, CBlastRPSInfo::fFrequenciesFile);
+
+        BlastRPSProfileHeader* header = rps_info()->freq_header;
+        if (header->magic_number != m_MagicNumber) {
+            Log(db, e_Brief) << "  [ERROR] Bad magic number of the "
+                " .wcounts file" << endl;
+            m_NumFailures++;
+        }
+
+        if (header->num_profiles != m_NumProfiles) {
+            Log(db, e_Brief) << "  [ERROR] Wrong number of profiles in the "
+                " .wcounts file: " << header->num_profiles << endl;
+            m_NumFailures++;
+        }
+
+        for (int i=0;i < m_NumProfiles;i++) {
+            if (header->start_offsets[i] != m_Offsets[i]) {
+                Log(db, e_Brief) << "  [ERROR] Header offsets differ between .rps "
+                    "and .wcounts files" << endl;
+                m_NumFailures++;
+            }
+        }
+    }
+    catch (exception& e) {
+        Log(db, e_Brief) << "  [ERROR] caught exception while reading "
+            ".wcounts freqs header " << endl;
+        Log(db, e_Details) << e.what() << endl;
+        m_NumFailures++;
+    }
+
+    return (m_NumFailures == 0);
+}
+
+bool CCddDeltaHeadersTest::x_TestObservations(CSeqDB& db)
+{
+    try {
+        const string& db_name = db.GetDBNameList();
+        CBlastRPSInfo rps_info(db_name, CBlastRPSInfo::fObservationsFile);
+
+        BlastRPSProfileHeader* header = rps_info()->obsr_header;
+        if (header->magic_number != m_MagicNumber) {
+            Log(db, e_Brief) << "  [ERROR] Bad magic number of the "
+                " .obsr file" << endl;
+            m_NumFailures++;
+        }
+
+        if (header->num_profiles != m_NumProfiles) {
+            Log(db, e_Brief) << "  [ERROR] Wrong number of profiles in the "
+                " .obsr file: " << header->num_profiles << endl;
+            m_NumFailures++;
+        }
+
+        // The observations file is compressed and the offsets will be
+        // different from other files, hence we only compare the number
+        // of rectords.
+
+    }
+    catch (exception& e) {
+        Log(db, e_Brief) << "  [ERROR] caught exception while reading "
+            ".obsr freqs header " << endl;
+        Log(db, e_Details) << e.what() << endl;
+        m_NumFailures++;
+    }
+
+    return (m_NumFailures == 0);
+}
+
 
 /////////////////////////////////////////////////////////////////////////////
 //  Run test (printout arguments obtained from command-line)
@@ -1278,6 +1471,12 @@ int CBlastDbCheckApplication::Run(void)
                 << "By default, testing " << random_sample << " randomly sampled OIDs." << endl;
             tests->Add(new CSampleTest(output, random_sample, flags));
         }
+
+        if (args["cdd_delta"]) {
+            output.Log(e_Summary)
+                << "Testing CDD files' headers for DELTA-BLAST" << endl;
+            tests->Add(new CCddDeltaHeadersTest(output, flags));
+        }
         
         //if (fork1) {
         //    output.Log(e_Summary)
