@@ -1,4 +1,4 @@
-/*  $Id: makeprofiledb.cpp 397706 2013-04-30 13:37:09Z fongah2 $
+/*  $Id: makeprofiledb.cpp 448400 2014-10-06 14:05:40Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -61,6 +61,7 @@ static char const rcsid[] = "";
 #include <algo/blast/core/ncbi_math.h>
 #include <objtools/blast/seqdb_writer/writedb.hpp>
 #include <objtools/blast/seqdb_writer/taxid_set.hpp>
+#include "../blast/blast_app_util.hpp"
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 USING_NCBI_SCOPE;
@@ -243,6 +244,8 @@ private:
     void x_UpdateFreqRatios(const CPssmWithParameters & pssm_p, Int4 seq_index, Int4 seq_size);
     bool x_IsUpdateFreqRatios(const CPssm & p);
 
+    int x_Run(void);
+
     // Data
     CNcbiOstream * m_LogFile;
     CNcbiIstream * m_InPssmList;
@@ -684,10 +687,7 @@ static bool s_DeleteMakeprofileDb(const string & name )
 void CMakeProfileDBApp::x_InitRPSDbInfo(Int4 num_files)
 {
 
-	 s_CreateDirectories(m_OutDbName);
-	 if ( s_DeleteMakeprofileDb(m_OutDbName)) {
-		 *m_LogFile << "Deleted existing BLAST database with identical name." << endl;
-	 }
+
      m_RpsDbInfo.num_seqs = num_files;
 
      string rps_str = m_OutDbName + ".rps";
@@ -1270,8 +1270,30 @@ void CMakeProfileDBApp::Init(void)
 	x_InitProgramParameters();
 }
 
-int CMakeProfileDBApp::Run(void)
+static bool s_HasDefline(const CBioseq & bio)
 {
+    if (bio.CanGetDescr()) {
+        return true;
+    }
+
+    return false;
+}
+
+static CRef<CBlast_def_line_set> s_GenerateBlastDefline(const CBioseq & bio)
+{
+	 CRef<CBlast_def_line_set> defline_set(new CBlast_def_line_set());
+	 CRef<CBlast_def_line> defline(new CBlast_def_line());
+	 defline->SetSeqid() = bio.GetId();
+	 defline_set->Set().push_back(defline);
+	 return defline_set;
+}
+
+int CMakeProfileDBApp::x_Run(void)
+{
+	 s_CreateDirectories(m_OutDbName);
+	 if ( s_DeleteMakeprofileDb(m_OutDbName)) {
+		 *m_LogFile << "Deleted existing BLAST database with identical name." << endl;
+	 }
 	vector<string> smpFilenames = (op_delta == m_op_mode )? x_CreateDeltaList():x_GetSMPFilenames();
 
 	x_InitRPSDbInfo((Int4) smpFilenames.size());
@@ -1312,7 +1334,14 @@ int CMakeProfileDBApp::Run(void)
 		int seq_size = pssm.GetQueryLength();
 
 		const CBioseq & bioseq = pssm.GetQuery().GetSeq();
-		CRef<CBlast_def_line_set> deflines = CWriteDB::ExtractBioseqDeflines(bioseq);
+		CRef<CBlast_def_line_set> deflines;
+		if(s_HasDefline(bioseq)) {
+			deflines = CWriteDB::ExtractBioseqDeflines(bioseq);
+		}
+		else {
+			deflines = s_GenerateBlastDefline(bioseq);
+		}
+
 		m_Taxids->FixTaxId(deflines);
 		m_OutputDb->AddSequence(bioseq);
 		m_OutputDb->SetDeflines(*deflines);
@@ -1655,6 +1684,32 @@ void CMakeProfileDBApp::x_WrapUpDelta(CTmpFile & tmp_obsr_file, CTmpFile & tmp_f
 }
 
 
+int CMakeProfileDBApp::Run(void)
+{
+	int status = 0;
+	try { x_Run(); }
+	catch(const blast::CInputException& e)  {
+		 LOG_POST(Error << "INPUT ERROR: " << e.GetMsg());
+		 status = BLAST_INPUT_ERROR;
+	}
+	catch (const CSeqDBException& e) {
+	     LOG_POST(Error << "ERROR: " << e.GetMsg());
+	     status = BLAST_DATABASE_ERROR;
+	}
+	catch (const blast::CBlastException& e) {
+		 LOG_POST(Error << "ERROR: " << e.GetMsg());
+		 status = BLAST_INPUT_ERROR;
+	}
+	catch (const CException& e) {
+	    LOG_POST(Error << "ERROR: " << e.GetMsg());
+	    status = BLAST_UNKNOWN_ERROR;
+	}
+	catch (...) {
+	    LOG_POST(Error << "Error: Unknown exception");
+	    status = BLAST_UNKNOWN_ERROR;
+	}
+	return status;
+}
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 int main(int argc, const char* argv[] /*, const char* envp[]*/)
