@@ -1,4 +1,4 @@
-/* $Id: msvc_prj_utils.cpp 408439 2013-07-30 16:41:12Z gouriano $
+/* $Id: msvc_prj_utils.cpp 430659 2014-03-27 18:13:30Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -575,7 +575,11 @@ string CMsvc7RegSettings::sm_MsvcPlatformName = "ppc";
 
 #elif defined(NCBI_COMPILER_MSVC)
 
-#if _MSC_VER >= 1700
+#if _MSC_VER >= 1800
+CMsvc7RegSettings::EMsvcVersion CMsvc7RegSettings::sm_MsvcVersion =
+CMsvc7RegSettings::eMsvc1200;
+string CMsvc7RegSettings::sm_MsvcVersionName = "1200";
+#elif _MSC_VER >= 1700
 CMsvc7RegSettings::EMsvcVersion CMsvc7RegSettings::sm_MsvcVersion =
     CMsvc7RegSettings::eMsvc1100;
 string CMsvc7RegSettings::sm_MsvcVersionName = "1100";
@@ -678,6 +682,10 @@ void CMsvc7RegSettings::IdentifyPlatform()
             sm_MsvcVersion = eMsvc1100;
             sm_MsvcVersionName = "1100";
             break;
+        case 1200:
+            sm_MsvcVersion = eMsvc1200;
+            sm_MsvcVersionName = "1200";
+            break;
         default:
             NCBI_THROW(CProjBulderAppException, eBuildConfiguration, "Unsupported IDE version");
             break;
@@ -737,7 +745,8 @@ string CMsvc7RegSettings::GetProjectFileFormatVersion(void)
     } else if (GetMsvcVersion() == eMsvc900) {
         return "9.00";
     } else if (GetMsvcVersion() == eMsvc1000 ||
-               GetMsvcVersion() == eMsvc1100) {
+               GetMsvcVersion() == eMsvc1100 ||
+               GetMsvcVersion() == eMsvc1200) {
         return "10.0.30319.1";
     }
     return "";
@@ -754,6 +763,8 @@ string CMsvc7RegSettings::GetSolutionFileFormatVersion(void)
         return "11.00\n# Visual Studio 2010";
     } else if (GetMsvcVersion() == eMsvc1100) {
         return "12.00\n# Visual Studio 2012";
+    } else if (GetMsvcVersion() == eMsvc1200) {
+        return "12.00\n# Visual Studio 2013";
     }
     return "";
 }
@@ -792,6 +803,10 @@ string CMsvc7RegSettings::GetTopBuilddir(void)
     string top( GetApp().GetConfig().GetString(section, "TopBuilddir", ""));
     if (!top.empty()) {
         top = CDirEntry::ConcatPath(CDirEntry(GetApp().m_Solution).GetDir(), top);
+        if (!CFile(top).Exists()) {
+            top.clear();
+        }
+
     }
     return top;
 }
@@ -1613,4 +1628,70 @@ CProjKey CDllSrcFilesDistr::GetFileLib(const string&   file_path,
 }
 
 
+const CDataToolGeneratedSrc*
+IsProducedByDatatool(const string&    src_path_abs,
+                     const CProjItem& project)
+{
+    if ( project.m_DatatoolSources.empty() ) {
+        ITERATE( list<CProjKey>, d, project.m_Depends) {
+            if (d->Type() == CProjKey::eDataSpec) {
+                CProjectItemsTree::TProjects::const_iterator n = 
+                               GetApp().GetWholeTree().m_Projects.find(*d);
+                if (n != GetApp().GetWholeTree().m_Projects.end()) {
+                    const CDataToolGeneratedSrc *pFound =
+                        IsProducedByDatatool(src_path_abs, n->second);
+                    if (pFound) {
+                        return pFound;
+                    }
+                }
+            }
+        }
+        return NULL;
+    }
+
+    string src_base;
+    CDirEntry::SplitPath(src_path_abs, NULL, &src_base);
+
+    // guess name.asn file name from name__ or name___
+    string asn_base;
+    if ( NStr::EndsWith(src_base, "___") ) {
+        asn_base = src_base.substr(0, src_base.length() -3);
+    } else if ( NStr::EndsWith(src_base, "__") ) {
+        asn_base = src_base.substr(0, src_base.length() -2);
+    } else {
+        return NULL;
+    }
+    string asn_name = asn_base + ".asn";
+    string dtd_name = asn_base + ".dtd";
+    string xsd_name = asn_base + ".xsd";
+    string wsdl_name = asn_base + ".wsdl";
+
+    //try to find this name in datatool generated sources container
+    ITERATE(list<CDataToolGeneratedSrc>, p, project.m_DatatoolSources) {
+        const CDataToolGeneratedSrc& asn = *p;
+        if ((asn.m_SourceFile == asn_name) ||
+            (asn.m_SourceFile == dtd_name) ||
+            (asn.m_SourceFile == xsd_name) ||
+            (asn.m_SourceFile == wsdl_name)) {
+            return &(*p);
+        }
+    }
+
+    try {
+        ITERATE(list<CDataToolGeneratedSrc>, p, project.m_DatatoolSources) {
+            const CDataToolGeneratedSrc& asn = *p;
+            string def( CDirEntry::ConcatPath(asn.m_SourceBaseDir, CDirEntry(asn.m_SourceFile).GetBase()) + ".def");
+            if (CFile(def).Exists()) {
+                CNcbiIfstream in(def.c_str(), IOS_BASE::in | IOS_BASE::binary);
+                CNcbiRegistry reg(in);
+                if (reg.GetString("-", "-oc", "") == asn_base) {
+                    return &(*p);
+                } 
+            }
+        }
+    } catch (...) {
+    }
+    return NULL;
+}
+
 END_NCBI_SCOPE
