@@ -1,4 +1,4 @@
-/* $Id: ncbi_crypt.c 400602 2013-05-22 16:31:50Z lavr $
+/* $Id: ncbi_crypt.c 424437 2014-01-15 22:44:09Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,6 +37,7 @@
 #include "../ncbi_ansi_ext.h"
 #include "../ncbi_priv.h"
 #include <connect/ext/ncbi_crypt.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -143,7 +144,7 @@ void CRYPT_Free(CRYPT_Key key)
         return;
     }
     if (key->magic != CRYPT_MAGIC) {
-        CORE_LOG_X(1, eLOG_Warning, "[CRYPT_Free]  Magic corrupted");
+        CORE_LOG_X(1, eLOG_Critical, "[CRYPT_Free]  Magic corrupted");
     }
     free(key);
 }
@@ -263,7 +264,7 @@ extern char* CRYPT_EncodeString(CRYPT_Key key, const char* str)
         return 0;
     }
     if (key->magic != CRYPT_MAGIC) {
-        CORE_LOG_X(3, eLOG_Error, "[CRYPT_Encode]  Bad key magic");
+        CORE_LOG_X(3, eLOG_Critical, "[CRYPT_Encode]  Bad key magic");
         return 0;
     }
     if (!str) {
@@ -345,7 +346,7 @@ extern char* CRYPT_DecodeString(CRYPT_Key key, const char* str)
         return 0;
     }
     if (key->magic != CRYPT_MAGIC) {
-        CORE_LOG_X(5, eLOG_Error, "[CRYPT_Decode]  Bad key magic");
+        CORE_LOG_X(5, eLOG_Critical, "[CRYPT_Decode]  Bad key magic");
         return 0;
     }
     if ((len = str ? strlen(str) : 0) < 3) {
@@ -354,8 +355,8 @@ extern char* CRYPT_DecodeString(CRYPT_Key key, const char* str)
 
     version = *str++ - '0';
     if (version & ~1) {
-        CORE_LOGF_X(6, eLOG_Error,
-                    ("[CRYPT_Decode]  Unknown crypt version `%u'", version));
+        CORE_TRACEF(("[CRYPT_Decode]  Unknown crypt version `%u'", version));
+        errno = EINVAL;
         return 0;
     }
     assert(0 <= version  &&  (size_t) version < SizeOf(s_Coders));
