@@ -1,4 +1,4 @@
-/* $Id: ncbi_lbsm.c 404445 2013-06-24 19:03:36Z lavr $
+/* $Id: ncbi_lbsm.c 440972 2014-07-18 16:07:40Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -178,26 +178,27 @@ const SLBSM_Service* LBSM_LookupService(HEAP                 heap,
 }
 
 
+/* NB: "hint" may point to a freed and merged entry */
 const SLBSM_Host* LBSM_LookupHost(HEAP               heap,
                                   unsigned int       addr,
                                   const SLBSM_Entry* hint)
 {
-    const SLBSM_Entry* end = hint;
-    const SLBSM_Entry* e   = hint;
-    for (;;) {
-        while ((e = (const SLBSM_Entry*) HEAP_Next(heap, &e->head)) != hint) {
-            if (!e)
+    int/*bool*/ wrap = hint ? 1/*true*/ : 0/*false*/;
+    const SLBSM_Entry* e = hint;
+    while ((e = (const SLBSM_Entry*) HEAP_Next(heap, &e->head)) != hint) {
+        if (!e) {
+            assert(hint);
+            if (!wrap)
                 break;
-            if (e->type == eLBSM_Host) {
-                const SLBSM_Host* h = (const SLBSM_Host*) e;
-                assert(h->addr);
-                if (!addr  ||  h->addr == addr)
-                    return h;
-            }
+            wrap = 0/*false*/;
+            continue;
+        }
+        if (e->type == eLBSM_Host) {
+            const SLBSM_Host* h = (const SLBSM_Host*) e;
+            assert(h->addr);
+            if (!addr  ||  h->addr == addr)
+                return h;
         }
-        if (e == end/*NB: == hint OR == 0*/)
-            break;
-        end = 0;
     }
     return 0;
 }
@@ -217,7 +218,7 @@ ELBSM_Type LBSM_PublishService(HEAP heap, const SLBSM_Service* s, int log)
     name = (const char*) s + s->name;
     type = s->info.quorum ? eLBSM_PendingOrService : eLBSM_Service;
     while ((t = s_Lookup(heap, name, 0, type, t)) != 0) {
-        char host1[40], host2[40];
+        char addr1[40], addr2[40];
         int/*bool*/ equal;
 
         if (s->info.type != t->info.type  ||
@@ -252,16 +253,16 @@ ELBSM_Type LBSM_PublishService(HEAP heap, const SLBSM_Service* s, int log)
             }
             break/*even definitions (by possibly 2 different hosts)*/;
         }
-        if (log  &&  SOCK_ntoa(s->addr, host1, sizeof(host1)) != 0)
-            strcpy(host1, "(unknown)");
-        if (log  &&  SOCK_ntoa(t->addr, host2, sizeof(host2)) != 0)
-            strcpy(host2, "(unknown)");
+        if (log  &&  SOCK_ntoa(s->addr, addr1, sizeof(addr1)) != 0)
+            sprintf(addr1, "%08X", s->addr);
+        if (log  &&  SOCK_ntoa(t->addr, addr2, sizeof(addr2)) != 0)
+            sprintf(addr2, "%08X", t->addr);
         if (equal) {
             /*even definitions but with different dynamic properties*/
             if (log) {
                 CORE_LOGF_X(2, eLOG_Warning,
                             ("Service `%s' defined by"
-                             " both %s and %s", name, host1, host2));
+                             " both %s and %s", name, addr1, addr2));
             }
             break;
         }
@@ -271,7 +272,7 @@ ELBSM_Type LBSM_PublishService(HEAP heap, const SLBSM_Service* s, int log)
             if (log) {
                 CORE_LOGF_X(3, eLOG_Warning,
                             ("Rejected attempt from %s to unevenly replace"
-                             " service `%s' from %s", host1, name, host2));
+                             " service `%s' from %s", addr1, name, addr2));
             }
             return type/*fake success*/;
         }
@@ -279,7 +280,7 @@ ELBSM_Type LBSM_PublishService(HEAP heap, const SLBSM_Service* s, int log)
         if (log) {
             CORE_LOGF_X(4, eLOG_Error,
                         ("Service `%s' announced unevenly by"
-                         " both %s and %s", name, host1, host2));
+                         " both %s and %s", name, addr1, addr2));
         }
         break;
     }
@@ -301,29 +302,30 @@ ELBSM_Type LBSM_PublishService(HEAP heap, const SLBSM_Service* s, int log)
 }
 
 
-TNCBI_Time LBSM_UnpublishHost(HEAP heap, unsigned int addr)
+TNCBI_Time LBSM_UnpublishHost(HEAP heap, unsigned int addr, unsigned int items)
 {
     SLBSM_Entry* p, *e = 0;
     TNCBI_Time joined = 0;
 
-    /* Delete host and [almost] all service entries announced by this host */
+    /* Delete a host and [almost] all service entries announced by that host */
     for (p = 0;  (e = (SLBSM_Entry*) HEAP_Walk(heap, &e->head)) != 0;  p = e) {
         if (!(e->head.flag & 1))
             continue/*free block*/;
         if (e->type == eLBSM_Host) {
             SLBSM_Host* h = (SLBSM_Host*) e;
-            TNCBI_Time jt;
             assert(h->addr);
             if (h->addr != addr)
-                continue;
-            jt = h->sys.data.joined ? h->sys.data.joined : NCBI_TIME_INFINITE;
-            if (joined > jt  ||  !joined)
-                joined = jt;
+                continue/*not this host*/;
+            assert(!joined);
+            joined
+                = h->sys.data.joined ? h->sys.data.joined : NCBI_TIME_INFINITE;
+            --items;
         } else if (e->type == eLBSM_Service  ||  e->type == eLBSM_Pending) {
             SLBSM_Service* s = (SLBSM_Service*) e;
             assert(s->info.host);
             if (s->addr != addr)
                 continue/*not by this host*/;
+            --items;
             if (s->entry.type == eLBSM_Service
                 &&  (s->info.rate < 0.0  ||  s->info.quorum)) {
                 continue/*keep static / active backup entry*/;
@@ -331,6 +333,8 @@ TNCBI_Time LBSM_UnpublishHost(HEAP heap, unsigned int addr)
         } else
             continue;
         HEAP_FreeFast(heap, &e->head, &p->head);
+        if (!items)
+            break;
         if (p  &&  !(p->head.flag & 1))
             e = p;
     }
@@ -346,9 +350,18 @@ int/*bool*/ LBSM_PublishHost(HEAP heap, const SLBSM_Host* h)
     if (!h  ||  h->entry.type != eLBSM_Host  ||  !h->addr)
         return 0/*failure*/;
 
-    assert(!LBSM_LookupHost(heap, h->addr, 0));
+#if defined(_DEBUG)  &&  !defined(NDEBUG)
+    /* This used to be an assert(!LBSM_LookupHost(heap, h->addr, 0)); */
+    b = (SLBSM_Host*) LBSM_LookupHost(heap, h->addr, 0);
+    if (b) {
+        char addr[40];
+        if (SOCK_ntoa(h->addr, addr, sizeof(addr)) != 0)
+            sprintf(addr, "%08X", h->addr);
+        CORE_LOGF(eLOG_Fatal, ("Host %s found at %p", addr, b));
+    }
+#endif /*_DEBUG && !NDEBUG*/
 
-    /* Then, allocate new host entry */
+    /* Then, allocate a new host entry */
     size  = sizeof(*h) + (h->env ? strlen((const char*) h + h->env) + 1 : 0);
     size -= sizeof(h->entry.head);
     if (!(b = (SLBSM_Host*) HEAP_Alloc(heap, size, 1)))
@@ -388,17 +401,17 @@ size_t LBSM_Expire(HEAP heap, TNCBI_Time time, size_t count)
             assert(((const SLBSM_Host*) e)->addr);
             if (count) {
                 const SLBSM_Host* h = (const SLBSM_Host*) e;
-                char host[64];
+                char addr[40];
                 char buf[32];
-                if (SOCK_ntoa(h->addr, host, sizeof(host)) != 0)
-                    strcpy(host, "(unknown)");
-                if (count != (size_t)(-1)) {
+                if (SOCK_ntoa(h->addr, addr, sizeof(addr)) != 0)
+                    sprintf(addr, "%08X", h->addr);
+                if (count != (size_t)(-1L)) {
                     sprintf(buf, " (%lu)", (unsigned long) --count);
                     if (!count)
-                        count = (size_t)(-1);
+                        count = (size_t)(-1L);
                 } else
                     *buf = '\0';
-                CORE_LOGF_X(5, eLOG_Warning, ("Host %s expired%s", host, buf));
+                CORE_LOGF_X(5, eLOG_Warning, ("Host %s expired%s", addr, buf));
             }
             n++;
         }
@@ -428,7 +441,7 @@ static int s_BackupWatchByName(HEAP heap, const char* name, int/*bool*/ log)
         if (s->info.quorum) {
             const SLBSM_Service* t = 0;
             if (s->entry.type == eLBSM_Service) {
-                s->entry.type =  eLBSM_Pending;
+                s->entry.type  = eLBSM_Pending;
                 backed_up = 1;
             }
             if (!s->entry.good) {
@@ -462,11 +475,11 @@ static int s_BackupWatchByName(HEAP heap, const char* name, int/*bool*/ log)
             if (s->info.rate > 0.0) {
                 const SLBSM_Host* h = LBSM_LookupHost(heap, s->info.host, 0);
                 if (h) {
-                    host_status += (s->info.flag == fSERV_Blast
+                    host_status += (s->info.flag & eSERV_Blast
                                     ? h->sys.load.statusBLAST
                                     : h->sys.load.status);
                 } else
-                    host_status += 1.0/LBSM_DEFAULT_RATE;
+                    host_status += 1.0 / LBSM_DEFAULT_RATE;
                 serv_status += s->info.rate;
             }
             n_active++;
@@ -501,15 +514,15 @@ static int s_BackupWatchByName(HEAP heap, const char* name, int/*bool*/ log)
                 memcpy(s->info.reserved + sizeof(s->info.reserved)
                        - sizeof(rate), &rate, sizeof(rate));
                 if (h) {
-                    rate = (s->info.flag == fSERV_Blast
+                    rate = (s->info.flag & eSERV_Blast
                             ? h->sys.load.statusBLAST
                             : h->sys.load.status);
                 } else
-                    rate = 1.0/LBSM_DEFAULT_RATE;
+                    rate = 1.0 / LBSM_DEFAULT_RATE;
                 s->info.rate = -rate * serv_status / host_status;
             }
         } else {
-            assert(s->entry.type == eLBSM_Pending && s->info.quorum);
+            assert(s->entry.type == eLBSM_Pending  &&  s->info.quorum);
             if (!s->info.reserved[sizeof(s->info.reserved) - 1]
                 ||  n_active >= quorum) {
                 continue;
@@ -565,10 +578,10 @@ double LBSM_CalculateStatus(double rate, double fine, ESERV_Flag flag,
     if (rate < LBSM_STANDBY_THRESHOLD)
         status = rate < 0.0 ? -LBSM_DEFAULT_RATE : LBSM_DEFAULT_RATE;
     else
-        status = flag == fSERV_Blast ? load->statusBLAST : load->status;
-    status *= rate/LBSM_DEFAULT_RATE;
+        status = flag & eSERV_Blast ? load->statusBLAST : load->status;
+    status *= rate / LBSM_DEFAULT_RATE;
     /* accurately apply fine: avoid any imperfections with 100% */
-    status *= (100.0 - (fine < 0.0 ? 0.0 : fine > 100.0 ? 100.0 : fine))/100.0;
+    status *= (100. - (fine < 0. ? 0. : fine > 100. ? 100. : fine)) / 100.0;
     return fabs(status); /*paranoid but safe*/
 }
 
@@ -608,14 +621,22 @@ int/*bool*/ LBSM_SubmitPenaltyOrRerate(const char*    name,
     }
     if (fine)
         NCBI_simple_ftoa(value, min(max(0.0, rate), 100.0), 0);
-    else if (rate <= -LBSM_RERATE_DEFAULT  ||  LBSM_RERATE_DEFAULT <= rate)
+    else if (LBSM_RERATE_RESERVE <= rate)
+        strcpy(value, "RESERVE");
+    else if  (rate <= LBSM_RERATE_DEFAULT)
         strcpy(value, "DEFAULT");
-    else { /*NB: avoid -0.0*/
-        NCBI_simple_ftoa(value, !rate ? 0.0 :
+    else {
+        if (fabs(rate) < SERV_MINIMAL_RATE / 2.0)
+            rate = 0.0;
+        else if (rate > 0.0)
+            rate = max(rate,  SERV_MINIMAL_RATE);
+        else if (rate < 0.0)
+            rate = min(rate, -SERV_MINIMAL_RATE);
+        NCBI_simple_ftoa(value, !rate ? 0.0/*NB: avoid -0.0*/ :
                          min(max(-SERV_MAXIMAL_RATE, rate), SERV_MAXIMAL_RATE),
                          3/*decimal places*/);
     }
-    retval = 0; /*assume worst: failure*/
+    retval = 0;  /*assume worst, a failure*/
     len = sprintf(msg, "%u %s %s%s %s %s\n", (unsigned int) geteuid(),
                   name, fine ? "" : "RERATE ", type_name, addr, value);
     if (len > 0) {
