@@ -1,4 +1,4 @@
-/* $Id: ncbi_lbsm_ipc.c 408190 2013-07-26 18:12:57Z lavr $
+/* $Id: ncbi_lbsm_ipc.c 425983 2014-02-05 03:52:02Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -347,7 +347,7 @@ static HEAP s_Shmem_Attach(int which)
 #endif /*LBSM_DEBUG*/
     if ((shmid = shmget(k_ShmemKey[which], 0, 0)) >= 0  &&
         (shmid == s_Shmid[which]  ||
-         ((shmem = shmat(shmid, 0, SHM_RDONLY))  &&  shmem != (void*)(-1)))) {
+         ((shmem = shmat(shmid, 0, SHM_RDONLY))  &&  shmem != (void*)(-1L)))) {
         if (shmid != s_Shmid[which]) {
             struct shmid_ds shm_ds;
 #ifdef LBSM_DEBUG
@@ -388,11 +388,10 @@ HEAP LBSM_Shmem_Attach(int/*bool*/ fallback)
 #endif /*LBSM_DEBUG*/
     if (!(heap = s_Shmem_Attach(which))) {
         int error = errno;
-        int/*bool*/ attached = s_Shmem[which] != 0 ? 1/*true*/ : 0/*false*/;
         s_Shmem_RUnlock(which);
         CORE_LOGF_ERRNO_X(11, eLOG_Error, error,
-                          ("LBSM shmem[%d] %s", which + 1,
-                           attached ? "access failed" : "failed to attach"));
+                          ("LBSM shmem[%d] %s", which + 1, s_Shmem[which]
+                           ? "access failed" : "failed to attach"));
     }
 #ifdef LBSM_DEBUG
     else {
@@ -553,7 +552,7 @@ unsigned int LBSM_Shmem_Update(HEAP heap, int/*bool*/ wait)
             s_Shmem_Destroy(i, s_ShmemSize[i] ? 0 : getpid());
             if ((shmid = shmget(k_ShmemKey[i], heapsize,
                                 LBSM_SHM_PROT | IPC_CREAT | IPC_EXCL)) < 0
-                ||  !(shmem = shmat(shmid, 0, 0))  ||  shmem == (void*)(-1)) {
+                ||  !(shmem = shmat(shmid, 0, 0))  ||  shmem == (void*)(-1L)) {
                 CORE_LOGF_ERRNO_X(22, eLOG_Error, errno,
                                   ("LBSM shmem[%d] unable to re-create",
                                    i + 1));
@@ -623,7 +622,7 @@ pid_t LBSM_UnLBSMD(int/*bool*/ undaemon)
 
     if (s_Muxid >= 0) {
         if (undaemon <= 0) {
-            if (!undaemon) {
+            if (undaemon == 0) {
                 if ((which = s_Shmem_RLock(1)) >= 0) {
                     int shmid = shmget(k_ShmemKey[which], 0, 0);
                     struct shmid_ds shm_ds;
