@@ -1,4 +1,4 @@
-/* $Id: ncbi_lbsmd.c 408178 2013-07-26 18:07:30Z lavr $
+/* $Id: ncbi_lbsmd.c 440975 2014-07-18 16:15:14Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -105,6 +105,15 @@ static unsigned int s_Localhost(const SLBSM_Version* v)
 }
 
 
+#ifdef __GNUC__
+inline
+#endif /*__GNUC__*/
+static const char* s_HostEnv(const SLBSM_Host* h)
+{
+    return h->env ? (const char*) h + h->env : 0;
+}
+
+
 static unsigned int s_GetLocalHostAddress(HEAP heap)
 {
     unsigned int localhost;
@@ -586,7 +595,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
         if (iter->external  &&  svc->info.locl)
             continue; /* external mapping requested; local/private server */
 
-        if (svc->info.locl & 0xF0) {
+        if (svc->info.locl & 0x10) {
             /* private server */
             if (svc->info.host  &&
                 svc->info.host != s_GetLocalHostAddress(heap)) {
@@ -672,8 +681,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
             assert(iter->arg);
             if (!host)
                 host = s_LookupHost(heap, iter, svc);
-            if ( host  &&  host->env)
-                env = (const char*) host + host->env;
+            env = s_HostEnv(host);
             match = s_Match(env,
                             iter->arg, iter->arglen,
                             iter->val, iter->vallen, &a, &v);
@@ -691,7 +699,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
 
         if (svc->info.rate) {
             /* NB: server is _not_ down, but it may have been suppressed */
-            if (fabs(svc->info.rate) < 0.01) {
+            if (fabs(svc->info.rate) < LBSM_STANDBY_THRESHOLD) {
                 if (!standby) {
                     if (!iter->ok_suppressed)
                         continue;
@@ -835,10 +843,10 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
                     }
                     if (iter->external  &&  svc->info.locl)
                         continue;/* external mapping requested; local server */
-                    assert(!(svc->info.locl & 0xF0)); /* no private DNS */
+                    assert(!(svc->info.locl & 0x10)); /* no private DNS */
                     status = LBSM_CalculateStatus(!svc->info.rate ? 0.0
                                                   : -LBSM_DEFAULT_RATE,
-                                                  svc->fine, fSERV_Regular,
+                                                  svc->fine, eSERV_Regular,
                                                   NULL);
                     if (status > 0.0)
                         break;
@@ -873,9 +881,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
                 if ((host = cand[n].host) != 0) {
                     *host_info =
                         HINFO_Create(host->addr, &host->sys, sizeof(host->sys),
-                                     host->env
-                                     ? (const char*) host + host->env
-                                     : 0, cand[n].arg, cand[n].val);
+                                     s_HostEnv(host), cand[n].arg,cand[n].val);
                 } else
                     *host_info = 0;
             }
@@ -987,8 +993,8 @@ const SSERV_VTable* SERV_LBSMD_Open(SERV_ITER    iter,
 {
     static int s_Init = 0;
     SSERV_Info* tmp;
-    /* Daemon is running if LBSM_LBSMD returns 1: mutex exists but read
-       operation failed with errno == EAGAIN (the mutex is busy) */
+    /* Daemon is running if LBSM_LBSMD returns 1:  mutex exists but
+       read operation failed with errno == EAGAIN (the mutex is busy) */
     if (LBSM_LBSMD(0) <= 0  ||  errno != EAGAIN)
         return 0;
     if (!s_Init) {
@@ -1002,16 +1008,16 @@ const SSERV_VTable* SERV_LBSMD_Open(SERV_ITER    iter,
         srand(g_NCBI_ConnectRandomSeed);
     }
     if (!no_dispd_follows)
-        iter->data =  iter;
+        iter->data  = iter;
     tmp = s_GetNextInfo(iter, host_info);
     if (iter->data == iter)
-        iter->data =  0;
-    if (!tmp) {
+        iter->data  = 0;
+    if (!no_dispd_follows  &&  !tmp) {
         s_Close(iter);
         return 0;
     }
     if (info)
-        *info = tmp;
+        *info = tmp ? tmp : (SSERV_Info*)(-1L);
     else if (tmp)
         free(tmp);
     return &s_op;
@@ -1065,6 +1071,7 @@ const char* LBSMD_GetHostParameter(unsigned int addr,
 {
     size_t namelen = name  &&  *name ? strlen(name) : 0;
     const SLBSM_Host* host;
+    const char* env;
     HEAP heap;
 
     if (!namelen
@@ -1074,25 +1081,25 @@ const char* LBSMD_GetHostParameter(unsigned int addr,
     }
     if (addr == SERV_ANYHOST  ||  addr == SERV_LOCALHOST)
         addr  = s_GetLocalHostAddress(heap);
-    if ((host = LBSM_LookupHost(heap, addr, 0)) != 0  &&  host->env) {
-        const char* e = (char*) host + host->env;
-        const char* env;
-        for (env = e;  *env;  env = e) {
+    if ((host = LBSM_LookupHost(heap, addr, 0)) != 0
+        &&  (env = s_HostEnv(host)) != 0) {
+        const char* e;
+        for (e = env;  *e;  e = env) {
             const char* p;
-            size_t    len;
-            if (!(e = strchr(env, '\n'))) {
-                len = strlen(env);
-                e   = env + len;
+            size_t len;
+            if (!(env = strchr(e, '\n'))) {
+                len = strlen(e);
+                env = e + len;
             } else
-                len = (size_t)(e++ - env);
-            if (!(p = (const char*) memchr(env, '=', len)))
+                len = (size_t)(env++ - e);
+            if (!(p = (const char*) memchr(e, '=', len)))
                 continue;
-            len = (size_t)(p - env);
+            len = (size_t)(p - e);
             if (len != namelen)
                 continue;
-            if (strncasecmp(env, name, namelen) != 0)
+            if (strncasecmp(e, name, namelen) != 0)
                 continue;
-            len = (size_t)(e - ++p);
+            len = (size_t)(env - ++p);
             return strndup(p, len);
         }
     }
