@@ -1,4 +1,4 @@
-/* $Id: ncbi_pipe.cpp 401314 2013-05-29 18:10:52Z lavr $
+/* $Id: ncbi_pipe.cpp 431369 2014-04-03 12:52:44Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -63,14 +63,19 @@
 
 #define NCBI_USE_ERRCODE_X   Connect_Pipe
 
+
 #define IS_SET(flags, mask)  (((flags) & (mask)) == (mask))
 
 
 BEGIN_NCBI_SCOPE
 
 
-// Predefined timeout (in milliseconds)
-const unsigned long kWaitPrecision = 100;
+
+#if defined(NCBI_OS_MSWIN)
+    // Predefined timeout (in milliseconds)
+    const unsigned long kWaitPrecision = 100;
+#endif
+
 
 
 //////////////////////////////////////////////////////////////////////////////
@@ -339,17 +344,16 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
             stderr_handle  = INVALID_HANDLE_VALUE;
         }
         
-        // Flush std.output buffers before remap
+        // Flush stdio buffers before remap
         NcbiCout.flush();
-        ::fflush(stdout);
         if (stdout_handle != INVALID_HANDLE_VALUE) {
             ::FlushFileBuffers(stdout_handle);
         }
         NcbiCerr.flush();
-        ::fflush(stderr);
         if (stderr_handle != INVALID_HANDLE_VALUE) {
             ::FlushFileBuffers(stderr_handle);
         }
+        ::fflush(NULL);
 
         // Set base security attributes
         SECURITY_ATTRIBUTES attr;
@@ -1078,11 +1082,10 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
 
     int status_pipe[2] = {-1, -1};
     try {
-        // Flush std.output
+        // Flush stdio
         NcbiCout.flush();
-        ::fflush(stdout);
         NcbiCerr.flush();
-        ::fflush(stderr);
+        ::fflush(NULL);
 
         // Create pipe for child's stdin
         _ASSERT(CPipe::fStdIn_Close);
@@ -1156,6 +1159,7 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
                     ::close(pipe_in[0]);
                 }
                 ::close(pipe_in[1]);
+                ::fflush(stdin);
             } else {
                 (void) ::freopen("/dev/null", "r", stdin);
             }
@@ -1257,7 +1261,7 @@ EIO_Status CPipeHandle::Open(const string&         cmd,
         status_pipe[0] = -1;
 
         if (n > 0) {
-            // Child could not run -- reap it and exit with error
+            // Child could not run -- reap it and exit with an error
             ::waitpid(m_Pid, NULL, 0);
             string errmsg("Failed to execute \"" + cmd + '"');
             if ((size_t) n >= sizeof(errcode)  &&  errcode) {
