@@ -1,4 +1,4 @@
-/*  $Id: json_over_uttp.cpp 403271 2013-06-13 16:32:42Z kazimird $
+/*  $Id: json_over_uttp.cpp 449265 2014-10-14 21:00:35Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -732,14 +732,14 @@ void CJsonNode::SetByKey(const string& key, CJsonNode::TInstance value)
     SJsonObjectNodeImpl* impl(m_Impl->GetObjectNodeImpl("SetByKey()"));
 
     pair<TJsonObjectElements::iterator, bool> insertion =
-            impl->m_Elements.insert(SJsonObjectElement(key, value));
+            impl->m_Elements.insert(SJsonObjectElement(key, NULL));
 
     SJsonObjectElement* element =
             &const_cast<SJsonObjectElement&>(*insertion.first);
 
-    if (!insertion.second)
-        element->m_Node = value;
-    else {
+    element->m_Node = value;
+
+    if (insertion.second) {
         element->m_Order = impl->m_NextElementOrder++;
         impl->m_ElementOrder.insert(element);
     }
@@ -747,7 +747,7 @@ void CJsonNode::SetByKey(const string& key, CJsonNode::TInstance value)
 
 void CJsonNode::DeleteByKey(const string& key)
 {
-    SJsonObjectNodeImpl* impl(m_Impl->GetObjectNodeImpl("SetByKey()"));
+    SJsonObjectNodeImpl* impl(m_Impl->GetObjectNodeImpl("DeleteByKey()"));
 
     TJsonObjectElements::iterator it =
             impl->m_Elements.find(SJsonObjectElement(key, NULL));
@@ -1170,8 +1170,8 @@ bool CJsonOverUTTPReader::x_AddNewNode(CJsonNode::TInstance new_node)
         m_CurrentNode.SetByKey(m_HashKey, new_node);
     } else
         // The current node is either a JSON object or an array,
-        // because if it was a non-container node, this method
-        // would not be called.
+        // because if it was a non-container node then this method
+        // wouldn't have been called.
         if (m_CurrentNode.IsArray())
             m_CurrentNode.Append(new_node);
         else {
