@@ -1,4 +1,4 @@
-/*  $Id: netcache_api.cpp 420507 2013-12-03 19:01:36Z kazimird $
+/*  $Id: netcache_api.cpp 447943 2014-10-01 17:52:07Z vasilche $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -87,6 +87,63 @@ CRef<INetServerProperties> CNetCacheServerListener::AllocServerProperties()
     return CRef<INetServerProperties>(new SNetCacheServerProperties);
 }
 
+CConfig* CNetCacheServerListener::LoadConfigFromAltSource(CObject* api_impl,
+        string* new_section_name)
+{
+    SNetCacheAPIImpl* nc_impl = static_cast<SNetCacheAPIImpl*>(api_impl);
+
+    auto_ptr<CConfig::TParamTree> result;
+
+    if (nc_impl->m_NetScheduleAPI) {
+        CNetScheduleAPI::TQueueParams queue_params;
+
+        nc_impl->m_NetScheduleAPI.GetQueueParams(
+                nc_impl->m_NetScheduleAPI.GetQueueName(), queue_params);
+
+        ITERATE(CNetScheduleAPI::TQueueParams, param_it, queue_params) {
+            if (NStr::StartsWith(param_it->first, "nc.")) {
+                string param_name(param_it->first);
+                param_name.erase(0, sizeof("nc.") - 1);
+                if (result.get() == NULL) {
+                    result.reset(new CConfig::TParamTree);
+                    *new_section_name = "netcache_conf_from_netschedule";
+                }
+                result->AddNode(CConfig::TParamValue(param_name, param_it->second));
+            }
+        }
+
+        if (result.get() == NULL)
+            try {
+                nc_impl->m_NetScheduleAPI.GetQueueParams(queue_params);
+
+                ITERATE(CNetScheduleAPI::TQueueParams, param_it, queue_params) {
+                    if (NStr::StartsWith(param_it->first, "nc::")) {
+                        string param_name(param_it->first);
+                        param_name.erase(0, sizeof("nc::") - 1);
+                        if (result.get() == NULL) {
+                            result.reset(new CConfig::TParamTree);
+                            *new_section_name =
+                                    "netcache_conf_from_netschedule_GETP2";
+                        }
+                        result->AddNode(CConfig::TParamValue(param_name,
+                                param_it->second));
+                    }
+                }
+            }
+            catch (CNetScheduleException&) {
+            }
+    }
+
+    if (result.get() == NULL)
+        return NULL;
+
+    CConfig* config = new CConfig(result.get());
+
+    result.release();
+
+    return config;
+}
+
 void CNetCacheServerListener::OnInit(CObject* api_impl,
     CConfig* config, const string& config_section)
 {
@@ -117,35 +174,17 @@ void CNetCacheServerListener::OnInit(CObject* api_impl,
         nc_impl->m_CacheOutput = config->GetBool(config_section,
             "cache_output", CConfig::eErr_NoThrow, false);
 
-        string enable_mirroring(config->GetString(config_section,
-                "enable_mirroring", CConfig::eErr_NoThrow, kEmptyStr));
+        nc_impl->m_DefaultParameters.SetMirroringMode(
+            config->GetString(config_section, "enable_mirroring",
+                CConfig::eErr_NoThrow, kEmptyStr));
 
-        if (!enable_mirroring.empty()) {
-            NStr::ReplaceInPlace(enable_mirroring, "_", kEmptyStr);
+        nc_impl->m_DefaultParameters.SetServerCheck(
+            config->GetString(config_section,
+                "server_check", CConfig::eErr_NoThrow, kEmptyStr));
 
-            nc_impl->m_DefaultParameters.SetMirroringMode(
-                NStr::CompareNocase(enable_mirroring, "ifkeymirrored") == 0 ||
-                NStr::CompareNocase(enable_mirroring, "onread") == 0 ?
-                    CNetCacheAPI::eIfKeyMirrored :
-                        NStr::StringToBool(enable_mirroring) ?
-                            CNetCacheAPI::eMirroringEnabled :
-                            CNetCacheAPI::eMirroringDisabled);
-        }
-
-        string server_check(config->GetString(config_section,
-            "server_check", CConfig::eErr_NoThrow, kEmptyStr));
-
-        if (!server_check.empty())
-            nc_impl->m_DefaultParameters.SetServerCheck(
-                NStr::CompareNocase(server_check, "auto") == 0 ?
-                    eDefault : NStr::StringToBool(server_check) ? eOn : eOff);
-
-        server_check = config->GetString(config_section,
-            "server_check_hint", CConfig::eErr_NoThrow, kEmptyStr);
-
-        if (!server_check.empty())
-            nc_impl->m_DefaultParameters.SetServerCheckHint(
-                    NStr::StringToBool(server_check));
+        nc_impl->m_DefaultParameters.SetServerCheckHint(
+            config->GetString(config_section,
+                "server_check_hint", CConfig::eErr_NoThrow, kEmptyStr));
 
         if (config->GetBool(config_section,
                 "use_compound_id", CConfig::eErr_NoThrow, false))
@@ -157,21 +196,19 @@ void CNetCacheServerListener::OnInit(CObject* api_impl,
     }
 }
 
-void CNetCacheServerListener::OnConnected(
-        CNetServerConnection::TInstance conn_impl)
+void CNetCacheServerListener::OnConnected(CNetServerConnection& connection)
 {
     CRef<SNetCacheServerProperties> server_props(
-            x_GetServerProperties(conn_impl->m_Server));
+            x_GetServerProperties(connection->m_Server));
 
     CFastMutexGuard guard(server_props->m_Mutex);
 
     if (server_props->mirroring_checked) {
         guard.Release();
-        conn_impl->WriteLine(m_Auth);
+        connection->WriteLine(m_Auth);
     } else {
-        CNetServerConnection conn(conn_impl);
         try {
-            string version_info(conn.Exec(m_Auth + "\r\nVERSION"));
+            string version_info(connection.Exec(m_Auth + "\r\nVERSION", false));
 
             server_props->mirroring_checked = true;
 
@@ -187,8 +224,7 @@ void CNetCacheServerListener::OnConnected(
     }
 }
 
-void CNetCacheServerListener::OnError(
-    const string& err_msg, SNetServerImpl* server)
+void CNetCacheServerListener::OnError(const string& err_msg, CNetServer& server)
 {
     string message = server->m_ServerInPool->m_Address.AsString();
 
@@ -197,8 +233,13 @@ void CNetCacheServerListener::OnError(
 
     static const char s_BlobNotFoundMsg[] = "BLOB not found";
     if (NStr::strncmp(err_msg.c_str(), s_BlobNotFoundMsg,
-        sizeof(s_BlobNotFoundMsg) - 1) == 0)
-        NCBI_THROW(CNetCacheException, eBlobNotFound, message);
+            sizeof(s_BlobNotFoundMsg) - 1) == 0) {
+        if (strstr(err_msg.c_str(), "AGE=") != NULL) {
+            NCBI_THROW(CNetCacheBlobTooOldException, eBlobTooOld, message);
+        } else {
+            NCBI_THROW(CNetCacheException, eBlobNotFound, message);
+        }
+    }
 
     static const char s_AccessDenied[] = "Access denied";
     if (NStr::strncmp(err_msg.c_str(), s_AccessDenied,
@@ -214,7 +255,7 @@ void CNetCacheServerListener::OnError(
 }
 
 void CNetCacheServerListener::OnWarning(const string& warn_msg,
-        SNetServerImpl* server)
+        CNetServer& server)
 {
     if (m_EventHandler)
         m_EventHandler->OnWarning(warn_msg, server);
@@ -243,9 +284,11 @@ static const char* const s_NetCacheConfigSections[] = {
 };
 
 SNetCacheAPIImpl::SNetCacheAPIImpl(CConfig* config, const string& section,
-        const string& service, const string& client_name) :
+        const string& service, const string& client_name,
+        CNetScheduleAPI::TInstance ns_api) :
     m_Service(new SNetServiceImpl("NetCacheAPI", client_name,
         new CNetCacheServerListener)),
+    m_NetScheduleAPI(ns_api),
     m_DefaultParameters(eVoid)
 {
     m_Service->Init(this, service, config, section, s_NetCacheConfigSections);
@@ -258,6 +301,7 @@ SNetCacheAPIImpl::SNetCacheAPIImpl(SNetServerInPool* server,
     m_TempDir(parent->m_TempDir),
     m_CacheInput(parent->m_CacheInput),
     m_CacheOutput(parent->m_CacheOutput),
+    m_NetScheduleAPI(parent->m_NetScheduleAPI),
     m_DefaultParameters(parent->m_DefaultParameters)
 {
 }
@@ -265,43 +309,41 @@ SNetCacheAPIImpl::SNetCacheAPIImpl(SNetServerInPool* server,
 void SNetCacheAPIImpl::AppendClientIPSessionID(string* cmd)
 {
     CRequestContext& req = CDiagContext::GetRequestContext();
-    cmd->append(" \"");
+    cmd->append(" ip=\"");
     cmd->append(req.GetClientIP());
-    cmd->append("\" \"");
+    cmd->append("\" sid=\"");
     cmd->append(req.GetSessionID());
-    cmd->append("\"");
+    cmd->append(1, '\"');
 }
 
-void SNetCacheAPIImpl::AppendClientIPSessionIDPassword(string* cmd,
+void SNetCacheAPIImpl::AppendClientIPSessionIDPasswordAgeHitID(string* cmd,
         const CNetCacheAPIParameters* parameters)
 {
     AppendClientIPSessionID(cmd);
 
     string password(parameters->GetPassword());
-    if (!password.empty())
-        cmd->append(password);
-}
+    if (!password.empty()) {
+        cmd->append(" pass=\"");
+        cmd->append(NStr::PrintableString(password));
+        cmd->append(1, '\"');
+    }
 
-string SNetCacheAPIImpl::MakeCmd(const char* cmd)
-{
-    string result(cmd);
-    AppendClientIPSessionID(&result);
-    return result;
-}
+    unsigned max_age = parameters->GetMaxBlobAge();
+    if (max_age > 0) {
+        cmd->append(" age=");
+        cmd->append(NStr::NumericToString(max_age));
+    }
 
-string SNetCacheAPIImpl::MakeCmd(const char* cmd,
-        const CNetCacheAPIParameters* parameters)
-{
-    string result(cmd);
-    AppendClientIPSessionIDPassword(&result, parameters);
-    return result;
+    cmd->append(" ncbi_phid=\"");
+    cmd->append(CDiagContext::GetRequestContext().GetHitID());
+    cmd->append(1, '\"');
 }
 
 string SNetCacheAPIImpl::MakeCmd(const char* cmd_base, const CNetCacheKey& key,
         const CNetCacheAPIParameters* parameters)
 {
     string result(cmd_base + key.StripKeyExtensions());
-    AppendClientIPSessionIDPassword(&result, parameters);
+    AppendClientIPSessionIDPasswordAgeHitID(&result, parameters);
     return result;
 }
 
@@ -312,7 +354,8 @@ unsigned SNetCacheAPIImpl::x_ExtractBlobAge(
 
     if (pos == string::npos) {
         NCBI_THROW_FMT(CNetCacheException, eInvalidServerResponse,
-                "No AGE field in " << cmd_name << " output");
+                       "No AGE field in " << cmd_name <<
+                       " output: \"" << exec_result.response << "\"");
     }
 
     return NStr::StringToUInt(exec_result.response.c_str() + pos +
@@ -356,37 +399,79 @@ CNetServer SNetCacheMirrorTraversal::NextServer()
     return *(m_Iterator = m_Service.Iterate(m_PrimaryServer));
 }
 
-CNetService SNetCacheAPIImpl::FindOrCreateService(const CNetCacheKey& key)
-{
-    const string& service_name(key.GetServiceName());
-
-    if (service_name == m_Service.GetServiceName())
-        return m_Service;
-
-    pair<TNetServiceByName::iterator, bool> loc(m_ServicesFromKeys.insert(
-            TNetServiceByName::value_type(service_name, CNetService())));
-
-    if (!loc.second)
-        return loc.first->second;
-
-    CNetService new_service(new SNetServiceImpl(service_name, m_Service));
-
-    loc.first->second = new_service;
-
-    return new_service;
-}
-
 CNetServer::SExecResult SNetCacheAPIImpl::ExecMirrorAware(
     const CNetCacheKey& key, const string& cmd,
+    bool multiline_output,
     const CNetCacheAPIParameters* parameters,
     SNetServiceImpl::EServerErrorHandling error_handling)
 {
-    CNetServer primary_server(GetServer(key));
+
+    const string& service_name(key.GetServiceName());
 
     bool service_name_is_defined = !key.GetServiceName().empty();
 
+    CNetService service(m_Service);
+
+    if (service_name_is_defined && service_name != service.GetServiceName()) {
+        CFastMutexGuard guard(m_ServiceMapMutex);
+
+        pair<TNetServiceByName::iterator, bool> loc(m_ServicesFromKeys.insert(
+                TNetServiceByName::value_type(service_name, CNetService())));
+
+        service = !loc.second ? loc.first->second :
+                (loc.first->second =
+                        new SNetServiceImpl(service_name, service));
+    }
+
     bool key_is_mirrored = service_name_is_defined &&
-            !key.GetFlag(CNetCacheKey::fNCKey_SingleServer);
+            !key.GetFlag(CNetCacheKey::fNCKey_SingleServer) &&
+            parameters->GetMirroringMode() != CNetCacheAPI::eMirroringDisabled;
+
+    if (key.GetVersion() == 3) {
+        /* Version 3 - no server address, only CRC32 of it: */
+        if (service.GetServiceType() != CNetService::eLoadBalancedService) {
+            NCBI_THROW_FMT(CNetSrvConnException, eLBNameNotFound,
+                key.GetKey() << ": NetCache key version 3 "
+                "requires an LBSM service name.");
+        }
+
+        Uint4 crc32 = key.GetHostPortCRC32();
+
+        for (CNetServiceIterator it(service.Iterate()); it; ++it) {
+            CNetServer server(*it);
+
+            if (CNetCacheKey::CalculateChecksum(
+                    CSocketAPI::ntoa(server.GetHost()),
+                            server.GetPort()) == crc32) {
+                // The server with the checksum from the key
+                // has been found in the service.
+
+                // TODO: cache the calculated checksums to resolve
+                // them into host:port immediately.
+
+                if (!key_is_mirrored)
+                    return server.ExecWithRetry(cmd, multiline_output);
+
+                CNetServer::SExecResult exec_result;
+
+                SNetCacheMirrorTraversal mirror_traversal(service,
+                        server, eOff /* turn off server check since
+                                        the server is discovered
+                                        through this service */);
+
+                m_Service->IterateUntilExecOK(cmd, multiline_output,
+                        exec_result, &mirror_traversal, error_handling);
+
+                return exec_result;
+            }
+        }
+
+        NCBI_THROW_FMT(CNetSrvConnException, eServerNotInService,
+                key.GetKey() << ": unable to find a NetCache server "
+                "by the checksum from this key.");
+    }
+
+    CNetServer primary_server(service.GetServer(key.GetHost(), key.GetPort()));
 
     ESwitch server_check = eDefault;
     parameters->GetServerCheck(&server_check);
@@ -394,69 +479,63 @@ CNetServer::SExecResult SNetCacheAPIImpl::ExecMirrorAware(
         server_check = key.GetFlag(CNetCacheKey::fNCKey_NoServerCheck) ?
                 eOff : eOn;
 
-    if (!key_is_mirrored || parameters->GetMirroringMode() ==
-            CNetCacheAPI::eMirroringDisabled) {
-        if (service_name_is_defined && server_check != eOff &&
-                !FindOrCreateService(key)->IsInService(primary_server)) {
-            NCBI_THROW_FMT(CNetSrvConnException, eServerNotInService,
-                    key.GetKey() << ": NetCache server " <<
-                    primary_server.GetServerAddress() << " could not be "
-                    "accessed because it is not registered for the service.");
-        }
-        return primary_server.ExecWithRetry(cmd);
-    } else {
+    if (key_is_mirrored) {
         CNetServer::SExecResult exec_result;
 
-        SNetCacheMirrorTraversal mirror_traversal(FindOrCreateService(key),
+        SNetCacheMirrorTraversal mirror_traversal(service,
                 primary_server, server_check);
 
-        m_Service->IterateUntilExecOK(cmd, exec_result,
+        m_Service->IterateUntilExecOK(cmd, multiline_output, exec_result,
                 &mirror_traversal, error_handling);
 
         return exec_result;
     }
-}
 
-CNetCachePasswordGuard::CNetCachePasswordGuard(CNetCacheAPI::TInstance nc_api,
-    const string& password)
-{
-    if (!nc_api->m_DefaultParameters.GetPassword().empty()) {
-        NCBI_THROW(CNetCacheException, eAuthenticationError,
-            "Cannot reuse a password-protected NetCache API object.");
-    }
-    if (password.empty())
-        m_NetCacheAPI = nc_api;
-    else {
-        m_NetCacheAPI = new SNetCacheAPIImpl(*nc_api);
-        m_NetCacheAPI->m_DefaultParameters.SetPassword(password);
+    if (service_name_is_defined && server_check != eOff &&
+            !service->IsInService(primary_server)) {
+        NCBI_THROW_FMT(CNetSrvConnException, eServerNotInService,
+                key.GetKey() << ": NetCache server " <<
+                primary_server.GetServerAddress() << " could not be "
+                "accessed because it is not registered for the service.");
     }
+
+    return primary_server.ExecWithRetry(cmd, multiline_output);
 }
 
 CNetCacheAPI::CNetCacheAPI(CNetCacheAPI::EAppRegistry /* use_app_reg */,
-        const string& conf_section /* = kEmptyStr */) :
-    m_Impl(new SNetCacheAPIImpl(NULL, conf_section, kEmptyStr, kEmptyStr))
+        const string& conf_section /* = kEmptyStr */,
+        CNetScheduleAPI::TInstance ns_api) :
+    m_Impl(new SNetCacheAPIImpl(NULL, conf_section,
+            kEmptyStr, kEmptyStr, ns_api))
 {
 }
 
-CNetCacheAPI::CNetCacheAPI(const IRegistry& reg, const string& conf_section)
+CNetCacheAPI::CNetCacheAPI(const IRegistry& reg, const string& conf_section,
+        CNetScheduleAPI::TInstance ns_api)
 {
     CConfig conf(reg);
-    m_Impl = new SNetCacheAPIImpl(&conf, conf_section, kEmptyStr, kEmptyStr);
+    m_Impl = new SNetCacheAPIImpl(&conf, conf_section,
+            kEmptyStr, kEmptyStr, ns_api);
 }
 
-CNetCacheAPI::CNetCacheAPI(CConfig* conf, const string& conf_section) :
-    m_Impl(new SNetCacheAPIImpl(conf, conf_section, kEmptyStr, kEmptyStr))
+CNetCacheAPI::CNetCacheAPI(CConfig* conf, const string& conf_section,
+        CNetScheduleAPI::TInstance ns_api) :
+    m_Impl(new SNetCacheAPIImpl(conf, conf_section,
+            kEmptyStr, kEmptyStr, ns_api))
 {
 }
 
-CNetCacheAPI::CNetCacheAPI(const string& client_name) :
-    m_Impl(new SNetCacheAPIImpl(NULL, kEmptyStr, kEmptyStr, client_name))
+CNetCacheAPI::CNetCacheAPI(const string& client_name,
+        CNetScheduleAPI::TInstance ns_api) :
+    m_Impl(new SNetCacheAPIImpl(NULL,
+            kEmptyStr, kEmptyStr, client_name, ns_api))
 {
 }
 
 CNetCacheAPI::CNetCacheAPI(const string& service_name,
-        const string& client_name) :
-    m_Impl(new SNetCacheAPIImpl(NULL, kEmptyStr, service_name, client_name))
+        const string& client_name, CNetScheduleAPI::TInstance ns_api) :
+    m_Impl(new SNetCacheAPIImpl(NULL,
+            kEmptyStr, service_name, client_name, ns_api))
 {
 }
 
@@ -485,16 +564,16 @@ CNetServerConnection SNetCacheAPIImpl::InitiateWriteCmd(
         cmd.append(stripped_blob_id);
     }
 
-    AppendClientIPSessionIDPassword(&cmd, parameters);
+    AppendClientIPSessionIDPasswordAgeHitID(&cmd, parameters);
 
     CNetServer::SExecResult exec_result;
 
     if (write_existing_blob)
-        exec_result = ExecMirrorAware(key, cmd, parameters,
+        exec_result = ExecMirrorAware(key, cmd, false, parameters,
                 SNetServiceImpl::eIgnoreServerErrors);
     else {
         try {
-            exec_result = m_Service.FindServerAndExec(cmd);
+            exec_result = m_Service.FindServerAndExec(cmd, false);
         } catch (CNetSrvConnException& e) {
             SServerAddress* backup = s_GetFallbackServer();
 
@@ -507,7 +586,8 @@ CNetServerConnection SNetCacheAPIImpl::InitiateWriteCmd(
                 m_Service.GetServiceName() << ":" << e.what() <<
                 ". Connecting to backup server " << backup->AsString() << ".");
 
-            exec_result = m_Service.GetServer(*backup).ExecWithRetry(cmd);
+            exec_result =
+                    m_Service.GetServer(*backup).ExecWithRetry(cmd, false);
         }
     }
 
@@ -616,21 +696,16 @@ bool CNetCacheAPI::HasBlob(const string& blob_id,
     parameters.LoadNamedParameters(optional);
 
     try {
-        return m_Impl->ExecMirrorAware(key, m_Impl->MakeCmd("HASB ",
-                key, &parameters), &parameters).response[0] == '1';
+        return m_Impl->ExecMirrorAware(key,
+                m_Impl->MakeCmd("HASB ", key, &parameters),
+                false,
+                &parameters).response[0] == '1';
     }
     catch (CNetCacheException& e) {
         if (e.GetErrCode() == CNetCacheException::eBlobNotFound)
             return false;
         throw;
     }
-    catch (CNetServiceException& e) {
-        if (!TCGI_NetCacheUseHasbFallback::GetDefault() ||
-                e.GetErrCode() != CNetServiceException::eCommunicationError ||
-                e.GetMsg() != "Unknown request")
-            throw;
-        return !GetOwner(blob_id).empty();
-    }
 }
 
 
@@ -643,8 +718,11 @@ size_t CNetCacheAPI::GetBlobSize(const string& blob_id,
 
     parameters.LoadNamedParameters(optional);
 
-    return CheckBlobSize(NStr::StringToUInt8(m_Impl->ExecMirrorAware(key,
-        m_Impl->MakeCmd("GSIZ ", key, &parameters), &parameters).response));
+    return CheckBlobSize(NStr::StringToUInt8(
+            m_Impl->ExecMirrorAware(key,
+                    m_Impl->MakeCmd("GSIZ ", key, &parameters),
+                    false,
+                    &parameters).response));
 }
 
 
@@ -658,11 +736,15 @@ void CNetCacheAPI::Remove(const string& blob_id,
     CNetCacheKey key(blob_id, m_Impl->m_CompoundIDPool);
 
     try {
-        m_Impl->ExecMirrorAware(key, m_Impl->MakeCmd("RMV2 ",
-                key, &parameters), &parameters);
-    } catch (std::exception& e) {
+        m_Impl->ExecMirrorAware(key,
+                m_Impl->MakeCmd("RMV2 ", key, &parameters),
+                false,
+                &parameters);
+    }
+    catch (std::exception& e) {
         ERR_POST("Could not remove blob \"" << blob_id << "\": " << e.what());
-    } catch (...) {
+    }
+    catch (...) {
         ERR_POST("Could not remove blob \"" << blob_id << "\"");
     }
 }
@@ -686,7 +768,7 @@ CNetServerMultilineCmdOutput CNetCacheAPI::GetBlobInfo(const string& blob_id,
     parameters.LoadNamedParameters(optional);
 
     CNetServerMultilineCmdOutput output(m_Impl->ExecMirrorAware(key,
-            cmd, &parameters));
+            cmd, true, &parameters));
 
     output->SetNetCacheCompatMode();
 
@@ -722,28 +804,9 @@ void CNetCacheAPI::ProlongBlobLifetime(const string& blob_key, unsigned ttl,
 
     parameters.LoadNamedParameters(optional);
 
-    m_Impl->AppendClientIPSessionIDPassword(&cmd, &parameters);
+    m_Impl->AppendClientIPSessionIDPasswordAgeHitID(&cmd, &parameters);
 
-    m_Impl->ExecMirrorAware(key_obj, cmd, &parameters);
-}
-
-string CNetCacheAPI::GetOwner(const string& blob_id,
-        const CNamedParameterList* optional)
-{
-    CNetCacheKey key(blob_id, m_Impl->m_CompoundIDPool);
-
-    CNetCacheAPIParameters parameters(&m_Impl->m_DefaultParameters);
-
-    parameters.LoadNamedParameters(optional);
-
-    try {
-        return m_Impl->ExecMirrorAware(key, m_Impl->MakeCmd("GBOW ",
-                key, &parameters), &parameters).response;
-    } catch (CNetCacheException& e) {
-        if (e.GetErrCode() == CNetCacheException::eBlobNotFound)
-            return kEmptyStr;
-        throw;
-    }
+    m_Impl->ExecMirrorAware(key_obj, cmd, false, &parameters);
 }
 
 IReader* CNetCacheAPI::GetReader(const string& key, size_t* blob_size,
@@ -785,7 +848,11 @@ IReader* CNetCacheAPI::GetData(const string& key, size_t* blob_size,
 {
     try {
         return GetReader(key, blob_size, optional);
-    } catch (CNetCacheException& e) {
+    }
+    catch (CNetCacheBlobTooOldException&) {
+        return NULL;
+    }
+    catch (CNetCacheException& e) {
         switch (e.GetErrCode()) {
         case CNetCacheException::eBlobNotFound:
         case CNetCacheException::eAccessDenied:
@@ -795,7 +862,7 @@ IReader* CNetCacheAPI::GetData(const string& key, size_t* blob_size,
     }
 }
 
-IReader* SNetCacheAPIImpl::GetPartReader(const string& blob_id,
+CNetCacheReader* SNetCacheAPIImpl::GetPartReader(const string& blob_id,
     size_t offset, size_t part_size,
     size_t* blob_size_ptr, const CNamedParameterList* optional)
 {
@@ -820,7 +887,7 @@ IReader* SNetCacheAPIImpl::GetPartReader(const string& blob_id,
 
     parameters.LoadNamedParameters(optional);
 
-    AppendClientIPSessionIDPassword(&cmd, &parameters);
+    AppendClientIPSessionIDPasswordAgeHitID(&cmd, &parameters);
 
     unsigned max_age = parameters.GetMaxBlobAge();
     if (max_age > 0) {
@@ -828,7 +895,8 @@ IReader* SNetCacheAPIImpl::GetPartReader(const string& blob_id,
         cmd += NStr::NumericToString(max_age);
     }
 
-    CNetServer::SExecResult exec_result(ExecMirrorAware(key, cmd, &parameters));
+    CNetServer::SExecResult exec_result(
+            ExecMirrorAware(key, cmd, false, &parameters));
 
     unsigned* actual_age_ptr = parameters.GetActualBlobAgePtr();
     if (max_age > 0 && actual_age_ptr != NULL)
@@ -984,7 +1052,7 @@ public:
                     CVersionInfo::eNonCompatible) {
             CConfig config(params);
             return new SNetCacheAPIImpl(&config, m_DriverName,
-                kEmptyStr, kEmptyStr);
+                kEmptyStr, kEmptyStr, NULL);
         }
         return NULL;
     }
