@@ -1,4 +1,4 @@
-/*  $Id: neticache_client.cpp 419051 2013-11-21 17:52:07Z kazimird $
+/*  $Id: neticache_client.cpp 448050 2014-10-02 13:50:25Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -139,7 +139,7 @@ struct SNetICacheClientImpl : public SNetCacheAPIImpl, protected CConnIniter
             const string& client_name,
             const string& cache_name) :
         SNetCacheAPIImpl(new SNetServiceImpl(s_NetICacheAPIName,
-            client_name, new CNetICacheServerListener)),
+                client_name, new CNetICacheServerListener)),
         m_CacheName(cache_name),
         m_CacheFlags(ICache::fBestPerformance)
     {
@@ -148,7 +148,9 @@ struct SNetICacheClientImpl : public SNetCacheAPIImpl, protected CConnIniter
     }
 
     CNetServer::SExecResult StickToServerAndExec(const string& cmd,
-        CNetServer::TInstance server_to_use = NULL);
+            bool multiline_output,
+            const CNetCacheAPIParameters* parameters,
+            INetServerConnectionListener* conn_listener = NULL);
 
     string MakeStdCmd(const char* cmd_base, const string& blob_id,
         const CNetCacheAPIParameters* parameters,
@@ -162,7 +164,9 @@ struct SNetICacheClientImpl : public SNetCacheAPIImpl, protected CConnIniter
             const CNetCacheAPIParameters* parameters);
 
     IReader* ReadCurrentBlobNotOlderThan(
-        const string& key, const string& subkey, int* version,
+        const string& key, const string& subkey,
+        size_t* blob_size_ptr,
+        int* version,
         ICache::EBlobVersionValidity* validity,
         unsigned max_age, unsigned* actual_age);
 
@@ -175,8 +179,6 @@ struct SNetICacheClientImpl : public SNetCacheAPIImpl, protected CConnIniter
     string m_CacheName;
     string m_ICacheCmdPrefix;
 
-    CNetServer m_SelectedServer;
-
     ICache::TFlags m_CacheFlags;
 };
 
@@ -216,22 +218,6 @@ void CNetICacheServerListener::OnInit(CObject* api_impl,
     icache_impl->m_ICacheCmdPrefix.append(") ");
 }
 
-CNetICachePasswordGuard::CNetICachePasswordGuard(
-    CNetICacheClient::TInstance ic_client,
-    const string& password)
-{
-    if (!ic_client->m_DefaultParameters.GetPassword().empty()) {
-        NCBI_THROW(CNetCacheException, eAuthenticationError,
-            "Cannot reuse a password-protected CNetICacheClient object.");
-    }
-    if (password.empty())
-        m_NetICacheClient = ic_client;
-    else {
-        m_NetICacheClient = new SNetICacheClientImpl(*ic_client);
-        m_NetICacheClient->m_DefaultParameters.SetPassword(password);
-    }
-}
-
 CNetServerConnection SNetICacheClientImpl::InitiateWriteCmd(
     CNetCacheWriter* nc_writer, const CNetCacheAPIParameters* parameters)
 {
@@ -241,9 +227,9 @@ CNetServerConnection SNetICacheClientImpl::InitiateWriteCmd(
     cmd.append(nc_writer->GetBlobID());
     if (nc_writer->GetResponseType() == eNetCache_Wait)
         cmd.append(" confirm=1");
-    AppendClientIPSessionIDPassword(&cmd, parameters);
+    AppendClientIPSessionIDPasswordAgeHitID(&cmd, parameters);
 
-    return StickToServerAndExec(cmd).conn;
+    return StickToServerAndExec(cmd, false, parameters).conn;
 }
 
 CNetICacheClient::CNetICacheClient(EAppRegistry use_app_reg,
@@ -291,14 +277,17 @@ STimeout CNetICacheClient::GetCommunicationTimeout() const
 }
 
 CNetServer::SExecResult SNetICacheClientImpl::StickToServerAndExec(
-        const string& cmd, CNetServer::TInstance server_to_use)
+        const string& cmd,
+        bool multiline_output,
+        const CNetCacheAPIParameters* parameters,
+        INetServerConnectionListener* conn_listener)
 {
-    CNetServer selected_server(server_to_use != NULL ?
-            server_to_use : (CNetServer::TInstance) m_SelectedServer);
+    CNetServer selected_server(parameters->GetServerToUse());
 
     if (selected_server) {
         try {
-            return selected_server.ExecWithRetry(cmd);
+            return selected_server.ExecWithRetry(cmd,
+                    multiline_output, conn_listener);
         }
         catch (CNetSrvConnException& ex) {
             if (ex.GetErrCode() != CNetSrvConnException::eConnectionFailure &&
@@ -310,7 +299,8 @@ CNetServer::SExecResult SNetICacheClientImpl::StickToServerAndExec(
 
     for (CNetServiceIterator it = m_Service.Iterate(); it; ++it) {
         try {
-            return (m_SelectedServer = *it).ExecWithRetry(cmd);
+            m_DefaultParameters.SetServerToUse(*it);
+            return (*it).ExecWithRetry(cmd, multiline_output, conn_listener);
         }
         catch (CNetSrvConnException& ex) {
             if (ex.GetErrCode() != CNetSrvConnException::eConnectionFailure &&
@@ -464,22 +454,20 @@ IReader* SNetICacheClientImpl::GetReadStreamPart(
                     ' ' + NStr::UInt8ToString((Uint8) part_size));
         }
 
-        unsigned max_age = parameters.GetMaxBlobAge();
-        if (max_age > 0) {
-            cmd += " age=";
-            cmd += NStr::NumericToString(max_age);
-        }
-
         CNetServer::SExecResult exec_result(
-                StickToServerAndExec(cmd, parameters.GetServerToUse()));
+                StickToServerAndExec(cmd, false, &parameters));
 
         unsigned* actual_age_ptr = parameters.GetActualBlobAgePtr();
-        if (max_age > 0 && actual_age_ptr != NULL)
+        if (parameters.GetMaxBlobAge() > 0 && actual_age_ptr != NULL)
             *actual_age_ptr = x_ExtractBlobAge(exec_result, cmd_name);
 
         return new CNetCacheReader(this, blob_id,
             exec_result, blob_size_ptr, &parameters);
-    } catch (CNetCacheException& e) {
+    }
+    catch (CNetCacheBlobTooOldException&) {
+        return NULL;
+    }
+    catch (CNetCacheException& e) {
         if (e.GetErrCode() != CNetCacheException::eBlobNotFound)
             throw;
         return NULL;
@@ -523,7 +511,11 @@ void CNetICacheClient::GetBlobAccess(const string&     key,
                                      SBlobAccessDescr* blob_descr)
 {
     if (blob_descr->return_current_version) {
-        blob_descr->reader.reset(m_Impl->ReadCurrentBlobNotOlderThan(key, subkey,
+        blob_descr->return_current_version_supported = true;
+        blob_descr->reader.reset(m_Impl->ReadCurrentBlobNotOlderThan(
+                key,
+                subkey,
+                &blob_descr->blob_size,
                 &blob_descr->current_version,
                 &blob_descr->current_version_validity,
                 blob_descr->maximum_age,
@@ -700,27 +692,34 @@ IReader* CNetICacheClient::GetReadStream(const string& key,
         const string& subkey, int* version,
         ICache::EBlobVersionValidity* validity)
 {
-    return m_Impl->ReadCurrentBlobNotOlderThan(key, subkey, version,
-            validity, 0, NULL);
+    return m_Impl->ReadCurrentBlobNotOlderThan(key, subkey, NULL,
+            version, validity, 0, NULL);
 }
 
 IReader* SNetICacheClientImpl::ReadCurrentBlobNotOlderThan(const string& key,
-        const string& subkey, int* version,
+        const string& subkey,
+        size_t* blob_size_ptr,
+        int* version,
         ICache::EBlobVersionValidity* validity,
         unsigned max_age, unsigned* actual_age)
 {
     try {
         string blob_id(s_KeySubkeyToBlobID(key, subkey));
 
-        string cmd(MakeStdCmd("READLAST", blob_id,
-            &m_DefaultParameters));
+        string cmd;
 
-        if (max_age > 0) {
-            cmd += " age=";
-            cmd += NStr::NumericToString(max_age);
+        if (max_age == 0)
+            cmd = MakeStdCmd("READLAST", blob_id, &m_DefaultParameters);
+        else {
+            CNetCacheAPIParameters parameters(&m_DefaultParameters);
+
+            parameters.SetMaxBlobAge(max_age);
+
+            cmd = MakeStdCmd("READLAST", blob_id, &parameters);
         }
 
-        CNetServer::SExecResult exec_result(StickToServerAndExec(cmd));
+        CNetServer::SExecResult exec_result(StickToServerAndExec(cmd,
+                false, &m_DefaultParameters));
 
         string::size_type pos = exec_result.response.find("VER=");
 
@@ -756,21 +755,109 @@ IReader* SNetICacheClientImpl::ReadCurrentBlobNotOlderThan(const string& key,
             *actual_age = x_ExtractBlobAge(exec_result, "READLAST");
 
         return new CNetCacheReader(this, blob_id, exec_result,
-                NULL, &m_DefaultParameters);
-    } catch (CNetCacheException& e) {
-        if (e.GetErrCode() != CNetCacheException::eBlobNotFound)
-            throw;
+                blob_size_ptr, &m_DefaultParameters);
+    } catch (CNetCacheBlobTooOldException& e) {
+        if (actual_age != NULL)
+            *actual_age = e.GetAge();
+        *version = e.GetVersion();
+
         return NULL;
     }
 }
 
+class CSetValidWarningSuppressor : public CNetICacheServerListener
+{
+public:
+    CSetValidWarningSuppressor(
+            INetServerConnectionListener* delegate_listener,
+            const string& key,
+            const string& subkey,
+            int version) :
+        m_DelegateListener(delegate_listener),
+        m_Key(key),
+        m_Subkey(subkey),
+        m_Version(version)
+    {
+    }
+
+    virtual CRef<INetServerProperties> AllocServerProperties();
+    virtual CConfig* LoadConfigFromAltSource(CObject* api_impl,
+        string* new_section_name);
+    virtual void OnInit(CObject* api_impl,
+        CConfig* config, const string& config_section);
+    virtual void OnConnected(CNetServerConnection& connection);
+    virtual void OnError(const string& err_msg, CNetServer& server);
+    virtual void OnWarning(const string& warn_msg, CNetServer& server);
+
+    CRef<INetServerConnectionListener> m_DelegateListener;
+    string m_Key;
+    string m_Subkey;
+    int m_Version;
+};
+
+CRef<INetServerProperties> CSetValidWarningSuppressor::AllocServerProperties()
+{
+    return m_DelegateListener->AllocServerProperties();
+}
+
+CConfig* CSetValidWarningSuppressor::LoadConfigFromAltSource(CObject* api_impl,
+        string* new_section_name)
+{
+    return m_DelegateListener->LoadConfigFromAltSource(api_impl,
+            new_section_name);
+}
+
+void CSetValidWarningSuppressor::OnInit(CObject* api_impl,
+        CConfig* config, const string& config_section)
+{
+    m_DelegateListener->OnInit(api_impl, config, config_section);
+}
+
+void CSetValidWarningSuppressor::OnConnected(CNetServerConnection& connection)
+{
+    m_DelegateListener->OnConnected(connection);
+}
+
+void CSetValidWarningSuppressor::OnError(
+        const string& err_msg, CNetServer& server)
+{
+    m_DelegateListener->OnError(err_msg, server);
+}
+
+void CSetValidWarningSuppressor::OnWarning(
+        const string& warn_msg, CNetServer& server)
+{
+    SIZE_TYPE ver_pos = NStr::FindCase(warn_msg,
+            CTempString("VER=", sizeof("VER=") - 1));
+
+    if (ver_pos == NPOS)
+        m_DelegateListener->OnWarning(warn_msg, server);
+    else {
+        int version = atoi(warn_msg.c_str() + ver_pos + sizeof("VER=") - 1);
+        if (version < m_Version) {
+            ERR_POST("Cache actualization error (key \"" << m_Key <<
+                    "\", subkey \"" << m_Subkey <<
+                    "\"): the cached blob version downgraded from " <<
+                    m_Version << " to " << version);
+        }
+    }
+}
+
 void CNetICacheClient::SetBlobVersionAsCurrent(const string& key,
         const string& subkey, int version)
 {
-    CNetServer::SExecResult exec_result(m_Impl->StickToServerAndExec(
-        m_Impl->MakeStdCmd("SETVALID",
-                s_KeyVersionSubkeyToBlobID(key, version, subkey),
-                        &m_Impl->m_DefaultParameters)));
+    CRef<INetServerConnectionListener> warning_suppressor(
+            new CSetValidWarningSuppressor(m_Impl->m_Service->m_Listener,
+                    key, subkey, version));
+
+    CNetServer::SExecResult exec_result(
+            m_Impl->StickToServerAndExec(
+                    m_Impl->MakeStdCmd("SETVALID",
+                            s_KeyVersionSubkeyToBlobID(key, version, subkey),
+                            &m_Impl->m_DefaultParameters),
+                    false,
+                    &m_Impl->m_DefaultParameters,
+                    warning_suppressor));
 
     if (!exec_result.response.empty()) {
         ERR_POST("SetBlobVersionAsCurrent(\"" << key << "\", " <<
@@ -786,10 +873,13 @@ CNetServerMultilineCmdOutput CNetICacheClient::GetBlobInfo(const string& key,
 
     parameters.LoadNamedParameters(optional);
 
-    CNetServerMultilineCmdOutput output(m_Impl->StickToServerAndExec(
-        m_Impl->MakeStdCmd("GETMETA",
-            s_KeyVersionSubkeyToBlobID(key, version, subkey),
-                &parameters), parameters.GetServerToUse()));
+    CNetServerMultilineCmdOutput output(
+            m_Impl->StickToServerAndExec(
+                    m_Impl->MakeStdCmd("GETMETA",
+                            s_KeyVersionSubkeyToBlobID(key, version, subkey),
+                            &parameters),
+                    true,
+                    &parameters));
 
     output->SetNetCacheCompatMode();
 
@@ -813,7 +903,7 @@ void CNetICacheClient::PrintBlobInfo(const string& key,
 
 CNetServer CNetICacheClient::GetCurrentServer()
 {
-    return m_Impl->m_SelectedServer;
+    return m_Impl->m_DefaultParameters.GetServerToUse();
 }
 
 CNetService CNetICacheClient::GetService()
@@ -837,7 +927,7 @@ string SNetICacheClientImpl::MakeStdCmd(const char* cmd_base,
     if (!injection.empty())
         cmd.append(injection);
 
-    AppendClientIPSessionIDPassword(&cmd, parameters);
+    AppendClientIPSessionIDPasswordAgeHitID(&cmd, parameters);
 
     return cmd;
 }
@@ -846,9 +936,12 @@ string SNetICacheClientImpl::ExecStdCmd(const char* cmd_base,
     const string& key, int version, const string& subkey,
     const CNetCacheAPIParameters* parameters)
 {
-    return StickToServerAndExec(MakeStdCmd(cmd_base,
-            s_KeyVersionSubkeyToBlobID(key, version, subkey),
-                    parameters), parameters->GetServerToUse()).response;
+    return StickToServerAndExec(
+            MakeStdCmd(cmd_base,
+                    s_KeyVersionSubkeyToBlobID(key, version, subkey),
+                    parameters),
+            false,
+            parameters).response;
 }
 
 string CNetICacheClient::GetCacheName(void) const
@@ -862,6 +955,11 @@ bool CNetICacheClient::SameCacheParams(const TCacheParams* params) const
     return false;
 }
 
+void CNetICacheClient::SetEventHandler(INetEventHandler* event_handler)
+{
+    m_Impl->GetListener()->m_EventHandler = event_handler;
+}
+
 
 /// Class factory for NetCache implementation of ICache
 ///
