@@ -1,4 +1,4 @@
-/*  $Id: netschedule_api.cpp 420507 2013-12-03 19:01:36Z kazimird $
+/*  $Id: netschedule_api.cpp 447472 2014-09-25 21:00:24Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -46,6 +46,10 @@
 #include <memory>
 #include <stdio.h>
 
+#ifdef NCBI_OS_LINUX
+#include <sys/prctl.h>
+#endif
+
 
 #define COMPATIBLE_NETSCHEDULE_VERSION "4.10.0"
 
@@ -62,11 +66,50 @@ void g_AppendClientIPAndSessionID(string& cmd, const string* default_session)
         cmd += '"';
     }
 
-    if (req.IsSetSessionID() || default_session != NULL) {
+    if (req.IsSetSessionID()) {
         cmd += " sid=\"";
-        cmd += NStr::PrintableString(default_session != NULL ?
-                *default_session : req.GetSessionID());
+        cmd += NStr::PrintableString(req.GetSessionID());
         cmd += '"';
+    } else if (default_session != NULL) {
+        cmd += " sid=\"";
+        cmd += NStr::PrintableString(*default_session);
+        cmd += '"';
+    }
+}
+
+SNetScheduleNotificationThread::SNetScheduleNotificationThread(
+        SNetScheduleAPIImpl* ns_api) :
+    m_API(ns_api),
+    m_StopThread(false)
+{
+    m_UDPSocket.SetReuseAddress(eOn);
+
+    STimeout rto;
+    rto.sec = rto.usec = 0;
+    m_UDPSocket.SetTimeout(eIO_Read, &rto);
+
+    EIO_Status status = m_UDPSocket.Bind(0);
+    if (status != eIO_Success) {
+        NCBI_THROW_FMT(CException, eUnknown,
+            "Could not bind a UDP socket: " << IO_StatusStr(status));
+    }
+
+    m_UDPPort = m_UDPSocket.GetLocalPort(eNH_HostByteOrder);
+}
+
+void SNetScheduleNotificationThread::CmdAppendPortAndTimeout(
+        string* cmd, const CDeadline* deadline)
+{
+    if (deadline != NULL) {
+        unsigned remaining_seconds = s_GetRemainingSeconds(*deadline);
+
+        if (remaining_seconds > 0) {
+            *cmd += " port=";
+            *cmd += NStr::UIntToString(GetPort());
+
+            *cmd += " timeout=";
+            *cmd += NStr::UIntToString(remaining_seconds);
+        }
     }
 }
 
@@ -116,6 +159,126 @@ int g_ParseNSOutput(const string& attr_string, const char* const* attr_names,
     return -1;
 }
 
+#define JOB_NOTIF_ATTR_COUNT 3
+
+SNetScheduleNotificationThread::ENotificationType
+        SNetScheduleNotificationThread::CheckNotification(string* ns_node)
+{
+    static const char* const attr_names[JOB_NOTIF_ATTR_COUNT] =
+        {"reason", "ns_node", "queue"};
+
+    string attr_values[JOB_NOTIF_ATTR_COUNT];
+
+    int defined_attrs = g_ParseNSOutput(m_Message,
+            attr_names, attr_values, JOB_NOTIF_ATTR_COUNT);
+
+    if (defined_attrs == -1 || attr_values[2] != m_API->m_Queue)
+        return eNT_Unknown;
+
+    *ns_node = attr_values[1];
+
+    if (defined_attrs != JOB_NOTIF_ATTR_COUNT)
+        return eNT_GetNotification;
+    else if (NStr::CompareCase(attr_values[0], CTempString("get", 3)) == 0)
+        return eNT_GetNotification;
+    else if (NStr::CompareCase(attr_values[0], CTempString("read", 4)) == 0)
+        return eNT_ReadNotification;
+    else
+        return eNT_Unknown;
+}
+
+void SServerNotifications::InterruptWait()
+{
+    CFastMutexGuard guard(m_Mutex);
+
+    if (m_Interrupted)
+        m_NotificationSemaphore.TryWait();
+    else {
+        m_Interrupted = true;
+        if (!m_ReadyServers.empty())
+            m_NotificationSemaphore.TryWait();
+    }
+
+    m_NotificationSemaphore.Post();
+}
+
+void SServerNotifications::RegisterServer(const string& ns_node)
+{
+    CFastMutexGuard guard(m_Mutex);
+
+    if (!m_ReadyServers.empty())
+        m_Interrupted = false;
+    else {
+        x_ClearInterruptFlag();
+        m_NotificationSemaphore.Post();
+    }
+
+    m_ReadyServers.insert(ns_node);
+}
+
+bool SServerNotifications::GetNextNotification(string* ns_node)
+{
+    CFastMutexGuard guard(m_Mutex);
+
+    x_ClearInterruptFlag();
+
+    if (m_ReadyServers.empty())
+        return false;
+
+    TReadyServers::iterator next_server = m_ReadyServers.begin();
+    *ns_node = *next_server;
+    m_ReadyServers.erase(next_server);
+
+    if (m_ReadyServers.empty())
+        // Make sure the notification semaphore count is reset to zero.
+        m_NotificationSemaphore.TryWait();
+
+    return true;
+}
+
+void* SNetScheduleNotificationThread::Main()
+{
+#ifdef NCBI_OS_LINUX
+    prctl(PR_SET_NAME, (unsigned long)
+            (CNcbiApplication::Instance()->GetProgramDisplayName() +
+                    "_nt").c_str(), 0, 0, 0);
+#endif
+
+    static const STimeout two_seconds = {2, 0};
+
+    size_t msg_len;
+    string server_host;
+
+    while (!m_StopThread)
+        if (m_UDPSocket.Wait(&two_seconds) == eIO_Success) {
+            if (m_StopThread)
+                break;
+
+            if (m_UDPSocket.Recv(m_Buffer, sizeof(m_Buffer), &msg_len,
+                    &server_host, NULL) == eIO_Success) {
+                while (msg_len > 0 && m_Buffer[msg_len - 1] == '\0')
+                    --msg_len;
+                m_Message.assign(m_Buffer, msg_len);
+
+                string ns_node;
+
+                ENotificationType notif_type = CheckNotification(&ns_node);
+
+                switch (notif_type) {
+                case eNT_GetNotification:
+                    m_GetNotifications.RegisterServer(ns_node);
+                    break;
+                case eNT_ReadNotification:
+                    m_ReadNotifications.RegisterServer(ns_node);
+                    break;
+                default:
+                    break;
+                }
+            }
+        }
+    return NULL;
+}
+
 bool CNetScheduleNotificationHandler::ReceiveNotification(string* server_host)
 {
     size_t msg_len;
@@ -164,6 +327,62 @@ void CNetScheduleNotificationHandler::PrintPortNumber()
     printf("Using UDP port %hu\n", GetPort());
 }
 
+void SNetScheduleAPIImpl::AllocNotificationThread()
+{
+    CFastMutexGuard guard(m_NotificationThreadMutex);
+
+    if (m_NotificationThread == NULL)
+        m_NotificationThread = new SNetScheduleNotificationThread(this);
+}
+
+void SNetScheduleAPIImpl::StartNotificationThread()
+{
+    if (m_NotificationThreadStartStopCounter.Add(1) == 1)
+        m_NotificationThread->Run();
+}
+
+void SNetScheduleAPIImpl::StopNotificationThread()
+{
+    if (m_NotificationThreadStartStopCounter.Add(-1) == 0) {
+        CFastMutexGuard guard(m_NotificationThreadMutex);
+
+        if (m_NotificationThread != NULL) {
+            m_NotificationThread->m_StopThread = true;
+            CDatagramSocket().Send("INTERRUPT", sizeof("INTERRUPT"),
+                    "127.0.0.1", m_NotificationThread->m_UDPPort);
+            m_NotificationThread->Join();
+        }
+    }
+}
+
+void SNetScheduleAPIImpl::x_ClearNode()
+{
+    string cmd("CLRN");
+    g_AppendClientIPAndSessionID(cmd);
+
+    for (CNetServiceIterator it =
+            m_Service.Iterate(CNetService::eIncludePenalized); it; ++it) {
+        CNetServer server = *it;
+
+        try {
+            CNetServer::SExecResult exec_result;
+            server->ConnectAndExec(cmd, false, exec_result);
+        } catch (CNetSrvConnException& e) {
+            if (m_Service.IsLoadBalanced()) {
+                ERR_POST(server->m_ServerInPool->m_Address.AsString() <<
+                    ": " << e.what());
+            }
+        } catch (CNetServiceException& e) {
+            if (e.GetErrCode() != CNetServiceException::eCommunicationError)
+                throw;
+            else {
+                ERR_POST(server->m_ServerInPool->m_Address.AsString() <<
+                    ": " << e.what());
+            }
+        }
+    }
+}
+
 /**********************************************************************/
 
 void CNetScheduleServerListener::SetAuthString(SNetScheduleAPIImpl* impl)
@@ -188,10 +407,12 @@ void CNetScheduleServerListener::SetAuthString(SNetScheduleAPIImpl* impl)
         auth += '\"';
     }
 
-    auth += " client_version=\"";
-    auth += CNcbiApplication::Instance()->
-            GetFullVersion().GetVersionInfo().Print();
-    auth += '\"';
+    CNcbiApplication* app = CNcbiApplication::Instance();
+    if (app != NULL) {
+        auth += " client_version=\"";
+        auth += app->GetFullVersion().GetVersionInfo().Print();
+        auth += '\"';
+    }
 
     ITERATE(SNetScheduleAPIImpl::TAuthParams, it, impl->m_AuthParams) {
         auth += it->second;
@@ -226,8 +447,9 @@ void CNetScheduleServerListener::SetAffinitiesSynced(
     x_GetServerProperties(server_impl)->affs_synced = affs_synced;
 }
 
-CRef<SNetScheduleServerProperties> CNetScheduleServerListener::x_GetServerProperties(
-        SNetServerImpl* server_impl)
+CRef<SNetScheduleServerProperties>
+        CNetScheduleServerListener::x_GetServerProperties(
+                SNetServerImpl* server_impl)
 {
     return CRef<SNetScheduleServerProperties>(
             static_cast<SNetScheduleServerProperties*>(
@@ -240,24 +462,26 @@ CRef<INetServerProperties> CNetScheduleServerListener::AllocServerProperties()
     return CRef<INetServerProperties>(new SNetScheduleServerProperties);
 }
 
+CConfig* CNetScheduleServerListener::LoadConfigFromAltSource(
+    CObject* /*api_impl*/, string* /*new_section_name*/)
+{
+    return NULL;
+}
+
 void CNetScheduleServerListener::OnInit(
     CObject* api_impl, CConfig* config, const string& config_section)
 {
     SNetScheduleAPIImpl* ns_impl = static_cast<SNetScheduleAPIImpl*>(api_impl);
 
-    if (ns_impl->m_Queue.empty()) {
-        if (config == NULL) {
-            NCBI_THROW(CConfigException, eParameterMissing,
-                "Could not get queue name");
-        }
+    if (!ns_impl->m_Queue.empty())
+        SNetScheduleAPIImpl::VerifyQueueNameAlphabet(ns_impl->m_Queue);
+    else if (config != NULL) {
         ns_impl->m_Queue = config->GetString(config_section,
             "queue_name", CConfig::eErr_NoThrow, kEmptyStr);
-        if (ns_impl->m_Queue.empty())
-            ns_impl->m_Queue = "noname";
-        else
+        if (!ns_impl->m_Queue.empty())
             SNetScheduleAPIImpl::VerifyQueueNameAlphabet(ns_impl->m_Queue);
-    } else
-        SNetScheduleAPIImpl::VerifyQueueNameAlphabet(ns_impl->m_Queue);
+    }
+
     if (config == NULL) {
         ns_impl->m_AffinityPreference = CNetScheduleExecutor::eAnyJob;
         ns_impl->m_UseEmbeddedStorage = true;
@@ -271,17 +495,45 @@ void CNetScheduleServerListener::OnInit(
                 "use_embedded_input", CConfig::eErr_NoThrow, true);
         }
 
-        bool use_affinities = config->GetBool(config_section,
-                "use_affinities", CConfig::eErr_NoThrow, false);
-        bool claim_new_affinities = config->GetBool(config_section,
-                "claim_new_affinities", CConfig::eErr_NoThrow, false);
+        if (!config->GetBool(config_section, "use_affinities",
+                CConfig::eErr_NoThrow, false))
+            ns_impl->m_AffinityPreference = CNetScheduleExecutor::eAnyJob;
+        else {
+            ns_impl->m_AffinityPreference = config->GetBool(config_section,
+                    "claim_new_affinities", CConfig::eErr_NoThrow, false) ?
+                CNetScheduleExecutor::eClaimNewPreferredAffs :
+                config->GetBool(config_section,
+                    "process_any_job", CConfig::eErr_NoThrow, false) ?
+                        CNetScheduleExecutor::ePreferredAffsOrAnyJob :
+                        CNetScheduleExecutor::ePreferredAffinities;
 
-        ns_impl->m_AffinityPreference =
-                !use_affinities && !claim_new_affinities ?
-                        CNetScheduleExecutor::eAnyJob :
-                claim_new_affinities ?
-                        CNetScheduleExecutor::eClaimNewPreferredAffs :
-                        CNetScheduleExecutor::ePreferredAffsOrAnyJob;
+            string affinity_list = config->GetString(config_section,
+                    "affinity_list", CConfig::eErr_NoThrow, kEmptyStr);
+
+            if (!affinity_list.empty()) {
+                NStr::Split(affinity_list, ", ", ns_impl->m_AffinityList);
+                ITERATE(list<string>, it, ns_impl->m_AffinityList) {
+                    SNetScheduleAPIImpl::VerifyAffinityAlphabet(*it);
+                }
+            }
+
+            string affinity_ladder = config->GetString(config_section,
+                    "affinity_ladder", CConfig::eErr_NoThrow, kEmptyStr);
+            list<CTempString> affinities;
+            NStr::Split(affinity_ladder, ", ", affinities);
+
+            if (!affinities.empty()) {
+                list<CTempString>::const_iterator it = affinities.begin();
+                affinity_list = *it;
+                for (;;) {
+                    ns_impl->m_AffinityLadder.push_back(affinity_list);
+                    if (++it == affinities.end())
+                        break;
+                    affinity_list += ',';
+                    affinity_list += *it;
+                }
+            }
+        }
 
         ns_impl->m_ClientNode = config->GetString(config_section,
             "client_node", CConfig::eErr_NoThrow, kEmptyStr);
@@ -293,13 +545,10 @@ void CNetScheduleServerListener::OnInit(
     SetAuthString(ns_impl);
 }
 
-void CNetScheduleServerListener::OnConnected(
-        CNetServerConnection::TInstance conn_impl)
+void CNetScheduleServerListener::OnConnected(CNetServerConnection& connection)
 {
     if (!m_WorkerNodeCompatMode) {
-        CNetServerConnection conn(conn_impl);
-
-        string version_info(conn.Exec(m_Auth));
+        string version_info(connection.Exec(m_Auth, false));
 
         CNetServerInfo server_info(new SNetServerInfoImpl(version_info));
 
@@ -314,7 +563,7 @@ void CNetScheduleServerListener::OnConnected(
 
         if (!ns_node.empty() && !ns_session.empty()) {
             CRef<SNetScheduleServerProperties> server_props =
-                x_GetServerProperties(conn_impl->m_Server);
+                x_GetServerProperties(connection->m_Server);
 
             if (server_props->ns_node != ns_node ||
                     server_props->ns_session != ns_session) {
@@ -322,16 +571,16 @@ void CNetScheduleServerListener::OnConnected(
 
                 server_props->ns_node = ns_node;
                 server_props->ns_session = ns_session;
-                m_ServerByNode[ns_node] = conn_impl->m_Server->m_ServerInPool;
+                m_ServerByNode[ns_node] = connection->m_Server->m_ServerInPool;
                 server_props->affs_synced = false;
             }
         }
     } else
-        conn_impl->WriteLine(m_Auth);
+        connection->WriteLine(m_Auth);
 }
 
 void CNetScheduleServerListener::OnError(
-    const string& err_msg, SNetServerImpl* /*server*/)
+    const string& err_msg, CNetServer& server)
 {
     string code;
     string msg;
@@ -341,9 +590,7 @@ void CNetScheduleServerListener::OnError(
             NCBI_THROW(CNetScheduleException, eJobNotFound, err_msg);
         }
         code = err_msg;
-        msg = err_msg;
-    } else if (msg.empty())
-        msg = code;
+    }
 
     // Map code into numeric value
     CException::TErrCode err_code = CNetScheduleExceptionMap::GetCode(code);
@@ -353,20 +600,22 @@ void CNetScheduleServerListener::OnError(
         NCBI_THROW(CNetServiceException, eCommunicationError, err_msg);
 
     case CNetScheduleException::eGroupNotFound:
-        // Ignore this error.
+    case CNetScheduleException::eAffinityNotFound:
+        // Convert these errors into warnings.
+        OnWarning(msg, server);
         break;
 
     case CNetScheduleException::eJobNotFound:
-        msg = "Job not found";
-        /* FALL THROUGH */
+        NCBI_THROW(CNetScheduleException, eJobNotFound, "Job not found");
 
     default:
-        NCBI_THROW(CNetScheduleException, EErrCode(err_code), msg);
+        NCBI_THROW(CNetScheduleException, EErrCode(err_code), !msg.empty() ?
+                msg : CNetScheduleException::GetErrCodeDescription(err_code));
     }
 }
 
 void CNetScheduleServerListener::OnWarning(const string& warn_msg,
-        SNetServerImpl* server)
+        CNetServer& server)
 {
     if (m_EventHandler)
         m_EventHandler->OnWarning(warn_msg, server);
@@ -496,6 +745,65 @@ CNetScheduleAPI::StringToStatus(const CTempString& status_str)
     return eJobNotFound;
 }
 
+#define EXTRACT_WARNING_TYPE(warning_type) \
+    if (NStr::StartsWith(warn_msg, "e" #warning_type ":")) { \
+        warn_msg.erase(0, sizeof("e" #warning_type ":") - 1); \
+        return eWarn##warning_type; \
+    }
+
+CNetScheduleAPI::ENetScheduleWarningType
+        CNetScheduleAPI::ExtractWarningType(string& warn_msg)
+{
+    EXTRACT_WARNING_TYPE(AffinityNotFound);
+    EXTRACT_WARNING_TYPE(AffinityNotPreferred);
+    EXTRACT_WARNING_TYPE(AffinityAlreadyPreferred);
+    EXTRACT_WARNING_TYPE(GroupNotFound);
+    EXTRACT_WARNING_TYPE(JobNotFound);
+    EXTRACT_WARNING_TYPE(JobAlreadyCanceled);
+    EXTRACT_WARNING_TYPE(JobAlreadyDone);
+    EXTRACT_WARNING_TYPE(JobAlreadyFailed);
+    EXTRACT_WARNING_TYPE(JobPassportOnlyMatch);
+    EXTRACT_WARNING_TYPE(NoParametersChanged);
+    EXTRACT_WARNING_TYPE(ConfigFileNotChanged);
+    EXTRACT_WARNING_TYPE(AlertNotFound);
+    EXTRACT_WARNING_TYPE(AlertAlreadyAcknowledged);
+    EXTRACT_WARNING_TYPE(SubmitsDisabledForServer);
+    EXTRACT_WARNING_TYPE(QueueAlreadyPaused);
+    EXTRACT_WARNING_TYPE(QueueNotPaused);
+    EXTRACT_WARNING_TYPE(CommandObsolete);
+    return eWarnUnknown;
+}
+
+#define WARNING_TYPE_TO_STRING(warning_type) \
+    case eWarn##warning_type: \
+        return #warning_type;
+
+const char* CNetScheduleAPI::WarningTypeToString(
+        CNetScheduleAPI::ENetScheduleWarningType warning_type)
+{
+    switch (warning_type) {
+    WARNING_TYPE_TO_STRING(AffinityNotFound);
+    WARNING_TYPE_TO_STRING(AffinityNotPreferred);
+    WARNING_TYPE_TO_STRING(AffinityAlreadyPreferred);
+    WARNING_TYPE_TO_STRING(GroupNotFound);
+    WARNING_TYPE_TO_STRING(JobNotFound);
+    WARNING_TYPE_TO_STRING(JobAlreadyCanceled);
+    WARNING_TYPE_TO_STRING(JobAlreadyDone);
+    WARNING_TYPE_TO_STRING(JobAlreadyFailed);
+    WARNING_TYPE_TO_STRING(JobPassportOnlyMatch);
+    WARNING_TYPE_TO_STRING(NoParametersChanged);
+    WARNING_TYPE_TO_STRING(ConfigFileNotChanged);
+    WARNING_TYPE_TO_STRING(AlertNotFound);
+    WARNING_TYPE_TO_STRING(AlertAlreadyAcknowledged);
+    WARNING_TYPE_TO_STRING(SubmitsDisabledForServer);
+    WARNING_TYPE_TO_STRING(QueueAlreadyPaused);
+    WARNING_TYPE_TO_STRING(QueueNotPaused);
+    WARNING_TYPE_TO_STRING(CommandObsolete);
+    default:
+        return "eWarnUnknown";
+    }
+}
+
 CNetScheduleSubmitter CNetScheduleAPI::GetSubmitter()
 {
     return new SNetScheduleSubmitterImpl(m_Impl);
@@ -503,9 +811,16 @@ CNetScheduleSubmitter CNetScheduleAPI::GetSubmitter()
 
 CNetScheduleExecutor CNetScheduleAPI::GetExecutor()
 {
+    m_Impl->AllocNotificationThread();
     return new SNetScheduleExecutorImpl(m_Impl);
 }
 
+CNetScheduleJobReader CNetScheduleAPI::GetJobReader()
+{
+    m_Impl->AllocNotificationThread();
+    return new SNetScheduleJobReaderImpl(m_Impl);
+}
+
 CNetScheduleAdmin CNetScheduleAPI::GetAdmin()
 {
     return new SNetScheduleAdminImpl(m_Impl);
@@ -516,12 +831,26 @@ CNetService CNetScheduleAPI::GetService()
     return m_Impl->m_Service;
 }
 
-#define SKIP_SPACE(ptr) \
-    while (isspace((unsigned char) *(ptr))) \
-        ++ptr;
+static void s_SetJobExpTime(time_t* job_exptime, const string& time_str)
+{
+    if (job_exptime != NULL)
+        *job_exptime = (time_t) NStr::StringToUInt8(time_str,
+                NStr::fConvErr_NoThrow);
+}
 
-CNetScheduleAPI::EJobStatus
-    CNetScheduleAPI::GetJobDetails(CNetScheduleJob& job, time_t* job_exptime)
+static void s_SetPauseMode(ENetScheduleQueuePauseMode* pause_mode,
+        const string& mode_str)
+{
+    if (pause_mode != NULL)
+        *pause_mode = mode_str.empty() ? eNSQ_NoPause :
+                mode_str == "pullback" ? eNSQ_WithPullback :
+                        eNSQ_WithoutPullback;
+}
+
+CNetScheduleAPI::EJobStatus CNetScheduleAPI::GetJobDetails(
+        CNetScheduleJob& job,
+        time_t* job_exptime,
+        ENetScheduleQueuePauseMode* pause_mode)
 {
     string resp = m_Impl->x_ExecOnce("STATUS2", job.job_id);
 
@@ -539,7 +868,9 @@ CNetScheduleAPI::EJobStatus
             "input",            // 2
             "output",           // 3
             "ret_code",         // 4
-            "err_msg"};         // 5
+            "err_msg",          // 5
+            "pause",            // 6
+    };
 
 #define NUMBER_OF_STATUS_ATTRS (sizeof(s_JobStatusAttrNames) / \
     sizeof(*s_JobStatusAttrNames))
@@ -551,9 +882,8 @@ CNetScheduleAPI::EJobStatus
 
     EJobStatus status = StringToStatus(attr_values[0]);
 
-    if (job_exptime != NULL)
-        *job_exptime = (time_t) NStr::StringToUInt8(attr_values[1],
-                NStr::fConvErr_NoThrow);
+    s_SetJobExpTime(job_exptime, attr_values[1]);
+    s_SetPauseMode(pause_mode, attr_values[6]);
 
     switch (status) {
     case ePending:
@@ -578,7 +908,8 @@ CNetScheduleAPI::EJobStatus
 }
 
 CNetScheduleAPI::EJobStatus SNetScheduleAPIImpl::GetJobStatus(const string& cmd,
-        const string& job_key, time_t* job_exptime)
+        const string& job_key, time_t* job_exptime,
+        ENetScheduleQueuePauseMode* pause_mode)
 {
     string response;
 
@@ -597,7 +928,8 @@ CNetScheduleAPI::EJobStatus SNetScheduleAPIImpl::GetJobStatus(const string& cmd,
 
     static const char* const s_AttrNames[] = {
         "job_status",       // 0
-        "job_exptime"       // 1
+        "job_exptime",      // 1
+        "pause",            // 2
     };
 
 #define NUMBER_OF_ATTRS (sizeof(s_AttrNames) / sizeof(*s_AttrNames))
@@ -606,13 +938,37 @@ CNetScheduleAPI::EJobStatus SNetScheduleAPIImpl::GetJobStatus(const string& cmd,
 
     g_ParseNSOutput(response, s_AttrNames, attr_values, NUMBER_OF_ATTRS);
 
-    if (job_exptime != NULL)
-        *job_exptime = (time_t) NStr::StringToUInt8(attr_values[1],
-                NStr::fConvErr_NoThrow);
+    s_SetJobExpTime(job_exptime, attr_values[1]);
+    s_SetPauseMode(pause_mode, attr_values[2]);
 
     return CNetScheduleAPI::StringToStatus(attr_values[0]);
 }
 
+bool SNetScheduleAPIImpl::GetServerByNode(const string& ns_node,
+        CNetServer* server)
+{
+    SNetServerInPool* known_server; /* NCBI_FAKE_WARNING */
+
+    {{
+        CNetScheduleServerListener* listener = GetListener();
+
+        CFastMutexGuard guard(listener->m_ServerByNodeMutex);
+
+        CNetScheduleServerListener::TServerByNode::iterator
+            server_props_it(listener->m_ServerByNode.find(ns_node));
+
+        if (server_props_it == listener->m_ServerByNode.end())
+            return false;
+
+        known_server = server_props_it->second;
+    }}
+
+    *server = new SNetServerImpl(m_Service,
+            m_Service->m_ServerPool->ReturnServer(known_server));
+
+    return true;
+}
+
 const CNetScheduleAPI::SServerParams& SNetScheduleAPIImpl::GetServerParams()
 {
     CFastMutexGuard g(m_FastMutex);
@@ -631,7 +987,7 @@ const CNetScheduleAPI::SServerParams& SNetScheduleAPIImpl::GetServerParams()
     string cmd("QINF2 " + m_Queue);
     g_AppendClientIPAndSessionID(cmd);
 
-    CUrlArgs url_parser(m_Service.FindServerAndExec(cmd).response);
+    CUrlArgs url_parser(m_Service.FindServerAndExec(cmd, false).response);
 
     enum {
         eMaxInputSize,
@@ -665,6 +1021,43 @@ const CNetScheduleAPI::SServerParams& CNetScheduleAPI::GetServerParams()
     return m_Impl->GetServerParams();
 }
 
+void CNetScheduleAPI::GetQueueParams(
+        const string& queue_name, TQueueParams& queue_params)
+{
+    string cmd;
+
+    if (queue_name.empty())
+        cmd = "QINF2 " + m_Impl->m_Queue;
+    else {
+        SNetScheduleAPIImpl::VerifyQueueNameAlphabet(queue_name);
+
+        cmd = "QINF2 " + queue_name;
+    }
+
+    g_AppendClientIPAndSessionID(cmd);
+
+    CUrlArgs url_parser(m_Impl->m_Service.FindServerAndExec(cmd,
+            false).response);
+
+    ITERATE(CUrlArgs::TArgs, field, url_parser.GetArgs()) {
+        queue_params[field->name] = field->value;
+    }
+}
+
+void CNetScheduleAPI::GetQueueParams(
+        CNetScheduleAPI::TQueueParams& queue_params)
+{
+    string cmd("GETP2");
+    g_AppendClientIPAndSessionID(cmd);
+
+    CUrlArgs url_parser(m_Impl->m_Service.FindServerAndExec(cmd,
+            false).response);
+
+    ITERATE(CUrlArgs::TArgs, field, url_parser.GetArgs()) {
+        queue_params[field->name] = field->value;
+    }
+}
+
 void CNetScheduleAPI::GetProgressMsg(CNetScheduleJob& job)
 {
     job.progress_msg = NStr::ParseEscapes(
@@ -715,6 +1108,8 @@ void CNetScheduleAPI::SetClientSession(const string& client_session)
 
 void CNetScheduleAPI::UpdateAuthString()
 {
+    m_Impl->m_Service->m_ServerPool->ResetServerConnections();
+
     m_Impl->GetListener()->SetAuthString(m_Impl);
 }
 
