@@ -1,4 +1,4 @@
-/*  $Id: netschedule_api_executor.cpp 420510 2013-12-03 19:05:09Z kazimird $
+/*  $Id: netschedule_api_executor.cpp 448239 2014-10-03 14:17:45Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -189,17 +189,15 @@ bool g_ParseGetJobResponse(CNetScheduleJob& job, const string& response)
         return false;
 
     try {
-        if (s_DoParseGet2JobResponse(job, response))
-            return true;
+        return s_DoParseGet2JobResponse(job, response);
     }
     catch (CUrlParserException&) {
         if (s_DoParseGetJobResponse(job, response))
             return true;
-    }
 
-    NCBI_THROW(CNetScheduleException, eProtocolSyntaxError,
-            "Cannot parse server output for " +
-                    job.job_id + ":\n" + response);
+        NCBI_THROW(CNetScheduleException, eProtocolSyntaxError,
+                "Cannot parse server output for GET:\n" + response);
+    }
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -210,7 +208,7 @@ void CNetScheduleExecutor::JobDelayExpiration(const string& job_key,
     cmd += ' ';
     cmd += NStr::NumericToString(runtime_inc);
     g_AppendClientIPAndSessionID(cmd);
-    m_Impl->m_API->GetServer(job_key).ExecWithRetry(cmd);
+    m_Impl->m_API->GetServer(job_key).ExecWithRetry(cmd, false);
 }
 
 class CGetJobCmdExecutor : public INetServerFinder
@@ -242,30 +240,7 @@ const CNetScheduleAPI::SServerParams& CNetScheduleExecutor::GetServerParams()
 
 void CNetScheduleExecutor::ClearNode()
 {
-    string cmd("CLRN");
-    g_AppendClientIPAndSessionID(cmd);
-
-    for (CNetServiceIterator it = m_Impl->m_API->m_Service.
-            Iterate(CNetService::eIncludePenalized); it; ++it) {
-        CNetServer server = *it;
-
-        try {
-            CNetServer::SExecResult exec_result;
-            server->ConnectAndExec(cmd, exec_result);
-        } catch (CNetSrvConnException& e) {
-            if (m_Impl->m_API->m_Service.IsLoadBalanced()) {
-                ERR_POST(server->m_ServerInPool->m_Address.AsString() <<
-                    ": " << e.what());
-            }
-        } catch (CNetServiceException& e) {
-            if (e.GetErrCode() != CNetServiceException::eCommunicationError)
-                throw;
-            else {
-                ERR_POST_X(12, server->m_ServerInPool->m_Address.AsString() <<
-                    ": " << e.what());
-            }
-        }
-    }
+    m_Impl->m_API->x_ClearNode();
 }
 
 void CNetScheduleExecutor::SetAffinityPreference(
@@ -290,7 +265,7 @@ void SNetScheduleExecutorImpl::ClaimNewPreferredAffinity(
         for (CNetServiceIterator it =
                 m_API->m_Service.ExcludeServer(orig_server); it; ++it)
             try {
-                (*it).ExecWithRetry(new_preferred_aff_cmd);
+                (*it).ExecWithRetry(new_preferred_aff_cmd, false);
             }
             catch (CException& e) {
                 ERR_POST("Error while notifying " <<
@@ -331,7 +306,8 @@ bool SNetScheduleExecutorImpl::ExecGET(SNetServerImpl* server,
     CNetServer::SExecResult exec_result;
 
     try {
-        server->ConnectAndExec(get_cmd, exec_result, NULL, &get_cmd_listener);
+        server->ConnectAndExec(get_cmd, false,
+                exec_result, NULL, &get_cmd_listener);
     }
     catch (CNetScheduleException& e) {
         if (e.GetErrCode() != CNetScheduleException::ePrefAffExpired)
@@ -343,11 +319,12 @@ bool SNetScheduleExecutorImpl::ExecGET(SNetServerImpl* server,
             CFastMutexGuard guard(listener->m_AffinitySubmissionMutex);
 
             listener->SetAffinitiesSynced(server, false);
-            server->ConnectAndExec(MkSETAFFCmd(), exec_result);
+            server->ConnectAndExec(MkSETAFFCmd(), false,  exec_result);
             listener->SetAffinitiesSynced(server, true);
         }
 
-        server->ConnectAndExec(get_cmd, exec_result, NULL, &get_cmd_listener);
+        server->ConnectAndExec(get_cmd, false,
+                exec_result, NULL, &get_cmd_listener);
     }
 
     if (!g_ParseGetJobResponse(job, exec_result.response))
@@ -378,16 +355,21 @@ bool CNetScheduleExecutor::GetJob(CNetScheduleJob& job,
     while (m_Impl->m_NotificationHandler.WaitForNotification(*deadline)) {
         CNetServer server;
         if (m_Impl->m_NotificationHandler.CheckRequestJobNotification(m_Impl,
-                &server) && g_ParseGetJobResponse(job, server.ExecWithRetry(
-                    m_Impl->m_NotificationHandler.CmdAppendTimeoutAndClientInfo(
-                        base_cmd, deadline)).response)) {
+                &server) &&
+                g_ParseGetJobResponse(job,
+                        server.ExecWithRetry(
+                                m_Impl->m_NotificationHandler.
+                                        CmdAppendTimeoutAndClientInfo(
+                                                base_cmd, deadline),
+                                false).response)
+                ) {
             // Notify the rest of NetSchedule servers that
             // we are no longer listening on the UDP socket.
             string cancel_wget_cmd("CWGET");
             g_AppendClientIPAndSessionID(cancel_wget_cmd);
             for (CNetServiceIterator it =
                     m_Impl->m_API->m_Service.ExcludeServer(server); it; ++it)
-                (*it).ExecWithRetry(cancel_wget_cmd);
+                (*it).ExecWithRetry(cancel_wget_cmd, false);
 
             // Also, if a new preferred affinity is given by
             // the server, register it with the rest of servers.
@@ -495,7 +477,7 @@ bool CNetScheduleNotificationHandler::RequestJob(
     g_AppendClientIPAndSessionID(cancel_wget_cmd);
 
     while (--it)
-        (*it).ExecWithRetry(cancel_wget_cmd);
+        (*it).ExecWithRetry(cancel_wget_cmd, false);
 
     return true;
 }
@@ -516,26 +498,7 @@ bool CNetScheduleNotificationHandler::CheckRequestJobNotification(
             attr_values[1] != executor->m_API.GetQueueName())
         return false;
 
-    SNetServerInPool* known_server; /* NCBI_FAKE_WARNING */
-
-    {{
-        CNetScheduleServerListener* listener = executor->m_API->GetListener();
-
-        CFastMutexGuard guard(listener->m_ServerByNodeMutex);
-
-        CNetScheduleServerListener::TServerByNode::iterator
-            server_props_it(listener->m_ServerByNode.find(attr_values[0]));
-
-        if (server_props_it == listener->m_ServerByNode.end())
-            return false;
-
-        known_server = server_props_it->second;
-    }}
-
-    *server = new SNetServerImpl(executor->m_API->m_Service, executor->
-            m_API->m_Service->m_ServerPool->ReturnServer(known_server));
-
-    return true;
+    return executor->m_API->GetServerByNode(attr_values[0], server);
 }
 
 inline
@@ -553,11 +516,11 @@ void SNetScheduleExecutorImpl::ExecWithOrWithoutRetry(const string& job_key,
     CNetServer server(m_API->GetServer(job_key));
 
     if (!m_WorkerNodeMode)
-        server.ExecWithRetry(cmd);
+        server.ExecWithRetry(cmd, false);
     else {
         CNetServer::SExecResult exec_result;
 
-        server->ConnectAndExec(cmd, exec_result);
+        server->ConnectAndExec(cmd, false, exec_result);
     }
 }
 
@@ -596,7 +559,8 @@ void CNetScheduleExecutor::PutProgressMsg(const CNetScheduleJob& job)
     cmd += '\"';
     g_AppendClientIPAndSessionID(cmd);
     CNetServer::SExecResult exec_result;
-    m_Impl->m_API->GetServer(job.job_id)->ConnectAndExec(cmd, exec_result);
+    m_Impl->m_API->GetServer(job.job_id)->ConnectAndExec(cmd,
+            false, exec_result);
 }
 
 void CNetScheduleExecutor::GetProgressMsg(CNetScheduleJob& job)
@@ -604,7 +568,8 @@ void CNetScheduleExecutor::GetProgressMsg(CNetScheduleJob& job)
     m_Impl->m_API.GetProgressMsg(job);
 }
 
-void CNetScheduleExecutor::PutFailure(const CNetScheduleJob& job)
+void CNetScheduleExecutor::PutFailure(const CNetScheduleJob& job,
+        bool no_retries)
 {
     s_CheckOutputSize(job.output,
         m_Impl->m_API->GetServerParams().max_output_size);
@@ -631,13 +596,45 @@ void CNetScheduleExecutor::PutFailure(const CNetScheduleJob& job)
 
     g_AppendClientIPAndSessionID(cmd);
 
+    if (no_retries)
+        cmd.append(" no_retries=1");
+
+    m_Impl->ExecWithOrWithoutRetry(job.job_id, cmd);
+}
+
+void CNetScheduleExecutor::Reschedule(const CNetScheduleJob& job)
+{
+    string cmd("RESCHEDULE job_key=" + job.job_id);
+
+    SNetScheduleAPIImpl::VerifyAuthTokenAlphabet(job.auth_token);
+    cmd += " auth_token=";
+    cmd += job.auth_token;
+
+    if (!job.affinity.empty()) {
+        cmd += " aff=\"";
+        SNetScheduleAPIImpl::VerifyAffinityAlphabet(job.affinity);
+        cmd += NStr::PrintableString(job.affinity);
+        cmd += '"';
+    }
+
+    if (!job.group.empty()) {
+        cmd += " group=\"";
+        SNetScheduleAPIImpl::VerifyJobGroupAlphabet(job.group);
+        cmd += NStr::PrintableString(job.group);
+        cmd += '"';
+    }
+
+    g_AppendClientIPAndSessionID(cmd);
+
     m_Impl->ExecWithOrWithoutRetry(job.job_id, cmd);
 }
 
 CNetScheduleAPI::EJobStatus CNetScheduleExecutor::GetJobStatus(
-        const string& job_key, time_t* job_exptime)
+        const string& job_key, time_t* job_exptime,
+        ENetScheduleQueuePauseMode* pause_mode)
 {
-    return m_Impl->m_API->GetJobStatus("WST2", job_key, job_exptime);
+    return m_Impl->m_API->GetJobStatus("WST2",
+            job_key, job_exptime, pause_mode);
 }
 
 void CNetScheduleExecutor::ReturnJob(const string& job_key,
@@ -731,7 +728,7 @@ void CNetScheduleGETCmdListener::OnExec(
             CFastMutexGuard guard(listener->m_AffinitySubmissionMutex);
 
             if (listener->NeedToSubmitAffinities(conn_impl->m_Server)) {
-                conn.Exec(m_Executor->MkSETAFFCmd());
+                conn.Exec(m_Executor->MkSETAFFCmd(), false);
                 listener->SetAffinitiesSynced(conn_impl->m_Server, true);
             }
         }
