@@ -1,4 +1,4 @@
-/*  $Id: netservice_api.cpp 420507 2013-12-03 19:01:36Z kazimird $
+/*  $Id: netservice_api.cpp 447471 2014-09-25 20:59:33Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -246,12 +246,10 @@ void SNetServiceImpl::Construct()
 
         if (!NStr::SplitInTwo(m_ServiceName, ":", host, port))
             m_ServiceType = CNetService::eLoadBalancedService;
-        else {
+        else
             Construct(m_ServerPool->FindOrCreateServerImpl(
                     g_NetService_gethostbyname(host),
                     (unsigned short) NStr::StringToInt(port)));
-            return;
-        }
     }
 }
 
@@ -297,7 +295,10 @@ void SNetServiceImpl::Init(CObject* api_impl, const string& service_name,
     {
         class CInPlaceConnIniter : protected CConnIniter
         {
+        public:
+            void NoOp() {};
         } conn_initer;
+        conn_initer.NoOp();
     }
 
     const char* const* default_section = default_config_sections;
@@ -319,9 +320,7 @@ void SNetServiceImpl::Init(CObject* api_impl, const string& service_name,
 
                 if (section_param_tree != NULL)
                     app_reg_config.reset(new CConfig(section_param_tree));
-                else if (*default_section == NULL)
-                    app_reg_config.reset(new CConfig(*reg));
-                else {
+                else if (*default_section != NULL) {
                     section = *default_section++;
                     continue;
                 }
@@ -351,6 +350,11 @@ void SNetServiceImpl::Init(CObject* api_impl, const string& service_name,
     m_ServiceName = service_name;
     NStr::TruncateSpacesInPlace(m_ServiceName);
 
+    if (m_ServiceName.empty() && config == NULL &&
+            (config = m_Listener->LoadConfigFromAltSource(api_impl,
+                    &section)) != NULL)
+        app_reg_config.reset(config);
+
     if (config != NULL) {
         if (m_ServiceName.empty()) {
             m_ServiceName = config->GetString(section, "service",
@@ -406,25 +410,31 @@ void SNetServerPoolImpl::Init(CConfig* config, const string& section,
             m_LBSMAffinityName = config->GetString(section,
                 "use_lbsm_affinity", CConfig::eErr_NoThrow, kEmptyStr);
 
-        unsigned long timeout = s_SecondsToMilliseconds(config->GetString(
+        unsigned long conn_timeout = s_SecondsToMilliseconds(config->GetString(
             section, "connection_timeout", CConfig::eErr_NoThrow, "0"), 0);
 
-        NcbiMsToTimeout(&m_ConnTimeout, timeout > 0 ? timeout :
+        NcbiMsToTimeout(&m_ConnTimeout, conn_timeout > 0 ? conn_timeout :
             SECONDS_DOUBLE_TO_MS_UL(CONNECTION_TIMEOUT_DEFAULT));
 
-        timeout = s_SecondsToMilliseconds(config->GetString(
+        unsigned long comm_timeout = s_SecondsToMilliseconds(config->GetString(
             section, "communication_timeout", CConfig::eErr_NoThrow, "0"), 0);
 
-        if (timeout > 0)
-            NcbiMsToTimeout(&m_CommTimeout, timeout);
+        if (comm_timeout > 0)
+            NcbiMsToTimeout(&m_CommTimeout, comm_timeout);
         else
             m_CommTimeout = s_GetDefaultCommTimeout();
 
-        NcbiMsToTimeout(&m_FirstServerTimeout,
-                s_SecondsToMilliseconds(config->GetString(section,
-                "first_server_timeout", CConfig::eErr_NoThrow,
-                NCBI_AS_STRING(FIRST_SERVER_TIMEOUT_DEFAULT)),
-                SECONDS_DOUBLE_TO_MS_UL(FIRST_SERVER_TIMEOUT_DEFAULT)));
+        unsigned long first_srv_timeout = s_SecondsToMilliseconds(
+                config->GetString(section, "first_server_timeout",
+                CConfig::eErr_NoThrow, "0"), 0);
+
+        if (first_srv_timeout > 0)
+            NcbiMsToTimeout(&m_FirstServerTimeout, first_srv_timeout);
+        else if (comm_timeout > 0)
+            m_FirstServerTimeout = m_CommTimeout;
+        else
+            NcbiMsToTimeout(&m_FirstServerTimeout,
+                    SECONDS_DOUBLE_TO_MS_UL(FIRST_SERVER_TIMEOUT_DEFAULT));
 
         m_ServerThrottlePeriod = config->GetInt(section,
             "throttle_relaxation_period", CConfig::eErr_NoThrow,
@@ -506,8 +516,6 @@ void SNetServerPoolImpl::Init(CConfig* config, const string& section,
         m_RebalanceStrategy = CreateDefaultRebalanceStrategy();
     }
 
-    m_PermanentConnection = eOn;
-
     if (m_ClientName.empty() || m_ClientName == "noname" ||
             NStr::FindNoCase(m_ClientName, "unknown") != NPOS) {
         CNcbiApplication* app = CNcbiApplication::Instance();
@@ -610,24 +618,39 @@ void CNetService::PrintCmdOutput(const string& cmd,
         if (load_balanced)
             output_stream << '[' << (*it).GetServerAddress() << ']' << endl;
 
-        CNetServer::SExecResult exec_result((*it).ExecWithRetry(cmd));
+        switch (output_style) {
+        case eSingleLineOutput:
+            output_stream << (*it).ExecWithRetry(cmd, false).response << endl;
+            break;
 
-        if (output_style == eSingleLineOutput)
-            output_stream << exec_result.response << endl;
-        else {
-            CNetServerMultilineCmdOutput output(exec_result);
+        case eUrlEncodedOutput:
+            {
+                CUrlArgs url_parser((*it).ExecWithRetry(cmd, false).response);
 
-            if (output_style == eMultilineOutput_NetCacheStyle)
-                output->SetNetCacheCompatMode();
+                ITERATE(CUrlArgs::TArgs, field, url_parser.GetArgs()) {
+                    output_stream << field->name <<
+                            ": " << field->value << endl;
+                }
+            }
+            break;
 
-            string line;
+        default:
+            {
+                CNetServerMultilineCmdOutput output(
+                        (*it).ExecWithRetry(cmd, true));
 
-            while (output.ReadLine(line))
-                output_stream << line << endl;
+                if (output_style == eMultilineOutput_NetCacheStyle)
+                    output->SetNetCacheCompatMode();
 
-            if (load_balanced)
-                output_stream << endl;
+                string line;
+
+                while (output.ReadLine(line))
+                    output_stream << line << endl;
+            }
         }
+
+        if (load_balanced)
+            output_stream << endl;
     }
 }
 
@@ -705,7 +728,8 @@ CNetServer SRandomServiceTraversal::NextServer()
     return ++m_Iterator ? *m_Iterator : CNetServer();
 }
 
-CNetServer::SExecResult CNetService::FindServerAndExec(const string& cmd)
+CNetServer::SExecResult CNetService::FindServerAndExec(const string& cmd,
+        bool multiline_output)
 {
     switch (m_Impl->m_ServiceType) {
     default: // CNetService::eServiceNotDefined
@@ -719,7 +743,8 @@ CNetServer::SExecResult CNetService::FindServerAndExec(const string& cmd)
 
             SRandomServiceTraversal random_traversal(*this);
 
-            m_Impl->IterateUntilExecOK(cmd, exec_result, &random_traversal,
+            m_Impl->IterateUntilExecOK(cmd, multiline_output,
+                    exec_result, &random_traversal,
                     SNetServiceImpl::eIgnoreServerErrors);
 
             return exec_result;
@@ -731,7 +756,7 @@ CNetServer::SExecResult CNetService::FindServerAndExec(const string& cmd)
                     m_Impl->m_ServerPool->ReturnServer(
                     m_Impl->m_DiscoveredServers->m_Servers.front().first)));
 
-            return server.ExecWithRetry(cmd);
+            return server.ExecWithRetry(cmd, multiline_output);
         }
     }
 }
@@ -739,7 +764,7 @@ CNetServer::SExecResult CNetService::FindServerAndExec(const string& cmd)
 void CNetService::ExecOnAllServers(const string& cmd)
 {
     for (CNetServiceIterator it = Iterate(eIncludePenalized); it; ++it)
-        (*it).ExecWithRetry(cmd);
+        (*it).ExecWithRetry(cmd, false);
 }
 
 void SNetServiceImpl::DiscoverServersIfNeeded()
@@ -875,6 +900,7 @@ bool SNetServiceImpl::IsInService(CNetServer::TInstance server)
 }
 
 void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
+    bool multiline_output,
     CNetServer::SExecResult& exec_result,
     IServiceTraversal* service_traversal,
     SNetServiceImpl::EServerErrorHandling error_handling)
@@ -898,7 +924,7 @@ void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
 
     for (;;) {
         try {
-            server->ConnectAndExec(cmd, exec_result, timeout);
+            server->ConnectAndExec(cmd, multiline_output, exec_result, timeout);
             return;
         }
         catch (CNetCacheException& ex) {
@@ -963,11 +989,11 @@ void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
             }
             if (retry_count <= 0) {
                 NCBI_THROW_FMT(CNetSrvConnException, eSrvListEmpty,
-                        "Unable to execute [" << cmd <<
-                        "] on any of the discovered servers.");
+                        "Unable to send [" << cmd <<
+                        "] to any of the discovered servers.");
             }
 
-            LOG_POST(Warning << "Unable to execute [" << cmd << "] on any "
+            LOG_POST(Warning << "Unable to send [" << cmd << "] to any "
                     "of the discovered servers; will retry after delay.");
 
             SleepMilliSec(retry_delay);
@@ -985,6 +1011,15 @@ void SNetServiceImpl::IterateUntilExecOK(const string& cmd,
     }
 }
 
+void SNetServerPoolImpl::ResetServerConnections()
+{
+    CFastMutexGuard server_mutex_lock(m_ServerMutex);
+
+    NON_CONST_ITERATE(TNetServerByAddress, it, m_Servers) {
+        it->second->m_CurrentConnectionGeneration.Add(1);
+    }
+}
+
 SNetServerPoolImpl::~SNetServerPoolImpl()
 {
     // Clean up m_Servers
@@ -1018,11 +1053,6 @@ const STimeout& CNetServerPool::GetCommunicationTimeout() const
     return m_Impl->m_CommTimeout;
 }
 
-void CNetServerPool::SetPermanentConnection(ESwitch type)
-{
-    m_Impl->m_PermanentConnection = type;
-}
-
 CNetServiceIterator CNetService::Iterate(CNetService::EIterationMode mode)
 {
     CRef<SDiscoveredServers> servers;
