@@ -1,4 +1,4 @@
-/*  $Id: ns_output_parser.cpp 419046 2013-11-21 17:45:50Z kazimird $
+/*  $Id: ns_output_parser.cpp 447471 2014-09-25 20:59:33Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -329,7 +329,7 @@ CJsonNode CExecAndParseStructuredOutput::ExecOn(CNetServer server)
 {
     CNetScheduleStructuredOutputParser parser;
 
-    return parser.ParseObject(server.ExecWithRetry(m_Cmd).response);
+    return parser.ParseObject(server.ExecWithRetry(m_Cmd, false).response);
 }
 
 CJsonNode g_ExecStructuredNetScheduleCmdToJson(CNetScheduleAPI ns_api,
@@ -371,7 +371,7 @@ CJsonNode g_GenericStatToJson(CNetServer server,
     if (verbose)
         stat_cmd.append(" VERBOSE");
 
-    CNetServerMultilineCmdOutput output(server.ExecWithRetry(stat_cmd));
+    CNetServerMultilineCmdOutput output(server.ExecWithRetry(stat_cmd, true));
 
     CJsonNode entities(CJsonNode::NewArrayNode());
     CJsonNode entity_info;
@@ -413,11 +413,26 @@ CJsonNode g_GenericStatToJson(CNetServer server,
     return entities;
 }
 
+bool g_FixMisplacedPID(CJsonNode& stat_info, CTempString& executable_path,
+        const char* pid_key)
+{
+    SIZE_TYPE misplaced_pid = NStr::Find(executable_path, "; PID: ");
+    if (misplaced_pid == NPOS)
+        return false;
+
+    SIZE_TYPE pos = misplaced_pid + sizeof("; PID: ") - 1;
+    stat_info.SetInteger(pid_key, NStr::StringToInt8(
+            CTempString(executable_path.data() + pos,
+                    executable_path.length() - pos)));
+    executable_path.erase(misplaced_pid);
+    return true;
+}
+
 CJsonNode g_LegacyStatToJson(CNetServer server, bool verbose)
 {
     const string stat_cmd(verbose ? "STAT ALL" : "STAT");
 
-    CNetServerMultilineCmdOutput output(server.ExecWithRetry(stat_cmd));
+    CNetServerMultilineCmdOutput output(server.ExecWithRetry(stat_cmd, true));
 
     CJsonNode stat_info(CJsonNode::NewObjectNode());
     CJsonNode jobs_by_status(CJsonNode::NewObjectNode());;
@@ -452,21 +467,13 @@ CJsonNode g_LegacyStatToJson(CNetServer server, bool verbose)
                     CNetScheduleAPI::eJobNotFound)
                 jobs_by_status.SetInteger(key, NStr::StringToInt8(value));
             else {
-                if (key == "Executable path") {
-                    SIZE_TYPE misplaced_pid = NStr::Find(value, "; PID: ");
-                    if (misplaced_pid != NPOS) {
-                        SIZE_TYPE pos = misplaced_pid + sizeof("; PID: ") - 1;
-                        stat_info.SetInteger("PID", NStr::StringToInt8(
-                                CTempString(value.data() + pos,
-                                        value.length() - pos)));
-                        value.erase(misplaced_pid);
+                if (key == "Executable path" &&
+                        g_FixMisplacedPID(stat_info, value, "PID")) {
+                    if (!stat_info.HasKey("Version"))
+                        stat_info.SetString("Version", "Unknown");
 
-                        if (!stat_info.HasKey("Version"))
-                            stat_info.SetString("Version", "Unknown");
-
-                        if (!stat_info.HasKey("Build date"))
-                            stat_info.SetString("Build date", "Unknown");
-                    }
+                    if (!stat_info.HasKey("Build date"))
+                        stat_info.SetString("Build date", "Unknown");
                 }
                 stat_info.SetByKey(key, CJsonNode::GuessType(value));
             }
@@ -520,7 +527,7 @@ struct SQueueInfoToJson : public IExecToJson
 
 CJsonNode SQueueInfoToJson::ExecOn(CNetServer server)
 {
-    CNetServerMultilineCmdOutput output(server.ExecWithRetry(m_Cmd));
+    CNetServerMultilineCmdOutput output(server.ExecWithRetry(m_Cmd, true));
 
     CJsonNode queue_map(CJsonNode::NewObjectNode());
     CJsonNode queue_params;
@@ -574,7 +581,7 @@ CJsonNode g_ReconfAndReturnJson(CNetScheduleAPI ns_api,
 }
 
 CAttrListParser::ENextAttributeType CAttrListParser::NextAttribute(
-    CTempString& attr_name, string& attr_value)
+    CTempString* attr_name, string* attr_value, size_t* attr_column)
 {
     while (isspace(*m_Position))
         ++m_Position;
@@ -582,14 +589,15 @@ CAttrListParser::ENextAttributeType CAttrListParser::NextAttribute(
     if (*m_Position == '\0')
         return eNoMoreAttributes;
 
-    const char* start_pos = m_Position;
+    const char* attr_start = m_Position;
+    *attr_column = attr_start - m_Start + 1;
 
     for (;;)
         if (*m_Position == '=') {
-            attr_name.assign(start_pos, m_Position - start_pos);
+            attr_name->assign(attr_start, m_Position - attr_start);
             break;
         } else if (isspace(*m_Position)) {
-            attr_name.assign(start_pos, m_Position - start_pos);
+            attr_name->assign(attr_start, m_Position - attr_start);
             while (isspace(*++m_Position))
                 ;
             if (*m_Position == '=')
@@ -597,7 +605,7 @@ CAttrListParser::ENextAttributeType CAttrListParser::NextAttribute(
             else
                 return eStandAloneAttribute;
         } else if (*++m_Position == '\0') {
-            attr_name.assign(start_pos, m_Position - start_pos);
+            attr_name->assign(attr_start, m_Position - attr_start);
             return eStandAloneAttribute;
         }
 
@@ -605,7 +613,7 @@ CAttrListParser::ENextAttributeType CAttrListParser::NextAttribute(
     while (isspace(*++m_Position))
         ;
 
-    start_pos = m_Position;
+    attr_start = m_Position;
 
     switch (*m_Position) {
     case '\0':
@@ -616,16 +624,16 @@ CAttrListParser::ENextAttributeType CAttrListParser::NextAttribute(
     case '"':
         {
             size_t n_read;
-            attr_value = NStr::ParseQuoted(CTempString(start_pos,
-                m_EOL - start_pos), &n_read);
+            *attr_value = NStr::ParseQuoted(CTempString(attr_start,
+                m_EOL - attr_start), &n_read);
             m_Position += n_read;
         }
         break;
     default:
         while (*++m_Position != '\0' && !isspace(*m_Position))
             ;
-        attr_value = NStr::ParseEscapes(
-            CTempString(start_pos, m_Position - start_pos));
+        *attr_value = NStr::ParseEscapes(
+            CTempString(attr_start, m_Position - attr_start));
     }
 
     return eAttributeWithValue;
@@ -728,10 +736,11 @@ void g_ProcessJobInfo(CNetScheduleAPI ns_api, const string& job_key,
                         CAttrListParser::ENextAttributeType next_attr_type;
                         CTempString attr_name;
                         string attr_value;
+                        size_t attr_column;
 
                         while ((next_attr_type = attr_parser.NextAttribute(
-                                attr_name, attr_value)) !=
-                                CAttrListParser::eNoMoreAttributes)
+                                &attr_name, &attr_value, &attr_column)) !=
+                                        CAttrListParser::eNoMoreAttributes)
                             if (next_attr_type ==
                                     CAttrListParser::eAttributeWithValue)
                                 processor->ProcessJobEventField(attr_name,
