@@ -1,4 +1,4 @@
-/*  $Id: srv_connections.cpp 420507 2013-12-03 19:01:36Z kazimird $
+/*  $Id: srv_connections.cpp 447471 2014-09-25 20:59:33Z kazimird $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -79,10 +79,10 @@ bool CNetServerMultilineCmdOutput::ReadLine(string& output)
         m_Impl->m_FirstOutputLine = kEmptyStr;
         m_Impl->m_FirstLineConsumed = true;
     } else if (!m_Impl->m_NetCacheCompatMode)
-        m_Impl->m_Connection->ReadCmdOutputLine(output);
+        m_Impl->m_Connection->ReadCmdOutputLine(output, true);
     else {
         try {
-            m_Impl->m_Connection->ReadCmdOutputLine(output);
+            m_Impl->m_Connection->ReadCmdOutputLine(output, true);
         }
         catch (CNetSrvConnException& e) {
             if (e.GetErrCode() != CNetSrvConnException::eConnClosedByServer)
@@ -104,6 +104,7 @@ bool CNetServerMultilineCmdOutput::ReadLine(string& output)
 inline SNetServerConnectionImpl::SNetServerConnectionImpl(
         SNetServerImpl* server) :
     m_Server(server),
+    m_Generation(server->m_ServerInPool->m_CurrentConnectionGeneration.Get()),
     m_NextFree(NULL)
 {
     if (TServConn_UserLinger2::GetDefault())
@@ -113,8 +114,8 @@ inline SNetServerConnectionImpl::SNetServerConnectionImpl(
 void SNetServerConnectionImpl::DeleteThis()
 {
     // Return this connection to the pool.
-    if (m_Server->m_ServerInPool->m_ServerPool->m_PermanentConnection != eOff &&
-            m_Socket.GetStatus(eIO_Open) == eIO_Success) {
+    if (m_Server->m_ServerInPool->m_CurrentConnectionGeneration.Get() ==
+            m_Generation && m_Socket.GetStatus(eIO_Open) == eIO_Success) {
         TFastMutexGuard guard(
                 m_Server->m_ServerInPool->m_FreeConnectionListLock);
 
@@ -134,7 +135,13 @@ void SNetServerConnectionImpl::DeleteThis()
     delete this;
 }
 
-void SNetServerConnectionImpl::ReadCmdOutputLine(string& result)
+#define STRING_LEN(str) (sizeof(str) - 1)
+#define WARNING_PREFIX "WARNING:"
+#define WARNING_PREFIX_LEN STRING_LEN(WARNING_PREFIX)
+
+void SNetServerConnectionImpl::ReadCmdOutputLine(string& result,
+        bool multiline_output,
+        INetServerConnectionListener* conn_listener)
 {
     switch (m_Socket.ReadLine(result))
     {
@@ -162,27 +169,32 @@ void SNetServerConnectionImpl::ReadCmdOutputLine(string& result)
     }
 
     if (NStr::StartsWith(result, "OK:")) {
-        result.erase(0, sizeof("OK:") - 1);
-        if (NStr::StartsWith(result, "WARNING:")) {
-            string::size_type semicolon_pos =
-                result.find(';', sizeof("WARNING:") - 1);
-            if (semicolon_pos != string::npos) {
-                m_Server->m_Service->m_Listener->OnWarning(string(
-                        result.begin() + sizeof("WARNING:") - 1,
-                        result.begin() + semicolon_pos), m_Server);
-                result.erase(0, semicolon_pos + 1);
-            } else {
-                m_Server->m_Service->m_Listener->OnWarning(string(
-                        result.begin() + sizeof("WARNING:") - 1,
-                        result.end()), m_Server);
-                result.clear();
+        const char* reply = result.c_str() + STRING_LEN("OK:");
+        size_t reply_len = result.length() - STRING_LEN("OK:");
+        while (reply_len >= WARNING_PREFIX_LEN &&
+                memcmp(reply, WARNING_PREFIX, WARNING_PREFIX_LEN) == 0) {
+            reply += WARNING_PREFIX_LEN;
+            reply_len -= WARNING_PREFIX_LEN;
+            const char* semicolon = strchr(reply, ';');
+            if (conn_listener == NULL)
+                conn_listener = m_Server->m_Service->m_Listener;
+            if (semicolon == NULL) {
+                conn_listener->OnWarning(string(reply, reply + reply_len),
+                        m_Server);
+                reply_len = 0;
+                break;
             }
+            conn_listener->OnWarning(string(reply, semicolon), m_Server);
+            reply_len -= semicolon - reply + 1;
+            reply = semicolon + 1;
         }
+        result.erase(0, result.length() - reply_len);
     } else if (NStr::StartsWith(result, "ERR:")) {
-        result.erase(0, sizeof("ERR:") - 1);
+        result.erase(0, STRING_LEN("ERR:"));
         result = NStr::ParseEscapes(result);
-        m_Server->m_Service->m_Listener->OnError(result, m_Server);
-        result = END_OF_MULTILINE_OUTPUT;
+        (conn_listener != NULL ? conn_listener :
+                m_Server->m_Service->m_Listener)->OnError(result, m_Server);
+        result = multiline_output ? string(END_OF_MULTILINE_OUTPUT) : kEmptyStr;
     }
 }
 
@@ -261,7 +273,10 @@ namespace {
     };
 }
 
-string CNetServerConnection::Exec(const string& cmd, STimeout* timeout)
+string CNetServerConnection::Exec(const string& cmd,
+        bool multiline_output,
+        STimeout* timeout,
+        INetServerConnectionListener* conn_listener)
 {
     CTimeoutKeeper timeout_keeper(&m_Impl->m_Socket, timeout);
 
@@ -276,7 +291,7 @@ string CNetServerConnection::Exec(const string& cmd, STimeout* timeout)
 
     string output;
     try {
-        m_Impl->ReadCmdOutputLine(output);
+        m_Impl->ReadCmdOutputLine(output, multiline_output, conn_listener);
     }
     catch (CNetSrvConnException& e) {
         NCBI_RETHROW_SAME(e, "... CMD=" + cmd);
@@ -297,6 +312,8 @@ SNetServerInPool::SNetServerInPool(unsigned host, unsigned short port,
     m_Address(host, port),
     m_ServerProperties(server_properties)
 {
+    m_CurrentConnectionGeneration.Set(0);
+
     m_FreeConnectionListHead = NULL;
     m_FreeConnectionListSize = 0;
 
@@ -441,7 +458,9 @@ CNetServerConnection SNetServerImpl::GetConnectionFromPool()
         guard.Release();
 
         // Check if the socket is already connected.
-        if (conn->m_Socket.GetStatus(eIO_Open) != eIO_Success)
+        if (conn->m_Socket.GetStatus(eIO_Open) != eIO_Success ||
+                conn->m_Generation !=
+                        m_ServerInPool->m_CurrentConnectionGeneration.Get())
             continue;
 
         SSOCK_Poll conn_socket_poll_struct = {
@@ -467,7 +486,8 @@ inline static bool operator <(const STimeout& t1, const STimeout& t2)
     return t1.sec == t2.sec ? t1.usec < t2.usec : t1.sec < t2.sec;
 }
 
-CNetServerConnection SNetServerImpl::Connect(STimeout* timeout)
+CNetServerConnection SNetServerImpl::Connect(STimeout* timeout,
+        INetServerConnectionListener* conn_listener)
 {
     CNetServerConnection conn = new SNetServerConnectionImpl(this);
 
@@ -597,7 +617,8 @@ CNetServerConnection SNetServerImpl::Connect(STimeout* timeout)
     conn->m_Socket.DisableOSSendDelay();
     conn->m_Socket.SetReuseAddress(eOn);
 
-    m_Service->m_Listener->OnConnected(conn);
+    (conn_listener != NULL ? conn_listener :
+            m_Service->m_Listener)->OnConnected(conn);
 
     if (timeout != NULL)
         conn->m_Socket.SetTimeout(eIO_ReadWrite,
@@ -606,7 +627,8 @@ CNetServerConnection SNetServerImpl::Connect(STimeout* timeout)
     return conn;
 }
 
-void SNetServerImpl::TryExec(INetServerExecHandler& handler, STimeout* timeout)
+void SNetServerImpl::TryExec(INetServerExecHandler& handler,
+        STimeout* timeout, INetServerConnectionListener* conn_listener)
 {
     m_ServerInPool->CheckIfThrottled();
 
@@ -633,7 +655,7 @@ void SNetServerImpl::TryExec(INetServerExecHandler& handler, STimeout* timeout)
     }
 
     try {
-        handler.Exec(Connect(timeout), timeout);
+        handler.Exec(Connect(timeout, conn_listener), timeout);
     }
     catch (CNetSrvConnException&) {
         m_ServerInPool->AdjustThrottlingParameters(
@@ -646,11 +668,15 @@ class CNetServerExecHandler : public INetServerExecHandler
 {
 public:
     CNetServerExecHandler(const string& cmd,
+            bool multiline_output,
             CNetServer::SExecResult& exec_result,
-            INetServerExecListener* exec_listener) :
+            INetServerExecListener* exec_listener,
+            INetServerConnectionListener* conn_listener) :
         m_Cmd(cmd),
+        m_MultilineOutput(multiline_output),
         m_ExecResult(exec_result),
-        m_ExecListener(exec_listener)
+        m_ExecListener(exec_listener),
+        m_ConnListener(conn_listener)
     {
     }
 
@@ -658,8 +684,10 @@ public:
             STimeout* timeout);
 
     string m_Cmd;
+    bool m_MultilineOutput;
     CNetServer::SExecResult& m_ExecResult;
     INetServerExecListener* m_ExecListener;
+    INetServerConnectionListener* m_ConnListener;
 };
 
 void CNetServerExecHandler::Exec(CNetServerConnection::TInstance conn_impl,
@@ -670,16 +698,21 @@ void CNetServerExecHandler::Exec(CNetServerConnection::TInstance conn_impl,
     if (m_ExecListener != NULL)
         m_ExecListener->OnExec(m_ExecResult.conn, m_Cmd);
 
-    m_ExecResult.response = m_ExecResult.conn.Exec(m_Cmd, timeout);
+    m_ExecResult.response = m_ExecResult.conn.Exec(m_Cmd,
+            m_MultilineOutput,
+            timeout, m_ConnListener);
 }
 
 void SNetServerImpl::ConnectAndExec(const string& cmd,
+        bool multiline_output,
         CNetServer::SExecResult& exec_result, STimeout* timeout,
-        INetServerExecListener* exec_listener)
+        INetServerExecListener* exec_listener,
+        INetServerConnectionListener* conn_listener)
 {
-    CNetServerExecHandler exec_handler(cmd, exec_result, exec_listener);
+    CNetServerExecHandler exec_handler(cmd, multiline_output,
+            exec_result, exec_listener, conn_listener);
 
-    TryExec(exec_handler, timeout);
+    TryExec(exec_handler, timeout, conn_listener);
 }
 
 void SNetServerInPool::AdjustThrottlingParameters(EConnOpResult op_result)
@@ -765,7 +798,9 @@ void SNetServerInPool::ResetThrottlingParameters()
     m_Throttled = false;
 }
 
-CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd)
+CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd,
+        bool multiline_output,
+        INetServerConnectionListener* conn_listener)
 {
     CNetServer::SExecResult exec_result;
 
@@ -777,7 +812,8 @@ CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd)
 
     for (;;) {
         try {
-            m_Impl->ConnectAndExec(cmd, exec_result);
+            m_Impl->ConnectAndExec(cmd, multiline_output,
+                    exec_result, NULL, NULL, conn_listener);
             return exec_result;
         }
         catch (CNetSrvConnException& e) {
@@ -821,7 +857,7 @@ CNetServer::SExecResult CNetServer::ExecWithRetry(const string& cmd)
 
 CNetServerInfo CNetServer::GetServerInfo()
 {
-    string response(ExecWithRetry("VERSION").response);
+    string response(ExecWithRetry("VERSION", false).response);
 
     // Keep these two lines separate to let the Exec method
     // above succeed before constructing the object below.
@@ -829,4 +865,9 @@ CNetServerInfo CNetServer::GetServerInfo()
     return new SNetServerInfoImpl(response);
 }
 
+CNetServerInfo g_ServerInfoFromString(const string& server_info)
+{
+    return new SNetServerInfoImpl(server_info);
+}
+
 END_NCBI_SCOPE
