@@ -1,4 +1,4 @@
-/*  $Id: ncbiapp.cpp 419651 2013-11-27 15:33:49Z ivanov $
+/*  $Id: ncbiapp.cpp 448585 2014-10-07 16:52:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -91,6 +91,8 @@ CNcbiApplication* CNcbiApplication::Instance(void)
 
 
 CNcbiApplication::CNcbiApplication(void)
+    : m_ConfigLoaded(false),
+      m_LogFile(0)
 {
     // Initialize UID and start timer
     GetDiagContext().GetUID();
@@ -276,8 +278,9 @@ void CNcbiApplication::x_TryInit(EAppDiagStream diag,
     } else {
         LoadConfig(*m_Config, NULL);
     }
+    m_ConfigLoaded = true;
 
-    CDiagContext::SetupDiag(diag, m_Config, eDCM_Flush);
+    CDiagContext::SetupDiag(diag, m_Config, eDCM_Flush, m_LogFile);
     CDiagContext::x_FinalizeSetupDiag();
 
     // Setup the standard features from the config file.
@@ -486,6 +489,34 @@ int CNcbiApplication::AppMain
     }
     x_SetupStdio();
 
+    // Check if logfile is set in the args.
+    m_LogFile = 0;
+    if (!m_DisableArgDesc && argc > 1  &&  argv  &&  diag != eDS_User) {
+        for (int i = 1;  i < argc;  i++) {
+            if ( !argv[i] ) {
+                continue;
+            }
+            if ( NStr::strcmp(argv[i], s_ArgLogFile) == 0 ) {
+                if (!argv[++i]) {
+                    continue;
+                }
+                m_LogFile = argv[i];
+            }
+        }
+    }
+    // Setup logging as soon as possible.
+    // Setup for diagnostics
+    try {
+        CDiagContext::SetupDiag(diag, 0, eDCM_NoChange, m_LogFile);
+    } catch (CException& e) {
+        NCBI_RETHROW(e, CAppException, eSetupDiag,
+                     "Application diagnostic stream's setup failed");
+    } catch (exception& e) {
+        NCBI_THROW(CAppException, eSetupDiag,
+                   "Application diagnostic stream's setup failed: " +
+                   string(e.what()));
+    }
+
     // Get program executable's name & path.
     string exepath = FindProgramExecutablePath(argc, argv, &m_RealExePath);
     m_ExePath = exepath;
@@ -531,7 +562,6 @@ int CNcbiApplication::AppMain
 
     // Check command line for presence special arguments
     // "-logfile", "-conffile", "-version"
-    bool is_diag_setup = false;
     if (!m_DisableArgDesc && argc > 1  &&  argv) {
         const char** v = new const char*[argc];
         v[0] = argv[0];
@@ -549,10 +579,6 @@ int CNcbiApplication::AppMain
                 }
                 v[real_arg_index++] = argv[i - 1];
                 v[real_arg_index++] = argv[i];
-                if (SetLogFile(argv[i], eDiagFile_All, true)) {
-                    diag = eDS_User;
-                    is_diag_setup = true;
-                }
                 // Configuration file
             } else if ( NStr::strcmp(argv[i], s_ArgCfgFile) == 0 ) {
                 if (!argv[++i]) {
@@ -620,20 +646,6 @@ int CNcbiApplication::AppMain
     // Clear registry content
     m_Config->Clear();
 
-    // Setup for diagnostics
-    try {
-        if ( !is_diag_setup ) {
-            CDiagContext::SetupDiag(diag);
-        }
-    } catch (CException& e) {
-        NCBI_RETHROW(e, CAppException, eSetupDiag,
-                     "Application diagnostic stream's setup failed");
-    } catch (exception& e) {
-        NCBI_THROW(CAppException, eSetupDiag,
-                   "Application diagnostic stream's setup failed: " +
-                   string(e.what()));
-    }
-
     // Call:  Init() + Run() + Exit()
     int exit_code = 1;
     bool got_exception = false;
@@ -783,14 +795,14 @@ void CNcbiApplication::SetupArgDescriptions(CArgDescriptions* arg_desc)
 
 bool CNcbiApplication::SetupDiag(EAppDiagStream diag)
 {
-    CDiagContext::SetupDiag(diag, 0, eDCM_Flush);
+    CDiagContext::SetupDiag(diag, 0, eDCM_Flush, m_LogFile);
     return true;
 }
 
 
 bool CNcbiApplication::SetupDiag_AppSpecific(void)
 {
-    CDiagContext::SetupDiag(eDS_ToStderr, 0, eDCM_Flush);
+    CDiagContext::SetupDiag(eDS_ToStderr, 0, eDCM_Flush, m_LogFile);
     return true;
 }
 
@@ -808,6 +820,7 @@ bool CNcbiApplication::LoadConfig(CNcbiRegistry&        reg,
             m_ConfigPath = CMetaRegistry::FindRegistry
                 ("ncbi", CMetaRegistry::eName_RcOrIni);
         }
+        m_ConfigLoaded = true;
         return false;
     } else if (conf->empty()) {
         entry = CMetaRegistry::Load(basename, CMetaRegistry::eName_Ini, 0,
@@ -838,6 +851,7 @@ bool CNcbiApplication::LoadConfig(CNcbiRegistry&        reg,
             m_ConfigPath = CMetaRegistry::FindRegistry
                 ("ncbi", CMetaRegistry::eName_RcOrIni);
         }
+        m_ConfigLoaded = true;
         return false;
     } else if (entry.registry != static_cast<IRWRegistry*>(&reg)) {
         // should be impossible with new CMetaRegistry interface...
@@ -853,6 +867,7 @@ bool CNcbiApplication::LoadConfig(CNcbiRegistry&        reg,
         }
     }
     m_ConfigPath = entry.actual_name;
+    m_ConfigLoaded = true;
     return true;
 }
 
@@ -1292,12 +1307,12 @@ void CNcbiApplication::x_HonorStandardSettings( IRegistry* reg)
             mem_size_limit = (size_t)(GetPhysicalMemorySize() * percents / 100);
         } else {
             try {
-                // Size is specified in MB by default if no suffixes
+                // Size is specified in MiB by default if no suffixes
                 // (converted without exception)
                 mem_size_limit = NStr::StringToSizet(s) * 1024 * 1024;
             }
             catch (CStringException&) {
-                // Otherwise, size have suffix (MB, GB, etc)
+                // Otherwise, size have suffix (MiB, G, GB, etc)
                 Uint8 bytes = NStr::StringToUInt8_DataSize(s);
                 if ( bytes > get_limits(mem_size_limit).max() ) {
                     NCBI_THROW(CAppException, eLoadConfig,
