@@ -1,4 +1,4 @@
-/*  $Id: ncbimtx.cpp 408264 2013-07-28 21:07:11Z vakatov $
+/*  $Id: ncbimtx.cpp 448269 2014-10-03 15:55:25Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -80,7 +80,7 @@ void WriteMutexEvent(void* mutex_ptr, const char* message)
         mode, perm);
     CNcbiOstrstream str_os;
     str_os << mutex_ptr << " "
-        << CThreadSystemID::GetCurrent().m_ID << " "
+        << GetCurrentThreadSystemID() << " "
         << message << "\n";
     write(handle, str_os.str(), str_os.pcount());
     str_os.rdbuf()->freeze(false);
@@ -302,8 +302,8 @@ void SSystemMutex::Lock(SSystemFastMutex::ELockSemantics lock)
 {
     m_Mutex.CheckInitialized();
 
-    CThreadSystemID owner = CThreadSystemID::GetCurrent();
-    if ( m_Count > 0 && m_Owner.Is(owner) ) {
+    TThreadSystemID owner = GetCurrentThreadSystemID();
+    if ( m_Count > 0 && m_Owner == owner ) {
         // Don't lock twice, just increase the counter
         m_Count++;
         return;
@@ -312,7 +312,7 @@ void SSystemMutex::Lock(SSystemFastMutex::ELockSemantics lock)
     // Lock the mutex and remember the owner
     m_Mutex.Lock(lock);
     assert(m_Count == 0);
-    m_Owner.Set(owner);
+    m_Owner = owner;
     m_Count = 1;
 }
 
@@ -320,8 +320,8 @@ bool SSystemMutex::TryLock(void)
 {
     m_Mutex.CheckInitialized();
 
-    CThreadSystemID owner = CThreadSystemID::GetCurrent();
-    if ( m_Count > 0 && m_Owner.Is(owner) ) {
+    TThreadSystemID owner = GetCurrentThreadSystemID();
+    if ( m_Count > 0 && m_Owner == owner ) {
         // Don't lock twice, just increase the counter
         m_Count++;
         return true;
@@ -330,7 +330,7 @@ bool SSystemMutex::TryLock(void)
     // If TryLock is successful, remember the owner
     if ( m_Mutex.TryLock() ) {
         assert(m_Count == 0);
-        m_Owner.Set(owner);
+        m_Owner = owner;
         m_Count = 1;
         return true;
     }
@@ -345,8 +345,8 @@ void SSystemMutex::Unlock(SSystemFastMutex::ELockSemantics lock)
 
     // No unlocks by threads other than owner.
     // This includes no unlocks of unlocked mutex.
-    CThreadSystemID owner = CThreadSystemID::GetCurrent();
-    if ( m_Count == 0 || m_Owner.IsNot(owner) ) {
+    TThreadSystemID owner = GetCurrentThreadSystemID();
+    if ( m_Count == 0 || m_Owner != owner ) {
         ThrowNotOwned();
     }
 
@@ -608,9 +608,9 @@ CRWLock::~CRWLock(void)
 }
 
 
-inline bool CRWLock::x_MayAcquireForReading(CThreadSystemID self_id)
+inline bool CRWLock::x_MayAcquireForReading(TThreadSystemID self_id)
 {
-    _ASSERT(self_id == CThreadSystemID::GetCurrent());
+    _ASSERT(self_id == GetCurrentThreadSystemID());
     if (m_Count < 0) { // locked for writing, possibly by self
         // return m_Owner.Is(self_id);
         return false; // allow special handling of self-locked cases
@@ -759,7 +759,7 @@ void CRWLock::ReadLock(void)
     }
 #endif
 
-    CThreadSystemID self_id = CThreadSystemID::GetCurrent();
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
 
     // Lock mutex now, unlock before exit.
     // (in fact, it will be unlocked by the waiting function for a while)
@@ -769,7 +769,7 @@ void CRWLock::ReadLock(void)
     CFastMutexGuard guard(m_RW->m_Mutex);
 #endif
     if ( !x_MayAcquireForReading(self_id) ) {
-        if (m_Count < 0  &&  m_Owner.Is(self_id)) {
+        if (m_Count < 0  &&  m_Owner == self_id) {
             _VERIFY(interlocked_dec_max(&m_Count, 0));
         }
         else {
@@ -847,7 +847,7 @@ bool CRWLock::TryReadLock(void)
     }
 #endif
 
-    CThreadSystemID self_id = CThreadSystemID::GetCurrent();
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
 
 #if defined(NCBI_USE_CRITICAL_SECTION)
     CWin32MutexHandleGuard guard(m_RW->m_Mutex);
@@ -856,7 +856,7 @@ bool CRWLock::TryReadLock(void)
 #endif
 
     if ( !x_MayAcquireForReading(self_id) ) {
-        if (m_Count >= 0  ||  m_Owner.IsNot(self_id)) {
+        if (m_Count >= 0  ||  m_Owner != self_id) {
             // (due to be) W-locked by another thread
             return false;
         }
@@ -907,7 +907,7 @@ bool CRWLock::TryReadLock(const CTimeout& timeout)
     }
 #endif
 
-    CThreadSystemID self_id = CThreadSystemID::GetCurrent();
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
 
     // Lock mutex now, unlock before exit.
     // (in fact, it will be unlocked by the waiting function for a while)
@@ -917,7 +917,7 @@ bool CRWLock::TryReadLock(const CTimeout& timeout)
     CFastMutexGuard guard(m_RW->m_Mutex);
 #endif
     if ( !x_MayAcquireForReading(self_id) ) {
-        if (m_Count < 0  &&  m_Owner.Is(self_id)) {
+        if (m_Count < 0  &&  m_Owner == self_id) {
             _VERIFY(interlocked_dec_max(&m_Count, 0));
         }
         else {
@@ -1006,7 +1006,7 @@ void CRWLock::WriteLock(void)
     return;
 #else
 
-    CThreadSystemID self_id = CThreadSystemID::GetCurrent();
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
 
 #if defined(NCBI_USE_CRITICAL_SECTION)
     CWin32MutexHandleGuard guard(m_RW->m_Mutex);
@@ -1014,7 +1014,7 @@ void CRWLock::WriteLock(void)
     CFastMutexGuard guard(m_RW->m_Mutex);
 #endif
 
-    if ( m_Count < 0 && m_Owner.Is(self_id) ) {
+    if ( m_Count < 0 && m_Owner == self_id ) {
         // W-locked by the same thread
         _VERIFY(interlocked_dec_max(&m_Count, 0));
     }
@@ -1084,7 +1084,7 @@ void CRWLock::WriteLock(void)
         xncbi_Validate(m_Count >= 0,
                        "CRWLock::WriteLock() - invalid readers counter");
         interlocked_set(&m_Count, -1);
-        m_Owner.Set(self_id);
+        m_Owner = self_id;
     }
 
     // No readers allowed
@@ -1099,7 +1099,7 @@ bool CRWLock::TryWriteLock(void)
     return true;
 #else
 
-    CThreadSystemID self_id = CThreadSystemID::GetCurrent();
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
 
 #if defined(NCBI_USE_CRITICAL_SECTION)
     CWin32MutexHandleGuard guard(m_RW->m_Mutex);
@@ -1109,7 +1109,7 @@ bool CRWLock::TryWriteLock(void)
 
     if ( m_Count < 0 ) {
         // W-locked
-        if ( m_Owner.IsNot(self_id) ) {
+        if ( m_Owner != self_id ) {
             // W-locked by another thread
             return false;
         }
@@ -1134,7 +1134,7 @@ bool CRWLock::TryWriteLock(void)
                        "error locking R&W-semaphores");
 #endif
         interlocked_set(&m_Count, -1);
-        m_Owner.Set(self_id);
+        m_Owner = self_id;
     }
 
     // No readers allowed
@@ -1159,7 +1159,7 @@ bool CRWLock::TryWriteLock(const CTimeout& timeout)
         return TryWriteLock();
     }
 
-    CThreadSystemID self_id = CThreadSystemID::GetCurrent();
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
 
 #if defined(NCBI_USE_CRITICAL_SECTION)
     CWin32MutexHandleGuard guard(m_RW->m_Mutex);
@@ -1167,7 +1167,7 @@ bool CRWLock::TryWriteLock(const CTimeout& timeout)
     CFastMutexGuard guard(m_RW->m_Mutex);
 #endif
 
-    if ( m_Count < 0 && m_Owner.Is(self_id) ) {
+    if ( m_Count < 0 && m_Owner == self_id ) {
         // W-locked by the same thread
         _VERIFY(interlocked_dec_max(&m_Count, 0));
     }
@@ -1266,7 +1266,7 @@ bool CRWLock::TryWriteLock(const CTimeout& timeout)
         xncbi_Validate(m_Count >= 0,
                        "CRWLock::TryWriteLock() - invalid readers counter");
         interlocked_set(&m_Count, -1);
-        m_Owner.Set(self_id);
+        m_Owner = self_id;
     }
 
     // No readers allowed
@@ -1288,7 +1288,7 @@ void CRWLock::Unlock(void)
     }
 #endif
 
-    CThreadSystemID self_id = CThreadSystemID::GetCurrent();
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
 
 #if defined(NCBI_USE_CRITICAL_SECTION)
     CWin32MutexHandleGuard guard(m_RW->m_Mutex);
@@ -1298,7 +1298,7 @@ void CRWLock::Unlock(void)
 
     if (m_Count < 0) {
         // Check it is R-locked or W-locked by the same thread
-        xncbi_Validate(m_Owner.Is(self_id),
+        xncbi_Validate(m_Owner == self_id,
                        "CRWLock::Unlock() - "
                        "RWLock is locked by another thread");
         _VERIFY(interlocked_inc_max(&m_Count, 0));
@@ -1340,7 +1340,7 @@ void CRWLock::Unlock(void)
         }
         if (m_Flags & fTrackReaders) {
             // Check if the unlocking thread is in the owners list
-            vector<CThreadSystemID>::iterator found =
+            vector<TThreadSystemID>::iterator found =
                 find(m_Readers.begin(), m_Readers.end(), self_id);
             _ASSERT(found != m_Readers.end());
             m_Readers.erase(found);
@@ -1583,6 +1583,11 @@ bool CSemaphore::TryWait(unsigned int NCBI_THREADS_ARG(timeout_sec),
 
 bool CSemaphore::TryWait(const CTimeout& timeout)
 {
+    if (timeout.IsInfinite()) {
+        Wait();
+        return true;
+    }
+
     unsigned int s, ns;
     timeout.GetNano(&s, &ns);
     return TryWait(s, ns);
@@ -1953,112 +1958,11 @@ CSpinLock::Unlock(void)
 //  CONDITION VARIABLE
 //
 
-
 #if defined(NCBI_HAVE_CONDITIONAL_VARIABLE)
 
-
-#  if defined(NCBI_OS_MSWIN)
-#    if (_WIN32_WINNT < 0x0600)
-#    pragma warning (disable : 4191)
-class CCV_Dynlink
-{
-public:
-    typedef VOID (WINAPI *pInitializeConditionVariable)( PCONDITION_VARIABLE ConditionVariable);
-    typedef VOID (WINAPI *pWakeConditionVariable)(       PCONDITION_VARIABLE ConditionVariable);
-    typedef VOID (WINAPI *pWakeAllConditionVariable)(    PCONDITION_VARIABLE ConditionVariable);
-    typedef BOOL (WINAPI *pSleepConditionVariableCS)(    PCONDITION_VARIABLE ConditionVariable,
-                                                            PCRITICAL_SECTION CriticalSection,
-                                                            DWORD dwMilliseconds);
-    static bool IsSupported(void)
-    {
-        return NULL != GetProcAddress(
-            GetModuleHandleA("KERNEL32.DLL"),"InitializeConditionVariable");
-    }
-    static void unsupported(void)
-    {
-        NCBI_THROW(CConditionVariableException, eUnsupported,
-                   "Condition variable is not supported");
-    }
-    static void InitializeConditionVariable(PCONDITION_VARIABLE ConditionVariable)
-    {
-        static pInitializeConditionVariable p = 0;
-        if (!p) {
-            p = (pInitializeConditionVariable)GetProcAddress(
-                GetModuleHandleA("KERNEL32.DLL"),"InitializeConditionVariable");
-        }
-        if (!p) {
-            unsupported();
-        }
-        p(ConditionVariable);
-    }
-    static void WakeConditionVariable(PCONDITION_VARIABLE ConditionVariable)
-    {
-        static pWakeConditionVariable p = 0;
-        if (!p) {
-            p = (pWakeConditionVariable)GetProcAddress(
-                GetModuleHandleA("KERNEL32.DLL"),"WakeConditionVariable");
-            if (!p) {
-                unsupported();
-            }
-        }
-        p(ConditionVariable);
-    }
-    static void WakeAllConditionVariable(PCONDITION_VARIABLE ConditionVariable)
-    {
-        static pWakeAllConditionVariable p = 0;
-        if (!p) {
-            p = (pWakeAllConditionVariable)GetProcAddress(
-                GetModuleHandleA("KERNEL32.DLL"),"WakeAllConditionVariable");
-            if (!p) {
-                unsupported();
-            }
-        }
-        p(ConditionVariable);
-    }
-    static bool SleepConditionVariableCS(PCONDITION_VARIABLE ConditionVariable,
-                                              PCRITICAL_SECTION CriticalSection,
-                                              DWORD dwMilliseconds)
-    {
-        static pSleepConditionVariableCS p = 0;
-        if (!p) {
-            p = (pSleepConditionVariableCS)GetProcAddress(
-                GetModuleHandleA("KERNEL32.DLL"),"SleepConditionVariableCS");
-            if (!p) {
-                unsupported();
-            }
-        }
-        return p(ConditionVariable,CriticalSection,dwMilliseconds) != FALSE;
-    }
-};
-
-#      define CCV_Dynlink_IsSupported                 CCV_Dynlink::IsSupported
-#      define CCV_Dynlink_InitializeConditionVariable CCV_Dynlink::InitializeConditionVariable
-#      define CCV_Dynlink_WakeConditionVariable       CCV_Dynlink::WakeConditionVariable
-#      define CCV_Dynlink_WakeAllConditionVariable    CCV_Dynlink::WakeAllConditionVariable
-#      define CCV_Dynlink_SleepConditionVariableCS    CCV_Dynlink::SleepConditionVariableCS 
-
-#    else // _WIN32_WINNT < 0x0600
-
-#      error "Now it is time to get rid of this ugly workaround above and below"
-
-#      define CCV_Dynlink_IsSupported()               true
-#      define CCV_Dynlink_InitializeConditionVariable InitializeConditionVariable
-#      define CCV_Dynlink_WakeConditionVariable       WakeConditionVariable
-#      define CCV_Dynlink_WakeAllConditionVariable    WakeAllConditionVariable
-#      define CCV_Dynlink_SleepConditionVariableCS    SleepConditionVariableCS 
-
-#    endif // _WIN32_WINNT < 0x0600
-#  endif // NCBI_OS_MSWIN
-
-
-
 bool CConditionVariable::IsSupported(void)
 {
-#if defined(NCBI_OS_MSWIN)
-    return CCV_Dynlink_IsSupported();
-#else
     return true;
-#endif
 }
 
 
@@ -2067,7 +1971,7 @@ CConditionVariable::CConditionVariable(void)
       m_WaitMutex(NULL)
 {
 #if defined(NCBI_OS_MSWIN)
-    CCV_Dynlink_InitializeConditionVariable(&m_ConditionVar);
+    InitializeConditionVariable(&m_ConditionVar);
 #else
     int res = pthread_cond_init(&m_ConditionVar, NULL);
     switch (res) {
@@ -2192,8 +2096,7 @@ bool CConditionVariable::x_WaitForSignal
 #if defined(NCBI_OS_MSWIN)
     DWORD timeout_msec = deadline.IsInfinite() ?
         INFINITE : deadline.GetRemainingTime().GetAsMilliSeconds();
-    BOOL res = CCV_Dynlink_SleepConditionVariableCS
-        (&m_ConditionVar, &mutex.m_Handle, timeout_msec);
+    BOOL res = SleepConditionVariableCS(&m_ConditionVar, &mutex.m_Handle, timeout_msec);
     s_ThrowIfDifferentMutexes(mutex_guard);
 
     if ( res )
@@ -2250,7 +2153,7 @@ bool CConditionVariable::WaitForSignal(CMutex&           mutex,
                    "WaitForSignal: mutex lock count not 1");
     }
 #if defined(NCBI_OS_MSWIN)
-    if (!sys_mtx.m_Owner.Is(CThreadSystemID::GetCurrent())) {
+    if ( sys_mtx.m_Owner != GetCurrentThreadSystemID() ) {
         NCBI_THROW(CConditionVariableException, eMutexOwner,
                    "WaitForSignal: mutex not owned by the current thread");
     }
@@ -2276,7 +2179,7 @@ bool CConditionVariable::WaitForSignal(CFastMutex&       mutex,
 void CConditionVariable::SignalSome(void)
 {
 #if defined(NCBI_OS_MSWIN)
-    CCV_Dynlink_WakeConditionVariable(&m_ConditionVar);
+    WakeConditionVariable(&m_ConditionVar);
 #else
     int res = pthread_cond_signal(&m_ConditionVar);
     if (res != 0) {
@@ -2296,7 +2199,7 @@ void CConditionVariable::SignalSome(void)
 void CConditionVariable::SignalAll(void)
 {
 #if defined(NCBI_OS_MSWIN)
-    CCV_Dynlink_WakeAllConditionVariable(&m_ConditionVar);
+    WakeAllConditionVariable(&m_ConditionVar);
 #else
     int res = pthread_cond_broadcast(&m_ConditionVar);
     if (res != 0) {
