@@ -1,4 +1,4 @@
-/*  $Id: ncbireg.cpp 389857 2013-02-21 14:59:19Z zakharov $
+/*  $Id: ncbireg.cpp 428466 2014-03-04 18:53:29Z vakatov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -204,7 +204,9 @@ void IRegistry::SetModifiedFlag(bool modified, TFlags flags)
 bool IRegistry::Write(CNcbiOstream& os, TFlags flags) const
 {
     x_CheckFlags("IRegistry::Write", flags,
-                 (TFlags)fLayerFlags | fInternalSpaces | fCountCleared |fSectionlessEntries);
+                 (TFlags)fLayerFlags | fInternalSpaces | fCountCleared
+                 | fSectionlessEntries);
+
     if ( !(flags & fTransient) ) {
         flags |= fPersistent;
     }
@@ -285,7 +287,9 @@ bool IRegistry::HasEntry(const string& section, const string& name,
                          TFlags flags) const
 {
     x_CheckFlags("IRegistry::HasEntry", flags,
-                 (TFlags)fLayerFlags | fInternalSpaces | fCountCleared |fSectionlessEntries);
+                 (TFlags)fLayerFlags | fInternalSpaces | fCountCleared
+                 | fSections | fSectionlessEntries);
+
     if ( !(flags & fTPFlags) ) {
         flags |= fTPFlags;
     }
@@ -428,7 +432,9 @@ void IRegistry::EnumerateSections(list<string>* sections, TFlags flags) const
 {
     // Should clear fSectionlessEntries if set
     x_CheckFlags("IRegistry::EnumerateSections", flags,
-                 (TFlags)fLayerFlags | fInternalSpaces | fCountCleared | fSectionlessEntries);
+                 (TFlags)fLayerFlags | fInternalSpaces | fCountCleared
+                 | fSectionlessEntries);
+
     if ( !(flags & fTPFlags) ) {
         flags |= fTPFlags;
     }
@@ -443,7 +449,9 @@ void IRegistry::EnumerateEntries(const string& section, list<string>* entries,
                                  TFlags flags) const
 {
     x_CheckFlags("IRegistry::EnumerateEntries", flags,
-                 (TFlags)fLayerFlags | fInternalSpaces | fCountCleared | fSectionlessEntries |fSections);
+                 (TFlags)fLayerFlags | fInternalSpaces | fCountCleared
+                 | fSectionlessEntries | fSections);
+
     if ( !(flags & fTPFlags) ) {
         flags |= fTPFlags;
     }
@@ -533,7 +541,8 @@ IRWRegistry* IRWRegistry::Read(CNcbiIstream& is, TFlags flags,
 {
     x_CheckFlags("IRWRegistry::Read", flags,
                  fTransient | fNoOverride | fIgnoreErrors | fInternalSpaces
-                 | fWithNcbirc | fJustCore | fCountCleared | fSectionlessEntries);
+                 | fWithNcbirc | fJustCore | fCountCleared
+                 | fSectionlessEntries);
 
     if ( !is ) {
         return NULL;
@@ -553,12 +562,13 @@ IRWRegistry* IRWRegistry::Read(CNcbiIstream& is, TFlags flags,
 
 
 IRWRegistry* IRWRegistry::x_Read(CNcbiIstream& is, TFlags flags,
-                                 const string& /* path */)
+                                 const string& path)
 {
     // Whether to consider this read to be (unconditionally) non-modifying
     TFlags layer         = (flags & fTransient) ? fTransient : fPersistent;
     TFlags impact        = layer | fJustCore;
-    bool   non_modifying = Empty(impact)  &&  !Modified(impact);
+    bool   was_empty     = Empty(impact);
+    bool   non_modifying = was_empty  &&  !Modified(impact);
     bool   ignore_errors = (flags & fIgnoreErrors) > 0;
 
     // Adjust flags for Set() fSectionlessEntries must survive this change
@@ -568,6 +578,7 @@ IRWRegistry* IRWRegistry::x_Read(CNcbiIstream& is, TFlags flags,
     SIZE_TYPE line;         // # of the line being parsed
     string    section(kEmptyStr);      // current section name
     string    comment;      // current comment
+    string    in_path = path.empty() ? kEmptyStr : (" in " + path);
 
     for (line = 1;  NcbiGetlineEOL(is, str);  ++line) {
         try {
@@ -601,18 +612,18 @@ IRWRegistry* IRWRegistry::x_Read(CNcbiIstream& is, TFlags flags,
                 SIZE_TYPE end = str.find_first_of(']', beg + 1);
                 if (end == NPOS) {
                     NCBI_THROW2(CRegistryException, eSection,
-                                "Invalid registry section(']' is missing): `"
-                                + str + "'", line);
+                                "Invalid registry section" + in_path
+                                + " (']' is missing): `" + str + "'", line);
                 }
                 section = NStr::TruncateSpaces(str.substr(beg, end - beg));
                 if (section.empty()) {
                     NCBI_THROW2(CRegistryException, eSection,
-                                "Unnamed registry section: `" + str + "'",
-                                line);
+                                "Unnamed registry section" + in_path + ": `"
+                                + str + "'", line);
                 } else if ( !s_IsNameSection(section, flags) ) {
                     NCBI_THROW2(CRegistryException, eSection,
-                                "Invalid registry section name: `"
-                                + str + "'", line);
+                                "Invalid registry section name" + in_path
+                                + ": `" + str + "'", line);
                 }
                 // add section comment
                 if ( !comment.empty() ) {
@@ -626,14 +637,14 @@ IRWRegistry* IRWRegistry::x_Read(CNcbiIstream& is, TFlags flags,
                 string name, value;
                 if ( !NStr::SplitInTwo(str, "=", name, value) ) {
                     NCBI_THROW2(CRegistryException, eEntry,
-                                "Invalid registry entry format: '" + str + "'",
-                                line);
+                                "Invalid registry entry format" + in_path
+                                + ": '" + str + "'", line);
                 }
                 NStr::TruncateSpacesInPlace(name);
                 if ( !s_IsNameSection(name, flags) ) {
                     NCBI_THROW2(CRegistryException, eEntry,
-                                "Invalid registry entry name: '" + str + "'",
-                                line);
+                                "Invalid registry entry name" + in_path + ": '"
+                                + str + "'", line);
                 }
             
                 NStr::TruncateSpacesInPlace(value);
@@ -672,8 +683,8 @@ IRWRegistry* IRWRegistry::x_Read(CNcbiIstream& is, TFlags flags,
                     } else {
                         NCBI_THROW2(CRegistryException, eValue,
                                     "Single(unescaped) '\"' in the middle "
-                                    "of registry value: '" + str + "'",
-                                    line);
+                                    "of registry value" + in_path + ": '"
+                                    + str + "'", line);
                     }
                 }
 
@@ -681,11 +692,11 @@ IRWRegistry* IRWRegistry::x_Read(CNcbiIstream& is, TFlags flags,
                     value = NStr::ParseEscapes(value.substr(beg, end - beg));
                 } catch (CStringException&) {
                     NCBI_THROW2(CRegistryException, eValue,
-                                "Badly placed '\\' in the registry value: '"
-                                + str + "'", line);
+                                "Badly placed '\\' in the registry value"
+                                + in_path + ": '" + str + "'", line);
 
                 }
-                TFlags set_flags = flags;
+                TFlags set_flags = flags & ~fJustCore;
                 if (NStr::EqualNocase(section, "NCBI")
                     &&  NStr::EqualNocase(name, ".Inherits")
                     &&  HasEntry(section, name, flags)) {
@@ -697,6 +708,11 @@ IRWRegistry* IRWRegistry::x_Read(CNcbiIstream& is, TFlags flags,
                         value += ' ';
                         value += old_value;
                     }
+                } else if (was_empty  &&  HasEntry(section, name, flags)) {
+                    ERR_POST_X(8, Warning
+                               << "Found multiple [" << section << "] "
+                               << name << " settings" << in_path
+                               << "; using the one from line " << line);
                 }
                 Set(section, name, value, set_flags, comment);
                 comment.erase();
@@ -712,8 +728,8 @@ IRWRegistry* IRWRegistry::x_Read(CNcbiIstream& is, TFlags flags,
     }
 
     if ( !is.eof() ) {
-        ERR_POST_X(4, "Error reading the registry after line " << line << ": "
-                       << str);
+        ERR_POST_X(4, "Error reading the registry after line " << line
+                   << in_path << ": " << str);
     }
 
     if ( non_modifying ) {
@@ -873,11 +889,13 @@ const string& CMemoryRegistry::x_GetComment(const string& section,
 void CMemoryRegistry::x_Enumerate(const string& section, list<string>& entries,
                                   TFlags flags) const
 {
-    if (section.empty() && ( (flags & IRegistry::fSections) || !(flags & IRegistry::fSectionlessEntries))) {
+    if (section.empty() &&
+        ( (flags & IRegistry::fSections)
+          || !(flags & IRegistry::fSectionlessEntries))) {
         // Enumerate sections
         if(!(flags & (IRegistry::fSections | IRegistry::fSectionlessEntries)))
-            _TRACE(string("Deprecated call to x_Enumerate with empty section name, but")+
-           " with no fSections flag set");
+            _TRACE("Deprecated call to x_Enumerate with empty section name, "
+                   " but with no fSections flag set");
 
         ITERATE (TSections, it, m_Sections) {
             if (s_IsNameSection(it->first, flags)
@@ -1162,11 +1180,9 @@ void CCompoundRegistry::x_Enumerate(const string& section,
         if ((flags & fJustCore)  &&  (it->first < m_CoreCutoff)) {
             break;
         }
+
         list<string> tmp;
-        if(flags & fSections)
-            it->second->EnumerateEntries(section, &tmp, (flags & ~fJustCore)|fSections);
-        else
-            it->second->EnumerateEntries(section, &tmp, (flags & ~fJustCore));
+        it->second->EnumerateEntries(section, &tmp, flags & ~fJustCore);
 
         ITERATE (list<string>, it2, tmp) {
             accum.insert(*it2);
@@ -1412,7 +1428,8 @@ CNcbiRegistry::CNcbiRegistry(CNcbiIstream& is, TFlags flags,
     : m_RuntimeOverrideCount(0), m_Flags(flags)
 {
     x_CheckFlags("CNcbiRegistry::CNcbiRegistry", flags,
-                 fTransient | fInternalSpaces | fWithNcbirc | fCaseFlags | fSectionlessEntries);
+                 fTransient | fInternalSpaces | fWithNcbirc | fCaseFlags
+                 | fSectionlessEntries);
     x_Init();
     m_FileRegistry->Read(is, flags & ~(fWithNcbirc | fCaseFlags));
     LoadBaseRegistries(flags, 0, path);
@@ -1479,7 +1496,7 @@ IRWRegistry* CNcbiRegistry::x_Read(CNcbiIstream& is, TFlags flags,
     // file portion so that environment settings can take priority.
     CConstRef<IRegistry> main_reg(FindByName(sm_MainRegName));
     if (main_reg->Empty()  &&  m_FileRegistry->Empty()) {
-        m_FileRegistry->Read(is, flags);
+        m_FileRegistry->Read(is, flags & ~fWithNcbirc);
         LoadBaseRegistries(flags, 0, path);
         IncludeNcbircIfAllowed(flags);
         return NULL;
@@ -1745,12 +1762,9 @@ void CCompoundRWRegistry::x_Enumerate(const string& section,
         if ((flags & fJustCore)  &&  (it->first < GetCoreCutoff())) {
             break;
         }
-        list<string> tmp;
 
-        if(flags & fSections)
-            it->second->EnumerateEntries(section, &tmp, (flags & ~fJustCore)|fSections);
-        else
-            it->second->EnumerateEntries(section, &tmp, flags & ~fJustCore);
+        list<string> tmp;
+        it->second->EnumerateEntries(section, &tmp, flags & ~fJustCore);
 
         ITERATE (list<string>, it2, tmp) {
             // avoid reporting cleared entries
