@@ -1,4 +1,4 @@
-/* $Id: ncbistre.cpp 403421 2013-06-14 15:35:46Z lavr $
+/* $Id: ncbistre.cpp 438026 2014-06-12 13:06:37Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -215,11 +215,13 @@ CNcbiIstream& NcbiGetlineEOL(CNcbiIstream& is, string& str, SIZE_TYPE* count)
 
 bool NcbiStreamCopy(CNcbiOstream& os, CNcbiIstream& is)
 {
-    if (!is.good())
+    if (!os.good())
         return false;
 #ifndef NCBI_COMPILER_GCC
-    if (CT_EQ_INT_TYPE(is.peek(), CT_EOF))
-        return true; // NB: Workshop doesn't set eofbit (C++ Std 27.6.1.3.27)
+    if (CT_EQ_INT_TYPE(is.peek(), CT_EOF)) {
+        // NB: C++ Std says nothing about eofbit (27.6.1.3.27)
+        return true;
+    }
     os << is.rdbuf();
 #elif   NCBI_COMPILER_VERSION <= 330
     // GCC stdlib++ version <= 3.3.0 has a bug in implementation of streamcopy,
@@ -242,18 +244,15 @@ bool NcbiStreamCopy(CNcbiOstream& os, CNcbiIstream& is)
     } while (is.good());
 #else
     if (CT_EQ_INT_TYPE(is.peek(), CT_EOF)) {
-#  ifdef __GLIBCXX__
-        return is.eof() ? true : false; // NB: GLIBC++ sets eofbit
-#  else
-        return true; // The C++ Std does not say about eofbit (27.6.1.3.27)
-#  endif
+        // NB: C++ Std says nothing about eofbit (27.6.1.3.27)
+        return true;
     }
     os << is.rdbuf();
 #endif
     if (!os.good()  ||  !os.flush())
         return false;
     if (CT_EQ_INT_TYPE(is.peek(), CT_EOF))
-        return is.fail() ? false : true;
+        return true;
     os.clear(NcbiFailbit);
     return false;
 }
@@ -272,6 +271,66 @@ void NcbiStreamCopyThrow(CNcbiOstream& os, CNcbiIstream& is)
 }
 
 
+size_t NcbiStreamToString(string* str, CNcbiIstream& is, size_t pos)
+{
+    if (!is.good()) {
+        // Can't extract anything
+        if (str)
+            str->resize(pos);
+        is.setstate(NcbiFailbit);
+        return 0;
+    }
+
+    char   buf[5120];
+    size_t buf_size = sizeof(buf);
+    size_t str_size;
+
+    if (str) {
+        str_size = pos;
+        if (str->size() < str_size + buf_size)
+            str->resize(str_size + buf_size);
+    } else
+        str_size = pos = 0;
+
+    do {
+        try {
+            is.read(str ? &(*str)[str_size] : buf, buf_size);
+        } catch (...) {
+            if (str)
+                str->resize(str_size);
+            throw;
+        }
+        streamsize count = is.gcount();
+        str_size += (size_t) count;
+        if (str) {
+            if ((size_t) count == buf_size) {
+                if (buf_size < (1UL << 16))
+                    buf_size <<= 1;
+                str->resize(str_size + buf_size);
+            } else
+                _ASSERT(!is.good());
+        }
+    } while (is.good());
+
+    _ASSERT(str_size >= pos);
+    if (str)
+        str->resize(str_size);
+
+    if (!(str_size -= pos)) {
+        // Nothing extracted
+        is.setstate(NcbiFailbit);
+        return 0;
+    }
+
+    // NB: istream::read() sets both bits at EOF (27.6.1.3.28)
+    IOS_BASE::iostate iostate = is.rdstate();
+    if (iostate != (NcbiFailbit | NcbiEofbit))
+        return 0;
+    is.clear(iostate & ~NcbiFailbit);
+    return str_size;
+}
+
+
 bool NcbiStreamCompare(CNcbiIstream& is1, CNcbiIstream& is2)
 {
     while (is1 && is2) {
@@ -345,6 +404,9 @@ bool NcbiStreamCompareText(CNcbiIstream& is, const string& str,
 
 CNcbiOstrstreamToString::operator string(void) const
 {
+#ifdef NCBI_SHUN_OSTRSTREAM
+    return m_Out.str();
+#else
     SIZE_TYPE len = (SIZE_TYPE) m_Out.pcount();
     if ( !len ) {
         return string();
@@ -352,6 +414,7 @@ CNcbiOstrstreamToString::operator string(void) const
     const char* str = m_Out.str();
     m_Out.freeze(false);
     return string(str, len);
+#endif
 }
 
 
@@ -413,20 +476,25 @@ string Printable(char c)
 
     string s;
     switch ( c ) {
-    case '\0':  s = "\\0";   break;
-    case '\\':  s = "\\\\";  break;
-    case '\n':  s = "\\n";   break;
-    case '\t':  s = "\\t";   break;
-    case '\r':  s = "\\r";   break;
-    case '\v':  s = "\\v";   break;
+    case '\0':  s += "\\0";   break;
+    case '\t':  s += "\\t";   break;
+    case '\v':  s += "\\v";   break;
+    case '\b':  s += "\\b";   break;
+    case '\r':  s += "\\r";   break;
+    case '\f':  s += "\\f";   break;
+    case '\a':  s += "\\a";   break;
+    case '\n':  s += "\\n";   break;
+    case '\\':  s += "\\\\";  break;
+    case '\'':  s += "\\'";   break;
+    case '"':   s += "\\\"";  break;
     default:
-        if ( isprint((unsigned char) c) ) {
-            s = c;
-        } else {
-            s = "\\x";
+        if ( !isprint((unsigned char) c) ) {
+            s += "\\x";
             s += kHex[(unsigned char) c / 16];
             s += kHex[(unsigned char) c % 16];
-        }
+        } else
+            s += c;
+        break;
     }
     return s;
 }
@@ -464,7 +532,7 @@ void s_WritePrintable(CNcbiOstream& out, char c, char n)
         break;
     }
 
-    bool full = !s_IsQuoted(n)  &&  n >= '0'  &&  n <= '7' ? true : false;
+    bool full = !s_IsQuoted(n)  &&  '0' <= n  &&  n <= '7' ? true : false;
     unsigned char v;
     char octal[4];
     int k = 1;
@@ -726,6 +794,36 @@ EEncodingForm GetTextEncodingForm(CNcbiIstream& input,
     return ef;
 }
 
+CNcbiOstream& operator<< (CNcbiOstream& str, const CByteOrderMark&  bom)
+{
+    switch (bom.GetEncodingForm()) {
+    /// Stream has no BOM.
+    default:
+    case eEncodingForm_Unknown:
+    case eEncodingForm_ISO8859_1:
+    case eEncodingForm_Windows_1252:
+        break;
+    case eEncodingForm_Utf8:
+        str << Uint1(0xEF) << Uint1(0xBB) << Uint1(0xBF);
+        break;
+    case eEncodingForm_Utf16Native:
+#ifdef WORDS_BIGENDIAN 
+        str << Uint1(0xFE) << Uint1(0xFF);
+#else
+        str << Uint1(0xFF) << Uint1(0xFE);
+#endif
+        break;
+    case eEncodingForm_Utf16Foreign:
+#ifdef WORDS_BIGENDIAN 
+        str << Uint1(0xFF) << Uint1(0xFE);
+#else
+        str << Uint1(0xFE) << Uint1(0xFF);
+#endif
+        break;
+    }
+    return str;
+}
+
 
 #include "ncbi_base64.c"
 
@@ -736,6 +834,7 @@ END_NCBI_SCOPE
 // See in the header why it is outside of NCBI scope (SunPro bug workaround...)
 
 #if defined(NCBI_USE_OLD_IOSTREAM)
+
 extern NCBI_NS_NCBI::CNcbiOstream& operator<<(NCBI_NS_NCBI::CNcbiOstream& os,
                                               const NCBI_NS_STD::string& str)
 {
@@ -760,9 +859,8 @@ extern NCBI_NS_NCBI::CNcbiIstream& operator>>(NCBI_NS_NCBI::CNcbiIstream& is,
     for (ch = is.rdbuf()->sbumpc();
          ch != EOF  &&  !isspace((unsigned char) ch);
          ch = is.rdbuf()->sbumpc()) {
-        str.append(1, (char)ch);
-        i++;
-        if (i == end)
+        str.append(1, (char) ch);
+        if (++i == end)
             break;
     }
     if (ch == EOF) 
@@ -774,5 +872,4 @@ extern NCBI_NS_NCBI::CNcbiIstream& operator>>(NCBI_NS_NCBI::CNcbiIstream& is,
     return is;
 }
 
-
 #endif  /* NCBI_USE_OLD_IOSTREAM */
