@@ -1,4 +1,4 @@
-/*  $Id: dbapi_impl_connection.cpp 414412 2013-09-24 13:51:48Z ucko $
+/*  $Id: dbapi_impl_connection.cpp 440505 2014-07-14 19:00:33Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -95,12 +95,11 @@ CConnection::CConnection(CDriverContext& dc,
 , m_MsgHandlers(dc.GetConnHandlerStack())
 , m_Interface(NULL)
 , m_ResProc(NULL)
+, m_ExceptionContext(new TDbgInfo(params))
 , m_ServerType(params.GetServerType())
 , m_ServerTypeIsKnown(false)
-, m_Server(params.GetServerName())
 , m_Host(params.GetHost())
 , m_Port(params.GetPort())
-, m_User(params.GetUserName())
 , m_Passwd(params.GetPassword())
 , m_Pool(params.GetParam("pool_name"))
 , m_Reusable(params.GetParam("is_pooled") == "true")
@@ -112,6 +111,7 @@ CConnection::CConnection(CDriverContext& dc,
 {
     _ASSERT(m_MsgHandlers.GetSize() == dc.GetConnHandlerStack().GetSize());
     _ASSERT(m_MsgHandlers.GetSize() > 0);
+    m_OpeningMsgHandlers = params.GetOpeningMsgHandlers();
 
     CheckCanOpen();
 }
@@ -277,7 +277,7 @@ CDB_Result* CConnection::Create_Result(impl::CResult& result)
 
 const string& CConnection::ServerName(void) const
 {
-    return m_Server;
+    return m_ExceptionContext->server_name;
 }
 
 Uint4 CConnection::Host(void) const
@@ -293,7 +293,7 @@ Uint2 CConnection::Port(void) const
 
 const string& CConnection::UserName(void) const
 {
-    return m_User;
+    return m_ExceptionContext->username;
 }
 
 
@@ -351,14 +351,14 @@ CConnection::SetDatabaseName(const string& name)
         auto_stmt->Send();
         auto_stmt->DumpResults();
 
-        m_Database = name;
+        m_ExceptionContext->database_name = name;
     }
 }
 
 const string&
 CConnection::GetDatabaseName(void) const
 {
-    return m_Database;
+    return m_ExceptionContext->database_name;
 }
 
 I_ConnectionExtra::TSockHandle
