@@ -1,4 +1,4 @@
-/* $Id: exception.cpp 403742 2013-06-18 15:26:09Z grichenk $
+/* $Id: exception.cpp 441150 2014-07-21 19:10:46Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,6 +31,8 @@
 
 #include <ncbi_pch.hpp>
 #include <dbapi/driver/exception.hpp>
+#include <dbapi/driver/interfaces.hpp>
+#include <dbapi/driver/impl/dbapi_impl_connection.hpp>
 #include <dbapi/error_codes.hpp>
 #include <corelib/ncbi_safe_static.hpp>
 
@@ -45,6 +47,73 @@ BEGIN_NCBI_SCOPE
 //  CDB_Exception::
 //
 
+CDB_Exception::SParams::~SParams(void)
+{
+    ITERATE (TParams, it, params) {
+        if (it->value != NULL) {
+            delete it->value;
+        }
+    }
+}
+
+CDB_Exception::SContext::SContext(const CDBConnParams& params)
+    : server_name(params.GetServerName()),
+      username(params.GetUserName())
+      // database_name(params.GetDatabaseName())
+{
+}
+
+void CDB_Exception::SContext::UpdateFrom(const SContext& ctx)
+{
+    // favor new context
+    if ( !ctx.server_name.empty()   ) { server_name   = ctx.server_name;   }
+    if ( !ctx.username.empty()      ) { username      = ctx.username;      }
+    if ( !ctx.database_name.empty() ) { database_name = ctx.database_name; }
+    if ( !ctx.extra_msg.empty()     ) { extra_msg     = ctx.extra_msg;     }
+}
+
+ostream& operator<<(ostream &os, const CDB_Exception::SContext& ctx)
+{
+    const char* delim = kEmptyCStr;
+    if ( !ctx.server_name.empty() ) { 
+        os << delim << "SERVER: '" << ctx.server_name << '\'';
+        delim = " ";
+    }
+    if ( !ctx.username.empty() ) {
+        os << delim << "USER: '" << ctx.username << '\'';
+        delim = " ";
+    }
+    if ( !ctx.database_name.empty() ) {
+        os << delim << "DATABASE: '" << ctx.database_name << '\'';
+        delim = " ";
+    }
+    if ( !ctx.extra_msg.empty() ) {
+        os << delim << ctx.extra_msg;
+    }
+    return os;
+}
+
+CDB_Exception::SMessageInContext CDB_Exception::SMessageInContext::operator+
+(const SContext& new_context) const
+{
+    if (context.Empty()) {
+        return SMessageInContext(message, new_context);
+    } else {
+        CRef<SContext> merged_context(new SContext(*context));
+        merged_context->UpdateFrom(new_context);
+        return SMessageInContext(message, *merged_context);
+    }
+}
+
+ostream& operator<<(ostream &os, const CDB_Exception::SMessageInContext& msg)
+{
+    os << msg.message;
+    if (msg.context.NotEmpty()) {
+        os << ' ' << *msg.context;
+    }
+    return os;
+}
+
 EDB_Severity
 CDB_Exception::Severity(void) const
 {
@@ -105,6 +174,53 @@ const char* CDB_Exception::SeverityString(EDB_Severity sev)
 }
 
 
+void CDB_Exception::SetFromConnection(const impl::CConnection& connection)
+{
+    if (GetServerName().empty()  &&  !connection.ServerName().empty()) {
+        SetServerName(connection.ServerName());
+        // AddToMessage(" SERVER: '" + connection.ServerName() + '\'');
+    }
+    if (GetUserName().empty()  &&  !connection.UserName().empty()) {
+        SetUserName(connection.UserName());
+        // AddToMessage(" USER: '" + connection.UserName() + '\'');
+    }
+    if (GetDatabaseName().empty()  &&  !connection.GetDatabaseName().empty()) {
+        SetDatabaseName(connection.GetDatabaseName());
+        // AddToMessage(" DATABASE: '" + connection.GetDatabaseName() + '\'');
+    }
+}
+
+
+void
+CDB_Exception::SetParams(const CDBParams* params)
+{
+    unsigned int n = params ? params->GetNum() : 0;
+    if (n == 0) {
+        return;
+    }
+    if (m_Params.Empty()) {
+        m_Params.Reset(new SParams);
+    }
+    
+    SParams::TParams& my_params = m_Params->params;
+    my_params.resize(n);
+    for (unsigned int i = 0;  i < n;  ++i) {
+        my_params[i].value = NULL;
+    }
+    for (unsigned int i = 0;  i < n;  ++i) {
+        SParam& p = my_params[i];
+        p.name = params->GetName(i);
+        try {
+            const CDB_Object* v = params->GetValue(i);
+            if (v != NULL) {
+                p.value = v->ShallowClone();
+            }
+        } catch (exception&) {
+        }
+    }
+}
+
+
 void
 CDB_Exception::ReportExtra(ostream& out) const
 {
@@ -115,7 +231,8 @@ CDB_Exception::ReportExtra(ostream& out) const
 void
 CDB_Exception::x_StartOfWhat(ostream& out) const
 {
-    out << "[";
+    out << *m_Context;
+    out << " [";
     out << SeverityString();
     out << " #";
     out << NStr::IntToString( GetDBErrCode() );
@@ -131,6 +248,34 @@ CDB_Exception::x_EndOfWhat(ostream& out) const
 //     out << "<<<";
 //     out << GetMsg();
 //     out << ">>>";
+    if ( !m_Params.Empty()  &&  !m_Params->params.empty() ) {
+        out << " [Parameters: ";
+        const char* delim = kEmptyCStr;
+        ITERATE (SParams::TParams, it, m_Params->params) {
+            out << delim;
+            if ( !it->name.empty() ) {
+                out << it->name << " = ";
+            }
+            if (it->value) {
+                out << it->value->GetLogString();
+            } else {
+                out << "(null)";
+            }
+            delim = ", ";
+        }
+        out << ']';
+    }
+}
+
+static CSafeStatic<CDB_Exception::SContext> kEmptyContext;
+
+void CDB_Exception::x_Init(const CDiagCompileInfo& info, const string& message,
+                           const CException* prev_exception, EDiagSev severity)
+{
+    CException::x_Init(info, message, prev_exception, severity);
+    if (m_Context.Empty()) {
+        m_Context.Reset(&kEmptyContext.Get());
+    }
 }
 
 void
@@ -141,11 +286,19 @@ CDB_Exception::x_Assign(const CException& src)
     CException::x_Assign(src);
 
     m_DBErrCode = other.m_DBErrCode;
-    m_ServerName = other.m_ServerName;
-    m_UserName = other.m_UserName;
+    m_Context   = other.m_Context;
     m_SybaseSeverity = other.m_SybaseSeverity;
+    m_Params = other.m_Params;
 }
 
+CDB_Exception::SContext& CDB_Exception::x_SetContext(void)
+{
+    // Unshare if necessary
+    if ( !m_Context->ReferencedOnlyOnce() ) {
+        m_Context.Reset(new SContext(*m_Context));
+    }
+    return const_cast<SContext&>(*m_Context);
+}
 
 const char*
 CDB_Exception::GetErrCodeString(void) const
@@ -369,7 +522,11 @@ public:
 
     CDB_UserHandler* Set(CDB_UserHandler* h);
 
+    virtual bool HandleAll(const TExceptions& exceptions);
     virtual bool HandleIt(CDB_Exception* ex);
+    virtual bool HandleMessage(int severity, int msgnum,
+                               const string& message);
+
     virtual ~CDB_UserHandler_Wrapper();
 
 private:
@@ -406,10 +563,23 @@ CDB_UserHandler_Wrapper::~CDB_UserHandler_Wrapper()
 
 bool CDB_UserHandler_Wrapper::HandleIt(CDB_Exception* ex)
 {
-    return m_Handler ? m_Handler->HandleIt(ex) : true;
+    return m_Handler ? m_Handler->HandleIt(ex) : false;
 }
 
 
+bool CDB_UserHandler_Wrapper::HandleAll(const TExceptions& exceptions)
+{
+    return m_Handler ? m_Handler->HandleAll(exceptions) : false;
+}
+
+
+bool CDB_UserHandler_Wrapper::HandleMessage(int severity, int msgnum,
+                                            const string& message)
+{
+    return m_Handler ? m_Handler->HandleMessage(severity, msgnum, message)
+        : false;
+}
+
 
 /////////////////////////////////////////////////////////////////////////////
 //  CDB_UserHandler::
@@ -460,9 +630,25 @@ CDB_UserHandler* CDB_UserHandler::SetDefault(CDB_UserHandler* h)
 
 bool CDB_UserHandler::HandleAll(const TExceptions& /* exceptions */)
 {
+    // return x_HandleAll(exceptions);
     return false;
 }
 
+bool CDB_UserHandler::x_HandleAll(const TExceptions& exceptions)
+{
+#if 1
+    return false;
+#else
+    // Better done in CDBHandlerStack::HandleExceptions, which can
+    // keep track of which exceptions still need handling.
+    bool handled_all = true;
+    ITERATE (TExceptions, it, exceptions) {
+        handled_all &= HandleIt(*it);
+    }
+    return handled_all;
+#endif
+}
+
 bool CDB_UserHandler::HandleMessage(int /* severity */,
                                     int /* msgnum */,
                                     const string& /* message */)
@@ -527,6 +713,12 @@ bool CDB_UserHandler_Diag::HandleIt(CDB_Exception* ex)
 }
 
 
+bool CDB_UserHandler_Diag::HandleAll(const TExceptions& exceptions)
+{
+    return x_HandleAll(exceptions);
+}
+
+
 
 /////////////////////////////////////////////////////////////////////////////
 //  CDB_UserHandler_Stream::
@@ -582,6 +774,12 @@ bool CDB_UserHandler_Stream::HandleIt(CDB_Exception* ex)
 }
 
 
+bool CDB_UserHandler_Stream::HandleAll(const TExceptions& exceptions)
+{
+    return x_HandleAll(exceptions);
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 //  CDB_UserHandler_Exception::
 //
@@ -659,6 +857,80 @@ CDB_UserHandler_Exception::HandleAll(const TExceptions& exceptions)
 }
 
 
+////////////////////////////////////////////////////////////////////////////////
+//  CDB_UserHandler_Deferred::
+//
+
+CDB_UserHandler_Deferred::CDB_UserHandler_Deferred
+(const impl::CDBHandlerStack& ultimate_handlers)
+    : m_UltimateHandlers(ultimate_handlers)
+{
+}
+
+CDB_UserHandler_Deferred::~CDB_UserHandler_Deferred(void)
+{
+    if ( !m_SavedExceptions.empty() ) {
+        ERR_POST_X(8, Warning << "Internal bug: Found unreported exceptions."
+                   << CStackTrace());
+        bool done = false;
+        while ( !done ) {
+            try {
+                Flush();
+                done = true;
+            } catch (CException& ex) {
+                ERR_POST(ex);
+            }
+        }
+    }
+}
+
+bool CDB_UserHandler_Deferred::HandleIt(CDB_Exception* ex)
+{
+    if (ex == NULL) {
+        return false;
+    }
+
+    CFastMutexGuard LOCK(m_Mutex);
+    _TRACE(*ex);
+    m_SavedExceptions.push_back(ex->Clone());
+    return true;
+}
+
+bool CDB_UserHandler_Deferred::HandleAll(const TExceptions& exceptions)
+{
+    CFastMutexGuard LOCK(m_Mutex);
+    ITERATE (TExceptions, ex, exceptions) {
+        if (*ex == NULL) {
+            continue;
+        }
+        _TRACE(**ex);
+        m_SavedExceptions.push_back((*ex)->Clone());
+    }
+    return true;
+}
+
+void CDB_UserHandler_Deferred::Flush(EDiagSev max_severity)
+{
+    CFastMutexGuard LOCK(m_Mutex);
+    ERASE_ITERATE (TExceptions, it, m_SavedExceptions) {
+        auto_ptr<CDB_Exception> ex(*it);
+        VECTOR_ERASE(it, m_SavedExceptions);
+        if (ex->GetSeverity() > max_severity) {
+            ex->SetSeverity(max_severity);
+        }
+        if (max_severity < eDiag_Error) {
+            try {
+                m_UltimateHandlers.PostMsg(ex.get());
+            } catch (CDB_Exception& ex2) {
+                ERR_POST(ex2);
+            }
+        } else { 
+            m_UltimateHandlers.PostMsg(ex.get());
+        }    
+    }
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 //  CDB_UserHandler_Exception_ODBC::
 //
