@@ -1,4 +1,4 @@
-/* $Id: public.cpp 414412 2013-09-24 13:51:48Z ucko $
+/* $Id: public.cpp 441150 2014-07-21 19:10:46Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,13 @@
 #include <dbapi/driver/impl/dbapi_impl_connection.hpp>
 #include <dbapi/error_codes.hpp>
 
+#ifdef NCBI_OS_MSWIN
+#  include <winsock2.h>
+#elif !defined(NCBI_OS_SOLARIS)
+#  include <sys/fcntl.h>
+#  include <sys/types.h>
+#  include <sys/socket.h>
+#endif
 
 #define NCBI_USE_ERRCODE_X   Dbapi_DataServer
 
@@ -43,6 +50,36 @@
 BEGIN_NCBI_SCOPE
 
 
+
+#ifdef _DEBUG
+static void s_TraceParams(const CDBParams& params,
+                          const CDiagCompileInfo& info) {
+    if (dynamic_cast<const impl::CCachedRowInfo*>(&params)) {
+        return;
+    }
+    for (unsigned int i = 0, n = params.GetNum();  i < n;  ++i) {
+        const CDB_Object* obj   = params.GetValue(i);
+        const string&     name  = params.GetName(i);
+        string            value = obj ? obj->GetLogString() : string("(null)");
+        if (name.empty()) { // insertion, typically bulk
+            CNcbiDiag(info, eDiag_Trace).GetRef()
+                << "Column #" << (i + 1) << " = " << value
+                << Endm;
+        } else {
+            CNcbiDiag(info, eDiag_Trace).GetRef()
+                << "Parameter #" << (i + 1) << " (" << name << ") = " << value
+                << Endm;
+        }
+    }
+}
+// Cite caller.
+#  define TRACE_PARAMS(params) s_TraceParams(params, DIAG_COMPILE_INFO)
+#else
+#  define TRACE_PARAMS(params) ((void)0)
+#endif
+
+
+
 ////////////////////////////////////////////////////////////////////////////
 //  CDBParamVariant::
 //
@@ -162,7 +199,8 @@ CTempString CDBParamVariant::MakeName(const CTempString& name,
                             format = eNamedName;
                         }
                     } else {
-                        DATABASE_DRIVER_ERROR("Invalid parameter format.", 1);
+                        DATABASE_DRIVER_ERROR("Invalid parameter format: "
+                                              + string(name), 1);
                     }
                     break;
                 case '@' :
@@ -253,7 +291,57 @@ CDB_Connection::CDB_Connection(impl::CConnection* c)
 
 bool CDB_Connection::IsAlive()
 {
-    return (m_ConnImpl == 0) ? false : m_ConnImpl->IsAlive();
+    if (m_ConnImpl == NULL  ||  !m_ConnImpl->IsAlive()) {
+        return false;
+    }
+
+    // Otherwise, try to confirm that the network connection hasn't closed.
+    // (Done only when no separate network library is necessary.)
+    // XXX - consider caching GetLowLevelHandle result, or at least
+    // availability.
+#ifndef NCBI_OS_SOLARIS
+    try {
+        I_ConnectionExtra::TSockHandle s = m_ConnImpl->GetLowLevelHandle();
+        char c;
+#  ifdef NCBI_OS_UNIX
+        // On Windows, the non-blocking flag is write-only(!); leave it
+        // alone, since only the ftds driver implements
+        // GetLowLevelHandle, and FreeTDS normally enables non-blocking
+        // mode itself.  (It does so on Unix too, but explicitly
+        // restoring settings is still best practice.)
+        int orig_flags = fcntl(s, F_GETFL, 0);
+        fcntl(s, F_SETFL, orig_flags | O_NONBLOCK);
+#  endif
+        int n = recv(s, &c, 1, MSG_PEEK);
+#  ifdef NCBI_OS_UNIX
+        if ((orig_flags & O_NONBLOCK) != O_NONBLOCK) {
+            fcntl(s, F_SETFL, orig_flags);
+        }
+#  endif
+        if (n > 0) {
+            return true; // open, with unread data available
+        } else if (n == 0) {
+            return false; // closed
+        } else {
+#  ifdef NCBI_OS_MSWIN
+            return WSAGetLastError() == WSAEWOULDBLOCK;
+#  else
+            switch (errno) {
+            case EAGAIN:
+#    if defined(EWOULDBLOCK)  &&  EWOULDBLOCK != EAGAIN
+            case EWOULDBLOCK:
+#    endif
+                return true; // open, but no data immediately available
+            default:
+                return false; // something else is wrong
+            }
+#  endif
+        }
+    } catch (CDB_Exception&) { // Presumably unimplemented
+        return true;
+    }
+#endif    
+    return true;
 }
 
 #define CHECK_CONNECTION( conn ) \
@@ -262,18 +350,21 @@ bool CDB_Connection::IsAlive()
 CDB_LangCmd* CDB_Connection::LangCmd(const string& lang_query)
 {
     CHECK_CONNECTION(m_ConnImpl);
+    _TRACE("Sending SQL: " << lang_query);
     return m_ConnImpl->LangCmd(lang_query);
 }
 
 CDB_RPCCmd* CDB_Connection::RPC(const string& rpc_name)
 {
     CHECK_CONNECTION(m_ConnImpl);
+    // _TRACE-d in CDB_RPCCmd::Send, which performs the actual I/O.
     return m_ConnImpl->RPC(rpc_name);
 }
 
 CDB_BCPInCmd* CDB_Connection::BCPIn(const string& table_name)
 {
     CHECK_CONNECTION(m_ConnImpl);
+    _TRACE("Performing bulk insertion into " << table_name);
     return m_ConnImpl->BCPIn(table_name);
 }
 
@@ -282,6 +373,7 @@ CDB_CursorCmd* CDB_Connection::Cursor(const string& cursor_name,
                                       unsigned int  batch_size)
 {
     CHECK_CONNECTION(m_ConnImpl);
+    _TRACE("Opening cursor for " << query);
     return m_ConnImpl->Cursor(cursor_name, query, batch_size);
 }
 
@@ -291,18 +383,25 @@ CDB_SendDataCmd* CDB_Connection::SendDataCmd(I_ITDescriptor& desc,
                                              bool dump_results)
 {
     CHECK_CONNECTION(m_ConnImpl);
+    _TRACE("Sending " << data_size << " byte(s) of data");
     return m_ConnImpl->SendDataCmd(desc, data_size, log_it, dump_results);
 }
 
 bool CDB_Connection::SendData(I_ITDescriptor& desc, CDB_Stream& lob, bool log_it)
 {
     CHECK_CONNECTION(m_ConnImpl);
+    _TRACE("Sending " << lob.Size() << " byte(s) of data");
     return m_ConnImpl->SendData(desc, lob, log_it);
 }
 
 void CDB_Connection::SetDatabaseName(const string& name)
 {
+    if (name.empty()) {
+        return;
+    }
+
     CHECK_CONNECTION(m_ConnImpl);
+    _TRACE("Now using database " << name);
     m_ConnImpl->SetDatabaseName(name);
 }
 
@@ -613,6 +712,8 @@ CDBParams& CDB_LangCmd::GetDefineParams(void)
 bool CDB_LangCmd::Send()
 {
     CHECK_COMMAND( m_CmdImpl );
+    TRACE_PARAMS(m_CmdImpl->GetBindParams());
+    m_CmdImpl->SaveInParams();
     return m_CmdImpl->Send();
 }
 
@@ -705,6 +806,9 @@ CDBParams& CDB_RPCCmd::GetDefineParams(void)
 bool CDB_RPCCmd::Send()
 {
     CHECK_COMMAND( m_CmdImpl );
+    _TRACE("Calling remote procedure " << GetProcName());
+    TRACE_PARAMS(m_CmdImpl->GetBindParams());
+    m_CmdImpl->SaveInParams();
     return m_CmdImpl->Send();
 }
 
@@ -832,6 +936,10 @@ bool CDB_BCPInCmd::Bind(unsigned int column_num, CDB_Object* value)
 bool CDB_BCPInCmd::SendRow()
 {
     CHECK_COMMAND( m_CmdImpl );
+    if (m_CmdImpl->m_RowsSent++ == 0) {
+        TRACE_PARAMS(m_CmdImpl->GetBindParams());
+    }
+    m_CmdImpl->SaveInParams();
     return m_CmdImpl->Send();
 }
 
@@ -844,12 +952,19 @@ bool CDB_BCPInCmd::Cancel()
 bool CDB_BCPInCmd::CompleteBatch()
 {
     CHECK_COMMAND( m_CmdImpl );
+    if (m_CmdImpl->m_BatchesSent++ == 0  &&  m_CmdImpl->m_RowsSent > 1) {
+        _TRACE("Sent a batch of " << m_CmdImpl->m_RowsSent << " rows");
+    }
     return m_CmdImpl->CommitBCPTrans();
 }
 
 bool CDB_BCPInCmd::CompleteBCP()
 {
     CHECK_COMMAND( m_CmdImpl );
+    if (m_CmdImpl->m_BatchesSent > 1) {
+        _TRACE("Sent " << m_CmdImpl->m_RowsSent << " rows, in "
+               << m_CmdImpl->m_BatchesSent << " batches");
+    }
     return m_CmdImpl->EndBCP();
 }
 
@@ -895,12 +1010,14 @@ CDBParams& CDB_CursorCmd::GetDefineParams(void)
 CDB_Result* CDB_CursorCmd::Open()
 {
     CHECK_COMMAND( m_CmdImpl );
+    TRACE_PARAMS(m_CmdImpl->GetBindParams());
     return m_CmdImpl->OpenCursor();
 }
 
 bool CDB_CursorCmd::Update(const string& table_name, const string& upd_query)
 {
     CHECK_COMMAND( m_CmdImpl );
+    m_CmdImpl->SaveInParams();
     return m_CmdImpl->Update(table_name, upd_query);
 }
 
@@ -1099,13 +1216,21 @@ void CDB_ResultProcessor::ReleaseConn(void)
 
 
 ////////////////////////////////////////////////////////////////////////////////
-CAutoTrans::CAutoTrans(CDB_Connection& connection)
+CAutoTrans::CAutoTrans(const CSubject& subject)
 : m_Abort(true)
-, m_Conn(connection)
+, m_Conn(subject.m_Connection)
 , m_TranCount(0)
 {
     BeginTransaction();
     m_TranCount = GetTranCount();
+    if (m_TranCount > 1) {
+        m_SavepointName = "ncbi_dbapi_txn_"
+            + NStr::NumericToString(reinterpret_cast<intptr_t>(this), 0, 16);
+        auto_ptr<CDB_LangCmd> auto_stmt
+            (m_Conn.LangCmd("SAVE TRANSACTION " + m_SavepointName));
+        auto_stmt->Send();
+        auto_stmt->DumpResults();
+    }
 }
 
 
@@ -1157,9 +1282,17 @@ CAutoTrans::Commit(void)
 void
 CAutoTrans::Rollback(void)
 {
-    auto_ptr<CDB_LangCmd> auto_stmt(m_Conn.LangCmd("ROLLBACK"));
+    auto_ptr<CDB_LangCmd> auto_stmt
+        (m_Conn.LangCmd("ROLLBACK TRANSACTION " + m_SavepointName));
     auto_stmt->Send();
     auto_stmt->DumpResults();
+    if (m_SavepointName.empty()) {
+        _ASSERT(m_TranCount == 1);
+    } else {
+        // Formally unwind via an empty commit, as a rollback would
+        // also cancel outer transactions.
+        Commit();
+    }
 }
 
 
