@@ -1,4 +1,4 @@
-/* $Id: variant.cpp 408471 2013-07-30 18:55:35Z ucko $
+/* $Id: variant.cpp 420801 2013-12-05 16:20:55Z ucko $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -23,7 +23,7 @@
 *
 * ===========================================================================
 *
-* File Name:  $Id: variant.cpp 408471 2013-07-30 18:55:35Z ucko $
+* File Name:  $Id: variant.cpp 420801 2013-12-05 16:20:55Z ucko $
 *
 * Author:  Michael Kholodov
 *
@@ -238,7 +238,9 @@ CVariant::CVariant(EDB_Type type, size_t size)
     case eDB_UnsupportedType:
         break;
     }
-    NCBI_THROW(CVariantException, eVariant, "Unsupported type");
+    NCBI_THROW(CVariantException, eVariant,
+               string("Unsupported type: ")
+               + CDB_Object::GetTypeName(type, false));
 }
 
 
@@ -301,7 +303,9 @@ CVariant::CVariant(const CTime& v, EDateTimeFormat fmt)
         m_data = new CDB_DateTime(v);
         break;
     default:
-        NCBI_THROW(CVariantException, eVariant, "CVariant::ctor(): unsupported datetime type");
+        NCBI_THROW(CVariantException, eVariant,
+                   "CVariant::ctor(): unsupported datetime type "
+                   + NStr::IntToString(fmt));
     }
 
     if (v.IsEmpty()) {
@@ -834,7 +838,10 @@ bool operator<(const CVariant& v1, const CVariant& v2)
     }
     else {
         if( v1.GetType() != v2.GetType() ) {
-            NCBI_THROW(CVariantException, eVariant, "Cannot compare different types");
+            NCBI_THROW(CVariantException, eVariant,
+                       string("Cannot compare different types ")
+                       + CDB_Object::GetTypeName(v1.GetType(), false) + " and "
+                       + CDB_Object::GetTypeName(v2.GetType(), false));
         }
 
         switch( v1.GetType() ) {
@@ -866,7 +873,9 @@ bool operator<(const CVariant& v1, const CVariant& v2)
             less = v1.GetCTime() < v2.GetCTime();
             break;
         default:
-            NCBI_THROW(CVariantException, eVariant, "Type not supported");
+            NCBI_THROW(CVariantException, eVariant,
+                       string("Type not supported: ")
+                       + CDB_Object::GetTypeName(v1.GetType(), false));
         }
     }
     return less;
@@ -881,7 +890,10 @@ bool operator==(const CVariant& v1, const CVariant& v2)
     }
     else {
         if( v1.GetType() != v2.GetType() ) {
-            NCBI_THROW(CVariantException, eVariant, "Cannot compare different types");
+            NCBI_THROW(CVariantException, eVariant,
+                       string("Cannot compare different types ")
+                       + CDB_Object::GetTypeName(v1.GetType(), false) + " and "
+                       + CDB_Object::GetTypeName(v2.GetType(), false));
         }
 
         switch( v1.GetType() ) {
@@ -918,7 +930,9 @@ bool operator==(const CVariant& v1, const CVariant& v2)
             less = v1.GetCTime() == v2.GetCTime();
             break;
         default:
-            NCBI_THROW(CVariantException, eVariant, "Type not supported");
+            NCBI_THROW(CVariantException, eVariant,
+                       string("Type not supported: ")
+                       + CDB_Object::GetTypeName(v1.GetType(), false));
         }
     }
     return less;
