@@ -1,4 +1,4 @@
-/*  $Id: names.cpp 403214 2013-06-13 13:25:50Z fongah2 $
+/*  $Id: names.cpp 431529 2014-04-04 14:15:53Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -136,6 +136,9 @@ CBlast4Field & CBlast4Field::Get(EBlastOptIdx opt)
         case eBlastOpt_PercentIdentity:
             m_Fields[opt] = CBlast4Field("PercentIdentity",       CBlast4_value::e_Real);
             break;
+        case eBlastOpt_QueryCovHspPerc:
+            m_Fields[opt] = CBlast4Field("QueryCovHspPerc",       CBlast4_value::e_Real);
+            break;
         case eBlastOpt_SumStatisticsMode:
             m_Fields[opt] = CBlast4Field("SumStatistics",         CBlast4_value::e_Boolean);
             break;
