@@ -1,4 +1,4 @@
-/*  $Id: data_source.cpp 419128 2013-11-21 20:27:51Z vasilche $
+/*  $Id: data_source.cpp 448567 2014-10-07 16:45:01Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -144,7 +144,7 @@ void CDataSource::DropAllTSEs(void)
 
     // then drop all TSEs
     {{
-        TMainLock::TWriteLockGuard guard2(m_DSCacheLock);
+        TCacheLock::TWriteLockGuard guard2(m_DSCacheLock);
         // check if any TSE is locked by user 
         ITERATE ( TBlob_Map, it, m_Blob_Map ) {
             int lock_counter = it->second->m_LockCounter.Get();
@@ -246,7 +246,7 @@ CDataSource::TTSE_Lock CDataSource::AddTSE(CRef<CTSE_Info> info)
     _ASSERT(!info->IsLocked());
     _ASSERT(!info->HasDataSource());
     TMainLock::TWriteLockGuard guard(m_DSMainLock);
-    TMainLock::TWriteLockGuard guard2(m_DSCacheLock);
+    TCacheLock::TWriteLockGuard guard2(m_DSCacheLock);
     _ASSERT(!info->IsLocked());
     _ASSERT(!info->HasDataSource());
     TBlobId blob_id = info->GetBlobId();
@@ -382,6 +382,21 @@ CDataSource::FindTSE_Lock(const CSeq_entry& tse,
 }
 
 
+CDataSource::TSeq_entry_Lock
+CDataSource::GetSeq_entry_Lock(const CBlobIdKey& blob_id)
+{
+    TSeq_entry_Lock ret;
+    {{
+        TMainLock::TReadLockGuard guard(m_DSMainLock);
+        ret.first = m_Loader->GetBlobById(blob_id);
+        if ( ret.first ) {
+            x_SetLock(ret.second, ConstRef(&ret.first->GetTSE_Info()));
+        }
+    }}
+    return ret;
+}
+
+
 CDataSource::TSeq_entry_Lock
 CDataSource::FindSeq_entry_Lock(const CSeq_entry& entry,
                                 const TTSE_LockSet& /*history*/) const
@@ -488,7 +503,7 @@ void CDataSource::GetLoadedBlob_ids(const CSeq_id_Handle& idh,
         }
     }
     if ( types & fLoaded_annots ) {
-        TMainLock::TReadLockGuard guard(m_DSAnnotLock);
+        TAnnotLock::TReadLockGuard guard(m_DSAnnotLock);
         if ( types & fLoaded_bioseq_annots ) {
             TSeq_id2TSE_Set::const_iterator tse_set =
                 m_TSE_seq_annot.find(idh);
@@ -1275,6 +1290,21 @@ CSeq_inst::TMol CDataSource::GetSequenceType(const CSeq_id_Handle& idh)
 }
 
 
+int CDataSource::GetSequenceState(const CSeq_id_Handle& idh)
+{
+    int ret = CBioseq_Handle::fState_not_found|CBioseq_Handle::fState_no_data;
+    TTSE_LockSet locks;
+    SSeqMatch_DS match = x_GetSeqMatch(idh, locks);
+    if ( match ) {
+        ret = match.m_Bioseq->GetTSE_Info().GetBlobState();
+    }
+    else if ( m_Loader ) {
+        ret = m_Loader->GetSequenceState(idh);
+    }
+    return ret;
+}
+
+
 void CDataSource::GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret)
 {
     size_t count = ids.size(), remaining = 0;
@@ -1427,6 +1457,32 @@ void CDataSource::GetSequenceTypes(const TIds& ids, TLoaded& loaded,
 }
 
 
+void CDataSource::GetSequenceStates(const TIds& ids, TLoaded& loaded,
+                                    TSequenceStates& ret)
+{
+    size_t count = ids.size(), remaining = 0;
+    _ASSERT(ids.size() == loaded.size());
+    _ASSERT(ids.size() == ret.size());
+    TTSE_LockSet locks;
+    for ( size_t i = 0; i < count; ++i ) {
+        if ( loaded[i] ) {
+            continue;
+        }
+        SSeqMatch_DS match = x_GetSeqMatch(ids[i], locks);
+        if ( match ) {
+            ret[i] = match.m_Bioseq->GetTSE_Info().GetBlobState();
+            loaded[i] = true;
+        }
+        else {
+            ++remaining;
+        }
+    }
+    if ( remaining && m_Loader ) {
+        m_Loader->GetSequenceStates(ids, loaded, ret);
+    }
+}
+
+
 void CDataSource::GetBlobs(TSeqMatchMap& match_map)
 {
     if ( match_map.empty() ) {
@@ -1575,7 +1631,7 @@ CTSE_LoadLock CDataSource::GetTSE_LoadLock(const TBlobId& blob_id)
         CTSE_Lock lock;
         CRef<CTSE_Info::CLoadMutex> load_mutex;
         {{
-            TMainLock::TWriteLockGuard guard(m_DSCacheLock);
+            TCacheLock::TWriteLockGuard guard(m_DSCacheLock);
             TTSE_Ref& slot = m_Blob_Map[blob_id];
             if ( !slot ) {
                 slot.Reset(new CTSE_Info(blob_id));
@@ -1599,7 +1655,7 @@ CTSE_LoadLock CDataSource::GetTSE_LoadLockIfLoaded(const TBlobId& blob_id)
     {{
         CTSE_Lock lock;
         {{
-            TMainLock::TWriteLockGuard guard(m_DSCacheLock);
+            TCacheLock::TWriteLockGuard guard(m_DSCacheLock);
             TBlob_Map::const_iterator iter = m_Blob_Map.find(blob_id);
             if ( iter == m_Blob_Map.end() || !iter->second ||
                  !IsLoaded(*iter->second) ) {
@@ -1637,7 +1693,7 @@ void CDataSource::SetLoaded(CTSE_LoadLock& lock)
         detach_guard.Attach(this, &*lock);
     }}
     {{
-        TMainLock::TWriteLockGuard guard2(m_DSCacheLock);
+        TCacheLock::TWriteLockGuard guard2(m_DSCacheLock);
         lock->m_LoadState = CTSE_Info::eLoaded;
         lock->m_LoadMutex.Reset();
     }}
@@ -1651,9 +1707,8 @@ NCBI_PARAM_DEF_EX(unsigned, OBJMGR, BLOB_CACHE, 10,
 
 static unsigned s_GetCacheSize(void)
 {
-    static const unsigned sx_Value =
-        NCBI_PARAM_TYPE(OBJMGR, BLOB_CACHE)::GetDefault();
-    return sx_Value;
+    static CSafeStatic<NCBI_PARAM_TYPE(OBJMGR, BLOB_CACHE)> sx_Value;
+    return sx_Value->Get();
 }
 
 
@@ -1666,7 +1721,7 @@ void CDataSource::x_ReleaseLastTSELock(CRef<CTSE_Info> tse)
     _ASSERT(tse);
     vector<TTSE_Ref> to_delete;
     {{
-        TMainLock::TWriteLockGuard guard(m_DSCacheLock);
+        TCacheLock::TWriteLockGuard guard(m_DSCacheLock);
         if ( tse->IsLocked() ) { // already locked again
             return;
         }
@@ -1715,7 +1770,7 @@ void CDataSource::x_SetLock(CTSE_Lock& lock, CConstRef<CTSE_Info> tse) const
         return;
     }
 
-    TMainLock::TWriteLockGuard guard(m_DSCacheLock);
+    TCacheLock::TWriteLockGuard guard(m_DSCacheLock);
     if ( tse->m_CacheState == CTSE_Info::eInCache ) {
         _ASSERT(tse->m_CachePosition ==
                 find(m_Blob_Cache.begin(), m_Blob_Cache.end(), tse));
@@ -1820,23 +1875,7 @@ void CDataSource::x_ReleaseLastLoadLock(CTSE_LoadLock& lock)
     CRef<CTSE_Info> info = lock.m_Info;
     if ( info->m_LoadState == CTSE_Info::eNotLoaded ) {
         // reset TSE info into empty state
-        info->m_Bioseq_sets.clear();
-        info->m_Bioseqs.clear();
-        info->m_Removed_Bioseq_sets.clear();
-        info->m_Removed_Bioseqs.clear();
-        info->m_Split.Reset();
-        info->m_SetObjectInfo.Reset();
-        info->m_NamedAnnotObjs.clear();
-        info->m_IdAnnotInfoMap.clear();
-        info->m_FeatIdIndex.clear();
-        info->m_BaseTSE.reset();
-        info->m_EditSaver.Reset();
-        info->m_InternalBioObjNumber = 0;
-        info->m_BioObjects.clear();
-            
-        info->m_Object.Reset();
-        info->m_Which = CSeq_entry::e_not_set;
-        info->m_Contents.Reset();
+        info->x_Reset();
     }
     lock.m_Info.Reset();
     lock.m_DataSource.Reset();
