@@ -1,4 +1,4 @@
-/*  $Id: seq_feat_handle.cpp 397491 2013-04-26 17:42:27Z kornbluh $
+/*  $Id: seq_feat_handle.cpp 448093 2014-10-02 14:58:15Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -68,7 +68,7 @@ CSeq_feat_Handle::CSeq_feat_Handle(const CSeq_annot_Handle& annot,
                                    const SSNP_Info& snp_info,
                                    CCreatedFeat_Ref& created_ref)
     : m_Seq_annot(annot),
-      m_FeatIndex(annot.x_GetInfo().x_GetSNP_annot_Info().GetIndex(snp_info)
+      m_FeatIndex(TFeatIndex(annot.x_GetInfo().x_GetSNP_annot_Info().GetIndex(snp_info))
                   | kSNPTableBit),
       m_CreatedFeat(&created_ref)
 {
