@@ -1,4 +1,4 @@
-/*  $Id: tse_chunk_info.cpp 411339 2013-08-27 20:46:42Z vasilche $
+/*  $Id: tse_chunk_info.cpp 414605 2013-09-25 18:28:39Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -33,6 +33,7 @@
 
 #include <ncbi_pch.hpp>
 #include <objmgr/impl/tse_chunk_info.hpp>
+#include <objmgr/impl/tse_chunk_info.hpp>
 #include <objmgr/impl/tse_split_info.hpp>
 #include <objmgr/impl/tse_info.hpp>
 #include <objmgr/impl/tse_assigner.hpp>
@@ -45,8 +46,14 @@
 #include <objects/seq/Seq_literal.hpp>
 #include <objmgr/seq_map.hpp>
 #include <algorithm>
+#include <objmgr/error_codes.hpp>
+
+#define NCBI_USE_ERRCODE_X   ObjMgr_ObjSplitInfo
 
 BEGIN_NCBI_SCOPE
+
+NCBI_DEFINE_ERR_SUBCODE_X(11);
+
 BEGIN_SCOPE(objects)
 
 /////////////////////////////////////////////////////////////////////////////
@@ -490,6 +497,17 @@ void CTSE_Chunk_Info::x_InitObjectIndexList(void)
             ITERATE ( TLocationSet, lit, tit->second ) {
                 key.m_Handle = lit->first;
                 key.m_Range = lit->second;
+                if ( key.m_Range.Empty() ) {
+                    string name;
+                    if ( it->first.IsNamed() ) {
+                        name = " "+it->first.GetName();
+                    }
+                    ERR_POST_X(11, "Empty region in split-info on "<<
+                               key.m_Handle<<name<<" "<<
+                               GetSplitInfo().GetBlobId());
+                    // do not believe empty range - it must be an error
+                    key.m_Range = key.m_Range.GetWhole();
+                }
                 infos.AddMap(key, index);
             }
             size_t keys_end = infos.GetKeys().size();
