@@ -1,4 +1,4 @@
-/*  $Id: build_db.cpp 416466 2013-10-28 19:08:42Z rafanovi $
+/*  $Id: build_db.cpp 443518 2014-08-15 01:32:43Z mcelhany $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -31,7 +31,7 @@
   Code to build a database given various sources of sequence data.
 */
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: build_db.cpp 416466 2013-10-28 19:08:42Z rafanovi $";
+static char const rcsid[] = "$Id: build_db.cpp 443518 2014-08-15 01:32:43Z mcelhany $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -305,7 +305,7 @@ static CConstRef<CBioseq> s_FixBioseqDeltas(CConstRef<objects::CBioseq> bs)
         string msg = CNcbiOstrstreamToString(oss);
         NCBI_THROW(CMultisourceException, eArg, msg);
     }
-    
+
     try {
         const CDelta_ext & dext = bs->GetInst().GetExt().GetDelta();
         
@@ -326,6 +326,11 @@ static CConstRef<CBioseq> s_FixBioseqDeltas(CConstRef<objects::CBioseq> bs)
         string na8;
         
         ITERATE(TItems, item, dext.Get()) {
+        	if(((**item).IsLoc()) && ((**item).GetLoc().IsNull())) {
+                seq8na.append(1, 0x0f);
+        		continue;
+        	}
+
             const CSeq_literal & L = (**item).GetLiteral();
 
             if (!L.CanGetSeq_data()) {
@@ -700,13 +705,13 @@ CFastaBioseqSource::CFastaBioseqSource(CNcbiIstream & fasta_file,
     typedef CFastaReader::EFlags TFlags;
     
     int iflags = CFastaReader::fAllSeqIds |
-                 CFastaReader::fForceType |
-                 CFastaReader::fParseGaps;
+                 CFastaReader::fForceType;
     
     if (is_protein) {
         iflags |= CFastaReader::fAssumeProt;
     } else {
         iflags |= CFastaReader::fAssumeNuc;
+        iflags |=  CFastaReader::fParseGaps;
     }
     
     if (parse_ids) {
@@ -714,10 +719,16 @@ CFastaBioseqSource::CFastaBioseqSource(CNcbiIstream & fasta_file,
     } else {
         iflags |= CFastaReader::fNoParseID;
     }
+
+    iflags |= CFastaReader::fQuickIDCheck;
     
     TFlags flags = (TFlags) iflags;
     
     m_FastaReader = new CFastaReader(*m_LineReader, flags);
+    m_FastaReader->IgnoreProblem(ILineError::eProblem_ModifierFoundButNoneExpected);
+    m_FastaReader->IgnoreProblem(ILineError::eProblem_TooManyAmbiguousResidues);
+    m_FastaReader->IgnoreProblem(ILineError::eProblem_TooLong);
+    
 }
 
 CFastaBioseqSource::~CFastaBioseqSource()
@@ -1334,6 +1345,7 @@ bool CBuildDatabase::x_EndBuild(bool erase, const CException * close_exception)
             m_LogFile << "file: " << *iterf << endl;
             if (erase) {
                 CFile(*iterf).Remove();
+                _TRACE("Deleting " << *iterf);
             }
         }
     }
