@@ -1,4 +1,4 @@
-/*  $Id: cached_sequence.cpp 368104 2012-07-02 20:05:54Z rafanovi $
+/*  $Id: cached_sequence.cpp 401240 2013-05-29 14:08:40Z madden $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
  * Defines the CCachedSequence class
  */
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: cached_sequence.cpp 368104 2012-07-02 20:05:54Z rafanovi $";
+static char const rcsid[] = "$Id: cached_sequence.cpp 401240 2013-05-29 14:08:40Z madden $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -89,8 +89,8 @@ CCachedSequence::CCachedSequence(IBlastDbAdapter& db,
     m_TSE.Reset();
     m_Length = m_BlastDb.GetSeqLength(m_OID);
     
-    CRef<CBioseq> bioseq(m_BlastDb.GetBioseqNoData(m_OID,
-                                                  m_SIH.IsGi()? m_SIH.GetGi(): 0));
+    CRef<CBioseq> bioseq(m_BlastDb.GetBioseqNoData(m_OID, 0, &*(m_SIH.GetSeqId())));
+
     s_ReplaceProvidedSeqIdsForRequestedSeqIds(m_SIH, *bioseq);
     
     CConstRef<CSeq_id> first_id( bioseq->GetFirstId() );
