@@ -1,4 +1,4 @@
-/*  $Id: remote_blastdb_adapter.cpp 401240 2013-05-29 14:08:40Z madden $
+/*  $Id: remote_blastdb_adapter.cpp 435003 2014-05-13 15:08:30Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,7 +32,7 @@
  * Defines the CRemoteBlastDbAdapter class
  */
 #ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: remote_blastdb_adapter.cpp 401240 2013-05-29 14:08:40Z madden $";
+static char const rcsid[] = "$Id: remote_blastdb_adapter.cpp 435003 2014-05-13 15:08:30Z ucko $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <ncbi_pch.hpp>
@@ -57,7 +57,7 @@ CRemoteBlastDbAdapter::CRemoteBlastDbAdapter(const string& db_name,
     CBlastServices rmt_svc;
     const bool kIsProtein = (db_type == CSeqDB::eProtein) ? true : false;
     if ( !rmt_svc.IsValidBlastDb(db_name, kIsProtein) ) {
-        ostrstream out;
+        CNcbiOstrstream out;
         out << (kIsProtein ? "Protein" : "Nucleotide") <<  " BLAST database "
             << "'" << db_name << "' does not exist in the NCBI servers";
         NCBI_THROW(CSeqDBException, eArgErr, CNcbiOstrstreamToString(out));
