@@ -1,4 +1,4 @@
-/*  $Id: gbloader.cpp 406820 2013-07-16 15:10:40Z vasilche $
+/*  $Id: gbloader.cpp 447987 2014-10-01 19:13:39Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -33,8 +33,8 @@
 #include <corelib/ncbi_param.hpp>
 #include <objtools/data_loaders/genbank/gbloader.hpp>
 #include <objtools/data_loaders/genbank/gbloader_params.h>
-#include <objtools/data_loaders/genbank/dispatcher.hpp>
-#include <objtools/data_loaders/genbank/request_result.hpp>
+#include <objtools/data_loaders/genbank/impl/dispatcher.hpp>
+#include <objtools/data_loaders/genbank/impl/request_result.hpp>
 
 #include <objtools/data_loaders/genbank/reader_interface.hpp>
 #include <objtools/data_loaders/genbank/writer_interface.hpp>
@@ -86,7 +86,6 @@ NCBI_PARAM_DEF_EX(string, GENBANK, LOADER_METHOD, kEmptyStr,
                   eParam_NoThread, GENBANK_LOADER_METHOD);
 typedef NCBI_PARAM_TYPE(GENBANK, LOADER_METHOD) TGenbankLoaderMethod;
 
-
 #if defined(HAVE_PUBSEQ_OS)
 static const char* const DEFAULT_DRV_ORDER = "ID2:PUBSEQOS:ID1";
 #else
@@ -95,7 +94,8 @@ static const char* const DEFAULT_DRV_ORDER = "ID2:ID1";
 
 #define GBLOADER_NAME "GBLOADER"
 
-#define DEFAULT_ID_GC_SIZE 1000
+#define DEFAULT_ID_GC_SIZE 10000
+#define DEFAULT_ID_EXPIRATION_TIMEOUT 2*3600 // 2 hours
 
 class CGBReaderRequestResult : public CReaderRequestResult
 {
@@ -113,9 +113,6 @@ public:
 
     //virtual TConn GetConn(void);
     //virtual void ReleaseConn(void);
-    virtual CRef<CLoadInfoSeq_ids> GetInfoSeq_ids(const string& id);
-    virtual CRef<CLoadInfoSeq_ids> GetInfoSeq_ids(const CSeq_id_Handle& id);
-    virtual CRef<CLoadInfoBlob_ids> GetInfoBlob_ids(const TKeyBlob_ids& id);
     virtual CTSE_LoadLock GetTSE_LoadLock(const TKeyBlob& blob_id);
     virtual CTSE_LoadLock GetTSE_LoadLockIfLoaded(const TKeyBlob& blob_id);
     virtual void GetLoadedBlob_ids(const CSeq_id_Handle& idh,
@@ -125,6 +122,10 @@ public:
 
     CInitMutexPool& GetMutexPool(void) { return m_Loader->m_MutexPool; }
 
+    virtual TExpirationTime GetIdExpirationTimeout(void) const;
+
+    virtual bool GetAddWGSMasterDescr(void) const;
+
     friend class CGBDataLoader;
 
 private:
@@ -342,18 +343,6 @@ namespace {
         }
         return 0;
     }
-
-    void DumpParams(const TParams* params, const string& prefix = "")
-    {
-        if ( params ) {
-            for ( TParams::TNodeList_CI it = params->SubNodeBegin();
-                  it != params->SubNodeEnd(); ++it ) {
-                NcbiCout << prefix << (*it)->GetKey() << " = " << (*it)->GetValue().value
-                         << NcbiEndl;
-                DumpParams(static_cast<const TParams*>(*it), prefix + "  ");
-            }
-        }
-    }
 }
 
 
@@ -477,11 +466,61 @@ void CGBDataLoader::x_CreateDriver(const CGBLoaderParams& params)
         catch ( CException& /*ignored*/ ) {
         }
     }
-    m_LoadMapSeq_ids.SetMaxSize(queue_size);
-    m_LoadMapSeq_ids2.SetMaxSize(queue_size);
-    m_LoadMapBlob_ids.SetMaxSize(queue_size);
+
+    m_IdExpirationTimeout = DEFAULT_ID_EXPIRATION_TIMEOUT;
+    if ( gb_params ) {
+        string param =
+            GetParam(gb_params, NCBI_GBLOADER_PARAM_ID_EXPIRATION_TIMEOUT);
+        if ( !param.empty() ) {
+            try {
+                Uint4 timeout = NStr::StringToNumeric<Uint4>(param);
+                if ( timeout > 0 ) {
+                    m_IdExpirationTimeout = timeout;
+                }
+            }
+            catch ( CException& exc ) {
+                NCBI_RETHROW_FMT(exc, CLoaderException, eBadConfig,
+                                 "Bad value of parameter "
+                                 NCBI_GBLOADER_PARAM_ID_EXPIRATION_TIMEOUT
+                                 ": \""<<param<<"\"");
+            }
+        }
+    }
+    m_AlwaysLoadExternal = false;
+    if ( gb_params ) {
+        string param =
+            GetParam(gb_params, NCBI_GBLOADER_PARAM_ALWAYS_LOAD_EXTERNAL);
+        if ( !param.empty() ) {
+            try {
+                m_AlwaysLoadExternal = NStr::StringToBool(param);
+            }
+            catch ( CException& exc ) {
+                NCBI_RETHROW_FMT(exc, CLoaderException, eBadConfig,
+                                 "Bad value of parameter "
+                                 NCBI_GBLOADER_PARAM_ALWAYS_LOAD_EXTERNAL
+                                 ": \""<<param<<"\"");
+            }
+        }
+    }
+    m_AddWGSMasterDescr = true;
+    if ( gb_params ) {
+        string param =
+            GetParam(gb_params, NCBI_GBLOADER_PARAM_ADD_WGS_MASTER);
+        if ( !param.empty() ) {
+            try {
+                m_AddWGSMasterDescr = NStr::StringToBool(param);
+            }
+            catch ( CException& exc ) {
+                NCBI_RETHROW_FMT(exc, CLoaderException, eBadConfig,
+                                 "Bad value of parameter "
+                                 NCBI_GBLOADER_PARAM_ADD_WGS_MASTER
+                                 ": \""<<param<<"\"");
+            }
+        }
+    }
     
     m_Dispatcher = new CReadDispatcher;
+    m_InfoManager = new CGBInfoManager(queue_size);
     
     // now we create readers & writers
     if ( params.GetReaderPtr() ) {
@@ -701,7 +740,7 @@ CConstRef<CSeqref> CGBDataLoader::GetSatSatkey(const CSeq_id_Handle& sih)
     TBlobId id = GetBlobId(sih);
     if ( id ) {
         const TRealBlobId& blob_id = GetRealBlobId(id);
-        return ConstRef(new CSeqref(0, blob_id.GetSat(), blob_id.GetSatKey()));
+        return ConstRef(new CSeqref(ZERO_GI, blob_id.GetSat(), blob_id.GetSatKey()));
     }
     return CConstRef<CSeqref>();
 }
@@ -715,15 +754,16 @@ CConstRef<CSeqref> CGBDataLoader::GetSatSatkey(const CSeq_id& id)
 
 CDataLoader::TBlobId CGBDataLoader::GetBlobId(const CSeq_id_Handle& sih)
 {
+    if ( CReadDispatcher::CannotProcess(sih) ) {
+        return TBlobId();
+    }
     CGBReaderRequestResult result(this, sih);
-    CLoadLockSeq_ids seq_ids(result, sih);
-    CLoadLockBlob_ids blobs(result, sih, 0);
+    CLoadLockBlobIds blobs(result, sih, 0);
     m_Dispatcher->LoadSeq_idBlob_ids(result, sih, 0);
-
-    ITERATE ( CLoadInfoBlob_ids, it, *blobs ) {
-        const CBlob_Info& info = it->second;
-        if ( info.GetContentsMask() & fBlobHasCore ) {
-            return TBlobId(it->first.GetPointer());
+    CFixedBlob_ids blob_ids = blobs.GetBlob_ids();
+    ITERATE ( CFixedBlob_ids, it, blob_ids ) {
+        if ( it->Matches(fBlobHasCore, 0) ) {
+            return TBlobId(it->GetBlob_id().GetPointer());
         }
     }
     return TBlobId();
@@ -735,58 +775,85 @@ CDataLoader::TBlobId CGBDataLoader::GetBlobIdFromString(const string& str) const
 }
 
 
+CDataLoader::TBlobId CGBDataLoader::GetBlobIdFromSatSatKey(int sat,
+                                                           int sat_key,
+                                                           int sub_sat) const
+{
+    CRef<CBlob_id> blob_id(new CBlob_id);
+    blob_id->SetSat(sat);
+    blob_id->SetSatKey(sat_key);
+    blob_id->SetSubSat(sub_sat);
+    return TBlobId(blob_id);
+}
+
+
 void CGBDataLoader::GetIds(const CSeq_id_Handle& idh, TIds& ids)
 {
+    if ( CReadDispatcher::CannotProcess(idh) ) {
+        return;
+    }
     CGBReaderRequestResult result(this, idh);
-    CLoadLockSeq_ids seq_ids(result, idh);
-    if ( !seq_ids.IsLoaded() ) {
+    CLoadLockSeqIds lock(result, idh);
+    if ( !lock.IsLoaded() ) {
         m_Dispatcher->LoadSeq_idSeq_ids(result, idh);
     }
-    ids = seq_ids->m_Seq_ids;
+    ids = lock.GetSeq_ids();
 }
 
 
 CSeq_id_Handle CGBDataLoader::GetAccVer(const CSeq_id_Handle& idh)
 {
+    if ( CReadDispatcher::CannotProcess(idh) ) {
+        return null;
+    }
     CGBReaderRequestResult result(this, idh);
-    CLoadLockSeq_ids seq_ids(result, idh);
-    if ( !seq_ids->IsLoadedAccVer() ) {
+    CLoadLockAcc lock(result, idh);
+    if ( !lock.IsLoadedAccVer() ) {
         m_Dispatcher->LoadSeq_idAccVer(result, idh);
     }
-    return seq_ids->GetAccVer();
+    return lock.GetAccVer();
 }
 
 
 TGi CGBDataLoader::GetGi(const CSeq_id_Handle& idh)
 {
+    if ( CReadDispatcher::CannotProcess(idh) ) {
+        return ZERO_GI;
+    }
     CGBReaderRequestResult result(this, idh);
-    CLoadLockSeq_ids seq_ids(result, idh);
-    if ( !seq_ids->IsLoadedGi() ) {
+    CLoadLockGi lock(result, idh);
+    if ( !lock.IsLoadedGi() ) {
         m_Dispatcher->LoadSeq_idGi(result, idh);
     }
-    return seq_ids->GetGi();
+    return lock.GetGi();
 }
 
 
 string CGBDataLoader::GetLabel(const CSeq_id_Handle& idh)
 {
+    if ( CReadDispatcher::CannotProcess(idh) ) {
+        return string();
+    }
     CGBReaderRequestResult result(this, idh);
-    CLoadLockSeq_ids seq_ids(result, idh);
-    if ( !seq_ids->IsLoadedLabel() ) {
+    CLoadLockLabel lock(result, idh);
+    if ( !lock.IsLoadedLabel() ) {
         m_Dispatcher->LoadSeq_idLabel(result, idh);
     }
-    return seq_ids->GetLabel();
+    return lock.GetLabel();
 }
 
 
 int CGBDataLoader::GetTaxId(const CSeq_id_Handle& idh)
 {
+    if ( CReadDispatcher::CannotProcess(idh) ) {
+        return -1;
+    }
     CGBReaderRequestResult result(this, idh);
-    CLoadLockSeq_ids seq_ids(result, idh);
-    if ( !seq_ids->IsLoadedTaxId() ) {
+    CLoadLockTaxId lock(result, idh);
+    if ( !lock.IsLoadedTaxId() ) {
         m_Dispatcher->LoadSeq_idTaxId(result, idh);
     }
-    int taxid = seq_ids->IsLoadedTaxId()? seq_ids->GetTaxId(): -1;
+    int taxid = lock.IsLoadedTaxId()? lock.GetTaxId(): -1;
     if ( taxid == -1 ) {
         return CDataLoader::GetTaxId(idh);
     }
@@ -794,51 +861,237 @@ int CGBDataLoader::GetTaxId(const CSeq_id_Handle& idh)
 }
 
 
+int CGBDataLoader::GetSequenceState(const CSeq_id_Handle& sih)
+{
+    const int kNotFound = (CBioseq_Handle::fState_not_found |
+                           CBioseq_Handle::fState_no_data);
+
+    if ( CReadDispatcher::CannotProcess(sih) ) {
+        return kNotFound;
+    }
+    CGBReaderRequestResult result(this, sih);
+    CLoadLockBlobIds blobs(result, sih, 0);
+    m_Dispatcher->LoadSeq_idBlob_ids(result, sih, 0);
+    if ( blobs.GetState() & CBioseq_Handle::fState_no_data ) {
+        return blobs.GetState();
+    }
+
+    CFixedBlob_ids blob_ids = blobs.GetBlob_ids();
+    ITERATE ( CFixedBlob_ids, it, blob_ids ) {
+        if ( it->Matches(fBlobHasCore, 0) ) {
+            return blob_ids.GetState();
+        }
+    }
+    return kNotFound;
+}
+
+
 void CGBDataLoader::GetAccVers(const TIds& ids, TLoaded& loaded, TIds& ret)
 {
-    if ( std::find(loaded.begin(), loaded.end(), false) != loaded.end() ) {
-        CGBReaderRequestResult result(this, ids[0]);
+    for ( size_t i = 0; i < ids.size(); ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        CGBReaderRequestResult result(this, ids[i]);
         m_Dispatcher->LoadAccVers(result, ids, loaded, ret);
+        return;
     }
 }
 
 
 void CGBDataLoader::GetGis(const TIds& ids, TLoaded& loaded, TGis& ret)
 {
-    if ( std::find(loaded.begin(), loaded.end(), false) != loaded.end() ) {
-        CGBReaderRequestResult result(this, ids[0]);
+    for ( size_t i = 0; i < ids.size(); ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        CGBReaderRequestResult result(this, ids[i]);
         m_Dispatcher->LoadGis(result, ids, loaded, ret);
+        return;
     }
 }
 
 
 void CGBDataLoader::GetLabels(const TIds& ids, TLoaded& loaded, TLabels& ret)
 {
-    if ( std::find(loaded.begin(), loaded.end(), false) != loaded.end() ) {
-        CGBReaderRequestResult result(this, ids[0]);
+    for ( size_t i = 0; i < ids.size(); ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        CGBReaderRequestResult result(this, ids[i]);
         m_Dispatcher->LoadLabels(result, ids, loaded, ret);
+        return;
     }
 }
 
 
 void CGBDataLoader::GetTaxIds(const TIds& ids, TLoaded& loaded, TTaxIds& ret)
 {
-    if ( std::find(loaded.begin(), loaded.end(), false) != loaded.end() ) {
-        CGBReaderRequestResult result(this, ids[0]);
+    for ( size_t i = 0; i < ids.size(); ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        CGBReaderRequestResult result(this, ids[i]);
         m_Dispatcher->LoadTaxIds(result, ids, loaded, ret);
+        return;
     }
 }
 
 
+static const bool s_LoadBulkBlobs = true;
+
+
+void CGBDataLoader::GetSequenceLengths(const TIds& ids, TLoaded& loaded,
+                                       TSequenceLengths& ret)
+{
+    if ( !s_LoadBulkBlobs ) {
+        CDataLoader::GetSequenceLengths(ids, loaded, ret);
+        return;
+    }
+    if ( ids.empty() ) {
+        return;
+    }
+    CGBReaderRequestResult result(this, ids[0]);
+    vector<CSeq_id_Handle> load_ids;
+    size_t count = ids.size();
+    _ASSERT(ids.size() == loaded.size());
+    _ASSERT(ids.size() == ret.size());
+    for ( size_t i = 0; i < count; ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        // add into loading set
+        load_ids.push_back(ids[i]);
+    }
+    if ( load_ids.empty() ) {
+        // nothing to load
+        return;
+    }
+    sort(load_ids.begin(), load_ids.end());
+    m_Dispatcher->LoadBlobSet(result, load_ids);
+    // update sequence types
+    for ( size_t i = 0; i < count; ++i ) {
+        const CSeq_id_Handle& id = ids[i];
+        if ( loaded[i] || CReadDispatcher::CannotProcess(id) ) {
+            continue;
+        }
+        CLoadLockBlobIds blob_ids_lock(result, id, 0);
+        CFixedBlob_ids blob_ids = blob_ids_lock.GetBlob_ids();
+        ITERATE ( CFixedBlob_ids, it, blob_ids ) {
+            const CBlob_Info& info = *it;
+            const CBlob_id& blob_id = *info.GetBlob_id();
+            if ( !info.Matches(fBlobHasCore, 0) ) {
+                continue;
+            }
+            CLoadLockBlob blob(result, blob_id);
+            _ASSERT(blob.IsLoadedBlob());
+            CTSE_LoadLock& lock = blob.GetTSE_LoadLock();
+            _ASSERT(lock);
+            if ( CConstRef<CBioseq_Info> seq = lock->FindMatchingBioseq(id) ) {
+                ret[i] = seq->GetBioseqLength();
+                loaded[i] = true;
+                break;
+            }
+        }
+    }
+}
+
+
+void CGBDataLoader::GetSequenceTypes(const TIds& ids, TLoaded& loaded,
+                                     TSequenceTypes& ret)
+{
+    if ( !s_LoadBulkBlobs ) {
+        CDataLoader::GetSequenceTypes(ids, loaded, ret);
+        return;
+    }
+    TTSE_LockSets lock_sets;
+    size_t count = ids.size();
+    _ASSERT(ids.size() == loaded.size());
+    _ASSERT(ids.size() == ret.size());
+    for ( size_t i = 0; i < count; ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        // add into loading set
+        lock_sets[ids[i]];
+    }
+    if ( lock_sets.empty() ) {
+        // nothing to load
+        return;
+    }
+    GetBlobs(lock_sets);
+    // update sequence types
+    for ( size_t i = 0; i < count; ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        TTSE_LockSet& locks = lock_sets[ids[i]];
+        ITERATE(TTSE_LockSet, it, locks) {
+            CConstRef<CBioseq_Info> bs_info =
+                (*it)->FindMatchingBioseq(ids[i]);
+            if ( bs_info ) {
+                ret[i] = bs_info->GetInst_Mol();
+                loaded[i] = true;
+                break;
+            }
+        }
+    }
+}
+
+
+void CGBDataLoader::GetSequenceStates(const TIds& ids, TLoaded& loaded,
+                                      TSequenceStates& ret)
+{
+    if ( !s_LoadBulkBlobs ) {
+        CDataLoader::GetSequenceStates(ids, loaded, ret);
+        return;
+    }
+    for ( size_t i = 0; i < ids.size(); ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        CGBReaderRequestResult result(this, ids[i]);
+        m_Dispatcher->LoadStates(result, ids, loaded, ret);
+        return;
+    }
+    /*
+    set<CSeq_id_Handle> load_set;
+    size_t count = ids.size();
+    _ASSERT(ids.size() == loaded.size());
+    _ASSERT(ids.size() == ret.size());
+    CGBReaderRequestResult result(this, ids[0]);
+    for ( size_t i = 0; i < count; ++i ) {
+        if ( loaded[i] || CReadDispatcher::CannotProcess(ids[i]) ) {
+            continue;
+        }
+        CLoadLockBlobIds blobs(result, ids[i], 0);
+        if ( blobs.IsLoaded() ) {
+            continue;
+        }
+        // add into loading set
+        load_set.insert(ids[i]);
+    }
+    if ( !load_set.empty() ) {
+        result.SetRequestedId(*load_set.begin());
+        m_Dispatcher->LoadSeq_idsBlob_ids(result, load_set);
+    }
+    // update sequence states
+    for ( size_t i = 0; i < count; ++i ) {
+        CReadDispatcher::SetBlobState(i, result, ids, loaded, ret);
+    }
+    */
+}
+
+
 CDataLoader::TBlobVersion CGBDataLoader::GetBlobVersion(const TBlobId& id)
 {
     const TRealBlobId& blob_id = GetRealBlobId(id);
     CGBReaderRequestResult result(this, CSeq_id_Handle());
-    CLoadLockBlob blob(result, blob_id);
-    if ( !blob.IsSetBlobVersion() ) {
+    CLoadLockBlobVersion lock(result, blob_id);
+    if ( !lock.IsLoadedBlobVersion() ) {
         m_Dispatcher->LoadBlobVersion(result, blob_id);
     }
-    return blob.GetBlobVersion();
+    return lock.GetBlobVersion();
 }
 
 
@@ -915,7 +1168,7 @@ CGBDataLoader::ResolveConflict(const CSeq_id_Handle& handle,
             best.Reset();
     }
 */
-    if ( !best ) {
+    if ( !best || conflict ) {
         _TRACE("CGBDataLoader::ResolveConflict("<<handle.AsString()<<"): "
                "conflict");
     }
@@ -925,8 +1178,7 @@ CGBDataLoader::ResolveConflict(const CSeq_id_Handle& handle,
 
 bool CGBDataLoader::HaveCache(TCacheType cache_type)
 {
-    typedef CReaderCacheManager::TCaches TCaches;
-    ITERATE(TCaches, it, m_CacheManager.GetCaches()) {
+    ITERATE(CReaderCacheManager::TCaches, it, m_CacheManager.GetCaches()) {
         if ((it->m_Type & cache_type) != 0) {
             return true;
         }
@@ -939,8 +1191,7 @@ void CGBDataLoader::PurgeCache(TCacheType            cache_type,
                                time_t                access_timeout,
                                ICache::EKeepVersions keep_last_ver)
 {
-    typedef CReaderCacheManager::TCaches TCaches;
-    NON_CONST_ITERATE(TCaches, it, m_CacheManager.GetCaches()) {
+    ITERATE(CReaderCacheManager::TCaches, it, m_CacheManager.GetCaches()) {
         if ((it->m_Type & cache_type) != 0) {
             it->m_Cache->Purge(access_timeout, keep_last_ver);
         }
@@ -1080,11 +1331,12 @@ CGBDataLoader::GetBlobById(const TBlobId& id)
     const TRealBlobId& blob_id = GetRealBlobId(id);
 
     CGBReaderRequestResult result(this, CSeq_id_Handle());
-    if ( !result.IsBlobLoaded(blob_id) )
-        m_Dispatcher->LoadBlob(result, blob_id);
     CLoadLockBlob blob(result, blob_id);
-    _ASSERT(blob.IsLoaded());
-    return blob;
+    if ( !blob.IsLoadedBlob() ) {
+        m_Dispatcher->LoadBlob(result, blob_id);
+    }
+    _ASSERT(blob.IsLoadedBlob());
+    return blob.GetTSE_LoadLock();
 }
 
 
@@ -1191,6 +1443,17 @@ CGBDataLoader::GetExternalAnnotRecords(const CBioseq_Info& bioseq,
 }
 
 
+CDataLoader::TTSE_LockSet
+CGBDataLoader::GetOrphanAnnotRecords(const CSeq_id_Handle& idh,
+                                     const SAnnotSelector* sel)
+{
+    if ( !m_AlwaysLoadExternal ) {
+        return CDataLoader::GetOrphanAnnotRecords(idh, sel);
+    }
+    return GetExternalAnnotRecords(idh, sel);
+}
+
+
 CDataLoader::TTSE_LockSet
 CGBDataLoader::x_GetRecords(const CSeq_id_Handle& sih,
                             TBlobContentsMask mask,
@@ -1198,7 +1461,7 @@ CGBDataLoader::x_GetRecords(const CSeq_id_Handle& sih,
 {
     TTSE_LockSet locks;
 
-    if ( mask == 0 ) {
+    if ( mask == 0 || CReadDispatcher::CannotProcess(sih) ) {
         return locks;
     }
 
@@ -1208,31 +1471,35 @@ CGBDataLoader::x_GetRecords(const CSeq_id_Handle& sih,
     }
 
     CGBReaderRequestResult result(this, sih);
-    CLoadLockSeq_ids seq_ids(result, sih);
     m_Dispatcher->LoadBlobs(result, sih, mask, sel);
-    CLoadLockBlob_ids blobs(result, sih, sel);
+    CLoadLockBlobIds blobs(result, sih, sel);
     _ASSERT(blobs.IsLoaded());
 
-    if ((blobs->GetState() & CBioseq_Handle::fState_no_data) != 0) {
-        if ( blobs->GetState() == CBioseq_Handle::fState_no_data ) {
+    CFixedBlob_ids blob_ids = blobs.GetBlob_ids();
+    if ( (blob_ids.GetState() & CBioseq_Handle::fState_no_data) != 0 ) {
+        if ( blob_ids.GetState() == CBioseq_Handle::fState_no_data ) {
             // default state - return empty lock set
             return locks;
         }
         NCBI_THROW2(CBlobStateException, eBlobStateError,
-                    "blob state error for "+sih.AsString(), blobs->GetState());
+                    "blob state error for "+sih.AsString(),
+                    blob_ids.GetState());
     }
 
-    ITERATE ( CLoadInfoBlob_ids, it, *blobs ) {
-        const CBlob_Info& info = it->second;
-        if ( info.Matches(*it->first, mask, sel) ) {
-            CLoadLockBlob blob(result, *it->first);
-            _ASSERT(blob.IsLoaded());
-            if ((blob.GetBlobState() & CBioseq_Handle::fState_no_data) != 0) {
+    ITERATE ( CFixedBlob_ids, it, blob_ids ) {
+        const CBlob_Info& info = *it;
+        const CBlob_id& blob_id = *info.GetBlob_id();
+        if ( info.Matches(mask, sel) ) {
+            CLoadLockBlob blob(result, blob_id);
+            _ASSERT(blob.IsLoadedBlob());
+            CTSE_LoadLock& lock = blob.GetTSE_LoadLock();
+            _ASSERT(lock);
+            if ( lock->GetBlobState() & CBioseq_Handle::fState_no_data ) {
                 NCBI_THROW2(CBlobStateException, eBlobStateError,
-                            "blob state error for "+it->first->ToString(),
-                            blob.GetBlobState());
+                            "blob state error for "+blob_id.ToString(),
+                            lock->GetBlobState());
             }
-            locks.insert(blob);
+            locks.insert(lock);
         }
     }
     result.SaveLocksTo(locks);
@@ -1246,25 +1513,31 @@ CGBDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih)
     TNamedAnnotNames names;
 
     CGBReaderRequestResult result(this, sih);
-    CLoadLockSeq_ids seq_ids(result, sih);
     SAnnotSelector sel;
     sel.IncludeNamedAnnotAccession("NA*");
-    CLoadLockBlob_ids blobs(result, sih, &sel);
+    CLoadLockBlobIds blobs(result, sih, &sel);
     m_Dispatcher->LoadSeq_idBlob_ids(result, sih, &sel);
     _ASSERT(blobs.IsLoaded());
 
-    if ((blobs->GetState() & CBioseq_Handle::fState_no_data) != 0) {
-        if ( blobs->GetState() == CBioseq_Handle::fState_no_data ) {
+    CFixedBlob_ids blob_ids = blobs.GetBlob_ids();
+    if ( (blob_ids.GetState() & CBioseq_Handle::fState_no_data) != 0) {
+        if ( blob_ids.GetState() == CBioseq_Handle::fState_no_data ) {
             // default state - return empty name set
             return names;
         }
         NCBI_THROW2(CBlobStateException, eBlobStateError,
-                    "blob state error for "+sih.AsString(), blobs->GetState());
+                    "blob state error for "+sih.AsString(),
+                    blob_ids.GetState());
     }
 
-    ITERATE ( CLoadInfoBlob_ids, it, *blobs ) {
-        const CBlob_Info& info = it->second;
-        ITERATE(CBlob_Info::TNamedAnnotNames, jt, info.GetNamedAnnotNames()) {
+    ITERATE ( CFixedBlob_ids, it, blob_ids ) {
+        const CBlob_Info& info = *it;
+        if ( !info.IsSetAnnotInfo() ) {
+            continue;
+        }
+        CConstRef<CBlob_Annot_Info> annot_info = info.GetAnnotInfo();
+        ITERATE( CBlob_Annot_Info::TNamedAnnotNames, jt,
+                 annot_info->GetNamedAnnotNames()) {
             names.insert(*jt);
         }
     }
@@ -1279,7 +1552,6 @@ CGBDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih,
     TNamedAnnotNames names;
 
     CGBReaderRequestResult result(this, sih);
-    CLoadLockSeq_ids seq_ids(result, sih);
     SAnnotSelector sel;
     if ( !ExtractZoomLevel(named_acc, 0, 0) ) {
         sel.IncludeNamedAnnotAccession(CombineWithZoomLevel(named_acc, -1));
@@ -1287,22 +1559,29 @@ CGBDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih,
     else {
         sel.IncludeNamedAnnotAccession(named_acc);
     }
-    CLoadLockBlob_ids blobs(result, sih, &sel);
+    CLoadLockBlobIds blobs(result, sih, &sel);
     m_Dispatcher->LoadSeq_idBlob_ids(result, sih, &sel);
     _ASSERT(blobs.IsLoaded());
 
-    if ((blobs->GetState() & CBioseq_Handle::fState_no_data) != 0) {
-        if ( blobs->GetState() == CBioseq_Handle::fState_no_data ) {
+    CFixedBlob_ids blob_ids = blobs.GetBlob_ids();
+    if ( (blob_ids.GetState() & CBioseq_Handle::fState_no_data) != 0 ) {
+        if ( blob_ids.GetState() == CBioseq_Handle::fState_no_data ) {
             // default state - return empty name set
             return names;
         }
         NCBI_THROW2(CBlobStateException, eBlobStateError,
-                    "blob state error for "+sih.AsString(), blobs->GetState());
+                    "blob state error for "+sih.AsString(),
+                    blob_ids.GetState());
     }
 
-    ITERATE ( CLoadInfoBlob_ids, it, *blobs ) {
-        const CBlob_Info& info = it->second;
-        ITERATE(CBlob_Info::TNamedAnnotNames, jt, info.GetNamedAnnotNames()) {
+    ITERATE ( CFixedBlob_ids, it, blob_ids ) {
+        const CBlob_Info& info = *it;
+        if ( !info.IsSetAnnotInfo() ) {
+            continue;
+        }
+        CConstRef<CBlob_Annot_Info> annot_info = info.GetAnnotInfo();
+        ITERATE ( CBlob_Annot_Info::TNamedAnnotNames, jt,
+                  annot_info->GetNamedAnnotNames() ) {
             names.insert(*jt);
         }
     }
@@ -1313,7 +1592,7 @@ CGBDataLoader::GetNamedAnnotAccessions(const CSeq_id_Handle& sih,
 void CGBDataLoader::GetChunk(TChunk chunk)
 {
     CReader::TChunkId id = chunk->GetChunkId();
-    if ( id == CProcessor::kMasterWGS_ChunkId ) {
+    if ( id == kMasterWGS_ChunkId ) {
         CProcessor::LoadWGSMaster(this, chunk);
     }
     else {
@@ -1331,7 +1610,7 @@ void CGBDataLoader::GetChunks(const TChunkSet& chunks)
     TChunkIdMap chunk_ids;
     ITERATE(TChunkSet, it, chunks) {
         CReader::TChunkId id = (*it)->GetChunkId();
-        if ( id == CProcessor::kMasterWGS_ChunkId ) {
+        if ( id == kMasterWGS_ChunkId ) {
             CProcessor::LoadWGSMaster(this, *it);
         }
         else {
@@ -1353,27 +1632,34 @@ void CGBDataLoader::GetBlobs(TTSE_LockSets& tse_sets)
     TBlobContentsMask mask = fBlobHasCore;
     CReadDispatcher::TIds ids;
     ITERATE(TTSE_LockSets, tse_set, tse_sets) {
-        CLoadLockSeq_ids seq_ids_lock(result, tse_set->first);
-        CLoadLockBlob_ids blob_ids_lock(result, tse_set->first, 0);
-        ids.push_back(tse_set->first);
+        const CSeq_id_Handle& id = tse_set->first;
+        if ( CReadDispatcher::CannotProcess(id) ) {
+            continue;
+        }
+        ids.push_back(id);
     }
     m_Dispatcher->LoadBlobSet(result, ids);
 
     NON_CONST_ITERATE(TTSE_LockSets, tse_set, tse_sets) {
         const CSeq_id_Handle& id = tse_set->first;
-        CLoadLockBlob_ids blob_ids_lock(result, id, 0);
-        ITERATE (CLoadInfoBlob_ids, it, *blob_ids_lock) {
-            const CBlob_Info& info = it->second;
-            if ( info.Matches(*it->first, mask, 0) ) {
-                CLoadLockBlob blob(result, *it->first);
-                _ASSERT(blob.IsLoaded());
+        if ( CReadDispatcher::CannotProcess(id) ) {
+            continue;
+        }
+        CLoadLockBlobIds blob_ids_lock(result, id, 0);
+        CFixedBlob_ids blob_ids = blob_ids_lock.GetBlob_ids();
+        ITERATE ( CFixedBlob_ids, it, blob_ids ) {
+            const CBlob_Info& info = *it;
+            const CBlob_id& blob_id = *info.GetBlob_id();
+            if ( info.Matches(mask, 0) ) {
+                CLoadLockBlob blob(result, blob_id);
+                _ASSERT(blob.IsLoadedBlob());
                 /*
                 if ((blob.GetBlobState() & CBioseq_Handle::fState_no_data) != 0) {
                     // Ignore bad blobs
                     continue;
                 }
                 */
-                tse_set->second.insert(result.GetTSE_LoadLock(*it->first));
+                tse_set->second.insert(blob.GetTSE_LoadLock());
             }
         }
     }
@@ -1427,31 +1713,12 @@ void CGBDataLoader::DropTSE(CRef<CTSE_Info> /* tse_info */)
 }
 
 
-CRef<CLoadInfoSeq_ids>
-CGBDataLoader::GetLoadInfoSeq_ids(const string& key)
-{
-    return m_LoadMapSeq_ids.Get(key);
-}
-
-
-CRef<CLoadInfoSeq_ids>
-CGBDataLoader::GetLoadInfoSeq_ids(const CSeq_id_Handle& key)
-{
-    return m_LoadMapSeq_ids2.Get(key);
-}
-
-
-CRef<CLoadInfoBlob_ids>
-CGBDataLoader::GetLoadInfoBlob_ids(const TLoadKeyBlob_ids& key)
-{
-    return m_LoadMapBlob_ids.Get(key);
-}
-
-
 CGBReaderRequestResult::
 CGBReaderRequestResult(CGBDataLoader* loader,
                        const CSeq_id_Handle& requested_id)
-    : CReaderRequestResult(requested_id),
+    : CReaderRequestResult(requested_id,
+                           loader->GetDispatcher(),
+                           loader->GetInfoManager()),
       m_Loader(loader)
 {
 }
@@ -1468,27 +1735,6 @@ CGBDataLoader* CGBReaderRequestResult::GetLoaderPtr(void)
 }
 
 
-CRef<CLoadInfoSeq_ids>
-CGBReaderRequestResult::GetInfoSeq_ids(const string& key)
-{
-    return GetLoader().GetLoadInfoSeq_ids(key);
-}
-
-
-CRef<CLoadInfoSeq_ids>
-CGBReaderRequestResult::GetInfoSeq_ids(const CSeq_id_Handle& key)
-{
-    return GetLoader().GetLoadInfoSeq_ids(key);
-}
-
-
-CRef<CLoadInfoBlob_ids>
-CGBReaderRequestResult::GetInfoBlob_ids(const TKeyBlob_ids& key)
-{
-    return GetLoader().GetLoadInfoBlob_ids(key);
-}
-
-
 CTSE_LoadLock CGBReaderRequestResult::GetTSE_LoadLock(const TKeyBlob& blob_id)
 {
     CGBDataLoader::TBlobId id(new TKeyBlob(blob_id));
@@ -1515,6 +1761,21 @@ void CGBReaderRequestResult::GetLoadedBlob_ids(const CSeq_id_Handle& idh,
     }
 }
 
+
+CGBDataLoader::TExpirationTimeout
+CGBReaderRequestResult::GetIdExpirationTimeout(void) const
+{
+    return m_Loader->GetIdExpirationTimeout();
+}
+
+
+bool
+CGBReaderRequestResult::GetAddWGSMasterDescr(void) const
+{
+    return m_Loader->GetAddWGSMasterDescr();
+}
+
+
 /*
 bool CGBDataLoader::LessBlobId(const TBlobId& id1, const TBlobId& id2) const
 {
@@ -1531,12 +1792,6 @@ string CGBDataLoader::BlobIdToString(const TBlobId& id) const
 }
 */
 
-CReadDispatcher& CGBDataLoader::GetDispatcher(void)
-{
-    return *m_Dispatcher;
-}
-
-
 void CGBReaderCacheManager::RegisterCache(ICache& cache,
                                           ECacheType cache_type)
 {
