@@ -1,4 +1,4 @@
-/*  $Id: reader_pubseq.cpp 406821 2013-07-16 15:12:01Z vasilche $
+/*  $Id: reader_pubseq.cpp 448568 2014-10-07 16:45:31Z ivanov $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -33,8 +33,8 @@
 #include <objtools/data_loaders/genbank/pubseq/reader_pubseq_entry.hpp>
 #include <objtools/data_loaders/genbank/pubseq/reader_pubseq_params.h>
 #include <objtools/data_loaders/genbank/readers.hpp> // for entry point
-#include <objtools/data_loaders/genbank/request_result.hpp>
-#include <objtools/data_loaders/genbank/dispatcher.hpp>
+#include <objtools/data_loaders/genbank/impl/request_result.hpp>
+#include <objtools/data_loaders/genbank/impl/dispatcher.hpp>
 #include <objtools/error_codes.hpp>
 
 #include <objmgr/objmgr_exception.hpp>
@@ -81,20 +81,15 @@ BEGIN_SCOPE(objects)
 static CAtomicCounter s_pubseq_readers;
 #endif
 
-#ifdef _DEBUG
 NCBI_PARAM_DECL(int, GENBANK, PUBSEQOS_DEBUG);
 NCBI_PARAM_DEF_EX(int, GENBANK, PUBSEQOS_DEBUG, 0,
                   eParam_NoThread, GENBANK_PUBSEQOS_DEBUG);
 
 static int GetDebugLevel(void)
 {
-    static const int s_Value =
-        NCBI_PARAM_TYPE(GENBANK, PUBSEQOS_DEBUG)::GetDefault();
-    return s_Value;
+    static CSafeStatic<NCBI_PARAM_TYPE(GENBANK, PUBSEQOS_DEBUG)> s_Value;
+    return s_Value->Get();
 }
-#else
-# define GetDebugLevel() (0)
-#endif
 
 #define RPC_GET_ASN         "id_get_asn"
 #define RPC_GET_BLOB_INFO   "id_get_blob_prop"
@@ -103,6 +98,54 @@ enum {
     fZipType_gzipped = 2
 };
 
+
+template<typename Int> struct CDB_Int_For;
+
+template<>
+class CDB_Int_For<Int4> : public CDB_Int
+{
+public:
+    typedef CDB_Int TParent;
+    typedef Int4 value_type;
+    CDB_Int_For()
+        {
+        }
+    CDB_Int_For(const value_type& i)
+        : TParent(i)
+        {
+        }
+};
+
+
+template<>
+class CDB_Int_For<Int8> : public CDB_BigInt
+{
+public:
+    typedef CDB_BigInt TParent;
+    typedef Int8 value_type;
+    CDB_Int_For()
+        {
+        }
+    CDB_Int_For(const value_type& i)
+        : TParent(i)
+        {
+        }
+};
+
+
+struct SPubseqReaderReceiveData {
+    SPubseqReaderReceiveData()
+        : zip_type(0), blob_state(0), blob_version(0) 
+        {
+        }
+    
+    AutoPtr<CDB_Result> dbr;
+    int zip_type;
+    CReader::TBlobState blob_state;
+    CReader::TBlobVersion blob_version;
+};
+
+
 CPubseqReader::CPubseqReader(int max_connections,
                              const string& server,
                              const string& user,
@@ -286,7 +329,7 @@ namespace {
     {
         string str = rpc;
         str += " ";
-        str += NStr::IntToString(blob_id.GetSatKey());
+        str += NStr::NumericToString<TIntId>(blob_id.GetSatKey());
         str += ",";
         str += NStr::IntToString(blob_id.GetSat());
         str += ",";
@@ -394,8 +437,8 @@ void CPubseqReader::x_ConnectAtSlot(TConn conn_)
     }
     
     if ( GetDebugLevel() >= 2 ) {
-        NcbiCout << "CPubseqReader::Connected to " << conn->ServerName()
-                 << NcbiEndl;
+        CDebugPrinter s(conn_, "CPubseqReader");
+        s << "Connected to " << conn->ServerName();
     }
 
     m_Connections[conn_].reset(conn.release());
@@ -411,30 +454,52 @@ void CPubseqReader::x_ConnectAtSlot(TConn conn_)
 bool CPubseqReader::LoadSeq_idGi(CReaderRequestResult& result,
                                  const CSeq_id_Handle& seq_id)
 {
-    CLoadLockSeq_ids seq_ids(result, seq_id);
-    if ( seq_ids->IsLoadedGi() ) {
+    CLoadLockGi lock(result, seq_id);
+    if ( lock.IsLoadedGi() ) {
         return true;
     }
-
-    if ( !GetSeq_idInfo(result, seq_id, seq_ids, seq_ids.GetBlob_ids()) ) {
-        return false;
-    }
-    // gi is always loaded in GetSeq_idInfo()
-    _ASSERT(seq_ids->IsLoadedGi());
-    return true;
+    return LoadSeq_idInfo(result, seq_id, 0);
 }
 
 
 bool CPubseqReader::LoadSeq_idSeq_ids(CReaderRequestResult& result,
                                       const CSeq_id_Handle& seq_id)
 {
-    CLoadLockSeq_ids seq_ids(result, seq_id);
-    if ( seq_ids.IsLoaded() ) {
+    CLoadLockSeqIds lock(result, seq_id);
+    if ( lock.IsLoaded() ) {
         return true;
     }
 
-    GetSeq_idSeq_ids(result, seq_ids, seq_id);
-    SetAndSaveSeq_idSeq_ids(result, seq_id, seq_ids);
+    if ( seq_id.Which() == CSeq_id::e_Gi ) {
+        return LoadGiSeq_ids(result, seq_id);
+    }
+
+    CLoadLockGi gi_lock(result, seq_id);
+    if ( !gi_lock.IsLoadedGi() ) {
+        m_Dispatcher->LoadSeq_idGi(result, seq_id);
+        if ( lock.IsLoaded() ) {
+            return true;
+        }
+        if ( !gi_lock.IsLoadedGi() ) {
+            return false;
+        }
+    }
+    TGi gi = gi_lock.GetGi();
+    if (gi == ZERO_GI) {
+        // no gi -> no Seq-ids
+        SetAndSaveNoSeq_idSeq_ids(result, seq_id, 0);
+        return true;
+    }
+
+    CSeq_id_Handle gi_handle = CSeq_id_Handle::GetGiHandle(gi);
+    CLoadLockSeqIds gi_ids(result, gi_handle);
+    m_Dispatcher->LoadSeq_idSeq_ids(result, gi_handle);
+    if ( !gi_ids.IsLoaded() ) {
+        return false;
+    }
+    
+    // copy Seq-id list from gi to original seq-id
+    SetAndSaveSeq_idSeq_ids(result, seq_id, gi_ids);
     return true;
 }
 
@@ -443,24 +508,41 @@ bool CPubseqReader::LoadSeq_idBlob_ids(CReaderRequestResult& result,
                                        const CSeq_id_Handle& seq_id,
                                        const SAnnotSelector* sel)
 {
-    CLoadLockSeq_ids seq_ids(result, seq_id, sel);
-    CLoadLockBlob_ids& blob_ids = seq_ids.GetBlob_ids();
+    CLoadLockBlobIds blob_ids(result, seq_id, sel);
     if ( blob_ids.IsLoaded() ) {
         return true;
     }
-    if ( seq_ids.IsLoaded() &&
-         seq_ids->GetState() & CBioseq_Handle::fState_no_data ) {
-        // no such seq-id
-        blob_ids->SetState(seq_ids->GetState());
-        SetAndSaveSeq_idBlob_ids(result, seq_id, 0, blob_ids);
-        return true;
+
+    CLoadLockSeqIds seq_ids(result, seq_id, eAlreadyLoaded);
+    if ( seq_ids ) {
+        CFixedSeq_ids::TState state = seq_ids.GetState();
+        if ( state & CBioseq_Handle::fState_no_data ) {
+            // no such seq-id
+            SetAndSaveNoSeq_idBlob_ids(result, seq_id, 0, state);
+            return true;
+        }
+    }
+    
+    if ( !sel || !sel->IsIncludedAnyNamedAnnotAccession() ) {
+        // no named annot accessions
+        return LoadSeq_idInfo(result, seq_id, 0);
     }
 
-    if ( !GetSeq_idInfo(result, seq_id, seq_ids, blob_ids) ) {
+    if ( sel->IsIncludedNamedAnnotAccession("NA*") ) {
+        // all named annot accessions
+        return LoadSeq_idInfo(result, seq_id, sel);
+    }
+
+    // part of named annot accessions
+    SAnnotSelector all_sel;
+    all_sel.IncludeNamedAnnotAccession("NA*");
+    if ( !LoadSeq_idInfo(result, seq_id, &all_sel) ) {
         return false;
     }
-    // blob_ids are always loaded in GetSeq_idInfo()
-    _ASSERT(blob_ids.IsLoaded());
+    CLoadLockBlobIds all_blob_ids(result, seq_id, &all_sel);
+    _ASSERT(all_blob_ids.IsLoaded());
+    // filter accessions
+    SetAndSaveSeq_idBlob_ids(result, seq_id, sel, blob_ids, all_blob_ids);
     return true;
 }
 
@@ -468,8 +550,8 @@ bool CPubseqReader::LoadSeq_idBlob_ids(CReaderRequestResult& result,
 bool CPubseqReader::LoadSeq_idAccVer(CReaderRequestResult& result,
                                      const CSeq_id_Handle& seq_id)
 {
-    CLoadLockSeq_ids seq_ids(result, seq_id);
-    if ( seq_ids->IsLoadedAccVer() ) {
+    CLoadLockAcc lock(result, seq_id);
+    if ( lock.IsLoadedAccVer() ) {
         return true;
     }
 
@@ -490,7 +572,7 @@ bool CPubseqReader::LoadSeq_idAccVer(CReaderRequestResult& result,
                 CDB_Connection* db_conn = x_GetConnection(conn);
     
                 AutoPtr<CDB_RPCCmd> cmd(db_conn->RPC("id_get_accn_ver_by_gi"));
-                CDB_Int giIn = GI_TO(Int4, gi);
+                CDB_Int_For<TIntId> giIn(gi);
                 cmd->SetParam("@gi", &giIn);
                 cmd->Send();
                 
@@ -518,7 +600,8 @@ bool CPubseqReader::LoadSeq_idAccVer(CReaderRequestResult& result,
                         CDB_VarChar accVerGot;
                         dbr->GetItem(&accVerGot);
                         try {
-                            CSeq_id id(accVerGot.Value());
+                            CSeq_id_Handle id =
+                                CSeq_id_Handle::GetHandle(accVerGot.Value());
                             SetAndSaveSeq_idAccVer(result, seq_id, id);
                             while ( dbr->Fetch() )
                                 ;
@@ -542,7 +625,7 @@ bool CPubseqReader::LoadSeq_idAccVer(CReaderRequestResult& result,
         }
     }
 
-    if ( !seq_ids->IsLoadedAccVer() ) {
+    if ( !lock.IsLoadedAccVer() ) {
         return CId1ReaderBase::LoadSeq_idAccVer(result, seq_id);
     }
 
@@ -550,10 +633,9 @@ bool CPubseqReader::LoadSeq_idAccVer(CReaderRequestResult& result,
 }
 
 
-bool CPubseqReader::GetSeq_idInfo(CReaderRequestResult& result,
-                                  const CSeq_id_Handle& seq_id,
-                                  CLoadLockSeq_ids& seq_ids,
-                                  CLoadLockBlob_ids& blob_ids)
+bool CPubseqReader::LoadSeq_idInfo(CReaderRequestResult& result,
+                                   const CSeq_id_Handle& seq_id,
+                                   const SAnnotSelector* with_named_accs)
 {
     // Get gi by seq-id
     _TRACE("ResolveSeq_id to gi/sat/satkey: " << seq_id.AsString());
@@ -572,7 +654,7 @@ bool CPubseqReader::GetSeq_idInfo(CReaderRequestResult& result,
     }}
 
     int result_count = 0;
-    int named_gi = 0;
+    TGi named_gi = ZERO_GI;
     CConn conn(result, this);
     {{
         CDB_Connection* db_conn = x_GetConnection(conn);
@@ -580,13 +662,15 @@ bool CPubseqReader::GetSeq_idInfo(CReaderRequestResult& result,
         AutoPtr<CDB_RPCCmd> cmd(db_conn->RPC("id_gi_by_seqid_asn"));
         cmd->SetParam("@asnin", &asnIn);
         cmd->Send();
-    
+        
+        TBlobIds blob_ids;
+        
         while(cmd->HasMoreResults()) {
             AutoPtr<CDB_Result> dbr(cmd->Result());
             if ( !dbr.get() ) {
                 continue;
             }
-
+            
             if ( dbr->ResultType() != eDB_RowResult) {
                 while ( dbr->Fetch() )
                     ;
@@ -594,9 +678,9 @@ bool CPubseqReader::GetSeq_idInfo(CReaderRequestResult& result,
             }
         
             while ( dbr->Fetch() ) {
-                CDB_Int giGot;
+                CDB_Int_For<TIntId> giGot;
                 CDB_Int satGot;
-                CDB_Int satKeyGot;
+                CDB_Int_For<TIntId> satKeyGot;
                 CDB_Int extFeatGot;
                 CDB_Int namedAnnotsGot;
 
@@ -640,84 +724,73 @@ bool CPubseqReader::GetSeq_idInfo(CReaderRequestResult& result,
                     }
                 }
 
-                int gi = giGot.Value();
+                TGi gi = giGot.Value();
                 int sat = satGot.Value();
-                int sat_key = satKeyGot.Value();
+                TIntId sat_key = satKeyGot.Value();
                 
                 if ( GetDebugLevel() >= 5 ) {
-                    NcbiCout << "CPubseqReader::ResolveSeq_id"
+                    CDebugPrinter s(conn, "CPubseqReader");
+                    s << "ResolveSeq_id"
                         "(" << seq_id.AsString() << ")"
                         " gi=" << gi <<
                         " sat=" << sat <<
                         " satkey=" << sat_key <<
                         " extfeat=";
                     if ( extFeatGot.IsNULL() ) {
-                        NcbiCout << "NULL";
+                        s << "NULL";
                     }
                     else {
-                        NcbiCout << extFeatGot.Value();
-                    }
-                    NcbiCout << NcbiEndl;
-                }
-
-                if ( !blob_ids.IsLoaded() ) {
-                    if ( CProcessor::TrySNPSplit() && !IsAnnotSat(sat) ) {
-                        // main blob
-                        CBlob_id blob_id;
-                        blob_id.SetSat(sat);
-                        blob_id.SetSatKey(sat_key);
-                        blob_ids.AddBlob_id(blob_id,
-                                            CBlob_Info(fBlobHasAllLocal));
-                        if ( !extFeatGot.IsNULL() ) {
-                            int ext_feat = extFeatGot.Value();
-                            while ( ext_feat ) {
-                                int bit = ext_feat & ~(ext_feat-1);
-                                ext_feat -= bit;
-                                blob_id.SetSat(GetAnnotSat(bit));
-                                blob_id.SetSatKey(gi);
-                                blob_id.SetSubSat(bit);
-                                blob_ids.AddBlob_id(blob_id,
-                                                    CBlob_Info(fBlobHasExtAnnot));
-                            }
-                        }
-                    }
-                    else {
-                        // whole blob
-                        CBlob_id blob_id;
-                        blob_id.SetSat(sat);
-                        blob_id.SetSatKey(sat_key);
-                        if ( !extFeatGot.IsNULL() ) {
-                            blob_id.SetSubSat(extFeatGot.Value());
-                        }
-                        blob_ids.AddBlob_id(blob_id,
-                                            CBlob_Info(fBlobHasAllLocal));
-                    }
-                    if ( !named_gi ) {
-                        SetAndSaveSeq_idBlob_ids(result, seq_id, 0, blob_ids);
+                        s << extFeatGot.Value();
                     }
                 }
 
-                if ( giGot.IsNULL() || gi == 0 ) {
+                if ( giGot.IsNULL() || gi == ZERO_GI ) {
                     // no gi -> only one Seq-id - the one used as argument
-                    if ( !seq_ids.IsLoaded() ) {
-                        seq_ids.AddSeq_id(seq_id);
-                        SetAndSaveSeq_idSeq_ids(result, seq_id, seq_ids);
-                    }
+                    SetAndSaveSeq_idGi(result, seq_id, ZERO_GI);
                 }
                 else {
                     // we've got gi
-                    if ( !seq_ids->IsLoadedGi() ) {
-                        SetAndSaveSeq_idGi(result, seq_id, seq_ids,
-                                           GI_FROM(Int4, giGot.Value()));
+                    SetAndSaveSeq_idGi(result, seq_id, gi);
+                }
+                
+                if ( CProcessor::TrySNPSplit() && !IsAnnotSat(sat) ) {
+                    // main blob
+                    CRef<CBlob_id> blob_id(new CBlob_id);
+                    blob_id->SetSat(sat);
+                    blob_id->SetSatKey(sat_key);
+                    blob_ids.push_back(CBlob_Info(blob_id, fBlobHasAllLocal));
+                    if ( !extFeatGot.IsNULL() ) {
+                        int ext_feat = extFeatGot.Value();
+                        while ( ext_feat ) {
+                            int bit = ext_feat & ~(ext_feat-1);
+                            ext_feat -= bit;
+                            blob_id = new CBlob_id;
+                            blob_id->SetSat(GetAnnotSat(bit));
+                            blob_id->SetSatKey(gi);
+                            blob_id->SetSubSat(bit);
+                            blob_ids.push_back(CBlob_Info(blob_id,
+                                                          fBlobHasExtAnnot));
+                        }
                     }
                 }
+                else {
+                    // whole blob
+                    CRef<CBlob_id> blob_id(new CBlob_id);
+                    blob_id->SetSat(sat);
+                    blob_id->SetSatKey(sat_key);
+                    if ( !extFeatGot.IsNULL() ) {
+                        blob_id->SetSubSat(extFeatGot.Value());
+                    }
+                    blob_ids.push_back(CBlob_Info(blob_id, fBlobHasAllLocal));
+                }
             }
         }
 
         cmd.reset();
 
-        if ( named_gi ) {
-            CDB_Int giIn(named_gi);
+        if ( with_named_accs && named_gi != ZERO_GI ) {
+            // postponed read of named annot accessions
+            CDB_Int_For<TIntId> giIn(named_gi);
             AutoPtr<CDB_RPCCmd> cmd(db_conn->RPC("id_get_annot_types"));
             cmd->SetParam("@gi", &giIn);
             cmd->Send();
@@ -735,9 +808,9 @@ bool CPubseqReader::GetSeq_idInfo(CReaderRequestResult& result,
                 }
                 
                 while ( dbr->Fetch() ) {
-                    CDB_Int giGot;
+                    CDB_Int_For<TIntId> giGot;
                     CDB_Int satGot;
-                    CDB_Int satKeyGot;
+                    CDB_Int_For<TIntId> satKeyGot;
                     CDB_Int typeGot;
                     CDB_VarChar nameGot;
                     for ( unsigned pos = 0; pos < dbr->NofItems(); ++pos ) {
@@ -767,27 +840,26 @@ bool CPubseqReader::GetSeq_idInfo(CReaderRequestResult& result,
                             dbr->SkipItem();
                         }
                     }
-                    CBlob_id blob_id;
-                    blob_id.SetSat(satGot.Value());
-                    blob_id.SetSatKey(satKeyGot.Value());
-                    CBlob_Info info(fBlobHasNamedFeat);
-                    info.AddNamedAnnotName(nameGot.Value());
-                    blob_ids.AddBlob_id(blob_id, info);
-
-                    /*
-                    CRef<CID2S_Feat_type_Info> feat(new CID2S_Feat_type_Info);
-                    feat->SetType(typeGot.Value());
-                    CRef<CID2S_Seq_annot_Info> annot(new CID2S_Seq_annot_Info);
-                    annot->SetName(nameGot.Value());
-                    annot->SetFeat().push_back(feat);
-                    list<CRef<CID2S_Seq_annot_Info> > annot_info(1, annot);
-                    SetAndSaveBlobAnnotInfo(result, blob_id, annot_info);
-                    */
+                    CRef<CBlob_id> blob_id(new CBlob_id);
+                    blob_id->SetSat(satGot.Value());
+                    blob_id->SetSatKey(satKeyGot.Value());
+                    CBlob_Info info(blob_id, fBlobHasNamedFeat);
+                    CRef<CBlob_Annot_Info> annot_info(new CBlob_Annot_Info);
+                    annot_info->AddNamedAnnotName(nameGot.Value());
+                    info.SetAnnotInfo(annot_info);
+                    blob_ids.push_back(info);
                 }
             }
-            SAnnotSelector sel;
-            sel.IncludeNamedAnnotAccession("NA*");
-            SetAndSaveSeq_idBlob_ids(result, seq_id, &sel, blob_ids);
+        }
+
+        if ( result_count == 0 ) {
+            SetAndSaveSeq_idGi(result, seq_id, ZERO_GI);
+            SetAndSaveNoSeq_idSeq_ids(result, seq_id, 0);
+            SetAndSaveNoSeq_idBlob_ids(result, seq_id, with_named_accs, 0);
+        }
+        else {
+            SetAndSaveSeq_idBlob_ids(result, seq_id, with_named_accs,
+                                     CFixedBlob_ids(eTakeOwnership, blob_ids));
         }
     }}
 
@@ -796,79 +868,36 @@ bool CPubseqReader::GetSeq_idInfo(CReaderRequestResult& result,
 }
 
 
-bool CPubseqReader::GetSeq_idBlob_ids(CReaderRequestResult& result,
-                                      CLoadLockBlob_ids& ids,
-                                      const CSeq_id_Handle& seq_id,
-                                      const SAnnotSelector* sel)
+bool CPubseqReader::LoadGiSeq_ids(CReaderRequestResult& result,
+                                  const CSeq_id_Handle& seq_id)
 {
-    NCBI_THROW(CLoaderException, eLoaderFailed, "invalid call");
-}
-
-
-void CPubseqReader::GetSeq_idSeq_ids(CReaderRequestResult& result,
-                                     CLoadLockSeq_ids& ids,
-                                     const CSeq_id_Handle& seq_id)
-{
-    if ( ids.IsLoaded() ) {
-        return;
+    CLoadLockSeqIds load_lock(result, seq_id);
+    if ( load_lock.IsLoaded() ) {
+        return true;
     }
-
-    if ( seq_id.Which() == CSeq_id::e_Gi ) {
-        GetGiSeq_ids(result, seq_id, ids);
-        return;
-    }
-
-    m_Dispatcher->LoadSeq_idGi(result, seq_id);
-    if ( ids.IsLoaded() ) { // may be loaded as extra information for gi
-        return;
-    }
-    TGi gi = ids->GetGi();
-    if (gi == ZERO_GI) {
-        // no gi -> no Seq-ids
-        return;
-    }
-
-    CSeq_id_Handle gi_handle = CSeq_id_Handle::GetGiHandle(gi);
-    CLoadLockSeq_ids gi_ids(result, gi_handle);
-    m_Dispatcher->LoadSeq_idSeq_ids(result, gi_handle);
-    
-    // copy Seq-id list from gi to original seq-id
-    ids->m_Seq_ids = gi_ids->m_Seq_ids;
-    ids->SetState(gi_ids->GetState());
-}
-
-
-void CPubseqReader::GetGiSeq_ids(CReaderRequestResult& result,
-                                 const CSeq_id_Handle& seq_id,
-                                 CLoadLockSeq_ids& ids)
-{
     _ASSERT(seq_id.Which() == CSeq_id::e_Gi);
-    TGi gi;
-    if ( seq_id.IsGi() ) {
-        gi = seq_id.GetGi();
-    }
-    else {
-        gi = seq_id.GetSeqId()->GetGi();
-    }
+    TGi gi = seq_id.GetGi();
     if ( gi == ZERO_GI ) {
-        return;
+        SetAndSaveNoSeq_idSeq_ids(result, seq_id, 0);
+        return true;
     }
 
     _TRACE("ResolveGi to Seq-ids: " << gi);
 
     CConn conn(result, this);
     {{
+        TSeqIds ids;
         CDB_Connection* db_conn = x_GetConnection(conn);
     
         AutoPtr<CDB_RPCCmd> cmd(db_conn->RPC("id_seqid4gi"));
-        CDB_Int giIn = GI_TO(Int4, gi);
+        CDB_Int_For<TIntId> giIn(gi);
         CDB_TinyInt binIn = 1;
         cmd->SetParam("@gi", &giIn);
         cmd->SetParam("@bin", &binIn);
         cmd->Send();
-    
+
+        int state = 0;
         bool not_found = false;
-        int id_count = 0;
         while ( cmd->HasMoreResults() ) {
             AutoPtr<CDB_Result> dbr(cmd->Result());
             if ( !dbr.get() ) {
@@ -876,13 +905,18 @@ void CPubseqReader::GetGiSeq_ids(CReaderRequestResult& result,
             }
             
             if ( dbr->ResultType() == eDB_StatusResult ) {
-                dbr->Fetch();
-                CDB_Int v;
-                dbr->GetItem(&v);
-                int status = v.Value();
-                if ( status == 100 ) {
-                    // gi does not exist
-                    not_found = true;
+                while ( dbr->Fetch() ) {
+                    CDB_Int v;
+                    dbr->GetItem(&v);
+                    int status = v.Value();
+                    if ( status == 100 ) {
+                        // gi does not exist
+                        not_found = true;
+                    }
+                }
+                if ( not_found ) {
+                    cmd->Cancel();
+                    break;
                 }
             }
             else if ( dbr->ResultType() == eDB_RowResult &&
@@ -893,8 +927,7 @@ void CPubseqReader::GetGiSeq_ids(CReaderRequestResult& result,
                 CSeq_id id;
                 while ( in.HaveMoreData() ) {
                     in >> id;
-                    ids.AddSeq_id(id);
-                    ++id_count;
+                    ids.push_back(CSeq_id_Handle::GetHandle(id));
                 }
                 if ( in.HaveMoreData() ) {
                     ERR_POST_X(4, "CPubseqReader: extra seqid data");
@@ -907,12 +940,36 @@ void CPubseqReader::GetGiSeq_ids(CReaderRequestResult& result,
             while ( dbr->Fetch() )
                 ;
         }
-        if ( id_count == 0 && !not_found ) {
+        if ( ids.empty() && !not_found ) {
             // artificially add argument Seq-id if empty set was received
-            ids.AddSeq_id(seq_id);
+            ids.push_back(seq_id);
+        }
+        SetAndSaveSeq_idSeq_ids(result, seq_id,
+                                CFixedSeq_ids(eTakeOwnership, ids, state));
+    }}
+    conn.Release();
+    return true;
+}
+
+
+void CPubseqReader::GetBlobState(CReaderRequestResult& result, 
+                                 const CBlob_id& blob_id)
+{
+    CConn conn(result, this);
+    {{
+        CDB_Connection* db_conn = x_GetConnection(conn);
+        AutoPtr<I_BaseCmd> cmd
+            (x_SendRequest2(blob_id, db_conn, RPC_GET_BLOB_INFO));
+        SPubseqReaderReceiveData data;
+        x_ReceiveData(result, data, blob_id, *cmd, false);
+        if ( data.dbr ) {
+            ERR_POST_X(3, "CPubseqReader: unexpected blob data");
         }
     }}
     conn.Release();
+    if ( !blob_id.IsMainBlob() ) {
+        result.SetAndSaveBlobState(blob_id, 0);
+    }
 }
 
 
@@ -925,23 +982,20 @@ void CPubseqReader::GetBlobVersion(CReaderRequestResult& result,
             CDB_Connection* db_conn = x_GetConnection(conn);
             AutoPtr<I_BaseCmd> cmd
                 (x_SendRequest2(blob_id, db_conn, RPC_GET_BLOB_INFO));
-            pair<AutoPtr<CDB_Result>, int> dbr
-                (x_ReceiveData(result, blob_id, *cmd, false));
-            if ( dbr.first ) {
+            SPubseqReaderReceiveData data;
+            x_ReceiveData(result, data, blob_id, *cmd, false);
+            if ( data.dbr ) {
                 ERR_POST_X(5, "CPubseqReader: unexpected blob data");
             }
         }}
         conn.Release();
         if ( !blob_id.IsMainBlob() ) {
-            CLoadLockBlob blob(result, blob_id);
-            if ( !blob.IsSetBlobVersion() ) {
-                SetAndSaveBlobVersion(result, blob_id, 0);
-            }
+            result.SetAndSaveBlobVersion(blob_id, 0);
         }
     }
     catch ( exception& ) {
         if ( !blob_id.IsMainBlob() ) {
-            SetAndSaveBlobVersion(result, blob_id, 0);
+            result.SetAndSaveBlobVersion(blob_id, 0);
             return;
         }
         throw;
@@ -953,21 +1007,18 @@ void CPubseqReader::GetBlob(CReaderRequestResult& result,
                             const TBlobId& blob_id,
                             TChunkId chunk_id)
 {
-    CConn conn(result, this);
-    if ( chunk_id == CProcessor::kMain_ChunkId ) {
-        CLoadLockBlob blob(result, blob_id);
-        if ( blob.IsLoaded() ) {
-            conn.Release();
-            return;
-        }
+    CLoadLockBlob blob(result, blob_id, chunk_id);
+    if ( blob.IsLoadedChunk() ) {
+        return;
     }
+    CConn conn(result, this);
     {{
         CDB_Connection* db_conn = x_GetConnection(conn);
         AutoPtr<I_BaseCmd> cmd(x_SendRequest(blob_id, db_conn, RPC_GET_ASN));
-        pair<AutoPtr<CDB_Result>, int> dbr
-            (x_ReceiveData(result, blob_id, *cmd, true));
-        if ( dbr.first ) {
-            CDB_Result_Reader reader(dbr.first);
+        SPubseqReaderReceiveData data;
+        x_ReceiveData(result, data, blob_id, *cmd, true);
+        if ( data.dbr ) {
+            CDB_Result_Reader reader(data.dbr);
             CRStream stream(&reader);
             CProcessor::EType processor_type;
             if ( blob_id.GetSubSat() == eSubSat_SNP ) {
@@ -976,7 +1027,7 @@ void CPubseqReader::GetBlob(CReaderRequestResult& result,
             else {
                 processor_type = CProcessor::eType_Seq_entry;
             }
-            if ( dbr.second & fZipType_gzipped ) {
+            if ( data.zip_type & fZipType_gzipped ) {
                 CCompressionIStream unzip(stream,
                                           new CZipStreamDecompressor,
                                           CCompressionIStream::fOwnProcessor);
@@ -994,7 +1045,7 @@ void CPubseqReader::GetBlob(CReaderRequestResult& result,
             cmd->DumpResults();
         }
         else {
-            SetAndSaveNoBlob(result, blob_id, chunk_id);
+            SetAndSaveNoBlob(result, blob_id, chunk_id, data.blob_state);
         }
     }}
     conn.Release();
@@ -1007,7 +1058,7 @@ I_BaseCmd* CPubseqReader::x_SendRequest(const CBlob_id& blob_id,
 {
     AutoPtr<CDB_RPCCmd> cmd(db_conn->RPC(rpc));
     CDB_SmallInt satIn(blob_id.GetSat());
-    CDB_Int satKeyIn(blob_id.GetSatKey());
+    CDB_Int_For<TIntId> satKeyIn(blob_id.GetSatKey());
     CDB_Int ext_feat(blob_id.GetSubSat());
 
     _TRACE("x_SendRequest: "<<blob_id.ToString());
@@ -1020,23 +1071,18 @@ I_BaseCmd* CPubseqReader::x_SendRequest(const CBlob_id& blob_id,
 }
 
 
-pair<AutoPtr<CDB_Result>, int>
-CPubseqReader::x_ReceiveData(CReaderRequestResult& result,
-                             const TBlobId& blob_id,
-                             I_BaseCmd& cmd,
-                             bool force_blob)
+void CPubseqReader::x_ReceiveData(CReaderRequestResult& result,
+                                  SPubseqReaderReceiveData& ret,
+                                  const TBlobId& blob_id,
+                                  I_BaseCmd& cmd,
+                                  bool force_blob)
 {
-    pair<AutoPtr<CDB_Result>, int> ret;
-
     enum {
         kState_dead = 125
     };
-    TBlobState blob_state = 0;
-
-    CLoadLockBlob blob(result, blob_id);
 
     // new row
-    while( !ret.first && cmd.HasMoreResults() ) {
+    while( !ret.dbr && cmd.HasMoreResults() ) {
         _TRACE("next result");
         if ( cmd.HasFailed() ) {
             break;
@@ -1053,7 +1099,7 @@ CPubseqReader::x_ReceiveData(CReaderRequestResult& result,
             continue;
         }
         
-        while ( !ret.first && dbr->Fetch() ) {
+        while ( !ret.dbr && dbr->Fetch() ) {
             _TRACE("next fetch: " << dbr->NofItems() << " items");
             for ( unsigned pos = 0; pos < dbr->NofItems(); ++pos ) {
                 const string& name = dbr->ItemName(pos);
@@ -1063,7 +1109,7 @@ CPubseqReader::x_ReceiveData(CReaderRequestResult& result,
                     dbr->GetItem(&v);
                     _TRACE("confidential: "<<v.Value());
                     if ( v.Value() ) {
-                        blob_state |= CBioseq_Handle::fState_confidential;
+                        ret.blob_state |= CBioseq_Handle::fState_confidential;
                     }
                 }
                 else if ( name == "suppress" ) {
@@ -1071,7 +1117,7 @@ CPubseqReader::x_ReceiveData(CReaderRequestResult& result,
                     dbr->GetItem(&v);
                     _TRACE("suppress: "<<v.Value());
                     if ( v.Value() ) {
-                        blob_state |= (v.Value() & 4)
+                        ret.blob_state |= (v.Value() & 4)
                             ? CBioseq_Handle::fState_suppress_temp
                             : CBioseq_Handle::fState_suppress_perm;
                     }
@@ -1081,32 +1127,31 @@ CPubseqReader::x_ReceiveData(CReaderRequestResult& result,
                     dbr->GetItem(&v);
                     _TRACE("withdrawn: "<<v.Value());
                     if ( v.Value() ) {
-                        blob_state |= CBioseq_Handle::fState_withdrawn;
+                        ret.blob_state |= CBioseq_Handle::fState_withdrawn;
                     }
                 }
                 else if ( name == "last_touched_m" ) {
                     CDB_Int v;
                     dbr->GetItem(&v);
                     _TRACE("version: " << v.Value());
-                    m_Dispatcher->SetAndSaveBlobVersion(result, blob_id,
-                                                        v.Value());
+                    ret.blob_version = v.Value();
                 }
                 else if ( name == "state" ) {
                     CDB_Int v;
                     dbr->GetItem(&v);
                     _TRACE("state: "<<v.Value());
                     if ( v.Value() == kState_dead ) {
-                        blob_state |= CBioseq_Handle::fState_dead;
+                        ret.blob_state |= CBioseq_Handle::fState_dead;
                     }
                 }
                 else if ( name == "zip_type" ) {
                     CDB_Int v;
                     dbr->GetItem(&v);
                     _TRACE("zip_type: "<<v.Value());
-                    ret.second = v.Value();
+                    ret.zip_type = v.Value();
                 }
                 else if ( name == "asn1" ) {
-                    ret.first.reset(dbr.release());
+                    ret.dbr.reset(dbr.release());
                     break;
                 }
                 else {
@@ -1140,13 +1185,13 @@ CPubseqReader::x_ReceiveData(CReaderRequestResult& result,
             }
         }
     }
-    if ( !ret.first && force_blob ) {
+    if ( !ret.dbr && force_blob ) {
         // no data
         _TRACE("actually no data");
-        blob_state |= CBioseq_Handle::fState_no_data;
+        ret.blob_state |= CBioseq_Handle::fState_no_data;
     }
-    m_Dispatcher->SetAndSaveBlobState(result, blob_id, blob, blob_state);
-    return ret;
+    result.SetAndSaveBlobState(blob_id, ret.blob_state);
+    result.SetAndSaveBlobVersion(blob_id, ret.blob_version);
 }
 
 END_SCOPE(objects)
