@@ -1,4 +1,4 @@
-/*  $Id: reader_pubseq2.cpp 368555 2012-07-09 19:33:51Z vasilche $
+/*  $Id: reader_pubseq2.cpp 440725 2014-07-16 16:44:20Z vasilche $
 * ===========================================================================
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
@@ -33,8 +33,8 @@
 #include <objtools/data_loaders/genbank/pubseq2/reader_pubseq2_entry.hpp>
 #include <objtools/data_loaders/genbank/pubseq2/reader_pubseq2_params.h>
 #include <objtools/data_loaders/genbank/readers.hpp> // for entry point
-#include <objtools/data_loaders/genbank/request_result.hpp>
-#include <objtools/data_loaders/genbank/dispatcher.hpp>
+#include <objtools/data_loaders/genbank/impl/request_result.hpp>
+#include <objtools/data_loaders/genbank/impl/dispatcher.hpp>
 #include <objtools/error_codes.hpp>
 
 #include <objmgr/objmgr_exception.hpp>
@@ -302,7 +302,7 @@ namespace {
     bool sx_FetchNextItem(CDB_Result& result, const CTempString& name)
     {
         while ( result.Fetch() ) {
-            for ( size_t pos = 0; pos < result.NofItems(); ++pos ) {
+            for ( unsigned pos = 0; pos < result.NofItems(); ++pos ) {
                 if ( result.ItemName(pos) == name ) {
                     return true;
                 }
@@ -524,7 +524,9 @@ CPubseq2Reader::x_SendPacket(CDB_Connection& db_conn,
     char buffer[MAX_ASN_IN];
     size_t size;
     {{
-        CNcbiOstrstream mem_str(buffer, sizeof(buffer));
+        // NOT CNcbiOstrstream, because that can be ostringstream, which
+        // doesn't support setting up a fixed-length buffer in this manner.
+        ostrstream mem_str(buffer, sizeof(buffer));
         {{
             CRequestSerializer obj_str(mem_str);
             obj_str << packet;
