@@ -1,4 +1,4 @@
-/*  $Id: comment_item.cpp 371260 2012-08-07 14:57:55Z rafanovi $
+/*  $Id: comment_item.cpp 448165 2014-10-02 17:42:41Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -50,6 +50,7 @@
 #include <objects/general/Object_id.hpp>
 #include <objects/general/Date.hpp>
 #include <objects/general/Dbtag.hpp>
+#include <objects/misc/sequence_util_macros.hpp>
 
 #include <objmgr/seqdesc_ci.hpp>
 #include <objmgr/util/sequence.hpp>
@@ -172,6 +173,68 @@ void CCommentItem::AddPeriod(void)
     }
 }
 
+void CCommentItem::RemoveExcessNewlines(
+    const CCommentItem & next_comment )
+{
+    if(  m_Comment.empty() || next_comment.m_Comment.empty() ) {
+        return;
+    }
+
+    // check if next_comment starts with an empty line
+    const string & next_comment_first_string = next_comment.m_Comment.front();
+    bool next_comment_starts_with_empty_line = false;
+    ITERATE( string, next_com_line_it, next_comment_first_string ) {
+        const char ch = *next_com_line_it;
+        if( ch == '\n' ) {
+            next_comment_starts_with_empty_line = true;
+            break;
+        } else if( ! isspace(ch) ) {
+            break;
+        }
+    }
+
+    if( ! next_comment_starts_with_empty_line ) {
+        // we assume that this comment won't have excessive blank lines
+        return;
+    }
+
+    // see if we have too many newlines at the end (we assume we don't have more than
+    // one extra)
+    string & last_str_of_comment = m_Comment.back();
+    if( last_str_of_comment.empty() ) {
+        return;
+    }
+
+    string::size_type pos = (last_str_of_comment.length() - 1);
+    if( last_str_of_comment[pos] == '\n' ) {
+        // skip final newlines because lines without newline will get 
+        // a newline added, so we would assume it's there anyway
+        --pos;
+    }
+    for( ; pos < last_str_of_comment.length(); --pos ) {
+        const char ch = last_str_of_comment[pos];
+
+        if( ch == '\n' ) {
+            // extra newline found: remove it
+            last_str_of_comment.erase(pos);
+            return;
+        } else if( ! isspace(ch) ) {
+            return;
+        }
+    }
+}
+
+void CCommentItem::RemovePeriodAfterURL(void)
+{
+    if( m_Comment.empty() ) {
+        return;
+    }
+
+    // remove period if it's after a '/', though.
+    if( NStr::EndsWith(m_Comment.back(), "/.") ) {
+        m_Comment.back().resize( m_Comment.back().length() - 1 );
+    }
+}
 
 const string& CCommentItem::GetNsAreGapsStr(void)
 {
@@ -247,14 +310,15 @@ string CCommentItem::GetStringForTPA
 }
 
 
-string CCommentItem::GetStringForBankIt(const CUser_object& uo)
+string CCommentItem::GetStringForBankIt(const CUser_object& uo, bool dump_mode)
 {
     if ( !uo.CanGetType()  ||  !uo.GetType().IsStr()  ||
          uo.GetType().GetStr() != "Submission" ) {
         return kEmptyStr;
     }
 
-    const string* uvc = 0, *bic = 0;
+    const string* uvc = 0, *bic = 0, *smc = 0;
+
     if ( uo.HasField("UniVecComment") ) {
         const CUser_field& uf = uo.GetField("UniVecComment");
         if ( uf.CanGetData()  &&  uf.GetData().IsStr() ) {
@@ -267,14 +331,26 @@ string CCommentItem::GetStringForBankIt(const CUser_object& uo)
             bic = &(uf.GetData().GetStr());
         } 
     }
+    if ( uo.HasField("SmartComment") && dump_mode ) {
+        const CUser_field& uf = uo.GetField("SmartComment");
+        if ( uf.CanGetData()  &&  uf.GetData().IsStr() ) {
+            smc = &(uf.GetData().GetStr());
+        } 
+    }
 
     CNcbiOstrstream text;
-    if ( uvc != 0  &&  bic != 0 ) {
-        text << "Vector Explanation: " << *uvc << "~Bankit Comment: " << *bic;
-    } else if ( uvc != 0 ) {
-        text << "Vector Explanation: " << *uvc;
-    } else if ( bic != 0 ) {
-         text << "Bankit Comment: " << *bic;
+    string pfx = "";
+    if ( uvc != 0 ) {
+        text << pfx << "Vector Explanation: " << *uvc;
+        pfx = "~";
+    }
+    if ( bic != 0 ) {
+        text << pfx << "Bankit Comment: " << *bic;
+        pfx = "~";
+    }
+    if ( smc != 0 ) {
+        text << pfx << "Bankit Comment: " << *smc;
+        pfx = "~";
     }
 
     return CNcbiOstrstreamToString(text);
@@ -614,7 +690,7 @@ string CCommentItem::GetStringForRefTrack
          oss << s;
      }}
 
-    const static string kRefSeqGeneLink = "<a href=\"http://www.ncbi.nlm.nih.gov/RefSeq/RSG\">RefSeqGene</a>";
+    const static string kRefSeqGeneLink = "<a href=\"http://www.ncbi.nlm.nih.gov/refseq/rsg/\">RefSeqGene</a>";
     const static string kRefSeqGene = "RefSeqGene";
 
     /// check for a concomitant RefSeqGene item
@@ -638,6 +714,74 @@ string CCommentItem::GetStringForRefTrack
     return CNcbiOstrstreamToString(oss);
 }
 
+string CCommentItem::GetStringForRefSeqGenome(const CUser_object& uo)
+{
+    if ( ! FIELD_IS_SET_AND_IS(uo, Type, Str)  ||
+         uo.GetType().GetStr() != "RefSeqGenome") 
+    {
+        return kEmptyStr;
+    }
+
+    // this holds the value we return if no issues arise
+    CNcbiOstrstream result_oss;
+
+    const static string kRefSeqCat = "RefSeq Category";
+
+    // get category name
+    result_oss << kRefSeqCat << ": ";
+    CConstRef<CUser_field> pCategoryField = uo.GetFieldRef(kRefSeqCat);
+    if( pCategoryField &&
+        FIELD_IS_SET_AND_IS(*pCategoryField, Data, Str) ) 
+    {
+        const string & sCategory = pCategoryField->GetData().GetStr();
+        result_oss << sCategory << '\n';
+    } else {
+        result_oss << "(?UNKNOWN?)" << '\n';
+    }
+
+    // get details field
+    CConstRef<CUser_field> pDetailsField = uo.GetFieldRef("Details");
+
+    CUser_field::TMapFieldNameToRef mapFieldNameToRef;
+    if( pDetailsField ) {
+        pDetailsField->GetFieldsMap(mapFieldNameToRef,
+            CUser_field::fFieldMapFlags_ExcludeThis);
+
+        const static char * arrFieldNames[] = {
+            "CALC", "CCA", "CLI", "COM", "FGS", "MOD", "PHY", "PRT", "QfO", "TYS", "UPR"
+        };
+
+        ITERATE_0_IDX(field_idx, ArraySize(arrFieldNames) ) {
+            const CTempString sFieldName( arrFieldNames[field_idx] );
+            CUser_field::SFieldNameChain field_name;
+            field_name += sFieldName;
+
+            CUser_field::TMapFieldNameToRef::const_iterator find_iter =
+                mapFieldNameToRef.find(field_name);
+            if( find_iter == mapFieldNameToRef.end() ) {
+                // not found
+                continue;
+            }
+
+            if( ! FIELD_IS_SET_AND_IS(*find_iter->second, Data, Str) ) {
+                // only Str fields are supported at this time
+                continue;
+            }
+
+            // might need to pad
+            if( sFieldName.length() < kRefSeqCat.length() ) {
+                result_oss << string(
+                    (kRefSeqCat.length() - sFieldName.length()), ' ');
+            }
+
+            result_oss << sFieldName << ": "
+                       << find_iter->second->GetData().GetStr() << '\n';
+        }
+    }
+
+    return CNcbiOstrstreamToString(result_oss);
+}
+
 
 string CCommentItem::GetStringForWGS(CBioseqContext& ctx)
 {
@@ -730,7 +874,9 @@ string CCommentItem::GetStringForTSA(CBioseqContext& ctx)
     for (CSeqdesc_CI it(ctx.GetHandle(), CSeqdesc::e_User); it; ++it) {
         const CUser_object& uo = it->GetUser();
         if (uo.IsSetType()  &&  uo.GetType().IsStr()  &&
-            NStr::EqualNocase(uo.GetType().GetStr(), "TSA-mRNA-List")) {
+            ( NStr::EqualNocase(uo.GetType().GetStr(), "TSA-mRNA-List") ||
+              NStr::EqualNocase(uo.GetType().GetStr(), "TSA-RNA-List") ) ) 
+        {
             if (uo.HasField("Accession_first")) {
                 const CUser_field& uf = uo.GetField("Accession_first");
                 if (uf.IsSetData()  &&  uf.GetData().IsStr()  &&
@@ -889,7 +1035,7 @@ string s_HtmlWrapModelEvidenceName( const SModelEvidance& me )
 {
     stringstream strm;
     strm << "<a href=\"" << strLinkBaseNuc;
-    if( me.gi > 0 ) {
+    if( me.gi > ZERO_GI ) {
         strm << me.gi;
     } else {
         strm << me.name;
@@ -906,6 +1052,12 @@ string s_HtmlWrapModelEvidenceName( const SModelEvidance& me )
     return strm.str();
 }
 
+static
+string s_HtmlWrapTranscriptName( const string& name )
+{
+    return "<a href=\"" + strLinkBaseNuc + name + "\">" + name + "</a>";
+}
+
 string CCommentItem::GetStringForModelEvidance
 (const SModelEvidance& me,
  ECommentFormat format)
@@ -921,6 +1073,29 @@ string CCommentItem::GetStringForModelEvidance
     text << "MODEL " << *refseq << ":  " << "This record is predicted by "
          << "automated computational analysis. This record is derived from "
          << "a genomic sequence (" << me_name << ")";
+
+    if ( !me.assembly.empty() ) {
+        int num_assm = me.assembly.size();
+        text << " and transcript sequence";
+        if (num_assm > 1) {
+            text << "s";
+        }
+        text << " (";
+        int count = 0;
+        string prefix = "";
+        FOR_EACH_STRING_IN_LIST (str, me.assembly) {
+            const string tr_name = ( bHtml ? s_HtmlWrapTranscriptName(*str) : *str);
+            text << prefix << tr_name;
+            count++;
+            if (num_assm == count + 1) {
+                prefix = " and ";
+            } else {
+                prefix = ", ";
+            }
+        }
+        text << ")";
+    }
+
     if ( !me.method.empty() ) {
         text << " annotated using gene prediction method: " << me.method;
     }
@@ -939,7 +1114,7 @@ string CCommentItem::GetStringForModelEvidance
     }
 
     const char *documentation_str = ( bHtml ? 
-        "<a href=\"http://www.ncbi.nlm.nih.gov/genome/guide/build.shtml\">Documentation</a>" : 
+        "<a href=\"http://www.ncbi.nlm.nih.gov/genome/annotation_euk/process/\">Documentation</a>" : 
         "Documentation" );
 
     text << ".~Also see:~"
@@ -1034,6 +1209,141 @@ string CCommentItem::GetStringForEncode(CBioseqContext& ctx)
     return CNcbiOstrstreamToString(str);
 }
 
+// static
+string CCommentItem::GetStringForAuthorizedAccess(CBioseqContext& ctx)
+{
+    const bool bHtml = ctx.Config().DoHTML();
+
+    const string & sAuthorizedAccess = ctx.GetAuthorizedAccess();
+    if( sAuthorizedAccess.empty() ) {
+        return kEmptyStr;
+    }
+
+    CNcbiOstrstream str;
+
+    str << "These data are available through the dbGaP authorized access system. ";
+    if( bHtml ) {
+        str << "<a href=\""  
+            << "https://dbgap.ncbi.nlm.nih.gov/aa/wga.cgi?adddataset="
+            << sAuthorizedAccess << "&page=login\">";
+        str << "Request access";
+        str << "</a>";
+        str << " to Study ";
+        str << "<a href=\""  
+            << "http://www.ncbi.nlm.nih.gov/projects/gap/cgi-bin/study.cgi?study_id="
+            << sAuthorizedAccess << "\">";
+        str << sAuthorizedAccess;
+        str << "</a>";
+    } else {
+        str << "Request access to Study ";
+        str << sAuthorizedAccess;
+    }
+    str << "."; // always needs a period
+
+    return CNcbiOstrstreamToString(str);
+}
+
+string CCommentItem::GetStringForOpticalMap(CBioseqContext& ctx)
+{
+    const bool bHtml = ctx.Config().DoHTML();
+
+    const CPacked_seqpnt* pOpticalMapPoints = ctx.GetOpticalMapPoints();
+    if( ! pOpticalMapPoints || 
+        RAW_FIELD_IS_EMPTY_OR_UNSET(*pOpticalMapPoints, Points) )
+    {
+        return kEmptyStr;
+    }
+
+    const string & sFiletrackURL = ctx.GetFiletrackURL();
+
+    const bool bIsCircular = FIELD_EQUALS( ctx.GetHandle(), Inst_Topology, 
+        CSeq_inst::eTopology_circular );
+    const TSeqPos uBioseqLength =
+        GET_FIELD_OR_DEFAULT(ctx.GetHandle(), Inst_Length, 0);
+
+    CNcbiOstrstream str;
+
+    // vecOfPoints elements are 1-based
+    const CPacked_seqpnt::TPoints & vecOfPoints =
+        pOpticalMapPoints->GetPoints();
+    _ASSERT( ! vecOfPoints.empty() );
+
+    str << "This ";
+    if( bHtml && ! sFiletrackURL.empty() ) {
+        str << "<a href=\"" << sFiletrackURL << "\">";
+    }
+    str << "map";
+    if( bHtml && ! sFiletrackURL.empty() ) {
+        str << "</a>";
+    }
+    str << " has ";
+
+    size_t uNumFrags = pOpticalMapPoints->GetPoints().size();
+    if( ! bIsCircular )
+    {
+        // non-circular has an extra fragment because the
+        // last fragment does NOT wrap around to continue on
+        // the beginning of the bioseq.
+        if (uNumFrags > 1 && vecOfPoints[uNumFrags-1] < uBioseqLength - 1) {
+            ++uNumFrags;
+        }
+    }
+    str << uNumFrags;
+    str << " piece" << ( (uNumFrags > 1) ? "s" : "" ) << ":";
+
+    // prevEndPos and thisEndPos are 1-based
+    TSeqPos prevEndPos = 1;
+    TSeqPos thisEndPos = vecOfPoints[0] + 1;
+
+    // non-circular's first fragment is from 0 to the first rsite
+    if ( ! bIsCircular ) {
+        x_GetStringForOpticalMap_WriteFragmentLine(
+            str, prevEndPos, thisEndPos, uBioseqLength,
+            eFragmentType_Normal );
+    }
+    prevEndPos = thisEndPos + 1;
+
+    // regular fragments
+    for( size_t idx = 1; idx < vecOfPoints.size(); ++idx ) {
+        thisEndPos = vecOfPoints[idx] + 1;
+        x_GetStringForOpticalMap_WriteFragmentLine(
+            str, prevEndPos, thisEndPos, uBioseqLength,
+            eFragmentType_Normal );
+        prevEndPos = thisEndPos + 1;
+    }
+
+    // The last fragment for circular wraps around to the first rsite,
+    // but for non-circular it ends at the end of the bioseq
+    thisEndPos = ( bIsCircular ? vecOfPoints[0] + 1 : uBioseqLength );
+    if ( bIsCircular || prevEndPos < uBioseqLength - 1 ) {
+        x_GetStringForOpticalMap_WriteFragmentLine(
+                str, prevEndPos, thisEndPos, uBioseqLength,
+                ( bIsCircular ?
+                  eFragmentType_WrapAround :
+                  eFragmentType_Normal ) );
+    }
+
+    return CNcbiOstrstreamToString(str);
+}
+
+string CCommentItem::GetStringForUnique(CBioseqContext& ctx)
+{
+    if( ! ctx.IsRSUniqueProt() ) {
+        return kEmptyStr;
+    }
+
+    CNcbiOstrstream str;
+
+    // this will be more complex if HTML links ever need to be added
+    // or we have to cover nucs or whatever
+
+    str << "REFSEQ: This record represents a single, non-redundant, protein "
+        << "sequence which may be annotated on many different RefSeq "
+        << "genomes from the same, or different, species.";
+
+    return CNcbiOstrstreamToString(str);
+}
+
 /***************************************************************************/
 /*                                 PROTECTED                               */
 /***************************************************************************/
@@ -1069,11 +1379,26 @@ string s_HtmlizeStructuredCommentData( const bool is_html, const string &label_s
         return data_str;
     }
 
+    CNcbiOstrstream result;
     if( label_str == "GOLD Stamp ID" && NStr::StartsWith(data_str, "Gi") ) {
-        CNcbiOstrstream result;
         result << "<a href=\"http://genomesonline.org/cgi-bin/GOLD/bin/GOLDCards.cgi?goldstamp=" << data_str 
                << "\">" << data_str << "</a>";
         return CNcbiOstrstreamToString(result);
+    }
+    if ( label_str == "Annotation Software Version") {
+        result << "<a href=\"http://www.ncbi.nlm.nih.gov/genome/annotation_euk/release_notes/#version"
+               << data_str
+               << "\">" << data_str << "</a>";
+        return CNcbiOstrstreamToString(result);
+    } else if ( label_str == "Annotation Version") {
+        string fst;
+        string snd;
+        NStr::Replace( data_str, " Annotation Release ", "/", fst );
+        NStr::Replace( fst, " ", "_", snd );
+        result << "<a href=\"http://www.ncbi.nlm.nih.gov/genome/annotation_euk/"
+               << snd
+               << "\">" << data_str << "</a>";
+        return CNcbiOstrstreamToString(result);
     } else {
         // normalize case: nothing to do
         return data_str;
@@ -1334,6 +1659,45 @@ bool CCommentItem::x_IsCommentEmpty(void) const
     return true;
 }
 
+// static
+void CCommentItem::x_GetStringForOpticalMap_WriteFragmentLine(
+    ostream & str, TSeqPos prevEndPos, TSeqPos thisEndPos, 
+    TSeqPos uBioseqLength, EFragmentType eFragmentType)
+{
+    str << '\n';
+    str << "*  " 
+        << setw(7) << (prevEndPos) 
+        << ' ' 
+        << setw(7) << (thisEndPos)
+        << ": fragment of ";
+
+    bool bLengthIsOkay = true; // until proven otherwise
+    if( (eFragmentType == eFragmentType_Normal) &&
+        (thisEndPos <= prevEndPos) ) 
+    {
+        bLengthIsOkay = false;
+    } else if( (eFragmentType == eFragmentType_WrapAround) &&
+        (thisEndPos >= prevEndPos) ) 
+    {
+        bLengthIsOkay = false;
+    }
+
+    if( ! bLengthIsOkay ) {
+        str << "(ERROR: CANNOT CALCULATE LENGTH)";
+    } else if( (thisEndPos > uBioseqLength) || 
+        (prevEndPos > uBioseqLength) ) 
+    {
+        str << "(ERROR: FRAGMENT IS OUTSIDE BIOSEQ BOUNDS)";
+    } else {
+        if( eFragmentType == eFragmentType_Normal ) {
+            str << (thisEndPos - prevEndPos + 1);
+        } else {
+            str << (uBioseqLength + thisEndPos - prevEndPos + 1);
+        }
+    }
+    str << " bp in length";
+}
+
 /////////////////////////////////////////////////////////////////////////////
 //
 // Derived Classes
@@ -1467,7 +1831,7 @@ string s_CreateHistCommentString
 (const string& prefix,
  const string& suffix,
  const CSeq_hist_rec& hist,
- bool do_html)
+ const CBioseqContext& ctx)
 {
     //if (!hist.CanGetDate()  ||  !hist.CanGetIds()) {
     //    return "???";
@@ -1478,7 +1842,7 @@ string s_CreateHistCommentString
         hist.GetDate().GetDate(&date, "%{%3N%|???%} %{%D%|??%}, %{%4Y%|????%}");
     }
 
-    vector<int> gis;
+    vector<TGi> gis;
     ITERATE (CSeq_hist_rec::TIds, id, hist.GetIds()) {
         if ( (*id)->IsGi() ) {
             gis.push_back((*id)->GetGi());
@@ -1500,9 +1864,9 @@ string s_CreateHistCommentString
             text << ",";
         }
         text << " gi:";
-        NcbiId(text, gis[count], do_html);
+        NcbiId(text, gis[count], ctx.Config().DoHTML());
     }
-    text << '.' << endl;
+    text << '.' << '\n';
 
     return CNcbiOstrstreamToString(text);
 }
@@ -1518,13 +1882,13 @@ void CHistComment::x_GatherInfo(CBioseqContext& ctx)
                 "[WARNING] On",
                 "this project was updated. The new version is",
                 m_Hist->GetReplaced_by(),
-                ctx.Config().DoHTML()));
+                ctx));
         } else {
             x_SetComment(s_CreateHistCommentString(
                 "[WARNING] On",
                 "this sequence was replaced by",
                 m_Hist->GetReplaced_by(),
-                ctx.Config().DoHTML()));
+                ctx));
         }
         break;
     case eReplaces:
@@ -1532,7 +1896,7 @@ void CHistComment::x_GatherInfo(CBioseqContext& ctx)
             "On",
             "this sequence version replaced",
             m_Hist->GetReplaces(),
-            ctx.Config().DoHTML()));
+            ctx));
         break;
     }
 }
@@ -1588,6 +1952,37 @@ void CLocalIdComment::x_GatherInfo(CBioseqContext&)
     x_SetComment(CNcbiOstrstreamToString(msg));
 }
 
+// --- CFileIdComment
+
+CFileIdComment::CFileIdComment(const CObject_id& oid, CBioseqContext& ctx) :
+    CCommentItem(ctx, false), m_Oid(&oid)
+{
+    x_GatherInfo(ctx);
+}
+
+
+void CFileIdComment::x_GatherInfo(CBioseqContext&)
+{
+    CNcbiOstrstream msg;
+
+    switch ( m_Oid->Which() ) {
+    case CObject_id::e_Id:
+        msg << "FileID: " << m_Oid->GetId();    
+        break;
+    case CObject_id::e_Str:
+        if ( m_Oid->GetStr().length() < 1000 ) {
+            msg << "FileID: " << m_Oid->GetStr();
+        } else {
+            msg << "FileID string too large";
+        }
+        break;
+    default:
+        break;
+    }
+    x_SetComment(CNcbiOstrstreamToString(msg));
+}
+
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
 
