@@ -1,4 +1,4 @@
-/*  $Id: context.cpp 412095 2013-09-04 17:38:14Z rafanovi $
+/*  $Id: context.cpp 422633 2013-12-23 20:29:07Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -116,6 +116,7 @@ CBioseqContext::CBioseqContext
     m_HasOperon(false),
     m_HasMultiIntervalGenes(true), // true is the safe choice if we're not sure
     m_IsGenomeAssembly(false),
+    m_IsCrossKingdom(false),
     m_fUnverified(fUnverified_None),
     m_ShowAnnotCommentAsCOMMENT(false),
     m_FFCtx(ffctx),
@@ -172,6 +173,7 @@ CBioseqContext::CBioseqContext
     m_HasOperon(false),
     m_HasMultiIntervalGenes(true), // true is the safe choice if we're not sure
     m_IsGenomeAssembly(false),
+    m_IsCrossKingdom(false),
     m_fUnverified(fUnverified_None),
     m_ShowAnnotCommentAsCOMMENT(false),
     m_FFCtx(ffctx),
@@ -342,18 +344,56 @@ void CBioseqContext::x_SetHasMultiIntervalGenes(void)
 void CBioseqContext::x_SetTaxname(void)
 {
     // look for taxname in Seqdescs
+    int num_super_kingdom = 0;
+    bool super_kingdoms_different = false;
+    string super_kingdom_name;
     CSeqdesc_CI desc_ci( m_Handle, CSeqdesc::e_Source );
     for( ; desc_ci; ++desc_ci ) {
         if( desc_ci->IsSource() ) {
             const CBioSource &bsrc = desc_ci->GetSource();
+            if (bsrc.IsSetOrgname()) {
+                const COrgName& onp = bsrc.GetOrgname();
+                if (onp.IsSetName()) {
+                    const COrgName::TName& nam = onp.GetName();
+                    if (nam.IsPartial()) {
+                        const CPartialOrgName& pon = nam.GetPartial();
+                        if (pon.IsSet()) {
+                            const CPartialOrgName::Tdata& tx = pon.Get();
+                            ITERATE (CPartialOrgName::Tdata, itr, tx) {
+                                const CTaxElement& te = **itr;
+                                if (te.IsSetFixed_level()) {
+                                    if (te.GetFixed_level() == 0 && te.IsSetLevel()) {
+                                        const string& lvl = te.GetLevel();
+                                        if (NStr::EqualNocase (lvl, "superkingdom")) {
+                                            num_super_kingdom++;
+                                            if (super_kingdom_name.empty() && te.IsSetName()) {
+                                                super_kingdom_name = te.GetName();
+                                            } else if (te.IsSetName() && ! NStr::EqualNocase (super_kingdom_name, te.GetName())) {
+                                                super_kingdoms_different = true;
+                                            }
+                                            if (num_super_kingdom > 1 && super_kingdoms_different) {
+                                                m_IsCrossKingdom = true;
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
             if( bsrc.IsSetTaxname() && ! bsrc.GetTaxname().empty() ) {
-                // we found a taxname; we're done
+                // we found a taxname; but need to look at all descriptors to set m_IsCrossKingdom, so keep going
                 m_Taxname = bsrc.GetTaxname();
-                return;
+                // return;
             }
         }
     }
 
+    if (! m_Taxname.empty()) {
+        return;
+    }
+
     // fall back on the Seq-feats
 
     SAnnotSelector sel;
