@@ -1,4 +1,4 @@
-/*  $Id: keywords_item.cpp 413667 2013-09-17 17:07:09Z ivanov $
+/*  $Id: keywords_item.cpp 425372 2014-01-28 19:01:52Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -48,6 +48,10 @@
 #include <objtools/format/text_ostream.hpp>
 #include <objtools/format/items/keywords_item.hpp>
 #include <objtools/format/context.hpp>
+#include <objects/valid/Comment_set.hpp>
+#include <objects/valid/Comment_rule.hpp>
+
+#include <objects/misc/sequence_util_macros.hpp>
 
 
 BEGIN_NCBI_SCOPE
@@ -268,6 +272,19 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
         }
     }
 
+    CBioseq_Handle bsh = ctx.GetHandle();
+    for (CSeqdesc_CI di(bsh, CSeqdesc::e_User); di; ++di) {
+        const CUser_object& usr = di->GetUser();
+        if ( ! CComment_rule::IsStructuredComment (usr) ) continue;
+        try {
+            list<string> keywords = CComment_set::GetKeywords(usr);
+            FOR_EACH_STRING_IN_LIST ( s_itr, keywords ) {
+                x_AddKeyword(*s_itr);
+            }
+        } catch (CException) {
+        }
+    }
+
     if( ctx.GetUnverifiedType() != CBioseqContext::fUnverified_None ) {
         x_AddKeyword("UNVERIFIED");
     }
@@ -280,7 +297,6 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
         x_AddKeyword( ctx.GetFinishingStatus() );
     }
 
-    
     if ( ctx.IsTPA() ) {
         // add TPA keywords
         x_AddKeyword("Third Party Data");
@@ -289,7 +305,12 @@ void CKeywordsItem::x_GatherInfo(CBioseqContext& ctx)
         // add RefSeq keyword
         x_AddKeyword("RefSeq");
     }
-    
+
+    if ( ctx.IsCrossKingdom() && ctx.IsRSUniqueProt() ) {
+        // add CrossKingdom keyword
+        x_AddKeyword("CROSS_KINGDOM");
+    }
+
     for (CSeqdesc_CI it(ctx.GetHandle());  it;  ++it) {
         const list<string>* keywords = NULL;
         
