@@ -1,4 +1,4 @@
-/*  $Id: sam_formatter.cpp 358642 2012-04-04 15:33:22Z grichenk $
+/*  $Id: sam_formatter.cpp 431313 2014-04-02 20:01:20Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -123,7 +123,7 @@ string CSAM_CIGAR_Formatter::x_GetTargetIdString(void) const
 void CSAM_CIGAR_Formatter::x_AddLines(const TLines& lines)
 {
     ITERATE(TLines, it, lines) {
-        m_Out << *it << endl;
+        m_Out << *it << '\n';
     }
 }
 
