@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.cpp 412266 2013-09-05 17:37:26Z rafanovi $
+/*  $Id: bed_reader.cpp 448872 2014-10-09 16:35:32Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,6 +116,18 @@ CBedReader::~CBedReader()
 {
 }
 
+//  ----------------------------------------------------------------------------
+CRef< CSeq_annot >
+CBedReader::ReadSeqAnnot(
+    CNcbiIstream& istr,
+    IMessageListener* pMessageListener ) 
+//  ----------------------------------------------------------------------------
+{
+    xProgressInit(istr);
+    CStreamLineReader lr( istr );
+    return ReadSeqAnnot( lr, pMessageListener );
+}
+
 //  ----------------------------------------------------------------------------                
 CRef< CSeq_annot >
 CBedReader::ReadSeqAnnot(
@@ -123,7 +135,7 @@ CBedReader::ReadSeqAnnot(
     IMessageListener* pEC ) 
 //  ----------------------------------------------------------------------------                
 {
-    const int MAX_RECORDS = 100000;
+    const size_t MAX_RECORDS = 100000;
 
     CRef<CSeq_annot> annot;
     CRef<CAnnot_descr> desc;
@@ -135,34 +147,30 @@ CBedReader::ReadSeqAnnot(
 
     string line;
     int featureCount = 0;
-
-    while (!lr.AtEOF()) {
-        ++m_uLineNumber;
-        line = *++lr;
-        if (NStr::TruncateSpaces(line).empty()) {
+    while (xGetLine(lr, line)) {
+        if (IsCanceled()) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Info,
+                0,
+                "Reader stopped by user.",
+                ILineError::eProblem_ProgressInfo));
+            ProcessError(*pErr, pEC);
+            return CRef<CSeq_annot>();
+        }
+        xReportProgress(pEC);
+        if (xIsTrackLine(line)  &&  featureCount) {
+            xUngetLine(lr);
+            break;
+        }
+        if (xParseBrowserLine(line, annot, pEC)) {
             continue;
         }
-        if (xParseComment(line, annot)) {
+        if (xParseTrackLine(line, pEC)) {
             continue;
         }
-        if (x_ParseBrowserLine(line, annot, pEC)) {
-            continue;
-        }
-        if (NStr::StartsWith(line, "track ")) {
-            if (featureCount > 0) {
-                --m_uLineNumber;
-                lr.UngetLine();
-                break;
-            }
-            else {
-                if (xParseTrackLine(line, annot, pEC)) {
-                    continue;
-                }
-            }
-        }
 
-	    string record_copy = line;
-	    NStr::TruncateSpacesInPlace(record_copy);
+	    CTempString record_copy = NStr::TruncateSpaces_Unsafe(line);
 
         //  parse
         vector<string> fields;
@@ -174,14 +182,6 @@ CBedReader::ReadSeqAnnot(
             ProcessError(err, pEC);
             continue;
         }
-        if (fields[0] != m_currentId) {
-            //record id has changed
-            if (featureCount > 0) {
-                --m_uLineNumber;
-                lr.UngetLine();
-                break;
-            }
-        }
         if (xParseFeature(fields, annot, featureCount, pEC)) {
             ++featureCount;
             continue;
@@ -194,8 +194,8 @@ CBedReader::ReadSeqAnnot(
     if (0 == featureCount) {
         return CRef<CSeq_annot>();
     }
-    x_AddConversionInfo(annot, pEC);
-    x_AssignTrackData( annot );
+    xAddConversionInfo(annot, pEC);
+    xAssignTrackData( annot );
 
     if(m_columncount >= 3) {
         CRef<CUser_object> columnCountUser( new CUser_object() );
@@ -209,33 +209,6 @@ CBedReader::ReadSeqAnnot(
     return annot;
 }
 
-//  --------------------------------------------------------------------------- 
-void
-CBedReader::ReadSeqAnnots(
-    vector< CRef<CSeq_annot> >& annots,
-    CNcbiIstream& istr,
-    IMessageListener* pMessageListener )
-//  ---------------------------------------------------------------------------
-{
-    CStreamLineReader lr( istr );
-    ReadSeqAnnots( annots, lr, pMessageListener );
-}
- 
-//  ---------------------------------------------------------------------------                       
-void
-CBedReader::ReadSeqAnnots(
-    vector< CRef<CSeq_annot> >& annots,
-    ILineReader& lr,
-    IMessageListener* pMessageListener )
-//  ----------------------------------------------------------------------------
-{
-    CRef<CSeq_annot> annot = ReadSeqAnnot(lr, pMessageListener);
-    while (annot) {
-        annots.push_back(annot);
-        annot = ReadSeqAnnot(lr, pMessageListener);
-    }
-}
-                        
 //  ----------------------------------------------------------------------------                
 CRef< CSerialObject >
 CBedReader::ReadObject(
@@ -252,7 +225,6 @@ CBedReader::ReadObject(
 bool
 CBedReader::xParseTrackLine(
     const string& strLine,
-    CRef< CSeq_annot >& current,
     IMessageListener* pEC)
 //  ----------------------------------------------------------------------------
 {
@@ -275,13 +247,14 @@ CBedReader::xParseTrackLine(
         m_columncount = 0;
     }
     m_currentId.clear();
-    if (!CReaderBase::x_ParseTrackLine(strLine, current, pEC)) {
-        CObjReaderLineException err(
+    if (!CReaderBase::xParseTrackLine(strLine, pEC)) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Warning,
             0,
             "Bad track line: Expected \"track key1=value1 key2=value2 ...\". Ignored.",
-            ILineError::eProblem_BadTrackLine);
-        ProcessWarning(err , pEC);    
+            ILineError::eProblem_BadTrackLine) );
+        ProcessWarning(*pErr , pEC);    
     }
     return true;
 }
@@ -315,11 +288,12 @@ bool CBedReader::xParseFeature(
             m_columncount = fields.size();
         }
         else {
-            CObjReaderLineException err(
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Bad data line: Inconsistent column count." );
-            ProcessError(err, pEC );
+                "Bad data line: Inconsistent column count." ) );
+            ProcessError(*pErr, pEC );
             return false;
         }
     }
@@ -366,7 +340,7 @@ bool CBedReader::xAppendFeatureChrom(
     CRef<CSeq_feat> feature;
     feature.Reset(new CSeq_feat);
     try {
-        xSetFeatureTitle(feature, fields);
+        //xSetFeatureTitle(feature, fields);
         xSetFeatureLocationChrom(feature, fields);
         xSetFeatureIdsChrom(feature, fields, baseId);
         xSetFeatureBedData(feature, fields);
@@ -393,7 +367,7 @@ bool CBedReader::xAppendFeatureThick(
     CRef<CSeq_feat> feature;
     feature.Reset(new CSeq_feat);
     try {
-        xSetFeatureTitle(feature, fields);
+        //xSetFeatureTitle(feature, fields);
         xSetFeatureLocationThick(feature, fields);
         xSetFeatureIdsThick(feature, fields, baseId);
         xSetFeatureBedData(feature, fields);
@@ -419,7 +393,7 @@ bool CBedReader::xAppendFeatureBlock(
     CRef<CSeq_feat> feature;
     feature.Reset(new CSeq_feat);
     try {
-        xSetFeatureTitle(feature, fields);
+        //xSetFeatureTitle(feature, fields);
         xSetFeatureLocationBlock(feature, fields);
         xSetFeatureIdsBlock(feature, fields, baseId);
         xSetFeatureBedData(feature, fields);
@@ -432,7 +406,6 @@ bool CBedReader::xAppendFeatureBlock(
     ftable.push_back(feature);
     return true;
 }
-
 //  ----------------------------------------------------------------------------
 bool CBedReader::xParseFeatureUserFormat(
     const vector<string>& fields,
@@ -459,18 +432,6 @@ bool CBedReader::xParseFeatureUserFormat(
     return true;
 }
 
-//  ----------------------------------------------------------------------------
-bool CBedReader::xParseComment(
-    const string& record,
-    CRef<CSeq_annot>& annot ) /* throws CObjReaderLineException */
-//  ----------------------------------------------------------------------------
-{
-    if (NStr::StartsWith(record, "#")) {
-        return true;
-    }
-    return false;
-}
- 
 //  ----------------------------------------------------------------------------
 void CBedReader::x_SetFeatureDisplayData(
     CRef<CSeq_feat>& feature,
@@ -551,27 +512,30 @@ void CBedReader::xSetFeatureLocationThick(
 {
     CRef<CSeq_loc> location(new CSeq_loc);
     int from, to;
+    from = to = -1;
 
     //already established: We got at least three columns
     try {
         from = NStr::StringToInt(fields[6]);
     }
     catch (std::exception&) {
-        CObjReaderLineException err( 
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Invalid data line: Bad \"ThickStart\" value." );
-        throw(err);
+            "Invalid data line: Bad \"ThickStart\" value." ) );
+        pErr->Throw();
     }
     try {
         to = NStr::StringToInt(fields[7])-1;
     }
     catch (std::exception&) {
-        CObjReaderLineException err( 
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Invalid data line: Bad \"ThickStop\" value.");
-        throw(err);
+            "Invalid data line: Bad \"ThickStop\" value.") );
+        pErr->Throw();
     }
     if (from == to) {
         location->SetPnt().SetPoint(from);
@@ -580,16 +544,17 @@ void CBedReader::xSetFeatureLocationThick(
         location->SetInt().SetFrom(from);
         location->SetInt().SetTo(to);
     }
-    else {
-        CObjReaderLineException err( 
-            eDiag_Error,
-            0,
-            "Invalid data line: \"ThickStop\" less than \"ThickStart\"." );
-        throw(err);
+    else if (from > to) {
+        //below: flip commenting to switch from null locations to impossible 
+        // intervals
+        //location->SetInt().SetFrom(from);
+        //location->SetInt().SetTo(to);
+        location->SetNull();
     }
 
-    location->SetStrand(xGetStrand(fields));
-    
+    if (!location->IsNull()) {
+        location->SetStrand(xGetStrand(fields));
+    }
     CRef<CSeq_id> id = CReadUtil::AsSeqId(fields[0], m_iFlags, false);
     location->SetId(*id);
     feature->SetLocation(*location);
@@ -613,11 +578,12 @@ ENa_strand CBedReader::xGetStrand(
     if (strand_field < fields.size()) {
         string strand = fields[strand_field];
         if (strand != "+"  &&  strand != "-"  &&  strand != ".") {
-            CObjReaderLineException err( 
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Invalid data line: Invalid strand character." );
-            throw(err);
+                "Invalid data line: Invalid strand character." ) );
+            pErr->Throw();
         }
     }
     return (fields[strand_field] == "-" ? eNa_strand_minus : eNa_strand_plus);
@@ -630,9 +596,9 @@ void CBedReader::xSetFeatureLocationBlock(
 //  ----------------------------------------------------------------------------
 {
     //already established: there are sufficient columns to do this
-    int blockCount = NStr::StringToInt(fields[9]);
-    vector<int> blockSizes;
-    vector<int> blockStarts;
+    size_t blockCount = NStr::StringToUInt(fields[9]);
+    vector<size_t> blockSizes;
+    vector<size_t> blockStarts;
     {{
         blockSizes.reserve(blockCount);
         vector<string> vals; 
@@ -641,51 +607,55 @@ void CBedReader::xSetFeatureLocationBlock(
             vals.erase(vals.end()-1);
         }
         if (vals.size() != blockCount) {
-            CObjReaderLineException err( 
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Invalid data line: Bad value count in \"blockSizes\"." );
-            throw( err );
+                "Invalid data line: Bad value count in \"blockSizes\"." ) );
+            pErr->Throw();
         }
         try {
-            for (int i=0; i < blockCount; ++i) {
-                blockSizes.push_back(NStr::StringToInt(vals[i]));
+            for (size_t i=0; i < blockCount; ++i) {
+                blockSizes.push_back(NStr::StringToUInt(vals[i]));
             }
         }
         catch (std::exception&) {
-            CObjReaderLineException err( 
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Invalid data line: Malformed \"blockSizes\" column." );
-            throw( err );
+                "Invalid data line: Malformed \"blockSizes\" column." ) );
+            pErr->Throw();
         }
     }}
     {{
         blockStarts.reserve(blockCount);
         vector<string> vals; 
-        int baseStart = NStr::StringToInt(fields[1]);
+        size_t baseStart = NStr::StringToUInt(fields[1]);
         NStr::Tokenize(fields[11], ",", vals);
         if (vals.back() == "") {
             vals.erase(vals.end()-1);
         }
         if (vals.size() != blockCount) {
-            CObjReaderLineException err( 
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Invalid data line: Bad value count in \"blockStarts\"." );
-            throw( err );
+                "Invalid data line: Bad value count in \"blockStarts\"." ) );
+            pErr->Throw();
         }
         try {
-            for (int i=0; i < blockCount; ++i) {
-                blockStarts.push_back(baseStart + NStr::StringToInt(vals[i]));
+            for (size_t i=0; i < blockCount; ++i) {
+                blockStarts.push_back(baseStart + NStr::StringToUInt(vals[i]));
             }
         }
         catch (std::exception&) {
-            CObjReaderLineException err( 
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Invalid data line: Malformed \"blockStarts\" column." );
-            throw( err );
+                "Invalid data line: Malformed \"blockStarts\" column." ) );
+            pErr->Throw();
         }
     }}
 
@@ -693,13 +663,21 @@ void CBedReader::xSetFeatureLocationBlock(
     
     ENa_strand strand = xGetStrand(fields);
     CRef<CSeq_id> pId = CReadUtil::AsSeqId(fields[0], m_iFlags, false);
-    for (int i=0; i < blockCount; ++i) {
+
+    bool negative = fields[5] == "-";
+
+    CPacked_seqint::Tdata& blocks = location.Set();
+
+    for (size_t i=0; i < blockCount; ++i) {
         CRef<CSeq_interval> pInterval(new CSeq_interval);
         pInterval->SetId(*pId);
         pInterval->SetFrom(blockStarts[i]);
         pInterval->SetTo(blockStarts[i] + blockSizes[i]);
         pInterval->SetStrand(strand);
-        location.Set().push_back(pInterval);
+        if (negative)
+            blocks.insert(blocks.begin(), pInterval);
+        else
+            blocks.push_back(pInterval);
     }
 
     CRef<CUser_object> pBed(new CUser_object());
@@ -791,7 +769,7 @@ void CBedReader::xSetFeatureTitle(
 //  ----------------------------------------------------------------------------
 {
     if (fields.size() >= 4  &&  !fields[3].empty()  &&  fields[3] != ".") {
-        feature->SetTitle(fields[3]);
+        feature->SetTitle(fields[0]);
     }
     else {
         feature->SetTitle(string("line_") + NStr::IntToString(m_uLineNumber));
@@ -805,7 +783,12 @@ void CBedReader::xSetFeatureBedData(
 //  ----------------------------------------------------------------------------
 {
     CSeqFeatData& data = feature->SetData();
-    data.SetRegion() = fields[0];
+	if (fields.size() >= 4  &&  fields[3] != ".") {
+		data.SetRegion() = fields[3];
+	}
+	else {
+		data.SetRegion() = fields[0];
+	}
     
     CRef<CUser_object> pDisplayData(new CUser_object());
     if (fields.size() < 5  ||  fields[4] == ".") {
@@ -816,24 +799,90 @@ void CBedReader::xSetFeatureBedData(
     pDisplayData->SetType().SetStr("DisplaySettings");
     exts.push_front(pDisplayData);
 
-    try {
-        int score = NStr::StringToInt(fields[4]);
-        pDisplayData->AddField("score", score);
+    int int_score = NStr::StringToInt(fields[4], NStr::fConvErr_NoThrow );
+    double d_score = 0;
+
+    if (int_score == 0 && fields[4].compare("0") != 0)
+    {
+        try {
+            d_score = NStr::StringToDouble(fields[4]);
+        }
+        catch(std::exception&) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Invalid data line: Bad \"score\" value.") );
+            pErr->Throw();
+        }
     }
-    catch(std::exception&) {
-        CObjReaderLineException err( 
+
+    if (d_score < 0 || int_score < 0)
+    {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Invalid data line: Bad \"strand\" value.");
-        throw(err);
+            "Invalid data line: Bad \"score\" value.") );
+        pErr->Throw();
+    }
+    else
+    if (d_score > 0)
+    {
+        pDisplayData->AddField("score", d_score);
+    }
+    else
+    {
+        pDisplayData->AddField("score", int_score);
     }
 
     if (fields.size() < 9) {
         return;
     }
-    vector<string> rgb;
-    NStr::Tokenize(fields[8], ",", rgb);
-    string rgbValue = NStr::Join(rgb, " ");
+    vector<string> srgb;
+    if (fields[8] == "0"  ||  fields[8] == ".") {
+        srgb.push_back("0");
+        srgb.push_back("0");
+        srgb.push_back("0");
+    }
+    else {
+        NStr::Tokenize(fields[8], ",", srgb);
+    }
+    if (srgb.size() != 3)
+    {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad color value.") );
+        pErr->Throw();
+    }
+    try
+    {
+        for (int i=0; i < 3; i++)
+        {
+           int x = NStr::StringToInt(srgb[i]);
+           if (x<0 || x>255) {
+                AutoPtr<CObjReaderLineException> pErr(
+                    CObjReaderLineException::Create(
+                    eDiag_Error,
+                    0,
+                    "Invalid data line: Bad color value.") );
+                pErr->Throw();
+           }
+
+        }
+    }
+    catch(const std::exception&)
+    {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Invalid data line: Bad color value.") );
+        pErr->Throw();
+    }
+    string rgbValue = NStr::Join(srgb, " ");
     pDisplayData->AddField("color", rgbValue);
 }
 
@@ -853,27 +902,30 @@ void CBedReader::x_SetFeatureLocation(
 
     CRef<CSeq_loc> location(new CSeq_loc);
     int from, to;
+    from = to = -1;
 
     //already established: We got at least three columns
     try {
         from = NStr::StringToInt(fields[1]);
     }
     catch(std::exception&) {
-        CObjReaderLineException err( 
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Invalid data line: Bad \"SeqStart\" value." );
-        throw( err );
+            "Invalid data line: Bad \"SeqStart\" value." ) );
+        pErr->Throw();
     }
     try {
         to = NStr::StringToInt(fields[2]) - 1;
     }
     catch(std::exception&) {
-        CObjReaderLineException err( 
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Invalid data line: Bad \"SeqStop\" value.");
-        throw( err );
+            "Invalid data line: Bad \"SeqStop\" value.") );
+        pErr->Throw();
     }
     if (from == to) {
         location->SetPnt().SetPoint(from);
@@ -883,11 +935,12 @@ void CBedReader::x_SetFeatureLocation(
         location->SetInt().SetTo(to);
     }
     else {
-        CObjReaderLineException err( 
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Invalid data line: \"SeqStop\" less than \"SeqStart\"." );
-        throw(err);
+            "Invalid data line: \"SeqStop\" less than \"SeqStart\"." ) );
+        pErr->Throw();
     }
 
     size_t strand_field = 5;
@@ -897,11 +950,12 @@ void CBedReader::x_SetFeatureLocation(
     if (strand_field < fields.size()) {
         string strand = fields[strand_field];
         if (strand != "+"  &&  strand != "-"  &&  strand != ".") {
-            CObjReaderLineException err( 
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Invalid data line: Invalid strand character." );
-            throw(err);
+                "Invalid data line: Invalid strand character." ) );
+            pErr->Throw();
         }
         location->SetStrand(( fields[strand_field] == "+" ) ?
                            eNa_strand_plus : eNa_strand_minus );
@@ -913,61 +967,37 @@ void CBedReader::x_SetFeatureLocation(
 }
 
 //  ----------------------------------------------------------------------------
-void CBedReader::x_SetTrackData(
+void CBedReader::xSetTrackData(
     CRef<CSeq_annot>& annot,
     CRef<CUser_object>& trackdata,
     const string& strKey,
-    const string& strValue )
+    const string& strValue)
 //  ----------------------------------------------------------------------------
 {
     CAnnot_descr& desc = annot->SetDesc();
 
-    if ( strKey == "useScore" ) {
-        m_usescore = ( 1 == NStr::StringToInt( strValue ) );
-        trackdata->AddField( strKey, NStr::StringToInt( strValue ) );
+    if (strKey == "useScore") {
+        m_usescore = (1 == NStr::StringToInt(strValue));
+        trackdata->AddField( strKey, NStr::StringToInt(strValue));
         return;
     }
-    if ( strKey == "name" ) {
-        CRef<CAnnotdesc> name( new CAnnotdesc() );
-        name->SetName( strValue );
-        desc.Set().push_back( name );
+    if (strKey == "name") {
+        CRef<CAnnotdesc> name(new CAnnotdesc());
+        name->SetName(strValue);
+        desc.Set().push_back(name);
         return;
     }
-    if ( strKey == "description" ) {
-        CRef<CAnnotdesc> title( new CAnnotdesc() );
-        title->SetTitle( strValue );
-        desc.Set().push_back( title );
+    if (strKey == "description") {
+        CRef<CAnnotdesc> title(new CAnnotdesc());
+        title->SetTitle(strValue);
+        desc.Set().push_back(title);
         return;
     }
-    if ( strKey == "visibility" ) {
-        trackdata->AddField( strKey, NStr::StringToInt( strValue ) );
+    if (strKey == "visibility") {
+        trackdata->AddField(strKey, NStr::StringToInt(strValue));
         return;
     }
-    CReaderBase::x_SetTrackData( annot, trackdata, strKey, strValue );
-}
-
-//  ----------------------------------------------------------------------------
-bool CBedReader::xGetLine(
-    ILineReader& lr,
-    string& line)
-//  ----------------------------------------------------------------------------
-{
-    while (!lr.AtEOF()) {
-        line = *++lr;
-        if (!xCommentLine(line)) {
-            return true;
-        }
-    }
-	return false;
-}
-
-//  ----------------------------------------------------------------------------
-inline bool CBedReader::xCommentLine(
-    const string& line) const
-//  ----------------------------------------------------------------------------
-{
-    char c = line.data()[0];
-    return c == '#' || c == '\0';
+    CReaderBase::xSetTrackData(annot, trackdata, strKey, strValue);
 }
 
 //  ----------------------------------------------------------------------------
@@ -1033,11 +1063,12 @@ CBedReader::xReadBedRecordRaw(
             m_columncount = columns.size();
         }
         else {
-            CObjReaderLineException err(
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Bad data line: Inconsistent column count." );
-            ProcessError(err, pMessageListener);
+                "Bad data line: Inconsistent column count." ) );
+            ProcessError(*pErr, pMessageListener);
             return false;
         }
     }
@@ -1050,11 +1081,12 @@ CBedReader::xReadBedRecordRaw(
         start = NStr::StringToInt(columns[1]);
     }
     catch(std::exception&) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Bad data line: Invalid \"SeqStart\" (column 2) value." );
-        ProcessError(err, pMessageListener);
+            "Bad data line: Invalid \"SeqStart\" (column 2) value." ) );
+        ProcessError(*pErr, pMessageListener);
         return false;
     }
 
@@ -1063,11 +1095,12 @@ CBedReader::xReadBedRecordRaw(
         stop = NStr::StringToInt(columns[2]);
     }
     catch(std::exception&) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Bad data line: Invalid \"SeqStop\" (column 3) value." );
-        ProcessError(err, pMessageListener);
+            "Bad data line: Invalid \"SeqStop\" (column 3) value." ) );
+        ProcessError(*pErr, pMessageListener);
         return false;
     }
 
@@ -1077,11 +1110,12 @@ CBedReader::xReadBedRecordRaw(
             score = NStr::StringToInt(columns[6]);
         }
         catch(std::exception&) {
-            CObjReaderLineException err(
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Error,
                 0,
-                "Bad data line: Invalid \"Score\" (column 5) value." );
-            ProcessError(err, pMessageListener);
+                "Bad data line: Invalid \"Score\" (column 5) value." ) );
+            ProcessError(*pErr, pMessageListener);
             return false;
         }
     }
@@ -1157,11 +1191,12 @@ CBedReader::xCleanColumnValues(
         columns.erase(columns.begin());
     }
     if (columns.size() < 3) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Bad data line: Insufficient column count." );
-        throw(err);
+            "Bad data line: Insufficient column count." ) );
+        pErr->Throw();
     }
 
     try {
@@ -1169,11 +1204,12 @@ CBedReader::xCleanColumnValues(
         columns[1] = fixup;
     }
     catch(std::exception&) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Bad data line: Invalid \"SeqStart\" (column 2) value." );
-        throw( err );
+            "Bad data line: Invalid \"SeqStart\" (column 2) value." ) );
+        pErr->Throw();
     }
 
     try {
@@ -1181,11 +1217,12 @@ CBedReader::xCleanColumnValues(
         columns[2] = fixup;
     }
     catch(std::exception&) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Bad data line: Invalid \"SeqStop\" (column 3) value." );
-        throw( err );
+            "Bad data line: Invalid \"SeqStop\" (column 3) value." ) );
+        pErr->Throw();
     }
 }
 
