@@ -1,4 +1,4 @@
-/*  $Id: fasta.cpp 418830 2013-11-19 23:15:50Z rafanovi $
+/*  $Id: fasta.cpp 448717 2014-10-08 15:44:23Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -85,26 +85,9 @@
 
 #define FASTA_LINE_EXPT(_eSeverity, _uLineNum, _MessageStrmOps,  _eErrCode, _eProblem, _sFeature, _sQualName, _sQualValue) \
     do {                                                                \
-        const string sSeqId_49518053 = ( m_BestID ?                     \
-                                         m_BestID->AsFastaString() :    \
-                                         kEmptyStr  );                  \
-        const size_t uLineNum_49518053 = (_uLineNum);                   \
         stringstream err_strm_49518053;                                 \
         err_strm_49518053 << _MessageStrmOps;                           \
-        CObjReaderLineException lineExpt(                               \
-            (_eSeverity), uLineNum_49518053,                            \
-            err_strm_49518053.str(),                                    \
-            (_eProblem),                                                \
-            sSeqId_49518053, (_sFeature),                               \
-            (_sQualName), (_sQualValue),                                \
-            CObjReaderParseException::_eErrCode);                       \
-        if ( ! pMessageListener && (_eSeverity) <= eDiag_Warning ) {    \
-            ERR_POST_X(1, "FASTA-Reader: Warning: " + lineExpt.Message()); \
-        } else if ( ! pMessageListener || ! pMessageListener->PutError( lineExpt ) ) \
-        {                                                               \
-            NCBI_THROW2(CObjReaderParseException, _eErrCode,            \
-                        err_strm_49518053.str(), uLineNum_49518053 );   \
-        }                                                               \
+        PostWarning(pMessageListener, (_eSeverity), (_uLineNum), (err_strm_49518053.str()),  (_eErrCode), (_eProblem), (_sFeature), (_sQualName), (_sQualValue)); \
     } while(0)
 
 #define FASTA_PROGRESS(_MessageStrmOps)                                 \
@@ -118,10 +101,10 @@
 
 
 #define FASTA_WARNING(_uLineNum, _MessageStrmOps, _eProblem, _Feature) \
-    FASTA_LINE_EXPT(eDiag_Warning, _uLineNum, _MessageStrmOps, eFormat, _eProblem, _Feature, kEmptyStr, kEmptyStr)
+    FASTA_LINE_EXPT(eDiag_Warning, _uLineNum, _MessageStrmOps, CObjReaderParseException::eFormat, _eProblem, _Feature, kEmptyStr, kEmptyStr)
 
 #define FASTA_WARNING_EX(_uLineNum, _MessageStrmOps, _eProblem, _Feature, _sQualName, _sQualValue) \
-    FASTA_LINE_EXPT(eDiag_Warning, _uLineNum, _MessageStrmOps, eFormat, _eProblem, _Feature, _sQualName, _sQualValue)
+    FASTA_LINE_EXPT(eDiag_Warning, _uLineNum, _MessageStrmOps, CObjReaderParseException::eFormat, _eProblem, _Feature, _sQualName, _sQualValue)
 
 #define FASTA_ERROR(_uLineNum, _MessageStrmOps, _eErrCode) \
     FASTA_LINE_EXPT(eDiag_Error, _uLineNum, _MessageStrmOps, _eErrCode, ILineError::eProblem_GeneralParsingError, kEmptyStr, kEmptyStr, kEmptyStr)
@@ -259,28 +242,36 @@ inline bool s_ASCII_IsValidNuc(unsigned char c)
 
 CFastaReader::CFastaReader(ILineReader& reader, TFlags flags)
     : m_LineReader(&reader), m_MaskVec(0), 
-      m_IDGenerator(new CSeqIdGenerator), m_MaxIDLength(kMax_UI4)
+      m_IDGenerator(new CSeqIdGenerator), 
+      m_gapNmin(0), m_gap_Unknown_length(0),
+      m_MaxIDLength(kMax_UI4)
 {
     m_Flags.push(flags);
 }
 
 CFastaReader::CFastaReader(CNcbiIstream& in, TFlags flags)
     : m_LineReader(ILineReader::New(in)), m_MaskVec(0),
-      m_IDGenerator(new CSeqIdGenerator), m_MaxIDLength(kMax_UI4)
+      m_IDGenerator(new CSeqIdGenerator), 
+      m_gapNmin(0), m_gap_Unknown_length(0),
+      m_MaxIDLength(kMax_UI4)
 {
     m_Flags.push(flags);
 }
 
 CFastaReader::CFastaReader(const string& path, TFlags flags)
     : m_LineReader(ILineReader::New(path)), m_MaskVec(0),
-      m_IDGenerator(new CSeqIdGenerator), m_MaxIDLength(kMax_UI4)
+      m_IDGenerator(new CSeqIdGenerator), 
+      m_gapNmin(0), m_gap_Unknown_length(0),
+      m_MaxIDLength(kMax_UI4)
 {
     m_Flags.push(flags);
 }
 
 CFastaReader::CFastaReader(CReaderBase::TReaderFlags fBaseFlags, TFlags flags)
     : CReaderBase(fBaseFlags), m_MaskVec(0), 
-      m_IDGenerator(new CSeqIdGenerator), m_MaxIDLength(kMax_UI4)
+      m_IDGenerator(new CSeqIdGenerator), 
+      m_gapNmin(0), m_gap_Unknown_length(0),
+      m_MaxIDLength(kMax_UI4)
 {
     m_Flags.push(flags);
 }
@@ -290,6 +281,20 @@ CFastaReader::~CFastaReader(void)
     _ASSERT(m_Flags.size() == 1);
 }
 
+void CFastaReader::SetMinGaps(TSeqPos gapNmin, TSeqPos gap_Unknown_length)
+{
+    m_gapNmin = gapNmin; m_gap_Unknown_length = gap_Unknown_length;
+}
+
+inline
+void CFastaReader::CloseGap(bool atStartOfLine, IMessageListener * pMessageListener)
+{
+    if (m_CurrentGapLength > 0) {
+        x_CloseGap(m_CurrentGapLength, atStartOfLine, pMessageListener);
+        m_CurrentGapLength = 0;
+    }
+}
+
 CRef<CSerialObject>
 CFastaReader::ReadObject (ILineReader &lr, IMessageListener *pMessageListener)
 {
@@ -342,24 +347,25 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(IMessageListener * pMessageListener)
         if (GetLineReader().AtEOF()) {
             FASTA_ERROR(LineNumber(), 
                         "CFastaReader: Unexpected end-of-file around line " << LineNumber(),
-                        eEOF );
+                        CObjReaderParseException::eEOF );
             break;
         }
-        if (c == '>') {
-            if (need_defline) {
-                ParseDefLine(*++GetLineReader(), pMessageListener);
-                need_defline = false;
-                continue;
+        if (c == '>' ) {
+            CTempString next_line = *++GetLineReader();
+            if( NStr::StartsWith(next_line, ">?") ) {
+                // This is actually a data line. an assembly gap, in particular, which
+                // we handle farther below
+                GetLineReader().UngetLine();
             } else {
-                CTempString next_line = *++GetLineReader();
-                if (next_line.size() > 2  &&  next_line[1] == '?'
-                    &&  ParseGapLine(next_line, pMessageListener) ) {
+                if(need_defline) {
+                    ParseDefLine(next_line, pMessageListener);
+                    need_defline = false;
                     continue;
                 } else {
                     GetLineReader().UngetLine();
+                    // start of the next sequence
+                    break;
                 }
-                // start of the next sequence
-                break;
             }
         } else if (c == '[') {
             return x_ReadSegSet(pMessageListener);
@@ -367,7 +373,7 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(IMessageListener * pMessageListener)
             if (need_defline) {
                 FASTA_ERROR(LineNumber(), 
                     "CFastaReader: Reached unexpected end of segmented set around line " << LineNumber(),
-                    eEOF );
+                    CObjReaderParseException::eEOF );
             }
             break;
         }
@@ -418,7 +424,7 @@ CRef<CSeq_entry> CFastaReader::ReadOneSeq(IMessageListener * pMessageListener)
     if (need_defline  &&  GetLineReader().AtEOF()) {
         FASTA_ERROR(LineNumber(),
             "CFastaReader: Expected defline around line " << LineNumber(),
-            eEOF);
+            CObjReaderParseException::eEOF);
     }
 
     AssembleSeq(pMessageListener);
@@ -497,7 +503,8 @@ CRef<CSeq_entry> CFastaReader::ReadSet(int max_seqs, IMessageListener * pMessage
             if (max_seqs == 1) {
                 return entry2;
             }
-            entry->SetSet().SetSeq_set().push_back(entry2);
+            if (entry2.NotEmpty())
+              entry->SetSet().SetSeq_set().push_back(entry2);
         } catch (CObjReaderParseException& e) {
             if (e.GetErrCode() == CObjReaderParseException::eEOF) {
                 break;
@@ -648,7 +655,7 @@ void CFastaReader::ParseDefLine(const TStr& s, IMessageListener * pMessageListen
         if (TestFlag(fRequireID)) {
             FASTA_ERROR(LineNumber(),
                         "CFastaReader: Defline lacks a proper ID around line " << LineNumber(),
-                        eNoIDs );
+                        CObjReaderParseException::eNoIDs );
         }
         GenerateID();
     } else if ( !TestFlag(fForceType) ) {
@@ -715,7 +722,7 @@ void CFastaReader::ParseDefLine(const TStr& s, IMessageListener * pMessageListen
                 FASTA_ERROR(LineNumber(),
                     "CFastaReader: Seq-id " << h.AsString()
                     << " is a duplicate around line " << LineNumber(),
-                    eDuplicateID );
+                    CObjReaderParseException::eDuplicateID );
             }
         }
     }
@@ -739,11 +746,26 @@ bool CFastaReader::ParseIDs(
     if (TestFlag(fParseRawID)) {
         flags |= CSeq_id::fParse_RawText;
     }
+
     try {
-        count = CSeq_id::ParseIDs(ids, s, flags);
+        if (s.find(',') != TStr::npos && s.find('|') == TStr::npos)
+        {
+            string temp = NStr::Replace(s, ",", "_");
+            count = CSeq_id::ParseIDs(ids, temp, flags);
+            FASTA_WARNING(LineNumber(),
+                "CFastaReader: Near line " << LineNumber() 
+                << ", the sequence contains 'comma' symbol and replaced with 'underscore' "
+                << "symbol. Please find and correct the sequence id.",
+                ILineError::eProblem_GeneralParsingError, "");
+        }
+        else
+        {
+            count = CSeq_id::ParseIDs(ids, s, flags);
+        }
     } catch (CSeqIdException&) {
         // swap(ids, old_ids);
     }
+
     // numerics become local, if requested
     if( m_iFlags & CReaderBase::fNumericIdsAsLocal ) {
         NON_CONST_ITERATE(CBioseq::TId, id_it, ids) {
@@ -755,19 +777,24 @@ bool CFastaReader::ParseIDs(
         }
     }
     // recheck raw local IDs
-    if (count == 1  &&  ids.back()->IsLocal()
-        &&  !NStr::StartsWith(s, "lcl|", NStr::eNocase)
-        &&  !IsValidLocalID(s)) {
-        // swap(ids, old_ids);
-        ids.clear();
-        return false;
+    if (count == 1  &&  ids.back()->IsLocal())
+    {
+        string temp;
+        ids.back()->GetLabel(&temp, 0, CSeq_id::eContent);
+        if (!IsValidLocalID(temp))
+        {
+            //&&  !NStr::StartsWith(s, "lcl|", NStr::eNocase)
+            ids.clear();
+            return false;
+        }
     }
     // check if ID was too long
     if( count == 1 && s.length() > m_MaxIDLength ){ 
 
         // before throwing an ID-too-long error, check if what we
         // think is a "sequence ID" is actually sequence data
-        if( CreateWarningsForSeqDataInTitle(s, LineNumber(), pMessageListener) ) {
+        CMessageListenerLenient dummy_err_container; // so we can ignore them
+        if( CreateWarningsForSeqDataInTitle(s, LineNumber(), &dummy_err_container) ) {
             // it's actually seq data
             return false;
         }
@@ -777,7 +804,7 @@ bool CFastaReader::ParseIDs(
             << ", the sequence ID is too long.  Its length is " << s.length()
             << " but the max length allowed is " << m_MaxIDLength 
             << ".  Please find and correct all sequence IDs that are too long.",
-            eIDTooLong);
+            CObjReaderParseException::eIDTooLong);
     }
     return count > 0;
 }
@@ -906,7 +933,7 @@ void CFastaReader::CheckDataLine(
             "CFastaReader: Near line " << LineNumber()
             << ", there's a line that doesn't look like plausible data, "
             "but it's not marked as defline or comment.",
-            eFormat);
+            CObjReaderParseException::eFormat);
     }
     // warn if more than a certain percentage is ambiguous nucleotides
     const static size_t kWarnPercentAmbiguous = 40; // e.g. "40" means "40%"
@@ -924,6 +951,11 @@ void CFastaReader::CheckDataLine(
 void CFastaReader::ParseDataLine(
     const TStr& s, IMessageListener * pMessageListener)
 {
+    if( NStr::StartsWith(s, ">?") ) {
+        ParseGapLine(s, pMessageListener);
+        return;
+    }
+
     CheckDataLine(s, pMessageListener);
 
     size_t len = min(s.length(), s.find(';')); // ignore ;-delimited comments
@@ -965,11 +997,12 @@ void CFastaReader::ParseDataLine(
 
     bool bIgnorableHyphenSeen = false;
 
-    const char chLetterGap = ( bIsNuc ? 'N' : 'X');
+    const char chLetterGap_lc = ( bIsNuc ? 'N' : 'X');
+    const char chLetterGap_uc = ( bIsNuc ? 'n' : 'x');
     for (size_t pos = 0;  pos < len;  ++pos) {
         unsigned char c = s[pos];
         if ( ( (c == '-')         && bHyphensAreGaps) ||
-             ( (c == chLetterGap) && bAllowLetterGaps) )
+             ( (c == chLetterGap_lc || c == chLetterGap_uc) && bAllowLetterGaps) )
         {
             CloseMask();
             // open a gap
@@ -978,7 +1011,7 @@ void CFastaReader::ParseDataLine(
                 ++pos2;
             }
             m_CurrentGapLength += pos2 - pos;
-            m_CurrentGapChar = c;
+            m_CurrentGapChar = toupper(c);
             pos = pos2 - 1;
         } else if( c == '-' && bHyphensIgnoreAndWarn ) {
             bIgnorableHyphenSeen = true;
@@ -987,7 +1020,7 @@ void CFastaReader::ParseDataLine(
             ( ! bIsNuc &&  ( s_ASCII_IsAlpha(c) ||  c == '*' ) ) ||
             c == '-' ) 
         {
-            CloseGap();
+            CloseGap(pos == 0);
             if (s_ASCII_IsLower(c)) {
                 m_SeqData[m_CurrentPos] = s_ASCII_ToUpper(c);
                 OpenMask();
@@ -1001,7 +1034,7 @@ void CFastaReader::ParseDataLine(
                 bad_pos_line_num = LineNumber();
             }
             bad_pos_vec.push_back(pos);
-        } 
+        }
     }
 
     m_SeqData.resize(m_CurrentPos);
@@ -1038,9 +1071,19 @@ void CFastaReader::ParseDataLine(
 }
 
 void CFastaReader::x_CloseGap(
-    TSeqPos len, IMessageListener * pMessageListener)
+    TSeqPos len, bool atStartOfLine, IMessageListener * pMessageListener)
 {
     _ASSERT(len > 0  &&  TestFlag(fParseGaps));
+
+    if (m_CurrentGapLength < m_gapNmin)
+    {
+        // the run of N is too short to be assumed as a gap, put N as is.
+        m_SeqData.resize(m_SeqData.size() + m_CurrentGapLength, 'X');
+        memset(&m_SeqData.at(m_CurrentPos), m_CurrentGapChar, m_CurrentGapLength);
+        m_CurrentPos += m_CurrentGapLength;
+        return;
+    }
+
     if (TestFlag(fAligning)) {
         TSeqPos pos = GetCurrentPos(ePosWithGapsAndSegs);
         m_Starts[pos + m_Offset][m_Row] = CFastaAlignmentBuilder::kNoPos;
@@ -1049,20 +1092,32 @@ void CFastaReader::x_CloseGap(
     } else {
         TSeqPos pos = GetCurrentPos(eRawPos);
         SGap::EKnownSize eKnownSize = SGap::eKnownSize_Yes;
-        // Special case -- treat a lone hyphen at the end of a line as
-        // a gap of unknown length.
-        // (do NOT treat a lone 'N' or 'X' as unknown length)
-        if (len == 1 && m_CurrentGapChar == '-' ) {
-            TSeqPos l = m_SeqData.length();
-            if (l == pos  ||  l == pos + (*GetLineReader()).length()) {
-                len = 0;
-                eKnownSize = SGap::eKnownSize_No;
+        if (len == m_gap_Unknown_length)
+        {
+            eKnownSize = SGap::eKnownSize_No;
+            //len = 0;
+        }
+        else
+        {
+            // Special case -- treat a lone hyphen at the end of a line as
+            // a gap of unknown length.
+            // (do NOT treat a lone 'N' or 'X' as unknown length)
+            if (len == 1 && m_CurrentGapChar == '-' ) {
+                TSeqPos l = m_SeqData.length();
+                if ((l == pos  ||  l == pos + (*GetLineReader()).length())  && atStartOfLine) {
+                    //and it's not the first col of the line
+                    len = 0;
+                    eKnownSize = SGap::eKnownSize_No;
+                }
             }
         }
+
         TGapRef pGap( new SGap(
             pos, len,
             eKnownSize,
-            LineNumber()) );
+            LineNumber(),
+            m_gap_type,
+            m_gap_linkage_evidence));
 
         m_Gaps.push_back(pGap);
         m_TotalGapLength += len;
@@ -1129,17 +1184,6 @@ bool CFastaReader::ParseGapLine(
             // try to continue the best we can
             uGapSize = 1;
         }
-        if( eIsKnown != SGap::eKnownSize_Yes && 
-            uGapSize != 100) 
-        {
-            FASTA_WARNING(LineNumber(), 
-                "CFastaReader: Unknown-size gaps must have a nominal "
-                "length of 100, at line  " << LineNumber(),
-                ILineError::eProblem_InvalidLengthAutoCorrected,
-                "gapline" );
-            // force to correct value
-            uGapSize = 100;
-        }
         sRemainingLine = sRemainingLine.substr(sDigits.length());
         NStr::TruncateSpacesInPlace(sRemainingLine, NStr::eTrunc_Begin);
     }
@@ -1358,6 +1402,7 @@ void CFastaReader::AssembleSeq(IMessageListener * pMessageListener)
               CSeq_data::e_Iupacaa : 
               CSeq_data::e_Ncbieaa ) :
             CSeq_data::e_Iupacna);
+
     if (TestFlag(fValidate)) {
         CSeq_data tmp_data(m_SeqData, format);
         vector<TSeqPos> badIndexes;
@@ -1410,56 +1455,41 @@ void CFastaReader::AssembleSeq(IMessageListener * pMessageListener)
         m_CurrentGapChar = '\0';
     }
 
-    if (m_Gaps.empty()) {
+    if (m_Gaps.empty() && m_SeqData.empty()) {
+
         _ASSERT(m_TotalGapLength == 0);
-        if (m_SeqData.empty()) {
             inst.SetLength(0);
             inst.SetRepr(CSeq_inst::eRepr_virtual);
             // empty sequence triggers warning if seq data was expected
-            if( ! TestFlag(fNoSeqData) ) {
-                FASTA_WARNING(LineNumber(),
+            if( ! TestFlag(fDisableNoResidues) &&
+                ! TestFlag(fNoSeqData) ) {
+                FASTA_ERROR(LineNumber(),
                     "FASTA-Reader: No residues given",
-                    ILineError::eProblem_Missing,
-                    "sequence");
-            }
-        } else if (TestFlag(fNoSplit)) {
-            inst.SetLength(GetCurrentPos(eRawPos));
-            inst.SetRepr(CSeq_inst::eRepr_raw);
-            CRef<CSeq_data> data(new CSeq_data(m_SeqData, format));
-            if ( !TestFlag(fLeaveAsText) ) {
-                CSeqportUtil::Pack(data, inst.GetLength());
-            }
-            inst.SetSeq_data(*data);
-        } else {
-            inst.SetLength(GetCurrentPos(eRawPos));
-            CDelta_ext& delta_ext = inst.SetExt().SetDelta();
-            delta_ext.AddAndSplit(m_SeqData, format, inst.GetLength(),
-                                  TestFlag(fLetterGaps));
-            if (delta_ext.Get().size() > 1) {
-                inst.SetRepr(CSeq_inst::eRepr_delta);
-            } else { // simplify -- just one piece
-                inst.SetRepr(CSeq_inst::eRepr_raw);
-                inst.SetSeq_data(delta_ext.Set().front()
-                                 ->SetLiteral().SetSeq_data());
-                inst.ResetExt();
+                    CObjReaderParseException::eNoResidues);
             }
+    }
+    else 
+    if (m_Gaps.empty() && TestFlag(fNoSplit)) {
+        inst.SetLength(GetCurrentPos(eRawPos));
+        inst.SetRepr(CSeq_inst::eRepr_raw);
+        CRef<CSeq_data> data(new CSeq_data(m_SeqData, format));
+        if ( !TestFlag(fLeaveAsText) ) {
+            CSeqportUtil::Pack(data, inst.GetLength());
         }
+        inst.SetSeq_data(*data);
     } else {
         CDelta_ext& delta_ext = inst.SetExt().SetDelta();
         inst.SetRepr(CSeq_inst::eRepr_delta);
         inst.SetLength(GetCurrentPos(ePosWithGaps));
         SIZE_TYPE n = m_Gaps.size();
-        for (SIZE_TYPE i = 0;  i < n;  ++i) {
 
-            if (i == 0  &&  m_Gaps[i]->m_uPos > 0) {
-                TStr chunk(m_SeqData, 0, m_Gaps[i]->m_uPos);
-                if (TestFlag(fNoSplit)) {
-                    delta_ext.AddLiteral(chunk, inst.GetMol());
-                } else {
-                    delta_ext.AddAndSplit(chunk, format, m_Gaps[i]->m_uPos,
-                                          TestFlag(fLetterGaps));
-                }
-            }
+        if (n==0 || m_Gaps[0]->m_uPos > 0)
+        {
+            TStr chunk(m_SeqData, 0, (n>0 && m_Gaps[0]->m_uPos > 0)?m_Gaps[0]->m_uPos : inst.GetLength());
+            delta_ext.AddAndSplit(chunk, format, chunk.length(), false, !TestFlag(fLeaveAsText));
+        }
+
+        for (SIZE_TYPE i = 0;  i < n;  ++i) {
 
             // add delta-seq
             CRef<CDelta_seq> gap_ds(new CDelta_seq);
@@ -1502,14 +1532,17 @@ void CFastaReader::AssembleSeq(IMessageListener * pMessageListener)
             if (next_start != m_Gaps[i]->m_uPos) {
                 TSeqPos seq_len = next_start - m_Gaps[i]->m_uPos;
                 TStr chunk(m_SeqData, m_Gaps[i]->m_uPos, seq_len);
-                if (TestFlag(fNoSplit)) {
-                    delta_ext.AddLiteral(chunk, inst.GetMol());
-                } else {
-                    delta_ext.AddAndSplit(chunk, format, seq_len,
-                                          TestFlag(fLetterGaps));
-                }
+                delta_ext.AddAndSplit(chunk, format, chunk.length(), false, !TestFlag(fLeaveAsText));
             }
         }
+        if (delta_ext.Get().size() == 1) {
+            // simplify -- just one piece
+            inst.SetRepr(CSeq_inst::eRepr_raw);
+            inst.SetSeq_data(delta_ext.Set().front()
+                                ->SetLiteral().SetSeq_data());
+            inst.ResetExt();
+        }
+
     }
 }
 
@@ -1689,7 +1722,7 @@ CRef<CSeq_entry> CFastaReader::x_ReadSeqsToAlign(TIds& ids,
                 FASTA_ERROR(LineNumber(),
                             "CFastaReader::ReadAlignedSet: Rows have different "
                             "lengths. For example, look around line " << LineNumber(), 
-                            eFormat );
+                            CObjReaderParseException::eFormat );
             }
         }
     }
@@ -1954,6 +1987,7 @@ void CFastaReader::x_ApplyAllMods(
     CSourceModParser smp( TestFlag(fBadModThrow) ?
         CSourceModParser::eHandleBadMod_Throw : 
     CSourceModParser::eHandleBadMod_Ignore );
+    smp.SetModFilter( m_pModFilter );
     CRef<CSeqdesc> title_desc;
 
 
@@ -2129,6 +2163,49 @@ CFastaReader::SGap::SGap(
 {
 }
 
+
+void CFastaReader::SetGapLinkageEvidence(CSeq_gap::EType type, CLinkage_evidence::EType evidence)
+{
+   if (type == -1)
+      m_gap_type.Release();
+   else
+      m_gap_type.Reset(new SGap::TGapTypeObj(type));
+
+   m_gap_linkage_evidence.clear();
+   if (evidence != -1)
+      m_gap_linkage_evidence.insert(evidence);
+}
+
+void CFastaReader::PostWarning(
+            IMessageListener * pMessageListener,
+            EDiagSev _eSeverity, size_t _uLineNum, CTempString _MessageStrmOps, CObjReaderParseException::EErrCode _eErrCode, ILineError::EProblem _eProblem, CTempString _sFeature, CTempString _sQualName, CTempString _sQualValue)
+{
+    if (find(m_ignorable.begin(), m_ignorable.end(), _eProblem) != m_ignorable.end())
+        return;
+
+
+    string sSeqId_49518053 = ( m_BestID ? m_BestID->AsFastaString() : kEmptyStr);                  
+    AutoPtr<CObjReaderLineException> pLineExpt(                        
+        CObjReaderLineException::Create(                            
+        (_eSeverity), _uLineNum,                        
+        _MessageStrmOps,                                
+        (_eProblem),                                            
+        sSeqId_49518053, (_sFeature),                           
+        (_sQualName), (_sQualValue),                            
+        _eErrCode) );                 
+    if ( ! pMessageListener && (_eSeverity) <= eDiag_Warning ) {    
+        LOG_POST_X(1, Warning << pLineExpt->Message()); 
+    } else if ( ! pMessageListener || ! pMessageListener->PutError( *pLineExpt ) ) 
+    {                                       
+        throw CObjReaderParseException(DIAG_COMPILE_INFO, 0, _eErrCode, _MessageStrmOps, _uLineNum, _eSeverity);
+    }                                                               
+}
+
+void CFastaReader::IgnoreProblem(ILineError::EProblem problem)
+{
+    m_ignorable.push_back(problem);
+}
+
 END_SCOPE(objects)
 END_NCBI_SCOPE
 
