@@ -1,4 +1,4 @@
-/*  $Id: gff3_reader.cpp 419210 2013-11-22 15:41:39Z rafanovi $
+/*  $Id: gff3_reader.cpp 448853 2014-10-09 16:29:29Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -178,9 +178,6 @@ bool CGff3Reader::x_UpdateAnnotFeature(
 //  ----------------------------------------------------------------------------
 {
     CRef< CSeq_feat > pFeature(new CSeq_feat);
-    if (! xFeatureAddRoundTripInfo(record, pFeature)) {
-        return false;
-    }
 
     string type = record.Type();
     if (type == "exon"  ||  type == "five_prime_UTR"  ||  type == "three_prime_UTR") {
@@ -189,50 +186,10 @@ bool CGff3Reader::x_UpdateAnnotFeature(
     if (type == "CDS"  ||  type == "cds") {
         return xUpdateAnnotCds(record, pFeature, pAnnot, pEC);
     }
-
-    //  General case: brand new regular feature
-    if (!record.InitializeFeature(m_iFlags, pFeature)) {
-        return false;
+    if (type == "gene") {
+        return xUpdateAnnotGene(record, pFeature, pAnnot, pEC);
     }
-    
-    if (!x_FeatureSetXref(record, pFeature)) {
-        return false;
-    }
-
-    if (! x_AddFeatureToAnnot( pFeature, pAnnot )) {
-        return false;
-    }
-    string strId;
-    if ( record.GetAttribute( "ID", strId ) ) {
-        m_MapIdToFeature[ strId ] = pFeature;
-    }
-    return true;
-}
-
-//  ----------------------------------------------------------------------------
-bool CGff3Reader::xFeatureAddRoundTripInfo(
-    const CGff2Record& record,
-    CRef<CSeq_feat> pFeature)
-//  ----------------------------------------------------------------------------
-{
-    CRef<CGb_qual> pQual(new CGb_qual);
-    pQual->SetQual("gff_source");
-    pQual->SetVal(record.Source());
-    pFeature->SetQual().push_back(pQual);
-
-    pQual.Reset(new CGb_qual);
-    pQual->SetQual("gff_type");
-    pQual->SetVal(record.Type());
-    pFeature->SetQual().push_back(pQual);
-
-    if (record.IsSetScore()) {
-        pQual.Reset(new CGb_qual);
-        pQual->SetQual("gff_score");
-        pQual->SetVal(NStr::DoubleToString(record.Score()));
-        pFeature->SetQual().push_back(pQual);
-    }
-
-    return true;
+    return xUpdateAnnotGeneric(record, pFeature, pAnnot, pEC);
 }
 
 //  ----------------------------------------------------------------------------
@@ -267,13 +224,14 @@ bool CGff3Reader::xUpdateAnnotCds(
 //  ----------------------------------------------------------------------------
 {
     if (!xVerifyCdsParents(record)) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Fatal,
             0,
             "Bad data line: CDS record with bad parent assignments.",
-            ILineError::eProblem_FeatureBadStartAndOrStop);
-        err.SetLineNumber(m_uLineNumber);
-        ProcessError(err, pEC);
+            ILineError::eProblem_FeatureBadStartAndOrStop) );
+        pErr->SetLineNumber(m_uLineNumber);
+        ProcessError(*pErr, pEC);
     }
     list<string> parents;
     if (record.GetAttribute("Parent", parents)) {
@@ -281,8 +239,6 @@ bool CGff3Reader::xUpdateAnnotCds(
                 ++it) {
             IdToFeatureMap::iterator fit = m_MapIdToFeature.find(*it);
             if (fit != m_MapIdToFeature.end()) {
-                CRef<CSeq_feat> pF = fit->second;
-                const CSeq_loc& loc = pF->GetLocation();
                 if (!record.UpdateFeature(m_iFlags, fit->second)) {
                     return false;
                 }
@@ -331,16 +287,30 @@ bool CGff3Reader::xUpdateAnnotGeneric(
     if (!record.InitializeFeature(m_iFlags, pFeature)) {
         return false;
     }
-    if (! x_AddFeatureToAnnot( pFeature, pAnnot )) {
+    if (!x_FeatureSetXref(record, pFeature)) {
+        return false;
+    }
+    if (! x_AddFeatureToAnnot(pFeature, pAnnot)) {
         return false;
     }
     string strId;
-    if ( record.GetAttribute( "ID", strId ) ) {
-        m_MapIdToFeature[ strId ] = pFeature;
+    if ( record.GetAttribute("ID", strId)) {
+        m_MapIdToFeature[strId] = pFeature;
     }
     return true;
 }
 
+//  ----------------------------------------------------------------------------
+bool CGff3Reader::xUpdateAnnotGene(
+    const CGff2Record& record,
+    CRef<CSeq_feat> pFeature,
+    CRef<CSeq_annot> pAnnot,
+    IMessageListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    return xUpdateAnnotGeneric(record, pFeature, pAnnot, pEC);
+}
+
 //  ----------------------------------------------------------------------------
 bool CGff3Reader::x_AddFeatureToAnnot(
     CRef< CSeq_feat > pFeature,
