@@ -1,4 +1,4 @@
-/*  $Id: gtf_reader.cpp 413130 2013-09-12 16:02:37Z ivanov $
+/*  $Id: gtf_reader.cpp 448877 2014-10-09 16:37:03Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -207,22 +207,33 @@ CGtfReader::ReadSeqAnnots(
 //  ----------------------------------------------------------------------------
 {
     string line;
-    int linecount = 0;
 
-    while ( x_GetLine( lr, line, linecount ) ) {
+    while (xGetLine(lr, line)) {
+        if (IsCanceled()) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Info,
+                0,
+                "Reader stopped by user.",
+                ILineError::eProblem_ProgressInfo));
+            ProcessError(*pErr, pEC);
+            annots.clear();
+            return;
+        }
+        xReportProgress(pEC);
         try {
-            if ( x_ParseBrowserLineGff( line, m_CurrentBrowserInfo ) ) {
+            if (x_ParseBrowserLineGff(line, m_CurrentBrowserInfo)) {
                 continue;
             }
-            if ( x_ParseTrackLineGff( line, m_CurrentTrackInfo ) ) {
+            if (x_ParseTrackLineGff(line, m_CurrentTrackInfo)) {
                 continue;
             }
-            if ( ! x_ParseFeatureGff( line, annots, pEC) ) {
+            if (x_ParseFeatureGff(line, annots, pEC)) {
                 continue;
             }
         }
-        catch( CObjReaderLineException& err ) {
-            err.SetLineNumber( linecount );
+        catch(CObjReaderLineException& err) {
+            err.SetLineNumber(m_uLineNumber);
         }
     }
 }
@@ -236,39 +247,10 @@ CGtfReader::ReadSeqAnnots(
 //  ---------------------------------------------------------------------------
 {
     CStreamLineReader lr( istr );
+    xProgressInit(istr);
     ReadSeqAnnots( annots, lr, pMessageListener );
 }
 
-//  ---------------------------------------------------------------------------                       
-bool
-CGtfReader::x_GetLine(
-    ncbi::ILineReader& lr,
-    string& strLine,
-    int& iLineCount )
-//  ---------------------------------------------------------------------------
-{
-    while ( ! lr.AtEOF() ) {
-
-        string strBuffer = NStr::TruncateSpaces_Unsafe( *++lr );
-        ++iLineCount;
-
-        if ( strBuffer.empty() ) {
-            continue;
-        }
-        size_t uComment = strBuffer.find( '#' );
-        if ( uComment != NPOS ) {
-            strBuffer = strBuffer.substr( 0, uComment );
-            if ( strBuffer.empty() ) {
-                continue;
-            }
-        }  
-
-        strLine = strBuffer;
-        return true;    
-    }
-    return false;
-}
- 
 //  ----------------------------------------------------------------------------
 bool CGtfReader::x_UpdateAnnotFeature(
     const CGff2Record& gff,
@@ -790,7 +772,8 @@ bool CGtfReader::x_CreateParentGene(
     }
     m_GeneMap[ s_GeneKey( gff ) ] = pFeature;
 
-    return x_AddFeatureToAnnot( pFeature, pAnnot );
+    x_AddFeatureToAnnot( pFeature, pAnnot );
+    return true;
 }
     
 //  ----------------------------------------------------------------------------
@@ -1084,6 +1067,8 @@ bool CGtfReader::x_ProcessQualifierSpecialCase(
     CRef< CSeq_feat > pFeature )
 //  ----------------------------------------------------------------------------
 {
+    CRef<CGb_qual> pQual(0);
+
     if (0 == NStr::CompareNocase(it->first, "exon_id")) {
         return true;
     }
