@@ -1,4 +1,4 @@
-/*  $Id: reader_base.cpp 407662 2013-07-23 16:48:24Z ludwigf $
+/*  $Id: reader_base.cpp 448864 2014-10-09 16:32:57Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -95,8 +95,10 @@
 #include <objtools/readers/fasta.hpp>
 #include <objtools/readers/readfeat.hpp>
 #include <objtools/error_codes.hpp>
+#include <objtools/readers/ucscregion_reader.hpp>
 
 #include <algorithm>
+#include <ctime>
 
 #include "reader_data.hpp"
 
@@ -136,6 +138,8 @@ CReaderBase::GetReader(
         return new CFastaReader(flags);
     case CFormatGuess::eFiveColFeatureTable:
         return new CFeature_table_reader(flags);
+    case CFormatGuess::eUCSCRegion:
+        return new CUCSCRegionReader(flags);
     }
 }
 
@@ -144,6 +148,10 @@ CReaderBase::CReaderBase(
     TReaderFlags flags) :
 //  ----------------------------------------------------------------------------
     m_uLineNumber(0),
+    m_uProgressReportInterval(0),
+    m_uMaxFilePos(0),
+    m_pInStream(0),
+    m_pCanceler(0),
     m_iFlags(flags)
 {
     m_pTrackDefaults = new CTrackData;
@@ -163,6 +171,7 @@ CReaderBase::ReadObject(
     IMessageListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------
 {
+    xProgressInit(istr);
     CStreamLineReader lr( istr );
     return ReadObject( lr, pMessageListener );
 }
@@ -174,6 +183,7 @@ CReaderBase::ReadSeqAnnot(
     IMessageListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------
 {
+    xProgressInit(istr);
     CStreamLineReader lr( istr );
     return ReadSeqAnnot( lr, pMessageListener );
 }
@@ -188,6 +198,34 @@ CReaderBase::ReadSeqAnnot(
     return CRef<CSeq_annot>();
 }
                 
+//  --------------------------------------------------------------------------- 
+void
+CReaderBase::ReadSeqAnnots(
+    vector< CRef<CSeq_annot> >& annots,
+    CNcbiIstream& istr,
+    IMessageListener* pMessageListener )
+//  ---------------------------------------------------------------------------
+{
+    xProgressInit(istr);
+    CStreamLineReader lr( istr );
+    ReadSeqAnnots( annots, lr, pMessageListener );
+}
+ 
+//  ---------------------------------------------------------------------------                       
+void
+CReaderBase::ReadSeqAnnots(
+    vector< CRef<CSeq_annot> >& annots,
+    ILineReader& lr,
+    IMessageListener* pMessageListener )
+//  ----------------------------------------------------------------------------
+{
+    CRef<CSeq_annot> annot = ReadSeqAnnot(lr, pMessageListener);
+    while (annot) {
+        annots.push_back(annot);
+        annot = ReadSeqAnnot(lr, pMessageListener);
+    }
+}
+                        
 //  ----------------------------------------------------------------------------
 CRef< CSeq_entry >
 CReaderBase::ReadSeqEntry(
@@ -195,8 +233,11 @@ CReaderBase::ReadSeqEntry(
     IMessageListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------
 {
+    xProgressInit(istr);
     CStreamLineReader lr( istr );
-    return ReadSeqEntry( lr, pMessageListener );
+    CRef<CSeq_entry> pResult = ReadSeqEntry( lr, pMessageListener );
+    m_pInStream = 0;
+    return pResult;
 }
 
 //  ----------------------------------------------------------------------------
@@ -217,8 +258,18 @@ CReaderBase::ProcessError(
 //  ----------------------------------------------------------------------------
 {
     err.SetLineNumber( m_uLineNumber );
-    if (!pContainer  ||  !pContainer->PutError(err)) {
-        throw( err );
+    if (!pContainer) {
+        err.Throw();
+    }
+    if (!pContainer->PutError(err)) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Fatal,
+            0,
+            "Error allowance exceeded",
+            ILineError::eProblem_GeneralParsingError) );
+        pErr->Throw();
+        //err.Throw();
     }
 }
 
@@ -236,7 +287,7 @@ CReaderBase::ProcessWarning(
         return;
     }
     if (!pContainer->PutError(err)) {
-        throw(err);
+        err.Throw();
     }
 }
 
@@ -248,7 +299,7 @@ CReaderBase::ProcessError(
 //  ----------------------------------------------------------------------------
 {
     if (!pContainer  ||  !pContainer->PutError(err)) {
-        throw( err );
+        err.Throw();
     }
  }
 
@@ -265,52 +316,83 @@ CReaderBase::ProcessWarning(
         return;
     }
     if (!pContainer->PutError(err)) {
-        throw(err);
+        err.Throw();
     }
  }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_SetBrowserRegion(
+void CReaderBase::xSetBrowserRegion(
     const string& strRaw,
     CAnnot_descr& desc,
     IMessageListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     CRef<CSeq_loc> location( new CSeq_loc );
-    CSeq_interval& interval = location->SetInt();
 
     string strChrom;
     string strInterval;
     if ( ! NStr::SplitInTwo( strRaw, ":", strChrom, strInterval ) ) {
-        CObjReaderLineException err(
-            eDiag_Error,
-            0,
-            "Bad browser line: cannot parse browser position" );
-        ProcessError(err, pEC);
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Bad browser line: cannot parse browser position" ) );
+        ProcessError(*pErr, pEC);
     }
     CRef<CSeq_id> id( new CSeq_id( CSeq_id::e_Local, strChrom ) );
-    location->SetId( *id );
 
-    string strFrom;
-    string strTo;
-    if ( ! NStr::SplitInTwo( strInterval, "-", strFrom, strTo ) ) {
-        CObjReaderLineException err(
-            eDiag_Error,
-            0,
-            "Bad browser line: cannot parse browser position" );
-        ProcessError(err, pEC);
-    }    
-    interval.SetFrom( NStr::StringToInt( strFrom ) - 1);
-    interval.SetTo( NStr::StringToInt( strTo ) - 1 );
-    interval.SetStrand( eNa_strand_unknown );
+    if (NStr::Compare(strInterval, "start-stop") == 0 )
+    {
+        location->SetWhole(*id);
+    }
+    else
+    {
+        string strFrom;
+        string strTo;
+        if ( ! NStr::SplitInTwo( strInterval, "-", strFrom, strTo ) ) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Bad browser line: cannot parse browser position" ) );
+            ProcessError(*pErr, pEC);
+        }  
+        try
+        {
+            int n_from,n_to;
 
-    CRef<CAnnotdesc> region( new CAnnotdesc() );
-    region->SetRegion( *location );
-    desc.Set().push_back( region );
+            n_from = NStr::StringToInt(strFrom, NStr::fAllowCommas);
+            n_to   = NStr::StringToInt(strTo, NStr::fAllowCommas);
+
+            CSeq_interval& interval = location->SetInt();
+            interval.SetFrom(n_from-1);
+            interval.SetTo(n_to-1);
+            interval.SetStrand( eNa_strand_unknown );
+            location->SetId( *id );
+
+        }
+        catch (const CStringException&)
+        {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                "Bad browser line: cannot parse browser position" ) );
+            ProcessError(*pErr, pEC);
+            location.Release();
+        }
+    }
+
+    if (location.NotEmpty())
+    {
+        CRef<CAnnotdesc> region( new CAnnotdesc() );
+        region->SetRegion( *location );
+        desc.Set().push_back( region );
+    }
 }
     
 //  ----------------------------------------------------------------------------
-bool CReaderBase::x_ParseBrowserLine(
+bool CReaderBase::xParseBrowserLine(
     const string& strLine,
     CRef<CSeq_annot>& annot,
     IMessageListener* pEC)
@@ -327,13 +409,14 @@ bool CReaderBase::x_ParseBrowserLine(
         if ( *it == "position" ) {
             ++it;
             if ( it == fields.end() ) {
-                CObjReaderLineException err(
+                AutoPtr<CObjReaderLineException> pErr(
+                    CObjReaderLineException::Create(
                     eDiag_Error,
                     0,
-                    "Bad browser line: incomplete position directive" );
-                ProcessError(err, pEC);
+                    "Bad browser line: incomplete position directive" ) );
+                ProcessError(*pErr, pEC);
             }
-            x_SetBrowserRegion(*it, desc, pEC);
+            xSetBrowserRegion(*it, desc, pEC);
         }
     }
 
@@ -341,10 +424,13 @@ bool CReaderBase::x_ParseBrowserLine(
 }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_AssignTrackData(
+void CReaderBase::xAssignTrackData(
     CRef<CSeq_annot>& annot )
 //  ----------------------------------------------------------------------------
 {
+    if (!m_pTrackDefaults->ContainsData()) {
+        return;
+    }
     CAnnot_descr& desc = annot->SetDesc();
     CRef<CUser_object> trackdata( new CUser_object() );
     trackdata->SetType().SetStr( "Track Data" );
@@ -368,9 +454,8 @@ void CReaderBase::x_AssignTrackData(
 }
 
 //  ----------------------------------------------------------------------------
-bool CReaderBase::x_ParseTrackLine(
+bool CReaderBase::xParseTrackLine(
     const string& strLine,
-    CRef<CSeq_annot>& annot,
     IMessageListener* pEC)
 //  ----------------------------------------------------------------------------
 {
@@ -384,7 +469,16 @@ bool CReaderBase::x_ParseTrackLine(
 }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_SetTrackData(
+bool CReaderBase::xParseBrowserLine(
+    const string& strLine,
+    IMessageListener* pEC)
+//  ----------------------------------------------------------------------------
+{
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+void CReaderBase::xSetTrackData(
     CRef<CSeq_annot>& annot,
     CRef<CUser_object>& trackdata,
     const string& strKey,
@@ -395,51 +489,205 @@ void CReaderBase::x_SetTrackData(
 }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_AddConversionInfo(
+bool CReaderBase::xParseComment(
+    const CTempString& record,
+    CRef<CSeq_annot>& annot ) /* throws CObjReaderLineException */
+//  ----------------------------------------------------------------------------
+{
+    if (NStr::StartsWith(record, "#")) {
+        return true;
+    }
+    return false;
+}
+ 
+//  ----------------------------------------------------------------------------
+void CReaderBase::xAddConversionInfo(
     CRef<CSeq_annot >& annot,
-    IMessageListener *pMessageListener )
+    IMessageListener *pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    if ( !annot || !pMessageListener ) {
+    if (!annot || !pMessageListener) {
         return;
     }
-    CRef<CAnnotdesc> user( new CAnnotdesc() );
-    user->SetUser( *x_MakeAsnConversionInfo( pMessageListener ) );
-    annot->SetDesc().Set().push_back( user );
+    if (0 == pMessageListener->LevelCount(eDiag_Critical)  &&
+        0 == pMessageListener->LevelCount(eDiag_Error) &&
+        0 == pMessageListener->LevelCount(eDiag_Warning) &&
+        0 == pMessageListener->LevelCount(eDiag_Info)) {
+        return;
+    }
+
+
+    CRef<CAnnotdesc> user(new CAnnotdesc());
+    user->SetUser(*xMakeAsnConversionInfo(pMessageListener));
+    annot->SetDesc().Set().push_back(user);
 }
 
 //  ----------------------------------------------------------------------------
-void CReaderBase::x_AddConversionInfo(
+void CReaderBase::xAddConversionInfo(
     CRef<CSeq_entry >& entry,
-    IMessageListener *pMessageListener )
+    IMessageListener *pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    if ( !entry || !pMessageListener ) {
+    if (!entry || !pMessageListener) {
         return;
     }
-    CRef<CSeqdesc> user( new CSeqdesc() );
-    user->SetUser( *x_MakeAsnConversionInfo( pMessageListener ) );
-    entry->SetDescr().Set().push_back( 
-        user );
+    CRef<CSeqdesc> user(new CSeqdesc());
+    user->SetUser(*xMakeAsnConversionInfo(pMessageListener));
+    entry->SetDescr().Set().push_back(user);
 }
 
 //  ----------------------------------------------------------------------------
-CRef<CUser_object> CReaderBase::x_MakeAsnConversionInfo(
+CRef<CUser_object> CReaderBase::xMakeAsnConversionInfo(
     IMessageListener* pMessageListener )
 //  ----------------------------------------------------------------------------
 {
-    CRef<CUser_object> conversioninfo( new CUser_object() );
-    conversioninfo->SetType().SetStr( "Conversion Info" );    
+    CRef<CUser_object> conversioninfo(new CUser_object());
+    conversioninfo->SetType().SetStr("Conversion Info");    
     conversioninfo->AddField( 
-        "critical errors", int ( pMessageListener->LevelCount( eDiag_Critical ) ) );
+        "critical errors", int(pMessageListener->LevelCount(eDiag_Critical)));
     conversioninfo->AddField( 
-        "errors", int ( pMessageListener->LevelCount( eDiag_Error ) ) );
+        "errors", int(pMessageListener->LevelCount(eDiag_Error)));
     conversioninfo->AddField( 
-        "warnings", int ( pMessageListener->LevelCount( eDiag_Warning ) ) );
+        "warnings", int(pMessageListener->LevelCount(eDiag_Warning)));
     conversioninfo->AddField( 
-        "notes", int ( pMessageListener->LevelCount( eDiag_Info ) ) );
+        "notes", int(pMessageListener->LevelCount(eDiag_Info)));
     return conversioninfo;
 }
 
+//  ----------------------------------------------------------------------------
+void CReaderBase::SetProgressReportInterval(
+    unsigned int intv)
+//  ----------------------------------------------------------------------------
+{
+    m_uProgressReportInterval = intv;
+    m_uNextProgressReport = (unsigned int)time(0) + intv;
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsReportingProgress() const
+//  ----------------------------------------------------------------------------
+{
+    if (0 == m_uProgressReportInterval) {
+        return false;
+    }
+    if (0 == m_pInStream) {
+        return false;
+    }
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+void CReaderBase::xReportProgress(
+    IMessageListener* pProgress)
+//  ----------------------------------------------------------------------------
+{
+    if (!xIsReportingProgress()) { // progress reports disabled
+        return;
+    }
+    unsigned int uCurrentTime = (unsigned int)time(0);
+    if (uCurrentTime < m_uNextProgressReport) { // not time yet
+        return;
+    }
+    // report something
+    ios::streampos curPos = m_pInStream->tellg();
+    pProgress->PutProgress("Percent Progress", Uint8(100*curPos/m_uMaxFilePos), 100);
+
+    m_uNextProgressReport += m_uProgressReportInterval;
+}
+
+//  ============================================================================
+bool CReaderBase::xReadInit()
+//  ============================================================================
+{
+    return true;
+}
+
+//  ============================================================================
+bool CReaderBase::xProgressInit(
+    CNcbiIstream& istr)
+//  ============================================================================
+{
+    if (0 == m_uProgressReportInterval) {
+        return true;
+    }
+    m_pInStream = &istr;
+    ios::streampos curPos = istr.tellg();
+    istr.seekg(0, ios::end);
+    m_uMaxFilePos = istr.tellg();
+    istr.seekg(curPos, ios::beg);
+    return true;
+}
+
+//  ============================================================================
+void CReaderBase::SetCanceler(
+    ICanceled* pCanceler)
+//  ============================================================================
+{
+    m_pCanceler = pCanceler;
+}
+
+//  ============================================================================
+bool CReaderBase::xIsOperationCanceled() const
+//  ============================================================================
+{
+    if (!m_pCanceler) {
+        return false;
+    }
+    return m_pCanceler->IsCanceled();
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsCommentLine(
+    const CTempString& strLine)
+//  ----------------------------------------------------------------------------
+{
+    if (strLine.empty()) {
+        return true;
+    }
+    return (strLine[0] == '#' && strLine[1] != '#');
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsTrackLine(
+    const CTempString& strLine)
+//  ----------------------------------------------------------------------------
+{
+    return NStr::StartsWith(strLine, "track ");
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xIsBrowserLine(
+    const CTempString& strLine)
+//  ----------------------------------------------------------------------------
+{
+    return NStr::StartsWith(strLine, "browser ");
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xGetLine(
+    ILineReader& lr,
+    string& line)
+//  ----------------------------------------------------------------------------
+{
+    while (!lr.AtEOF()) {
+        line = *++lr;
+        ++m_uLineNumber;
+        NStr::TruncateSpacesInPlace(line);
+        if (!xIsCommentLine(line)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+//  ----------------------------------------------------------------------------
+bool CReaderBase::xUngetLine(
+    ILineReader& lr)
+//  ----------------------------------------------------------------------------
+{
+    lr.UngetLine();
+    --m_uLineNumber;
+    return true;
+}
 END_objects_SCOPE
 END_NCBI_SCOPE
