@@ -1,4 +1,4 @@
-/*  $Id: wiggle_reader.cpp 416067 2013-10-23 18:17:46Z rafanovi $
+/*  $Id: wiggle_reader.cpp 448873 2014-10-09 16:35:53Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -61,16 +61,67 @@
 #include <objects/seqres/Real_graph.hpp>
 #include <objects/seqres/Byte_graph.hpp>
 
+#include "reader_data.hpp"
+
 BEGIN_NCBI_SCOPE
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
+//  ----------------------------------------------------------------------------
+CRef<CSeq_loc> 
+CWiggleReader::xGetContainingLoc()
+//  ----------------------------------------------------------------------------
+{
+    if (m_Values.empty()) {
+        return CRef<CSeq_loc>();
+    }
+    CRef<CSeq_loc> pContainingLoc(new CSeq_loc);
+
+    int sortAndMergeFlags = CSeq_loc::fMerge_All;
+    if (this->xValuesAreFromSingleSequence()) {
+        sortAndMergeFlags = CSeq_loc::fMerge_SingleRange;
+    }
+    CWiggleReader::TValues::const_iterator cit = m_Values.begin();
+    CRef<CSeq_id> pId = CReadUtil::AsSeqId(cit->m_Chrom, m_iFlags);
+    CRef<CSeq_interval> pInterval(new CSeq_interval(*pId, cit->m_Pos,
+        cit->m_Pos + cit->m_Span));
+    pContainingLoc->SetInt(*pInterval);
+    for (cit++; cit != m_Values.end(); ++cit) {
+        CRef<CSeq_id> pId = CReadUtil::AsSeqId(cit->m_Chrom, m_iFlags);
+        CRef<CSeq_interval> pInterval(new CSeq_interval(*pId, cit->m_Pos,
+            cit->m_Pos + cit->m_Span));
+        CRef<CSeq_loc> pAdd(new CSeq_loc);
+        pAdd->SetInt(*pInterval);
+        pContainingLoc->Assign(
+            *pContainingLoc->Add(*pAdd, sortAndMergeFlags, 0));
+    }
+    return pContainingLoc;
+}
+
+//  ----------------------------------------------------------------------------
+bool
+CWiggleReader::xValuesAreFromSingleSequence() const
+//  ----------------------------------------------------------------------------
+{
+    if (m_Values.empty()) {
+        return false;
+    }
+    TValues::const_iterator cit = m_Values.begin();
+    string first = cit->m_Chrom;
+    for (cit++; cit != m_Values.end(); ++cit) {
+        if (cit->m_Chrom != first) {
+            return false;
+        }
+    }
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 CWiggleReader::CWiggleReader(
     TFlags flags ) :
 //  ----------------------------------------------------------------------------
     CReaderBase(flags),
-    m_TrackType(eTrackType_invalid),
-    m_OmitZeros(false)
+    m_OmitZeros(false),
+    m_TrackType(eTrackType_invalid)
 {
     m_uLineNumber = 0;
     m_GapValue = 0.0; 
@@ -100,73 +151,139 @@ CWiggleReader::ReadSeqAnnot(
     ILineReader& lr,
     IMessageListener* pMessageListener ) 
 //  ----------------------------------------------------------------------------                
+{
+    if (m_iFlags & CWiggleReader::fAsGraph) {
+        return xReadSeqAnnotGraph(lr, pMessageListener);
+    }
+    else {
+        return xReadSeqAnnotTable(lr, pMessageListener);
+    }
+}
+
+//  ----------------------------------------------------------------------------                
+CRef<CSeq_annot>
+CWiggleReader::xReadSeqAnnotGraph(
+    ILineReader& lr,
+    IMessageListener* pMessageListener ) 
+//  ----------------------------------------------------------------------------                
 {
     m_ChromId.clear();
     m_Values.clear();
     if (lr.AtEOF()) {
         return CRef<CSeq_annot>();
     }
-    while ( xGetLine(lr) ) {
-        CTempString s = xGetWord(pMessageListener);
-        if ( s == "browser" ) {
-            xReadBrowser();
+    bool haveData = false;
+    while (xGetLine(lr, m_CurLine)) {
+        if (IsCanceled()) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Info,
+                0,
+                "Reader stopped by user.",
+                ILineError::eProblem_ProgressInfo));
+            ProcessError(*pErr, pMessageListener);
+            return CRef<CSeq_annot>();
         }
-        else if ( s == "track" ) {
-            xReadTrack(pMessageListener);
+        xReportProgress(pMessageListener);
+        if (xIsTrackLine(m_CurLine)  &&  haveData) {
+            xUngetLine(lr);
+            break;
         }
-        else if ( s == "fixedStep" ) {
+        if (xProcessBrowserLine(pMessageListener)) {
+            continue;
+        }
+        if (xParseTrackLine(m_CurLine, pMessageListener)) {
+            continue;
+        }
+
+        string s = xGetWord(pMessageListener);
+        if ( s == "fixedStep" ) {
             SFixedStepInfo fixedStepInfo;
             xGetFixedStepInfo(fixedStepInfo, pMessageListener);
-            if (!m_ChromId.empty() && fixedStepInfo.mChrom != m_ChromId) {
-                cerr << fixedStepInfo.mChrom << endl;
-                lr.UngetLine();
-                return xGetAnnot();
-            }
             xReadFixedStepData(fixedStepInfo, lr, pMessageListener);
+            haveData = true;
         }
         else if ( s == "variableStep" ) {
             SVarStepInfo varStepInfo;
             xGetVarStepInfo(varStepInfo, pMessageListener);
-            if (!m_ChromId.empty() && varStepInfo.mChrom != m_ChromId) {
-                lr.UngetLine();
-                return xGetAnnot();
-            }
             xReadVariableStepData(varStepInfo, lr, pMessageListener);
+            haveData = true;
         }
         else {
             xReadBedLine(s, pMessageListener);
+            haveData = true;
+        }
+    }
+    xSetChrom("");
+    return m_Annot;
+}
+    
+//  ----------------------------------------------------------------------------                
+CRef<CSeq_annot>
+CWiggleReader::xReadSeqAnnotTable(
+    ILineReader& lr,
+    IMessageListener* pMessageListener ) 
+//  ----------------------------------------------------------------------------                
+{
+    m_ChromId.clear();
+    m_Values.clear();
+    if (lr.AtEOF()) {
+        return CRef<CSeq_annot>();
+    }
+    xResetChromValues();
+
+    bool haveData = false;
+    while (xGetLine(lr, m_CurLine)) {
+        if (IsCanceled()) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Info,
+                0,
+                "Reader stopped by user.",
+                ILineError::eProblem_ProgressInfo));
+            ProcessError(*pErr, pMessageListener);
+            return CRef<CSeq_annot>();
+        }
+        xReportProgress(pMessageListener);
+        if (xIsTrackLine(m_CurLine)  &&  haveData) {
+            xUngetLine(lr);
+            break;
+        }
+        if (xProcessBrowserLine(pMessageListener)) {
+            continue;
+        }
+        if (xParseTrackLine(m_CurLine, pMessageListener)) {
+            continue;
+        }
+
+        string s = xGetWord(pMessageListener);
+        if ( s == "fixedStep" ) {
+            SFixedStepInfo fixedStepInfo;
+            xGetFixedStepInfo(fixedStepInfo, pMessageListener);
+            //if (!m_ChromId.empty() && fixedStepInfo.mChrom != m_ChromId) {
+            //    lr.UngetLine();
+            //    return xGetAnnot();
+            //}
+            xReadFixedStepData(fixedStepInfo, lr, pMessageListener);
+            haveData = true;
+        }
+        else if ( s == "variableStep" ) {
+            SVarStepInfo varStepInfo;
+            xGetVarStepInfo(varStepInfo, pMessageListener);
+            //if (!m_ChromId.empty() && varStepInfo.mChrom != m_ChromId) {
+            //    lr.UngetLine();
+            //    return xGetAnnot();
+            //}
+            xReadVariableStepData(varStepInfo, lr, pMessageListener);
+            haveData = true;
+        }
+        else {
+            xReadBedLine(s, pMessageListener);
+            haveData = true;
         }
     }
     return xGetAnnot();
 }
-    
-//  --------------------------------------------------------------------------- 
-void
-CWiggleReader::ReadSeqAnnots(
-    vector< CRef<CSeq_annot> >& annots,
-    CNcbiIstream& istr,
-    IMessageListener* pMessageListener )
-//  ---------------------------------------------------------------------------
-{
-    CStreamLineReader lr( istr );
-    ReadSeqAnnots( annots, lr, pMessageListener );
-}
- 
-//  ---------------------------------------------------------------------------                       
-void
-CWiggleReader::ReadSeqAnnots(
-    vector< CRef<CSeq_annot> >& annots,
-    ILineReader& lr,
-    IMessageListener* pMessageListener )
-//  ----------------------------------------------------------------------------
-{
-    while ( ! lr.AtEOF() ) {
-        CRef<CSeq_annot> pAnnot = ReadSeqAnnot( lr, pMessageListener );
-        if ( pAnnot ) {
-            annots.push_back( pAnnot );
-        }
-    }
-}
 
 //  ----------------------------------------------------------------------------
 bool 
@@ -176,8 +293,8 @@ CWiggleReader::ReadTrackData(
     IMessageListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    while (xGetLine(lr)) {
-        CTempString word = xGetWord(pMessageListener);
+    while (xGetLine(lr, m_CurLine)) {
+        string word = xGetWord(pMessageListener);
         if (word == "browser") {
             continue;
         }
@@ -208,9 +325,11 @@ CWiggleReader::xReadFixedStepDataRaw(
 
     SFixedStepInfo fixedStepInfo;
     xGetFixedStepInfo(fixedStepInfo, pMessageListener);
-    CRef<CSeq_id> id = CReadUtil::AsSeqId(fixedStepInfo.mChrom);
+    CRef<CSeq_id> id =
+        CReadUtil::AsSeqId(fixedStepInfo.mChrom, m_iFlags);
+
     unsigned int pos(fixedStepInfo.mStart);
-    while (xGetLine(lr)) {
+    while (xGetLine(lr, m_CurLine)) {
         double value(0);
         if (!xTryGetDouble(value, pMessageListener)) {
             lr.UngetLine();
@@ -235,8 +354,9 @@ CWiggleReader::xReadVariableStepDataRaw(
 
     SVarStepInfo varStepInfo;
     xGetVarStepInfo(varStepInfo, pMessageListener);
-    CRef<CSeq_id> id = CReadUtil::AsSeqId(varStepInfo.mChrom);
-    while (xGetLine(lr)) {
+    CRef<CSeq_id> id =
+        CReadUtil::AsSeqId(varStepInfo.mChrom, m_iFlags);
+    while (xGetLine(lr, m_CurLine)) {
         unsigned int pos(0);
         if (!xTryGetPos(pos, pMessageListener)) {
             lr.UngetLine();
@@ -345,7 +465,8 @@ void CWiggleReader::xPreprocessValues(SWiggleStat& stat)
 CRef<CSeq_id> CWiggleReader::xMakeChromId()
 //  ----------------------------------------------------------------------------
 {
-    CRef<CSeq_id> id = CReadUtil::AsSeqId(m_ChromId);
+    CRef<CSeq_id> id =
+        CReadUtil::AsSeqId(m_ChromId, m_iFlags);
     return id;
 }
 
@@ -368,8 +489,9 @@ void CWiggleReader::xSetTotalLoc(CSeq_loc& loc, CSeq_id& chrom_id)
 CRef<CSeq_table> CWiggleReader::xMakeTable(void)
 //  ----------------------------------------------------------------------------
 {
-    CRef<CSeq_table> table(new CSeq_table);
+    size_t size = m_Values.size();
 
+    CRef<CSeq_table> table(new CSeq_table);
     table->SetFeat_type(0);
 
     CRef<CSeq_id> chrom_id = xMakeChromId();
@@ -379,14 +501,26 @@ CRef<CSeq_table> CWiggleReader::xMakeTable(void)
         CRef<CSeqTable_column> col_id(new CSeqTable_column);
         table->SetColumns().push_back(col_id);
         col_id->SetHeader().SetField_name("Seq-table location");
-        col_id->SetDefault().SetLoc(*table_loc);
+        col_id->SetDefault().SetLoc(*xGetContainingLoc());
     }
 
     { // Seq-id
         CRef<CSeqTable_column> col_id(new CSeqTable_column);
         table->SetColumns().push_back(col_id);
         col_id->SetHeader().SetField_id(CSeqTable_column_info::eField_id_location_id);
-        col_id->SetDefault().SetId(*chrom_id);
+        //col_id->SetDefault().SetId(*chrom_id);
+        if (this->xValuesAreFromSingleSequence()) {
+            CRef<CSeq_id> pId = CReadUtil::AsSeqId(m_Values.front().m_Chrom, m_iFlags);
+            col_id->SetDefault().SetId(*pId);
+        }
+        else {
+            CSeqTable_multi_data::TId& seq_id = col_id->SetData().SetId();
+            seq_id.reserve(size);
+            for (TValues::const_iterator cit = m_Values.begin(); cit != m_Values.end(); ++cit) {
+                CRef<CSeq_id> pId = CReadUtil::AsSeqId(cit->m_Chrom, m_iFlags);
+                seq_id.push_back(pId);
+            }
+        }
     }
     
     // position
@@ -400,7 +534,6 @@ CRef<CSeq_table> CWiggleReader::xMakeTable(void)
     
     xSetTotalLoc(*table_loc, *chrom_id);
 
-    size_t size = m_Values.size();
     table->SetNum_rows(size);
     pos.reserve(size);
 
@@ -499,9 +632,11 @@ CRef<CSeq_graph> CWiggleReader::xMakeGraph(void)
     
     xSetTotalLoc(*graph_loc, *chrom_id);
 
-    if ( !m_TrackName.empty() ) {
-        graph->SetTitle(m_TrackName);
+    string trackName = m_pTrackDefaults->Name();
+    if (!trackName.empty()) {
+        graph->SetTitle(trackName);
     }
+       
     graph->SetComp(stat.m_Span);
     graph->SetA(stat.m_Step);
     graph->SetB(stat.m_Min);
@@ -542,28 +677,20 @@ CRef<CSeq_annot> CWiggleReader::xMakeAnnot(void)
 //  ----------------------------------------------------------------------------
 {
     CRef<CSeq_annot> annot(new CSeq_annot);
-    if ( !m_TrackDescription.empty() ) {
+    string trackDescription = m_pTrackDefaults->Description();
+    if (!trackDescription.empty()) {
         CRef<CAnnotdesc> desc(new CAnnotdesc);
-        desc->SetTitle(m_TrackDescription);
+        desc->SetTitle(trackDescription);
         annot->SetDesc().Set().push_back(desc);
     }
-    if ( !m_TrackName.empty() ) {
+
+    string trackName = m_pTrackDefaults->Name();
+    if (!trackName.empty()) {
         CRef<CAnnotdesc> desc(new CAnnotdesc);
-        desc->SetName(m_TrackName);
+        desc->SetName(trackName);
         annot->SetDesc().Set().push_back(desc);
     }
-    if ( !m_TrackParams.empty() ) {
-        CRef<CAnnotdesc> desc(new CAnnotdesc);
-        annot->SetDesc().Set().push_back(desc);
-        CUser_object& user = desc->SetUser();
-        user.SetType().SetStr("Track Data");
-        ITERATE ( TTrackParams, it, m_TrackParams ) {
-            CRef<CUser_field> field(new CUser_field);
-            field->SetLabel().SetStr(it->first);
-            field->SetData().SetStr(it->second);
-            user.SetData().push_back(field);
-        }
-    }
+    xAssignTrackData(annot);
     return annot;
 }
 
@@ -597,7 +724,7 @@ void CWiggleReader::xResetChromValues(void)
 bool CWiggleReader::xSkipWS(void)
 //  ----------------------------------------------------------------------------
 {
-    const char* ptr = m_CurLine.data();
+    const char* ptr = m_CurLine.c_str();
     size_t skip = 0;
     for ( size_t len = m_CurLine.size(); skip < len; ++skip ) {
         char c = ptr[skip];
@@ -613,16 +740,16 @@ bool CWiggleReader::xSkipWS(void)
 inline bool CWiggleReader::xCommentLine(void) const
 //  ----------------------------------------------------------------------------
 {
-    char c = m_CurLine.data()[0];
+    char c = m_CurLine.c_str()[0];
     return c == '#' || c == '\0';
 }
 
 //  ----------------------------------------------------------------------------
-CTempString CWiggleReader::xGetWord(
+string CWiggleReader::xGetWord(
     IMessageListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    const char* ptr = m_CurLine.data();
+    const char* ptr = m_CurLine.c_str();
     size_t skip = 0;
     for ( size_t len = m_CurLine.size(); skip < len; ++skip ) {
         char c = ptr[skip];
@@ -631,62 +758,68 @@ CTempString CWiggleReader::xGetWord(
         }
     }
     if ( skip == 0 ) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Warning,
             0,
-            "Identifier expected");
-        ProcessError(err, pMessageListener);
+            "Identifier expected") );
+        ProcessError(*pErr, pMessageListener);
     }
+    string word(ptr, skip);
     m_CurLine = m_CurLine.substr(skip);
-    return CTempString(ptr, skip);
+    return word;
 }
 
 //  ----------------------------------------------------------------------------
-CTempString CWiggleReader::xGetParamName(
+string CWiggleReader::xGetParamName(
     IMessageListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    const char* ptr = m_CurLine.data();
+    const char* ptr = m_CurLine.c_str();
     size_t skip = 0;
     for ( size_t len = m_CurLine.size(); skip < len; ++skip ) {
         char c = ptr[skip];
         if ( c == '=' ) {
+            string name(ptr, skip);
             m_CurLine = m_CurLine.substr(skip+1);
-            return CTempString(ptr, skip);
+            return name;
         }
         if ( c == ' ' || c == '\t' ) {
             break;
         }
     }
-    CObjReaderLineException err(
+    AutoPtr<CObjReaderLineException> pErr(
+        CObjReaderLineException::Create(
         eDiag_Warning,
         0,
-        "\"=\" expected");
-    ProcessWarning(err, pMessageListener);
-    return CTempString();
+        "\"=\" expected") );
+    ProcessWarning(*pErr, pMessageListener);
+    return string();
 }
 
 //  ----------------------------------------------------------------------------
-CTempString CWiggleReader::xGetParamValue(
+string CWiggleReader::xGetParamValue(
     IMessageListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    const char* ptr = m_CurLine.data();
+    const char* ptr = m_CurLine.c_str();
     size_t len = m_CurLine.size();
     if ( len && *ptr == '"' ) {
         size_t pos = 1;
         for ( ; pos < len; ++pos ) {
             char c = ptr[pos];
             if ( c == '"' ) {
+                string value(ptr, pos);
                 m_CurLine = m_CurLine.substr(pos+1);
-                return CTempString(ptr+1, pos-1);
+                return value;
             }
         }
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Warning,
             0,
-            "Open quotes");
-        ProcessError(err, pMessageListener);
+            "Open quotes") );
+        ProcessError(*pErr, pMessageListener);
     }
     return xGetWord(pMessageListener);
 }
@@ -698,7 +831,7 @@ void CWiggleReader::xGetPos(
 //  ----------------------------------------------------------------------------
 {
     TSeqPos ret = 0;
-    const char* ptr = m_CurLine.data();
+    const char* ptr = m_CurLine.c_str();
     for ( size_t skip = 0; ; ++skip ) {
         char c = ptr[skip];
         if ( c >= '0' && c <= '9' ) {
@@ -710,11 +843,12 @@ void CWiggleReader::xGetPos(
             return;
         }
         else {
-        CObjReaderLineException err(
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Integer value expected");
-        ProcessError(err, pMessageListener);
+            "Integer value expected") );
+        ProcessError(*pErr, pMessageListener);
         }
     }
 }
@@ -724,7 +858,7 @@ bool CWiggleReader::xTryGetDoubleSimple(double& v)
 //  ----------------------------------------------------------------------------
 {
     double ret = 0;
-    const char* ptr = m_CurLine.data();
+    const char* ptr = m_CurLine.c_str();
     size_t skip = 0;
     bool negate = false, digits = false;
     for ( ; ; ++skip ) {
@@ -771,10 +905,7 @@ bool CWiggleReader::xTryGetDoubleSimple(double& v)
         }
         else if ( (c == ' ' || c == '\t' || c == '\0') && digits ) {
             m_CurLine.clear();
-            v = ret;
-            if ( negate ) {
-                ret = -ret;
-            }
+            v = (negate ? -ret : ret);
             return true;
         }
         else {
@@ -792,18 +923,19 @@ bool CWiggleReader::xTryGetDouble(
     if ( xTryGetDoubleSimple(v) ) {
         return true;
     }
-    const char* ptr = m_CurLine.data();
+    const char* ptr = m_CurLine.c_str();
     char* endptr = 0;
     v = strtod(ptr, &endptr);
     if ( endptr == ptr ) {
         return false;
     }
     if ( *endptr ) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Warning,
             0,
-            "Extra text on line");
-        ProcessError(err, pMessageListener);
+            "Extra text on line") );
+        ProcessError(*pErr, pMessageListener);
     }
     m_CurLine.clear();
     return true;
@@ -815,7 +947,7 @@ inline bool CWiggleReader::xTryGetPos(
     IMessageListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    char c = m_CurLine.data()[0];
+    char c = m_CurLine.c_str()[0];
     if ( c < '0' || c > '9' ) {
         return false;
     }
@@ -830,28 +962,15 @@ inline void CWiggleReader::xGetDouble(
 //  ----------------------------------------------------------------------------
 {
     if ( !xTryGetDouble(v, pMessageListener) ) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Floating point value expected");
-        ProcessError(err, pMessageListener);
+            "Floating point value expected") );
+        ProcessError(*pErr, pMessageListener);
     }
 }
 
-//  ----------------------------------------------------------------------------
-bool CWiggleReader::xGetLine(
-    ILineReader& lr)
-//  ----------------------------------------------------------------------------
-{
-    while (!lr.AtEOF()) {
-        m_CurLine = *++lr;
-        if (!xCommentLine()) {
-            return true;
-        }
-    }
-	return false;
-}
-
 //  ----------------------------------------------------------------------------
 CRef<CSeq_annot> CWiggleReader::xGetAnnot()
 //  ----------------------------------------------------------------------------
@@ -861,7 +980,8 @@ CRef<CSeq_annot> CWiggleReader::xGetAnnot()
     }
     CRef<CSeq_annot> pAnnot = xMakeAnnot();
     if (m_iFlags & fAsGraph) {
-        pAnnot->SetData().SetGraph().push_back(xMakeGraph());
+        //pAnnot->SetData().SetGraph().push_back(xMakeGraph());
+        pAnnot = xMakeGraphAnnot();
     }
     else {
         pAnnot->SetData().SetSeq_table(*xMakeTable());
@@ -877,7 +997,7 @@ void CWiggleReader::xDumpChromValues(void)
     if ( m_ChromId.empty() ) {
         return;
     }
-    LOG_POST("Chrom: "<<m_ChromId<<" "<<m_Values.size());
+    //LOG_POST("Chrom: "<<m_ChromId<<" "<<m_Values.size());
     if ( !m_Annot ) {
         m_Annot = xMakeAnnot();
     }
@@ -890,71 +1010,61 @@ void CWiggleReader::xDumpChromValues(void)
     if ( !m_SingleAnnot ) {
 //        xDumpAnnot();
     }
-    xResetChromValues();
+    //xResetChromValues();
 }
 
 //  ----------------------------------------------------------------------------
-void CWiggleReader::xSetChrom(CTempString chrom)
+void CWiggleReader::xSetChrom(
+    const string& chrom)
 //  ----------------------------------------------------------------------------
 {
     if ( chrom != m_ChromId ) {
         xDumpChromValues();
+        if (m_iFlags & CWiggleReader::fAsGraph) {
+            m_Values.clear();
+        }
         m_ChromId = chrom;
     }
 }
 
 //  ----------------------------------------------------------------------------
-void CWiggleReader::xReadBrowser(void)
+bool CWiggleReader::xProcessBrowserLine(
+    IMessageListener* pMl)
 //  ----------------------------------------------------------------------------
 {
+    if (!NStr::StartsWith(m_CurLine, "browser")) {
+        return false;
+    }
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
-void CWiggleReader::xReadTrack(
+bool CWiggleReader::xParseTrackLine(
+    const string& line,
     IMessageListener* pMessageListener)
 //  ----------------------------------------------------------------------------
 {
-    m_TrackName = "User Track";
-    m_TrackDescription.clear();
-    m_TrackTypeValue.clear();
+    if (!xIsTrackLine(line)) {
+        return false;
+    }
+    CReaderBase::xParseTrackLine(line, pMessageListener);
+
     m_TrackType = eTrackType_invalid;
-    m_TrackParams.clear();
-    while ( xSkipWS() ) {
-        CTempString name = xGetParamName(pMessageListener);
-        CTempString value = xGetParamValue(pMessageListener);
-        if ( name == "type" ) {
-            m_TrackTypeValue = value;
-            if ( value == "wiggle_0" ) {
-                m_TrackType = eTrackType_wiggle_0;
-            }
-            else if ( value == "bedGraph" ) {
-                m_TrackType = eTrackType_bedGraph;
-            }
-            else {
-                CObjReaderLineException err(
-                    eDiag_Warning,
-                    0,
-                    "Invalid track type");
-                ProcessError(err, pMessageListener);
-            }
-        }
-        else if ( name == "name" ) {
-            m_TrackName = value;
-        }
-        else if ( name == "description" ) {
-            m_TrackDescription = value;
-        }
-        else {
-            m_TrackParams[name] = value;
-        }
+    if (m_pTrackDefaults->Type() == "wiggle_0") {
+        m_TrackType = eTrackType_wiggle_0;
     }
-    if ( m_TrackType == eTrackType_invalid ) {
-        CObjReaderLineException err(
-            eDiag_Error,
+    else if (m_pTrackDefaults->Type() == "bedGraph") {
+        m_TrackType = eTrackType_bedGraph;
+    }
+    else {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Warning,
             0,
-            "Unknown track type");
-        ProcessError(err, pMessageListener);
+            "Invalid track type") );
+        ProcessError(*pErr, pMessageListener);
     }
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
@@ -965,11 +1075,12 @@ void CWiggleReader::xGetFixedStepInfo(
 {
     if ( m_TrackType != eTrackType_wiggle_0 ) {
         if ( m_TrackType != eTrackType_invalid ) {
-            CObjReaderLineException err(
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Warning,
                 0,
-                "Track \"type=wiggle_0\" is required");
-            ProcessError(err, pMessageListener);
+                "Track \"type=wiggle_0\" is required") );
+            ProcessError(*pErr, pMessageListener);
         }
         else {
             m_TrackType = eTrackType_wiggle_0;
@@ -978,8 +1089,8 @@ void CWiggleReader::xGetFixedStepInfo(
 
     fixedStepInfo.Reset();
     while ( xSkipWS() ) {
-        CTempString name = xGetParamName(pMessageListener);
-        CTempString value = xGetParamValue(pMessageListener);
+        string name = xGetParamName(pMessageListener);
+        string value = xGetParamValue(pMessageListener);
         if ( name == "chrom" ) {
             fixedStepInfo.mChrom = value;
         }
@@ -993,33 +1104,37 @@ void CWiggleReader::xGetFixedStepInfo(
             fixedStepInfo.mSpan = NStr::StringToUInt(value);
         }
         else {
-            CObjReaderLineException err(
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Warning,
                 0,
-                "Bad parameter name");
-            ProcessError(err, pMessageListener);
+                "Bad parameter name") );
+            ProcessError(*pErr, pMessageListener);
         }
     }
     if ( fixedStepInfo.mChrom.empty() ) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Missing chrom parameter");
-        ProcessError(err, pMessageListener);
+            "Missing chrom parameter") );
+        ProcessError(*pErr, pMessageListener);
     }
     if ( fixedStepInfo.mStart == 0 ) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Missing start value");
-        ProcessError(err, pMessageListener);
+            "Missing start value") );
+        ProcessError(*pErr, pMessageListener);
     }
     if ( fixedStepInfo.mStep == 0 ) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Missing step value");
-        ProcessError(err, pMessageListener);
+            "Missing step value") );
+        ProcessError(*pErr, pMessageListener);
     }
 }
 
@@ -1032,9 +1147,10 @@ void CWiggleReader::xReadFixedStepData(
 {
     xSetChrom(fixedStepInfo.mChrom);
     SValueInfo value;
+    value.m_Chrom = fixedStepInfo.mChrom;
     value.m_Pos = fixedStepInfo.mStart-1;
     value.m_Span = fixedStepInfo.mSpan;
-    while ( xGetLine(lr) ) {
+    while ( xGetLine(lr, m_CurLine) ) {
         if ( !xTryGetDouble(value.m_Value, pMessageListener) ) {
             lr.UngetLine();
             break;
@@ -1052,11 +1168,12 @@ void CWiggleReader::xGetVarStepInfo(
 {
     if ( m_TrackType != eTrackType_wiggle_0 ) {
         if ( m_TrackType != eTrackType_invalid ) {
-            CObjReaderLineException err(
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Warning,
                 0,
-                "Track \"type=wiggle_0\" is required");
-            ProcessError(err, pMessageListener);
+                "Track \"type=wiggle_0\" is required") );
+            ProcessError(*pErr, pMessageListener);
         }
         else {
             m_TrackType = eTrackType_wiggle_0;
@@ -1065,8 +1182,8 @@ void CWiggleReader::xGetVarStepInfo(
 
     varStepInfo.Reset();
     while ( xSkipWS() ) {
-        CTempString name = xGetParamName(pMessageListener);
-        CTempString value = xGetParamValue(pMessageListener);
+        string name = xGetParamName(pMessageListener);
+        string value = xGetParamValue(pMessageListener);
         if ( name == "chrom" ) {
             varStepInfo.mChrom = value;
         }
@@ -1074,19 +1191,21 @@ void CWiggleReader::xGetVarStepInfo(
             varStepInfo.mSpan = NStr::StringToUInt(value);
         }
         else {
-            CObjReaderLineException err(
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
                 eDiag_Warning,
                 0,
-                "Bad parameter name");
-            ProcessError(err, pMessageListener);
+                "Bad parameter name") );
+            ProcessError(*pErr, pMessageListener);
         }
     }
     if ( varStepInfo.mChrom.empty() ) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Error,
             0,
-            "Missing chrom parameter");
-        ProcessError(err, pMessageListener);
+            "Missing chrom parameter") );
+        ProcessError(*pErr, pMessageListener);
     }
 }
 
@@ -1099,8 +1218,9 @@ void CWiggleReader::xReadVariableStepData(
 {
     xSetChrom(varStepInfo.mChrom);
     SValueInfo value;
+    value.m_Chrom = varStepInfo.mChrom;
     value.m_Span = varStepInfo.mSpan;
-    while ( xGetLine(lr) ) {
+    while ( xGetLine(lr, m_CurLine) ) {
         if ( !xTryGetPos(value.m_Pos, pMessageListener) ) {
             lr.UngetLine();
             break;
@@ -1114,17 +1234,18 @@ void CWiggleReader::xReadVariableStepData(
 
 //  =========================================================================
 void CWiggleReader::xReadBedLine(
-    CTempString chrom,
+    const string& chrom,
     IMessageListener* pMessageListener)
 //  =========================================================================
 {
     if ( m_TrackType != eTrackType_bedGraph &&
         m_TrackType != eTrackType_invalid ) {
-        CObjReaderLineException err(
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
             eDiag_Warning,
             0,
-            "Track \"type=bedGraph\" is required");
-        ProcessError(err, pMessageListener);
+            "Track \"type=bedGraph\" is required") );
+        ProcessError(*pErr, pMessageListener);
     }
     xSetChrom(chrom);
     SValueInfo value;
