@@ -1,4 +1,4 @@
-/*  $Id: variant.cpp 103491 2007-05-04 17:18:18Z kazimird $
+/*  $Id: variant.cpp 358154 2012-03-29 15:05:12Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -815,12 +815,6 @@ void CVariantInfo::SetPathWriteHook(CObjectOStream* out, const string& path,
     m_WriteHookData.SetPathHook(out,path,hook);
 }
 
-void CVariantInfo::SetGlobalSkipHook(CSkipChoiceVariantHook* hook)
-{
-    CMutexGuard guard(GetTypeInfoMutex());
-    m_SkipHookData.SetGlobalHook(hook);
-}
-
 void CVariantInfo::SetLocalSkipHook(CObjectIStream& stream,
                                     CSkipChoiceVariantHook* hook)
 {
@@ -828,12 +822,6 @@ void CVariantInfo::SetLocalSkipHook(CObjectIStream& stream,
     m_SkipHookData.SetLocalHook(stream.m_ChoiceVariantSkipHookKey, hook);
 }
 
-void CVariantInfo::ResetGlobalSkipHook(void)
-{
-    CMutexGuard guard(GetTypeInfoMutex());
-    m_SkipHookData.ResetGlobalHook();
-}
-
 void CVariantInfo::ResetLocalSkipHook(CObjectIStream& stream)
 {
     CMutexGuard guard(GetTypeInfoMutex());
