@@ -1,4 +1,4 @@
-/*  $Id: dictionary_util.cpp 369170 2012-07-17 13:20:38Z ivanov $
+/*  $Id: dictionary_util.cpp 403742 2013-06-18 15:26:09Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -627,7 +627,7 @@ private:
 
 static inline ECharType s_GetCharType(int c)
 {
-    static CSafeStaticPtr<CFillTypes> fill_types;
+    static CSafeStatic<CFillTypes> fill_types;
     _ASSERT(c < 256  &&  c >= 0);
     return fill_types->GetChar(c);
 }
