@@ -1,4 +1,4 @@
-/*  $Id: format_guess.cpp 390749 2013-03-01 18:22:26Z rafanovi $
+/*  $Id: format_guess.cpp 448875 2014-10-09 16:36:04Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -66,7 +66,19 @@ static unsigned char symbol_type_table[256];
 static bool s_IsTokenPosInt(
     const string& strToken )
 {
-    return ( -1 != NStr::StringToNonNegativeInt( strToken ) );
+    size_t tokenSize = strToken.size();
+    if (tokenSize == 0) {
+        return false;
+    }
+    if (strToken[0] < '1'  ||  '9' < strToken[0]) {
+        return false;
+    }
+    for (size_t i=1; i<tokenSize; ++i) {
+        if (strToken[i] < '0'  ||  '9' < strToken[i]) {
+            return false;
+        }
+    }
+    return true;
 }
 
 //  ----------------------------------------------------------------------------
@@ -157,9 +169,9 @@ CFormatGuess::s_CheckOrder[] =
     eAgp,
     eXml,
     eWiggle,
+    eNewick,
     eBed,
     eBed15,
-    eNewick,
     eHgvs,
     eAlignment,
     eDistanceMatrix,
@@ -172,6 +184,7 @@ CFormatGuess::s_CheckOrder[] =
     ePhrapAce,
     eTable,
     eBinaryASN,
+    eUCSCRegion,
 };
 
 
@@ -187,9 +200,9 @@ const char* const CFormatGuess::sm_FormatNames[CFormatGuess::eFormat_max] = {
     "AGP",
     "XML",
     "WIGGLE",
+    "Newick",
     "BED",
     "BED15",
-    "Newick",
     "alignment",
     "distance matrix",
     "flat-file sequence",
@@ -212,6 +225,7 @@ const char* const CFormatGuess::sm_FormatNames[CFormatGuess::eFormat_max] = {
     "SRA",
     "BAM",
     "VCF",
+    "UCSC Region"
 };
 
 const char*
@@ -332,7 +346,7 @@ CFormatGuess::CFormatGuess()
 //  ----------------------------------------------------------------------------
 CFormatGuess::CFormatGuess(
     const string& FileName )
-    : m_Stream( * new CNcbiIfstream( FileName.c_str() ) )
+    : m_Stream( * new CNcbiIfstream( FileName.c_str(), ios::binary ) )
     , m_bOwnsStream( true )
 {
     Initialize();
@@ -442,14 +456,14 @@ bool CFormatGuess::x_TestFormat(EFormat format, EMode mode)
         return TestFormatAgp( mode );
     case eXml:
         return TestFormatXml( mode );
+    case eNewick:
+        return TestFormatNewick( mode );
     case eWiggle:
         return TestFormatWiggle( mode );
     case eBed:
         return TestFormatBed( mode );
     case eBed15:
         return TestFormatBed15( mode );
-    case eNewick:
-        return TestFormatNewick( mode );
     case eAlignment:
         return TestFormatAlignment( mode );
     case eDistanceMatrix:
@@ -486,9 +500,12 @@ bool CFormatGuess::x_TestFormat(EFormat format, EMode mode)
         return TestFormatBam( mode );
     case eVcf:
         return TestFormatVcf( mode );
+    case eUCSCRegion:
+        return false;
     default:
         NCBI_THROW( CCoreException, eInvalidArg,
-            "CFormatGuess::x_TestFormat(): Unsupported format ID." );
+            "CFormatGuess::x_TestFormat(): Unsupported format ID (" +
+            NStr::NumericToString((int)format) + ")." );
     }
 }
 
@@ -715,6 +732,9 @@ CFormatGuess::TestFormatGvf(
         //  start of data
         //
         if ( it->empty() || (*it)[0] == '#' ) {
+            if (NStr::StartsWith(*it, "##gvf-version")) {
+                return true;
+            }
 			continue;
 		}
 		if ( !uGvfLineCount && NStr::StartsWith( *it, "browser " ) ) {
@@ -840,7 +860,11 @@ CFormatGuess::TestFormatAgp(
         return false;
     }
     ITERATE( list<string>, it, m_TestLines ) {
-        if ( !IsLineAgp( *it ) ) {
+        try {
+            if ( !IsLineAgp( *it ) ) {
+                return false;
+            }
+        } catch(...) {
             return false;
         }
     }
@@ -853,6 +877,41 @@ CFormatGuess::TestFormatNewick(
     EMode /* not used */ )
 {
 //  -----------------------------------------------------------------------------
+    // newick trees can be found in nexus files. check for that first as a special case
+    if ( ! EnsureTestBuffer() || ! EnsureSplitLines() ) {
+        const int BUFFSIZE = 8096;
+        m_pTestBuffer = new char[BUFFSIZE];
+        m_Stream.read( m_pTestBuffer, BUFFSIZE );
+        m_iTestDataSize = m_Stream.gcount();
+        m_Stream.clear();  // in case we reached eof
+        CStreamUtils::Stepback( m_Stream, m_pTestBuffer, m_iTestDataSize );
+        m_TestLines.push_back(m_pTestBuffer);
+    }
+
+    // Note: We can live with false negatives. Avoid false positives
+    // at all cost.
+
+    bool is_nexus = false;
+    bool has_trees = false;
+    ITERATE( list<string>, it, m_TestLines ) {
+        if ( NPOS != it->find( "#NEXUS" ) ) {
+            is_nexus = true;
+        }
+
+        // case insensitive
+        if (NPOS != NStr::FindNoCase(*it, "begin trees;")) { 
+            has_trees = true;
+        }
+    }
+
+    // In a nexus file with a tree, we will just read in the tree (ignoring for now
+    // the alignment)
+    if (is_nexus ) {
+        if (has_trees)
+            return true;
+        return false;
+    }
+
     //  special newick consideration:
     //  newick files may come with all data cramped into a single run-on line,
     //  that single oversized line may not have a line terminator
@@ -1073,13 +1132,6 @@ CFormatGuess::TestFormatAlignment(
     list<string>::const_iterator iter = m_TestLines.begin();
     list<string> toks;
 
-    // Merge delims if > 1.  Do not merge single delims (since they could 
-    // more easily represent blank fields
-    NStr::EMergeDelims  merge_delims = NStr::eMergeDelims;
-    if (delims.size() == 1)
-        merge_delims = NStr::eNoMergeDelims;
-
-
     // Skip initial lines since not all headers start with comments like # or ;:
     // Don't skip though if file is very short - add up to 3, 1 for each line 
     // over 5:
@@ -1423,8 +1475,14 @@ bool
 CFormatGuess::TestFormatHgvs(
     EMode /* not used */ )
 {
-    if ( ! EnsureTestBuffer() || ! EnsureSplitLines() ) {
-        return false;
+    if ( ! EnsureStats() || ! EnsureSplitLines() ) {
+        const int BUFFSIZE = 1024;
+        m_pTestBuffer = new char[BUFFSIZE];
+        m_Stream.read( m_pTestBuffer, BUFFSIZE );
+        m_iTestDataSize = m_Stream.gcount();
+        m_Stream.clear();  // in case we reached eof
+        CStreamUtils::Stepback( m_Stream, m_pTestBuffer, m_iTestDataSize );
+        m_TestLines.push_back(m_pTestBuffer);
     }
 
     unsigned int uHgvsLineCount = 0;
@@ -2021,6 +2079,7 @@ bool CFormatGuess::IsLineGtf(
 //  ----------------------------------------------------------------------------
 bool CFormatGuess::IsLineGvf(
     const string& line )
+//  ----------------------------------------------------------------------------
 {
     vector<string> tokens;
     if ( NStr::Tokenize( line, " \t", tokens, NStr::eMergeDelims ).size() < 8 ) {
@@ -2032,7 +2091,10 @@ bool CFormatGuess::IsLineGvf(
     if ( ! s_IsTokenPosInt( tokens[4] ) ) {
         return false;
     }
+
+    //make sure that "type" is a GVF admissible value:
 	{{
+        bool typeOk = false;
 		list<string> terms;
 		terms.push_back("snv");
 		terms.push_back("cnv");
@@ -2060,9 +2122,14 @@ bool CFormatGuess::IsLineGvf(
 		terms.push_back("structural_variation");
 		terms.push_back("sequence_alteration");
 		ITERATE(list<string>, termiter, terms) {
-			if(NStr::EqualNocase(*termiter, tokens[2]))
-				return true;
+			if(NStr::EqualNocase(*termiter, tokens[2])) {
+				typeOk = true;
+                break;
+            }
 		}
+        if (!typeOk) {
+            return false;
+        }
 	}}
 	if ( ! s_IsTokenDouble( tokens[5] ) ) {
         return false;
@@ -2073,19 +2140,15 @@ bool CFormatGuess::IsLineGvf(
     if ( tokens[7].size() != 1 || NPOS == tokens[7].find_first_of( ".0123" ) ) {
         return false;
     }
-	if(tokens.size() >= 9) {
-		list<string> terms;
-		terms.push_back("start_range");
-		terms.push_back("end_range");
-		terms.push_back("variant_seq");
-		terms.push_back("genotype");
-		ITERATE(list<string>, termiter, terms) {
-			if(NStr::EqualNocase(*termiter, tokens[8]))
-				return true;
-		}
-	}
 
-    return false;
+    //make sure all the mandatory attributes are present:
+    string attrs = tokens[8];
+    if (string::npos == attrs.find("ID="))
+        return false;
+    if (string::npos == attrs.find("Variant_seq=")) {
+        return false;
+    }
+    return true;
 }
 
 
@@ -2324,8 +2387,10 @@ CFormatGuess::EnsureSplitLines()
 
 //  ----------------------------------------------------------------------------
 bool
-CFormatGuess::IsAllComment()
+CFormatGuess::IsAsciiText()
 {
+    const double REQUIRED_ASCII_RATIO = 0.9;
+
     // first stab - are we text?  comments are only valid if we are text
     size_t count = 0;
     size_t count_print = 0;
@@ -2334,10 +2399,19 @@ CFormatGuess::IsAllComment()
             ++count_print;
         }
     }
-    if (count_print < count * 0.9) {
-        // 10% non-printing at least; likely not text
+    if (count_print < count * REQUIRED_ASCII_RATIO) {
 		return false;
     }
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+bool
+CFormatGuess::IsAllComment()
+{
+    if (!IsAsciiText()) {
+        return false;
+    }
 
     m_bSplitDone = false;
     m_TestLines.clear();
@@ -2366,7 +2440,7 @@ bool CFormatGuess::IsLineHgvs(
     const string& line )
 {
     // This simple check can mistake Newwick, so Newwick is checked first
-    //  /:(g|c|r|p|m|mt|n)\./  as in NC_000001.9:g.1234567C>T
+    //  /[:alnum:]+:(g|c|r|p|m|mt|n)\.[:alnum:]+/  as in NC_000001.9:g.1234567C>T
     int State = 0;
     ITERATE(string, Iter, line) {
         char Char = *Iter;
@@ -2377,27 +2451,37 @@ bool CFormatGuess::IsLineHgvs(
             Next = *NextI;
         
         if(State == 0) {
-            if(Char == ':')
-                State = 1;
+            if( isalnum(Char) )
+                State++;
         } else if(State == 1) {
+            if(Char == ':')
+                State++;
+        } else if(State == 2) {
             if (Char == 'g' ||
                 Char == 'c' ||
                 Char == 'r' ||
                 Char == 'p' ||
                 Char == 'n' ||
                 Char == 'm' ) {
-                State = 2;
+                State++;
                 if (Char=='m' && Next == 't') {
                     ++Iter;
                 }
+            } else {
+                return false;
             }
-        } else if(State == 2) {
+        } else if(State == 3) {
             if(Char == '.') 
-                State = 3;
+                State++;
+            else
+                return false;
+        } else if(State == 4) {
+            if( isalnum(Char) )
+                State++;
         }
     }
     
-    return (State == 3);    
+    return (State == 5);    
 }
 
 
