@@ -1,4 +1,4 @@
-/*  $Id: smalldns.cpp 369170 2012-07-17 13:20:38Z ivanov $
+/*  $Id: smalldns.cpp 403742 2013-06-18 15:26:09Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -117,7 +117,7 @@ string CSmallDNS::GetLocalIP(void) const
 
 string CSmallDNS::GetLocalHost(void)
 {
-    static CSafeStaticPtr<string> s_LocalHostName;
+    static CSafeStatic<string> s_LocalHostName;
 
     if ( s_LocalHostName->empty() ) {
 #if !defined(MAXHOSTNAMELEN)
